/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.5 $
 * $Date: 2004/07/27 20:37:26 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graph.engine;

import java.util.*;
import java.awt.*;
import java.awt.geom.*;

import org.concord.graph.event.*;

/**
 * @author Ingrid Moncada<p>
*/

public class AxisScale extends DefaultSelectable
	implements MouseControllable, Cursorable
{
	//Temporary information to store the state when started dragging
	/**
	 * Point where the mouse was clicked when started dragging operation
	 * in WORLD coordinates
	 */
	protected Point2D startLocationW;
	/**
	 * Point where the mouse was clicked when started dragging operation
	 * in DISPLAY coordinates
	 */
	protected Point2D startLocationD;
	/**
	 * Original scale when started the dragging operation 
	 */
	protected CoordinateSystem startCoord;
	
	private boolean mouseClicked = false;
	
	/**
	 * Drag Modes
	 */
	public static final int DRAGMODE_NONE = 0;			//Move axis origin (pan)
	public static final int DRAGMODE_TRANSLATE = 1;		//Move axis origin (pan)
	public static final int DRAGMODE_TRANSLATE_X = 6;	//Move X axis
	public static final int DRAGMODE_TRANSLATE_Y = 7;	//Move Y axis
	public static final int DRAGMODE_DILATE = 2;		//Scale x and y axis
	public static final int DRAGMODE_DILATE_X = 8;		//Scale X axis
	public static final int DRAGMODE_DILATE_Y = 9;		//Scale Y Axis
	public static final int DRAGMODE_ZOOM = 13;		//Zoom 
	public static final int DRAGMODE_ZOOM_IN = 14;		//Zoom in
	public static final int DRAGMODE_ZOOM_OUT = 15;		//Zoom out

	private int dragMode;

	private Cursor cursor;
	private Vector cursorListeners;
	
	private boolean showMessage = true;
	private boolean showCover = true;
	
	/** 
	 * Draws this object on Graphics g. 
	 * The graph area's coordinate system
	 * shoule be used to transform coordinates.
	 */
	public AxisScale()
	{
		cursorListeners = new Vector();
		dragMode = DRAGMODE_NONE;
	}
	
	/** 
	 * Draws this object on Graphics g. 
	 * The graph area's coordinate system
	 * shoule be used to transform coordinates.
	 */
	public void draw(Graphics2D g)
	{
		if (dragMode == DRAGMODE_NONE) return;
		
		if (graphArea == null) return;

		if (showCover){
			Composite oldComp = g.getComposite();
			
			//g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
			//g.setColor(new Color(110,110,128));
			
			g.setColor(new Color(150,150,160,128));
			g.fillRect(0,0,(int)graphArea.getSize().getWidth(),
				(int)graphArea.getSize().getHeight());
			
			g.setComposite(oldComp);
		}
		
		if (showMessage){
			String strMessage = "";
			if (dragMode == DRAGMODE_DILATE_X){
				strMessage = getString("MessageDilateX");
			}
			else if (dragMode == DRAGMODE_DILATE_Y){
				strMessage = getString("MessageDilateY");
			}
			
			if (!strMessage.equals("")){
				
				Color oldColor = g.getColor();
				
				FontMetrics fm = g.getFontMetrics();
				int w = fm.stringWidth(strMessage);
				int h = fm.getHeight();
				Point2D right = graphArea.getUpperRightCornerDisplay();
				Point2D low = graphArea.getLowerLeftCornerDisplay();
				int px = (int)(right.getX() - w - 30);
				int py = (int)(low.getY() - 30);
				
				//Box under the message
				g.setColor(Color.white);
				g.fillRect(px, py - (h*3) / 4, w, h);

				//Message
				g.setColor(Color.black);
				g.drawString(strMessage, px, py);
				
				g.setColor(oldColor);
			}
		}
/*			
		CoordinateSystem coord = graphArea.getCoordinateSystem();

			Point2D origin = coord.getOriginDisplay();
			Point2D left = graphArea.getLowerLeftCornerDisplay();
			Point2D right = graphArea.getUpperRightCornerDisplay();
			
			Color oldColor = g.getColor();
			
			g.setColor(Color.yellow);
			
			Rectangle2D rect = new Rectangle2D.Double(left.getX(),origin.getY()-10,right.getX()-left.getX(),20.0);
			//g.fillRect((int)left.getX(),(int)origin.getY()-10,(int)right.getX(),(int)origin.getY()+10);
			g.draw(rect);
			
			g.setColor(oldColor);
*/
	}

	/**
	 * Checks if the location is within enough proximity
	 * If the location is close enough of the object, it
	 * returns true. 
	 * The location is given in screen coordinates. 
	 * The axis is provided to transform coordinates.
	 */
	public boolean isPointInProximity(Point location)
	{
		if (dragMode == DRAGMODE_NONE) return false;

/*
		double d = 15.0;
				
		if (dragMode == this.DRAGMODE_TRANSLATE){
			return true;
		}
		if (dragMode == this.DRAGMODE_DILATE){
			if (isPointInXAxis(location, d) || isPointInYAxis(location, d)){
				return true;
			}
			return false;
		}
		if (dragMode == this.DRAGMODE_DILATE_X || dragMode == this.DRAGMODE_TRANSLATE_X){
			if (isPointInXAxis(location, d)){
				return true;
			}
			return false;
		}
		if (dragMode == this.DRAGMODE_DILATE_Y || dragMode == this.DRAGMODE_TRANSLATE_Y){
			if (isPointInYAxis(location, d)){
				return true;
			}
			return false;
		}
*/		
		return true;
	}

	/**
	 * Returns if p is near the X axis
	*/
	private boolean isPointInXAxis(Point p, double threshold)
	{
		if (graphArea == null) return false;
		
		CoordinateSystem coord = graphArea.getCoordinateSystem();

		Point2D origin = coord.getOriginOffsetDisplay();
		if (Math.abs(p.getY() - origin.getY()) <= threshold ){
			return true;
		}
		return false;
	}

	/**
	 * Returns if p is near the Y axis
	*/
	private boolean isPointInYAxis(Point p, double threshold)
	{
		if (graphArea == null) return false;

		CoordinateSystem coord = graphArea.getCoordinateSystem();

		Point2D origin = coord.getOriginOffsetDisplay();
		if (Math.abs(p.getX() - origin.getX()) <= threshold ){
			return true;
		}
		return false;
	}

	/**
	 * Handles the mouse press event. 
	 * The location is given in screen coordinates. 
	 * The axis is provided to transform coordinates. 
	 */
	public boolean mousePressed(Point location)
	{
		if (graphArea == null) return false;

		CoordinateSystem coord = graphArea.getCoordinateSystem();
		
		coord.setChanging(true);
		startCoord = coord.getCopy();
		startLocationD = location;
		startLocationW = coord.transformToWorld(location);

		mouseClicked = true;
		return true;
	}
	
	/** Handles the mouse drag event. 
	 * The location is given in screen coordinates. 
	 * The axis is provided to transform coordinates. 
	 */
	public boolean mouseDragged(Point location)
	{
		if (graphArea == null) return false;

		CoordinateSystem coord = graphArea.getCoordinateSystem();
				
		if (dragMode == DRAGMODE_DILATE || dragMode == DRAGMODE_DILATE_X ||
				dragMode == DRAGMODE_DILATE_Y){

			Point2D dragLocation = startCoord.transformToWorld(location);
			Point2D startScale = startCoord.getScale();
			double newScaleX = startScale.getX();
			double newScaleY = startScale.getY();
			double checkedScaleX = newScaleX;
			double checkedScaleY = newScaleY;
			
			if (dragMode == DRAGMODE_DILATE || dragMode == DRAGMODE_DILATE_X){
				newScaleX = (dragLocation.getX() * startScale.getX()) / startLocationW.getX();
				checkedScaleX = (newScaleX > 0.0) ? newScaleX : checkedScaleX;
				
			}
			if (dragMode == DRAGMODE_DILATE || dragMode == DRAGMODE_DILATE_Y){
				newScaleY = (dragLocation.getY() * startScale.getY()) / startLocationW.getY();

				checkedScaleY = (newScaleX > 0.0) ? newScaleY : checkedScaleY;
			}
			
			coord.setScale(new Point2D.Double(newScaleX, newScaleY));
			
			return true;
		}
		else if (dragMode == DRAGMODE_TRANSLATE || dragMode == DRAGMODE_TRANSLATE_X ||
				dragMode == DRAGMODE_TRANSLATE_Y){
			
			Point2D startOrigin = startCoord.getOriginOffsetDisplay();			
			double newOriginX = startOrigin.getX();
			double newOriginY = startOrigin.getY();

			if (dragMode == DRAGMODE_TRANSLATE || dragMode == DRAGMODE_TRANSLATE_X){
				newOriginX = startOrigin.getX() + (location.getX() - startLocationD.getX());
			}
			if (dragMode == DRAGMODE_TRANSLATE || dragMode == DRAGMODE_TRANSLATE_Y){
				newOriginY = startOrigin.getY() + (location.getY() - startLocationD.getY());
			}
			
			coord.setOriginOffsetDisplay(new Point2D.Double(newOriginX, newOriginY));
		}
		else if (dragMode == DRAGMODE_ZOOM){
			
			Point2D dragLocation = startCoord.transformToWorld(location);
			Point2D startScale = startCoord.getScale();
			double newScaleX = startScale.getX();
			double newScaleY = startScale.getY();
			
			if (startLocationW.getY() == 0.0){
				return true;
			}
				
			newScaleX = (dragLocation.getY() * startScale.getX()) / startLocationW.getY();
			newScaleY = (dragLocation.getY() * startScale.getY()) / startLocationW.getY();
			
//			coord.setScale(new Point2D.Double(newScaleX, newScaleY));

			//startLocationW should be in the same startLocationD
			//This only works for linear transformations
			Point2D startOrigin = startCoord.getOriginOffsetDisplay();			
			Point2D startOriginOffsetWorld = startCoord.transformToWorld(new Point2D.Double(0.0,0.0));
			
			startOriginOffsetWorld.setLocation(startLocationW);
			
			Point2D originOffset = coord.transformToDisplay(startOriginOffsetWorld);
			Point2D newOrigin = new Point2D.Double(startOrigin.getX()-originOffset.getX(), startOrigin.getY()-originOffset.getY());

//System.out.println("-- startOriginOffsetWorld:"+startOriginOffsetWorld);
//System.out.println("-- originOffset:"+originOffset);
//System.out.println("origin old:"+startCoord.getOriginDisplay()+" new:"+newOrigin);
			
			coord.setScale(new Point2D.Double(newScaleX, newScaleY));
			coord.setOriginOffsetDisplay(newOrigin);
			
			return true;
		}
		
		return false;
	}
	
	/** Handles the mouse released event. 
	 * The location is given in screen coordinates. 
	 * The axis is provided to transform coordinates. 
	 */
	public boolean mouseReleased(Point location)
	{
		if (graphArea == null) return false;

		mouseClicked = false;
		
		CoordinateSystem coord = graphArea.getCoordinateSystem();
		coord.setChanging(false);

		return true;
	}
	
	/**  
	 * Returns if the object is currently being
	 * controlled by the mouse 
	 */
	public boolean isMouseControlled()
	{
		return mouseClicked;
	}

	/** 
	 * Returns a copy of itself
	 * Not implemented yet
	 */
	public Graphable getCopy()
	{
		return null;
	}
	
	/**
	 * Sets the drag mode 
	*/
	public void setDragMode(int dragMode)
	{
		this.dragMode = dragMode;
		
		if (dragMode == DRAGMODE_NONE){
			setCursor(null);
		}
		else if (dragMode == DRAGMODE_TRANSLATE || dragMode == DRAGMODE_TRANSLATE_X ||
				dragMode == DRAGMODE_TRANSLATE_Y){
			setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
		}
		else if (dragMode == DRAGMODE_DILATE){
			//TEMP XXX FIXME
			//DILATE doesn't work yet, so:
//			setDragMode(DRAGMODE_NONE);
			
			setCursor(Cursor.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR));
		}
		else if (dragMode == DRAGMODE_DILATE_X){
			setCursor(Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR));
		}
		else if (dragMode == DRAGMODE_DILATE_Y){
			setCursor(Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
		}
		else if (dragMode == DRAGMODE_ZOOM){
			setCursor(Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR));
		}
		else if (dragMode == DRAGMODE_ZOOM_IN){
			setCursor(Cursor.getPredefinedCursor(Cursor.SW_RESIZE_CURSOR));
		}
		else if (dragMode == DRAGMODE_ZOOM_OUT){
			setCursor(Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR));
		}
		else{
			setCursor(null);
		}
		
		notifyChange();
	}

	/**
	 * Returns the drag mode of this object
	*/
	public int getDragMode()
	{
		return this.dragMode;
	}
	
	public void setShowMessage(boolean show)
	{
		showMessage = show;
	}
	
	public void setShowCover(boolean show)
	{
		showCover = show;
	}

	/////////////////////////
	// Cursorable interface
	/////////////////////////
	/**
	 * Sets the cursor that should be shown when the mouse is over
	 */
	public void setCursor(Cursor cursor)
	{
		if (this.cursor != cursor){
			this.cursor = cursor;
			notifyCursorListeners();
		}
	}
	
	/**
	 * Returns the cursor that should be shown when the mouse is over
	 */
	public Cursor getCursor()
	{
		return cursor;
	}

	/**
	 * Adds a listener that will listen to a cursor change
	 */
	public void addCursorListener(CursorListener l)
	{
		cursorListeners.add(l);
	}

	/**
	 * Removes a cursor change listener 
	 */
	public void removeCursorListener(CursorListener l)
	{
		cursorListeners.remove(l);
	}
	
	protected void notifyCursorListeners()
	{
		CursorListener l;
		for (int i=0;i<cursorListeners.size();i++){
			l = (CursorListener)cursorListeners.elementAt(i);
			l.cursorChanged(new EventObject(this));
		}
	}
	
	/**
	 * STRINGS --- This should be moved to a resource file!!!
	 * (for Localization)
	 */
	private String getString(String strStringName)
	{
		if (strStringName.equals("MessageDilateX")){
			return "Click and drag on the graph to scale the X axis";
		}
		if (strStringName.equals("MessageDilateY")){
			return "Click and drag on the graph to scale the Y axis";
		}
		return "";
	}
}

