/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/27 07:10:09 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graph.engine;

import java.awt.geom.Point2D;

import javax.swing.event.*;

/**
 * Coordinate system interface for transformation between world coordinates
 * and display coordinates<br>
 *
 * Date created: 04/07/2004
 *
 * @author Ingrid C Moncada<p>
 *
 */

public interface CoordinateSystem
{
	/*
	 * Transform From display coordinates To world coordinates 
	 */
	public Point2D transformToWorld(Point2D d);
	
	/*
	 * Transform From display coordinates To world coordinates 
	 */
	public Point2D transformToWorld(Point2D d, Point2D w);
	
	/*
	 * Transform From world coordinates To display coordinates 
	 */
	public Point2D transformToDisplay(Point2D w);
	
	/*
	 * Transform From world coordinates To display coordinates 
	 */
	public Point2D transformToDisplay(Point2D w, Point2D d);

	/*
	 * Sets the scale (# of display coordinates in each world coordinate) 
	 * FIXME: Note that this method is only valid for linear coordinate systems
	 * (Should we make a subinterface?)
	 */
	public void setScale(Point2D p);

	/*
	 * Returns the scale (# of display coordinates in each world coordinate) 
	 * FIXME: Note that this method is only valid for linear coordinate systems
	 * (Should we make a subinterface?)
	 */
	public Point2D getScale();

	/*
	 * Sets the origin in display coordinates 
	 * The origin coordinates are interpreted AS
	 * the distance between the world origin (0,0)
	 * and the upper left corner of the graph area
	 * in display coordinates
	 */
	public void setOriginOffsetDisplay(Point2D p);

	/*
	 * Returns the origin in display coordinates 
	 * The origin coordinates are interpreted AS
	 * the distance between the world origin (0,0)
	 * and the upper left corner of the graph area
	 * in display coordinates
	 */
	public Point2D getOriginOffsetDisplay();

	/**
	 * Add a listener to any change in the coordinate system
	 */
	public void addChangeListener(ChangeListener l);

	/**
	 * Removes a listener to changes in the coordinate system
	 */
	public void removeChangeListener(ChangeListener l);

	/**
	 * Returns a copy of itself
	 */
	public CoordinateSystem getCopy();
	
	/**
	 * Returns if the coordinate system is currently being changed 
	 */
	public boolean isChanging();

	/**
	 * Sets that the coordinate system is currently being changed or not 
	 */
	public void setChanging(boolean b);
}
    
