/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/*
 * Last modification information:
 * $Revision: 1.1 $
 * $Date: 2004/06/07 17:55:42 $
 * $Author: imoncada $
 *
 * Licence Information
 * Copyright 2004 The Concord Consortium 
*/
package org.concord.graph.engine;

import java.awt.Point;
import java.awt.geom.Point2D;


/**
 * A DefaultControllable is a MouseControllable,
 * Selectable, MouseSensitive Graphable object
 * that has a location and is able to move
 * (translate) with mouse drag events.
 *
 * Date created: May 28, 2004
 *
 * @author imoncada<p>
 *
 */
public abstract class DefaultControllable extends DefaultSelectable
	implements MouseControllable
{
	private boolean mouseClicked = false;	//was the mouse pressed in this graphable
	
	protected Point2D location;	//location of the object in WORLD coordinates
	
	protected Point2D clickPointDisplay;	//point where the mouse was clicked in display coordinates
	protected Point2D clickPointWorld;		//point where the mouse was clicked in world coordinates
	protected Point2D originalLocation;		//location of the object when the mouse was clicked in world coordinates
	
	/**
	 * 
	 */
	public DefaultControllable()
	{
		super();
		
		location = new Point2D.Double();
		clickPointWorld = new Point2D.Double();
		originalLocation = new Point2D.Double();
	}

	/**
	 * @see org.concord.graph.engine.MouseControllable#mousePressed(java.awt.Point)
	 */
	public boolean mousePressed(Point p)
	{
		CoordinateSystem coord = graphArea.getCoordinateSystem();
		
		mouseClicked = true;
		//Store location of the mouse
		clickPointDisplay = p;
		clickPointWorld = coord.transformToWorld(p, clickPointWorld);
		//Store my original location
		originalLocation.setLocation(this.location);
		
		return true;
	}

	/**
	 * @see org.concord.graph.engine.MouseControllable#mouseDragged(java.awt.Point)
	 */
	public boolean mouseDragged(Point p)
	{
		Point2D newLocation;
		
		newLocation = graphArea.getCoordinateSystem().transformToWorld(p);
		
		Point2D newP = new Point2D.Double(originalLocation.getX() + (newLocation.getX() - clickPointWorld.getX()),
			originalLocation.getY() + (newLocation.getY() - clickPointWorld.getY()));
			
		setLocation(newP);
		
		return true;
	}

	/**
	 * @see org.concord.graph.engine.MouseControllable#mouseReleased(java.awt.Point)
	 */
	public boolean mouseReleased(Point p)
	{
		mouseClicked = false;
		return true;
	}

	/**
	 * @see org.concord.graph.engine.MouseControllable#isMouseControlled()
	 */
	public boolean isMouseControlled()
	{
		return mouseClicked;
	}

	/**
	 * returns the location of the object
	 * @return
	 */
	public Point2D getLocation()
	{
		return this.location;
	}
	
	/**
	 * sets the location of the object
	 * @param location
	 */
	public void setLocation(Point2D location)
	{
		this.location = location;
		notifyChange();
	}
}
