/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/27 07:10:09 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graph.engine;

import java.util.Vector;
import java.util.Enumeration;
import java.util.EventObject;

import org.concord.graph.event.*;

/**
 * A default Selectable is a MouseSensitive Selectable Graphable
 * that implements all the basic methods of these interfaces
 * The only ones left are draw(), getCopy(), isPointInProximity()
 *
 * Date created: 04/07/2004
 *
 * @author Scott Cytacki<p>
 *
 */

public abstract class DefaultSelectable extends DefaultGraphable
	implements MouseSensitive, Selectable
{
	//////////////////////////////////////////////////////
	//Properties for Selectable interface
	protected Vector selectableListeners;			//selectable listeners		
	
	private boolean selected = false;	// is the graphable currently selected
	private boolean selectable = true;  // is the graphable selectable
	//////////////////////////////////////////////////////
	
	public DefaultSelectable()
	{
		selectableListeners = new Vector();
	}
	
	/**
	 * Selects this object. 
	 * This also fires an objectSelected(obj) event.
	 */
	public void select()
	{
		if (!selected){
			selected = true;
			notifySelect();
		}
	}
	
	/** Deselects this object
	 * This also fires an objectDeselected(obj) event.
	 */
	public void deselect()
	{
		if (selected){
			selected = false;
			notifyDeselect();
		}
	}
	
	/**
	 * Returns if the object is selected or not 
	 */
	public boolean isSelected()
	{
		return selected;
	}

	/**
	 * Sets if the object can be selected or not 
	 */
	public void setSelectionEnabled(boolean b)
	{
		selectable = b;
	}

	/**
	 * Returns if the object can be selected or not 
	 */
	public boolean isSelectionEnabled()
	{
		return selectable;
	}

	/*
	 * Adds a SelectableListener to listen for selection events 
	 */
	public void addSelectableListener(SelectableListener l)
	{
		selectableListeners.add(l);
	}
	
	/*
	 * Removes a SelectableListener.
	 */
	public void removeSelectableListener(SelectableListener l)
	{
		selectableListeners.remove(l);
	}

	protected void notifySelect() 
	{		
		EventObject e = new EventObject(this);
		Enumeration lists = selectableListeners.elements();
		while (lists.hasMoreElements()) {			
			GraphableListener gl = (GraphableListener)lists.nextElement();
			if (gl instanceof SelectableListener) {
				((SelectableListener)gl).graphableSelected(e);
			}
		}
	}

	protected void notifyDeselect() 
	{
		EventObject e = new EventObject(this);
		Enumeration lists = selectableListeners.elements();
		while (lists.hasMoreElements()) {			
			GraphableListener gl = (GraphableListener)lists.nextElement();
			if (gl instanceof SelectableListener) {
				((SelectableListener)gl).graphableDeselected(e);
			}
		}
	}
}
    
