/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.1 $
 * $Date: 2004/05/24 20:16:30 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graph.engine;

import java.awt.*;

/**
 * An interface for simple objects that are just drawn in a graph window.
 * A Drawable is an object that can draw itself in a graph area and a Graphics2D context 
 * Objects that want simply to be displayed in a Graph Window 
 * and that won't change, should implement this interface.
 * It cannot notify the container that is has changed, 
 * so if it changes, it won't be redrawn.
 * If you need this functionality, use the Graphable interface
 * <br>
 * Date created: 04/07/2004
 *
 * @author Eric Brown-Munoz<p>
 * @author Ingrid C Moncada<p>
 *
 */

public interface Drawable 
{
	/** 
	 * Draws this object on Graphics g. 
	 * The graph area's coordinate system
	 * shoule be used to transform coordinates.
	 */
	public void draw(Graphics2D g);	

	/** 
	 * Sets the graph area where the graphable will be drawn 
	 */
	public void setGraphArea(GraphArea area);

	/** 
	 * Returns the graph area where the graphable will be drawn 
	 */
	public GraphArea getGraphArea();
}

