/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.1 $
 * $Date: 2004/05/24 20:16:30 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graph.engine;

import org.concord.graph.event.*;

/**
 * An interface for objects that are graphed or drawn in a graph window.
 * A Graphable is an object on a graph area can draw itself in a Graphics2D context 
 * It can also keep a buffered version of itself for speed purposes when drawing. 
 * It can be removed, be visible or invisible, and it can return a copy (clone) of itself.
 * Objects that want to be displayed in a Graph Window should implement this interface.
 * A Graphable also fires events when it has been changed or removed.
 * <br>
 * Date created: 04/07/2004
 *
 * @author Eric Brown-Munoz<p>
 * @author Ingrid C Moncada<p>
 *
 */

public interface Graphable extends Drawable
{
	/*
	 * Adds a GraphableListener to listen for change and remove events 
	 */
	public void addGraphableListener(GraphableListener gl);
	
	/*
	 * Removes a GraphableListener.
	 */
	public void removeGraphableListener(GraphableListener gl);
	
	/** 
	 * Forces classes to redraw any internal image buffer (if implemented). 
	 * This makes sure the background is updated if there is a fundamental 
	 * change that is not checked by this class.
	 */
	public void update();

	/** 
	 * Removes the object from any enclosing view.
	 * This also fires an objectRemoved(obj) event.
	 */
	public void remove();
	
	/** 
	 * Sets whether it is visible in a graph.
	 * This also fires an objectChanged(obj) event.
	 */
	public void setVisible(boolean visible);
	
	/** 
	 * Returns true if it is visible in a graph 
	 */
	public boolean isVisible();

	/** 
	 * Returns a copy of itself 
	 */
	public Graphable getCopy();	
}

