/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * @(#)MouseControllable.java
 * 
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/06/07 17:55:42 $
 * $Author: imoncada $
 *
 * Copyright 
*/
package org.concord.graph.engine;

import java.awt.*;

/** 
 * Interface for objects that after being selected they can also be
 * controlled by mouse events (mouse pressed, dragged and released).
 * MouseControllable objects don't fire any additional type of events,
 * other than the event fired by Selectables and Graphables (changed,
 * removed, selected, deselected)
 * <br>
 * Date created: 04/07/2004
 *
 * @author Ingrid C Moncada<p>
 *
 */

public interface MouseControllable extends MouseSensitive
{
	/**
	 * Handles the mouse press event. 
	 * The location is given in display coordinates. 
	 */
	public boolean mousePressed(Point p);
	
	/** Handles the mouse drag event. 
	 * The location is given in display coordinates. 
	 */
	public boolean mouseDragged(Point p);
	
	/** Handles the mouse released event. 
	 * The location is given in display coordinates. 
	 */
	public boolean mouseReleased(Point p);
	
	/**  
	 * Returns if the object is currently being
	 * controlled by the mouse (dragged)
	 */
	public boolean isMouseControlled();
	
}

