/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/06/07 17:55:42 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graph.engine;

import java.awt.*;

/** 
 * Interface for objects that can receive motion events
 * from the mouse, if the mouse is close enough.
 * <br>
 * Date created: 04/07/2004
 *
 * @author Ingrid C Moncada<p>
 *
 */

public interface MouseMotionReceiver extends MouseSensitive
{
	/**
	 * Handles the mouse moved event. 
	 * The location is given in display coordinates. 
	 */
	public boolean mouseMoved(Point p);
	
	/**
	 * Fired when the mouse goes inside of the component
	 * while mouse moved 
	 * The location is given in display coordinates. 
	 */
	public boolean mouseEntered(Point p);

	/**
	 * Fired when the mouse goes outside of the component
	 * while mouse moved 
	 * The location is given in display coordinates. 
	 */
	public boolean mouseExited(Point p);
	
	/**
	 * Returns if the mouse is currently inside of the object 
	 * (has entered but hasn't exit yet)
	 */
	public boolean isMouseReceiving();
}

