/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/27 07:10:09 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graph.engine;

import org.concord.graph.event.*;

/** 
 * The Selectable interface is for objects that can be selected 
 * by clicking with the mouse within a proximity.
 * Selectable objects also fire events when they are selected and deselected.
 * <br>
 * Date created: 04/07/2004
 *
 * @author Ingrid C Moncada<p>
 *
 */

public interface Selectable 
{
	/*
	 * Adds a SelectableListener to listen for selection events 
	 */
	public void addSelectableListener(SelectableListener l);
	
	/*
	 * Removes a SelectableListener.
	 */
	public void removeSelectableListener(SelectableListener l);

	/**
	 * Selects this object. 
	 * This also fires an objectSelected(obj) event.
	 */
	public void select();
	
	/** Deselects this object
	 * This also fires an objectDeselected(obj) event.
	 */
	public void deselect();
	
	/**
	 * Returns if the object is selected or not 
	 */
	public boolean isSelected();

	/**
	 * Sets if the object can be selected or not 
	 * If the object is not selectable, the select()
	 * method won't select it
	 */
	public void setSelectionEnabled(boolean b);

	/**
	 * Returns if the object can be selected or not 
	 * If the object is not selectable, the select()
	 * method won't select it
	 */
	public boolean isSelectionEnabled();
}

