/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.1 $
 * $Date: 2004/05/24 20:16:30 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graph.event;

import java.util.*;

/**
 * Objects that want to listen to Graphable list events 
 * should implement this interface. 
 * The GraphableList fires events when new objects are added,
 * when objects change and when objects are removed
 *
 * Date created: 04/07/2004
 *
 * @author Ingrid Moncada<p>
 *
*/
public interface GraphableListListener extends EventListener
{
	/** 
	 * Fired when a new Graphable has been added to the list. 
	 * The event objects source is the new object added.
	 */
	public void listGraphableAdded(EventObject e);

	/**
	 * Fired when a Graphable has been removed to the list. 
	 *The event objects source is the object removed.
	 */
	public void listGraphableRemoved(EventObject e);

	/**
	 * Fired when a Graphable has changed. 
	 * The event objects source is the object that has changed.
	 */
	public void listGraphableChanged(EventObject e);
}

