/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.6 $
 * $Date: 2004/06/07 17:55:42 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graph.examples;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import org.concord.graph.engine.*;
import org.concord.graph.ui.*;

/**
 * Another test class for graph
 * 
 * @author Ingrid Moncada<p>
 *
   $Revision: 1.6 $
*/
public class BoxGraphableMainPanel extends JPanel
{
	private GraphWindow graph;
	private Grid grid;
	private AxisScale axisScale;
	
	public BoxGraphableMainPanel()
	{
		grid = new Grid();
		//grid.setInterval(1.0,1.0);
		//grid.setLabelFormat(new DecimalFormat("#"));
		grid.setShowAxisLabels(false);
		
		graph = new GraphWindow();
		
		//temp
		//graph.setDefaultGraphArea(new GraphArea(new AffineTransformCoordinateSystem()));
		
		//graph.getDefaultGraphArea().getCoordinateSystem().setScale(new Point2D.Double(1.0,-10.0));
		
		//grid.setUseBufferedImage(true);
		graph.getDefaultGraphArea().setOriginCentered(true);
		//graph.add(grid);
		
		Grid2D grid2d = new Grid2D();
		//graph.addDecoration(grid2d);

		SingleAxisGrid xGrid;
		//xGrid = new SingleAxisGrid(1);
		xGrid = grid2d.getXGrid();
		
		xGrid.setColorAxisLines(Color.black);
		xGrid.setColorMajorGridLines(Color.red);
		xGrid.setColorMinorGridLines(Color.pink);
		xGrid.setMajorInterval(4);
		xGrid.setAxisLabel("Hi X AXIS!!");
		xGrid.setShowLabelsOnMajorOnly(true);

		SingleAxisGrid yGrid;
		//yGrid = new SingleAxisGrid(2);
		yGrid = grid2d.getYGrid();
		yGrid.setColorAxisLines(Color.black);
		yGrid.setColorMajorGridLines(Color.blue);
		yGrid.setColorMinorGridLines(Color.cyan);
		yGrid.setMajorInterval(3);
//		yGrid.setShowAxisLines(false);
		yGrid.setAxisLabel("LABEL FOR YYYYYY");
		yGrid.setColorAxisLabel(Color.red);
		yGrid.setAxisLabelSize(20.0f); //Make the axis label BIG
		yGrid.setGridLabelSize(16.0f);
		yGrid.setDrawGridOnAxis(false);

		//graph.addDecoration(xGrid);
		//graph.addDecoration(yGrid);


		setLayout(new BorderLayout());
		add(graph);
		
		//Add a box
		BoxGraphableTest box;

		axisScale = new AxisScale();
		axisScale.setDragMode(AxisScale.DRAGMODE_NONE);
		axisScale.setShowMessage(false);
		axisScale.setShowCover(false);
		
//		graph.add(axisScale);

		box = new BoxGraphableTest("Box 11");
		graph.add(box);

		box = new BoxGraphableTest("Box 2");
		box.setLocation(5,5);
		graph.add(box);

		box = new BoxGraphableTest("Box 3");
		box.setLocation(-7,4);
		graph.add(box);

		graph.addDecoration(grid);
		
		graph.add(axisScale);
		
		JPanel dragModeContainer = new JPanel();
		dragModeContainer.setLayout(new BoxLayout(dragModeContainer, BoxLayout.Y_AXIS));
		
		ButtonGroup buttonGroup = new ButtonGroup();
		
		JRadioButton dragMode;
		
		dragMode = new JRadioButton("Nothing");
		dragMode.setSelected(true);
		dragMode.setActionCommand("0");
		dragMode.addActionListener(buttonHandler);
		buttonGroup.add(dragMode);
		dragModeContainer.add(dragMode);
		
		dragMode = new JRadioButton("Translate all");
		dragMode.setActionCommand("1");
		dragMode.addActionListener(buttonHandler);
		buttonGroup.add(dragMode);
		dragModeContainer.add(dragMode);
		
		dragMode = new JRadioButton("Translate X");
		dragMode.setActionCommand("6");
		dragMode.addActionListener(buttonHandler);
		buttonGroup.add(dragMode);
		dragModeContainer.add(dragMode);
		
		dragMode = new JRadioButton("Translate Y");
		dragMode.setActionCommand("7");
		dragMode.addActionListener(buttonHandler);
		buttonGroup.add(dragMode);
		dragModeContainer.add(dragMode);
		
		dragMode = new JRadioButton("Scale all");
		dragMode.setActionCommand("2");
		dragMode.addActionListener(buttonHandler);
		buttonGroup.add(dragMode);
		dragModeContainer.add(dragMode);
		
		dragMode = new JRadioButton("Scale X");
		dragMode.setActionCommand("8");
		dragMode.addActionListener(buttonHandler);
		buttonGroup.add(dragMode);
		dragModeContainer.add(dragMode);
		
		dragMode = new JRadioButton("Scale Y");
		dragMode.setActionCommand("9");
		dragMode.addActionListener(buttonHandler);
		buttonGroup.add(dragMode);
		dragModeContainer.add(dragMode);
		
//		add(dragModeContainer, BorderLayout.WEST);
		
		GraphWindowToolBar toolBar = new GraphWindowToolBar();
		toolBar.setGraphWindow(graph);
		toolBar.setAxisScale(axisScale);
		add(toolBar, BorderLayout.EAST);
		
	}
	
	ActionListener buttonHandler = new ActionListener()
	{
		public void actionPerformed(ActionEvent e)
		{
			axisScale.setDragMode(Integer.parseInt(e.getActionCommand()));
		}
	};

	public GraphWindow getGraph()
	{
		return graph;
	}
	
	public Grid getGrid()
	{
		return grid;
	}

	//Testing purposes
    public static void main(String args[]) 
    {
		final JFrame frame = new JFrame();
		final JPanel fa = new BoxGraphableMainPanel();
		frame.getContentPane().add(fa);
		frame.setSize(600,600);
		frame.show();
		
		frame.addWindowListener( new WindowAdapter() 
		{
			public void windowClosing(WindowEvent e)
			{				
				System.exit(0);
			}			
		});
    }
}
