/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/06/07 17:55:42 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graph.examples;

import java.util.*;
import java.awt.*;
import java.awt.geom.*;

import org.concord.graph.engine.*;
import org.concord.graph.event.*;

/**
 * Test class for graphables in general
 *
 * Date created: 04/07/2004
 *
 * @author Ingrid C Moncada<p>
 *
 */

public class BoxGraphableTest extends DefaultControllable
	implements MouseMotionReceiver, Cursorable
{	
	private boolean mouseClicked = false;	//was the mouse pressed in this graphable
	
	private boolean mouseInside = false;	//is the mouse curently inside of the graphable

	private Cursor currentCursor;
	private Vector cursorListeners;
	
	////////////////////////////	
//	private Point2D position;	//lower left corner of the box!
	private String message;
	private Dimension size;
	private Color backColor;
	private Color foreColor;
	private Stroke borderStroke;
	///////////////////////////
	
//	private Point2D clickLocation;
//	private Point2D originalLocation;
	
	//caching
	private Point2D displayPositionIni;
	private Point2D displayPositionFin;
	
	private boolean needUpdate = true;
	
	public BoxGraphableTest()
	{
		this("Box");
	}
	
	public BoxGraphableTest(String msg)
	{
		super();
		
		location = new Point(0,0);
		size = new Dimension(4,3);
		backColor = new Color(255,255,200);
		foreColor = new Color(0,0,150);
		message = msg;
		borderStroke = new BasicStroke(1.0f);
		
		displayPositionIni = new Point();
		displayPositionFin = new Point();
//		clickLocation = new Point();
		
		cursorListeners = new Vector();
	}
	
	public void setLocation(int x, int y)
	{
		setLocation(new Point2D.Double(x,y));
	}
	
	public void setLocation(Point2D pos)
	{
		super.setLocation(pos);
		needUpdate = true;
	}
	
	//////////////////////////////////////////
	// Methods from Graphable interface

	public void update()
	{
		CoordinateSystem cs = graphArea.getCoordinateSystem();
		
		//Cache in display coordinates
		displayPositionIni = cs.transformToDisplay(location);
		displayPositionFin = cs.transformToDisplay(new Point2D.Double(location.getX() + size.width, location.getY() + size.height));
		
		//Swicth the cached initial position and final position (display coordinates), if necessary
		if (displayPositionIni.getX() >= displayPositionFin.getX()){
			double temp = displayPositionFin.getX();
			displayPositionFin.setLocation(displayPositionIni.getX(), displayPositionFin.getY());
			displayPositionIni.setLocation(temp, displayPositionIni.getY());
		}
		if (displayPositionIni.getY() >= displayPositionFin.getY()){
			double temp = displayPositionFin.getY();
			displayPositionFin.setLocation(displayPositionFin.getX(), displayPositionIni.getY());
			displayPositionIni.setLocation(displayPositionIni.getX(), temp);
		}
	}
	
	/** 
	 * Draws this object on Graphics g. 
	 */
	public void draw(Graphics2D g)
	{
		if (needUpdate){
			update();
			needUpdate = false;
		}
		
		Color oldColor = g.getColor();
		Stroke oldStroke = g.getStroke();
				
		g.setColor(backColor);
		
		g.fillRect((int)displayPositionIni.getX(), (int)displayPositionIni.getY(), 
		           (int)(displayPositionFin.getX() - displayPositionIni.getX()), 
				   (int)(displayPositionFin.getY() - displayPositionIni.getY()) );
		
		g.setColor(foreColor);
		g.drawString(message, (int)(displayPositionIni.getX()+8), (int)(displayPositionIni.getY()+15) );

		g.setStroke(borderStroke);
		g.drawRect((int)displayPositionIni.getX(), (int)displayPositionIni.getY(), 
		           (int)(displayPositionFin.getX() - displayPositionIni.getX()), 
				   (int)(displayPositionFin.getY() - displayPositionIni.getY()) );
		
		g.setColor(oldColor);
		g.setStroke(oldStroke);
	}
	
	/** 
	 * Returns a copy of itself 
	 */
	public Graphable getCopy()
	{
		return null;
	}
	//////////////////////////////////////////

	//////////////////////////////////////////
	// Methods from Selectable interface

	/**
	 * Selects this object. 
	 * This also fires an objectSelected(obj) event.
	 */
	public void select() 
	{
		super.select();
		if (isSelected()){
			borderStroke = new BasicStroke(5.0f);
		}
	}
	
	/** Deselects this object
	 * This also fires an objectDeselected(obj) event.
	 */
	public void deselect() 
	{
		super.deselect();
		if (!isSelected()){
			borderStroke = new BasicStroke(1.0f);
		}
	}
	
	/**
	 * Checks if the location is within enough proximity to be selected. 
	 * If the location is close enough of the object and
	 * the object wants to be selected, returns true. 
	 * The location is given in screen coordinates. 
	 */
	public boolean isPointInProximity(Point location)
	{
		if (location.getX() >= displayPositionIni.getX() && location.getY() >= displayPositionIni.getY() &&
		    location.getX() <= displayPositionFin.getX() && location.getY() <= displayPositionFin.getY()){
			return true;
		}
		
		return false;
	}
	//////////////////////////////////////////

	//////////////////////////////////////////
	// Methods from MouseControllable interface
	
	/**
	 * Handles the mouse press event. 
	 * The location is given in screen coordinates. 
	 */
/*	public boolean mousePressed(Point location)
	{
		clickLocation = graphArea.getCoordinateSystem().transformToWorld(location);
		originalLocation = position;

		setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
		mouseClicked = true;
		return false;
	}
*/	
	/** Handles the mouse drag event. 
	 * The location is given in screen coordinates. 
	 */
/*	public boolean mouseDragged(Point location)
	{
		Point2D newLocation;
		
		newLocation = graphArea.getCoordinateSystem().transformToWorld(location);
		
		Point2D newP = new Point2D.Double(originalLocation.getX() + (newLocation.getX() - clickLocation.getX()),
			originalLocation.getY() + (newLocation.getY() - clickLocation.getY()));
			
		setPosition(newP);
		return false;
	}
*/	
	/** Handles the mouse released event. 
	 * The location is given in screen coordinates. 
	 */
/*	public boolean mouseReleased(Point location)
	{
		setCursor(null);
		mouseClicked = false;
		return false;
	}
*/	
	/**  
	 * Returns if the object is currently being
	 * controlled by the mouse 
	 */
/*	public boolean isMouseControlled()
	{
		return mouseClicked;
	}
*/
	//////////////////////////////////////////

	//////////////////////////////////////////
	// Methods from MouseMotionReceiver interface
	
	/**
	 * Handles the mouse moved event. 
	 * The location is given in screen coordinates. 
	 */
	public boolean mouseMoved(Point location)
	{
		return false;
	}
	
	/**
	 * Fired when the mouse goes inside of the component
	 * while mouse moved 
	 * The location is given in screen coordinates. 
	 */
	public boolean mouseEntered(Point location)
	{
		backColor = new Color(255,200,150);
		mouseInside = true;
		notifyChange();
		return false;
	}

	/**
	 * Fired when the mouse goes outside of the component
	 * while mouse moved 
	 * The location is given in screen coordinates. 
	 */
	public boolean mouseExited(Point location)
	{
		backColor = new Color(255,255,200);
		mouseInside = false;
		notifyChange();
		return false;
	}
	
	/**
	 * Returns if the mouse is currently inside of the object 
	 */
	public boolean isMouseReceiving()
	{
		return mouseInside;
	}
	//////////////////////////////////////////

	//////////////////////////////////////////
	//Methods of interface Cursorable
	
	/**
	 * Sets the current cursor
	 */
	public void setCursor(Cursor cursor)
	{
		if (currentCursor != cursor){
			currentCursor = cursor;
			notifyCursorListeners();
		}
	}

	/**
	 * Returns the current cursor
	 */
	public Cursor getCursor()
	{
		return currentCursor;
	}
	
	/**
	 * Adds a listener that will listen to a cursor change
	 */
	public void addCursorListener(CursorListener l)
	{
		cursorListeners.add(l);
	}

	/**
	 * Removes a cursor change listener 
	 */
	public void removeCursorListener(CursorListener l)
	{
		cursorListeners.remove(l);
	}
	
	protected void notifyCursorListeners()
	{
		CursorListener l;
		for (int i=0;i<cursorListeners.size();i++){
			l = (CursorListener)cursorListeners.elementAt(i);
			l.cursorChanged(new EventObject(this));
		}
	}
	
	//////////////////////////////////////////
	
	public String toString()
	{
		return super.toString() + " " + message;
	}
}

