/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.5 $
 * $Date: 2004/08/05 19:41:36 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graph.examples;

import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import java.net.*;

import javax.swing.*;

import org.concord.graph.engine.*;
import org.concord.graph.event.*;
import org.concord.graph.ui.*;

/**
 * Toolbar test class for igraph
 * 
 * @author Ingrid Moncada<p>
 *
*/

public class GraphWindowToolBar extends JToolBar
	implements GraphWindowListener
{
	private GraphWindow gw;
	private CoordinateSystem coord;
	private AxisScale ax;
	private AbstractButton selButton;
	private DashedBox dashBox;
	
	private Point2D originalOrigin;
	private Point2D originalScale;
	
	public GraphWindowToolBar()
	{
		super();
		this.setOrientation(VERTICAL);
		
		addButton("move.gif", "1", "Move al the graph");
		addButton("scaleX.gif", "8", "Scale the X axis");
		addButton("scaleY.gif", "9", "Scale the Y Axis");
		addButton("scaleXY.gif", "2", "Scale both axis");
		addButton("zoom.gif", "13", "Zoom to a point");
		addButton("dashedbox.gif", "dashedbox", "Draw a box");
		addButton("zoom.gif", "zoombox", "Zoom to the box", false);
		addButton("restorescale.gif", "restorescale", "Restore initial scale", false);
	}
	
	private AbstractButton addButton(String filename, String action, String toolTip)
	{
		return addButton(filename, action, toolTip, true);
	}
	
	private AbstractButton addButton(String filename, String action, String toolTip, boolean toggle)
	{
		AbstractButton button;
		if (toggle){
			button = new JToggleButton();
		}
		else{
			button = new JButton();
		}
		
		button.setToolTipText(toolTip);
		URL url = getClass().getResource("images/"+filename);
		if (url != null){
			button.setIcon(new ImageIcon(url));
		}
		
		button.addActionListener(buttonHandler);
		button.setActionCommand(action);
		
		add(button);
		
		return button;
	}
	
	ActionListener buttonHandler = new ActionListener()
	{
		public void actionPerformed(ActionEvent e)
		{
			String command = e.getActionCommand();
			AbstractButton button = (AbstractButton)e.getSource();
			int dragMode = -1;
			boolean doDragMode = false;
			try{
				dragMode = Integer.parseInt(command);
				doDragMode = true;
			}
			catch(Exception ex){
			}
			
			if (doDragMode){
				if (button.isSelected()){
					if (selButton != null){
						selButton.doClick();
						//selButton.setSelected(false);
					}
					selButton = (AbstractButton)e.getSource();
					ax.setDragMode(dragMode);
				}
				else{
					selButton = null;
					ax.setDragMode(AxisScale.DRAGMODE_NONE);
				}
			}
			else{
				if (command.equals("restorescale")){
					coord.setOriginOffsetDisplay(originalOrigin);
					coord.setScale(originalScale);
				}
				else if(command.equals("dashedbox")){
					showDashedBox(button.isSelected());
				}
				else if(command.equals("zoombox")){
					if (dashBox != null){
						dashBox.zoom();
					}
				}
			}
		}
	};

	/**
	 * Sets the GraphWindow to work with
	 */
	public void setGraphWindow(GraphWindow gw)
	{
		if (this.gw != null){
			this.gw.removeGraphWindowListener(this);
			this.coord = null;
		}
		this.gw = gw;
		if (this.gw != null){
			this.gw.addGraphWindowListener(this);
			this.coord = gw.getDefaultGraphArea().getCoordinateSystem();
		}
	}
	
	public void showDashedBox(boolean b)
	{
		//if (dashBox == null){
		if (b){
			dashBox = new DashedBox(true);
			dashBox.select();
			gw.add(dashBox);
		}
		else{
			dashBox.remove();
		}
		//dashBox.setVisible(b);
	}
	
	public void windowChanged(EventObject e)
	{
	}
	
	public void windowResized(GraphWindowResizeEvent e)
	{
		recordOriginalScale();
	}
	
	private void recordOriginalScale()
	{
		originalOrigin = coord.getOriginOffsetDisplay();
		originalScale = coord.getScale();
	}
	
	/**
	 * Sets the GraphWindow to work with
	 */
	public void setAxisScale(AxisScale ax)
	{
		this.ax = ax;
	}
	
}
