/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
   $Revision: 1.1 $
*/

package org.concord.graph.ui;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;

import org.concord.graph.event.*;
import org.concord.graph.engine.*;


public class AxisLabelControl 
	extends JFrame 
	implements ActionListener
{

	Grid grid;
	LabelTextField xLabelField;
	LabelTextField yLabelField;



	JComponent reference = null;

	public AxisLabelControl( Grid graphable) {
		super("Axis Labels");

		this.grid = graphable;

		xLabelField = new LabelTextField() {
				public void change() {
					String text = getText();
					grid.setLabelXAxis(text);
				}
			};

		yLabelField = new LabelTextField() {
				public void change() {
					String text = getText();
					grid.setLabelYAxis(text);
				}
			};


		JPanel main = new JPanel();
		main.setLayout(new BoxLayout(main,BoxLayout.Y_AXIS));
		main.add(new JLabel("Change Axis Labels"));

		JPanel xBorder = new JPanel();
		xBorder.setBorder(BorderFactory.createEtchedBorder());
		xBorder.setLayout(new BoxLayout(xBorder,BoxLayout.Y_AXIS));
		xBorder.add(new JLabel("x-axis:"));

		JPanel yBorder = new JPanel();
		yBorder.setBorder(BorderFactory.createEtchedBorder());
		yBorder.setLayout(new BoxLayout(yBorder,BoxLayout.Y_AXIS));
		yBorder.add(new JLabel("y-axis:"));

		JPanel xpanel = new JPanel();
		xpanel.setLayout(new BoxLayout(xpanel, BoxLayout.X_AXIS));
		xpanel.add(xLabelField);

		JPanel ypanel = new JPanel();
		ypanel.setLayout(new BoxLayout(ypanel, BoxLayout.X_AXIS));
		ypanel.add(yLabelField);

		xBorder.add(xpanel);
		main.add(xBorder);

		yBorder.add(ypanel);
		main.add(yBorder);


		JPanel buttons = new JPanel();

		JButton ok = new JButton("OK");
		ok.setActionCommand("ok");
		ok.addActionListener(this);


		buttons.add(ok);

		main.add(buttons);


		getContentPane().add(main);

		
		pack();
		validate();

	}

	public void show() {


		xLabelField.setText( grid.getLabelXAxis() );
		yLabelField.setText( grid.getLabelYAxis() );

		if (reference != null) {
			System.out.println ("ScaleControl setting location   ");
			Point screenLocation = reference.getLocationOnScreen();
			setLocation(screenLocation.x + 20,
						screenLocation.y + 20);
		}


		super.show();


	}

	public void sync (JComponent reference) {
		this.reference = reference;
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();

		if (command.equals("ok")) {
			xLabelField.change();
			yLabelField.change();
			hide();
		}
	
	}

}

abstract class LabelTextField 
	extends JTextField
	implements ActionListener
{
	String text; // Save for undo

	abstract void change() ;

	public LabelTextField() {
		
		addActionListener(this);
		
	}

	public void actionPerformed (ActionEvent e) {
		change();
	}

}


