/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.4 $
 * $Date: 2004/06/30 18:29:52 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graph.ui;

import java.awt.*;
import java.util.*;

import org.concord.graph.engine.*;

/**
 * Handler of the mouse events in the 
 * Graph Window
 *
 * Date created: 04/07/2004
 *
 * @author Ingrid Moncada<p>
*/
public class DefaultGraphDrawManager
	implements GraphDrawManager 
{
	protected GraphableList graphableObjects;

	/**
	 * Draws the graphables in the graphics context g
	 * If the graphable list is a SelectableList
	 * it draw the selected objects last
	 */
	public void draw(Graphics2D g)
	{
		if (graphableObjects == null) return;

		if (graphableObjects instanceof SelectableList){
			drawSelectables(g,(SelectableList)graphableObjects);
		}
		else{
			drawGraphables(g,graphableObjects);
		}
	}

	private void drawGraphables(Graphics2D g2, Vector gs)
	{	
		Object obj;
		Graphable g;
		Drawable d;
		int i;
		for (i=0;i<gs.size();i++){
			obj = gs.elementAt(i);
			if (obj instanceof SelectableList){
				drawSelectables(g2, (SelectableList)obj);
			}
			else if (obj instanceof GraphableList){
				drawGraphables(g2, (GraphableList)obj);
			}
			else if (obj instanceof Drawable){
				d = (Drawable)obj;
				if (obj instanceof Graphable){
					g = (Graphable)obj;
					//Don't paint invisible graphables
					if (!g.isVisible()) continue;
				}
				d.draw(g2);
			}
		}
	}	

	// This function is a bandaid to a design problem in the
	// Drawing and selection model. The correct way to draw a list
	// that contains a Selectable list has not been flushed out. 
	// This temporary fix makes
    // sure that Lists (which may return true to isSelected()) are
	// not drawn on top of Objects that are later in the Graphable
	// list but aren't selected.
	//
	// This function should be put out of its misery as soon as
	// these issues are resolved. In fact it  should never have
	// existed in the first place. May the Java gods have mercy 
    // on our souls.
	private void drawNonListGraphables(Graphics2D g2, Vector gs)
	{	
		Object obj;
		Graphable g;
		Drawable d;
		int i;
		for (i=0;i<gs.size();i++){
			obj = gs.elementAt(i);
			if (obj instanceof SelectableList){
				// Do nothing
			}
			else if (obj instanceof GraphableList){
				// ... still do nothing (this is a bandaid after all)
			}
			else if (obj instanceof Drawable){
				d = (Drawable)obj;
				if (obj instanceof Graphable){
					g = (Graphable)obj;
					//Don't paint invisible graphables
					if (!g.isVisible()) continue;
				}
				d.draw(g2);
			}
		}
	}	

	private void drawSelectables(Graphics2D g2, SelectableList gs)
	{	
		//Paint the selected on top
		Object obj;
		Drawable d;

		//Paint the non-selected objects
		boolean sel = false;
		int i;
		for (i=0;i<gs.size();i++){
			sel = false;
			obj = gs.elementAt(i);
			if (obj instanceof SelectableList){
				drawSelectables(g2, (SelectableList)obj);
			}
			else if (obj instanceof GraphableList){
				drawGraphables(g2, (GraphableList)obj);
			}
			else if (obj instanceof Drawable){
				d = (Drawable)obj;
				if (obj instanceof Graphable){
					Graphable graph = (Graphable)obj;
					//Don't paint invisible graphables
					if (!graph.isVisible()) continue;
				}
				if (obj instanceof Selectable){
					if (((Selectable)obj).isSelected()){
						sel = true;
					}
				}
				if (!sel){
					d.draw(g2);
				}
			}
		}

		//Paint the selected objects
		drawNonListGraphables(g2,gs.getSelectedObjects(false));
	}	

	/**
	 * Sets the list of Graphable objects
	 * that need to be drawn. 
	 * The objects in this list are 
	 * Graphable objects.
	 * This list is maintained by the Graph Window
	 * itself, so it should not be changed
	 */
	public void setGraphableList(GraphableList graphables)
	{
		graphableObjects = graphables;
	}
}
