/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
   $Revision: 1.2 $
*/

package org.concord.graph.ui;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;

import org.concord.graph.event.*;
import org.concord.graph.engine.*;


public class ScaleControl 
	extends JFrame 
	implements ActionListener
{

	SCTextField xMaxField = new SCTextField(this);
	SCTextField yMaxField = new SCTextField(this);

	GraphArea area;

	JComponent reference = null;

	public ScaleControl( GraphArea area) {
		super("Scale");

		this.area = area;

		JPanel main = new JPanel();
		main.setLayout(new BoxLayout(main,BoxLayout.Y_AXIS));
		main.add(new JLabel("Change Scale"));

		JPanel xBorder = new JPanel();
		xBorder.setBorder(BorderFactory.createEtchedBorder());
		xBorder.setLayout(new BoxLayout(xBorder,BoxLayout.Y_AXIS));
		xBorder.add(new JLabel("x-axis:"));

		JPanel yBorder = new JPanel();
		yBorder.setBorder(BorderFactory.createEtchedBorder());
		yBorder.setLayout(new BoxLayout(yBorder,BoxLayout.Y_AXIS));
		yBorder.add(new JLabel("y-axis:"));

		JPanel xpanel = new JPanel();
		xpanel.setLayout(new BoxLayout(xpanel, BoxLayout.X_AXIS));
		xpanel.add(new JLabel("From 0 to "));
		xpanel.add(xMaxField);

		JPanel ypanel = new JPanel();
		ypanel.setLayout(new BoxLayout(ypanel, BoxLayout.X_AXIS));
		ypanel.add(new JLabel("From 0 to "));
		ypanel.add(yMaxField);

		xBorder.add(xpanel);
		main.add(xBorder);

		yBorder.add(ypanel);
		main.add(yBorder);


		JPanel buttons = new JPanel();

		JButton ok = new JButton("OK");
		ok.setActionCommand("ok");
		ok.addActionListener(this);


		buttons.add(ok);

		main.add(buttons);


		getContentPane().add(main);

		
		pack();
		validate();

	}

	public void show() {

		CoordinateSystem cs = area.getCoordinateSystem();

		Point2D upperRight = area.getUpperRightCornerWorld();
		double xmax = upperRight.getX();
		double ymax = upperRight.getY();

		double roundXMax = Math.round(xmax*10.0)/10.0;
		xMaxField.changeText(Double.toString(roundXMax));

		double roundYMax = Math.round(ymax*10.0)/10.0;
		yMaxField.changeText(Double.toString(roundYMax));

		if (reference != null) {
			Point screenLocation = reference.getLocationOnScreen();
			setLocation(screenLocation.x + 20,
						screenLocation.y + 20);
		}


		super.show();


	}

	public void sync (JComponent reference) {
		this.reference = reference;
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();

		if (command.equals("ok")) {
			changeCS();
			hide();
		}
	
	}

	void changeCS () {

		CoordinateSystem cs = area.getCoordinateSystem();
		Dimension size = area.getSize();
		Point2D origin = cs.getOriginOffsetDisplay();
		Point2D.Double newscale;

		int xpixels = (size.width - (int)origin.getX());
		int ypixels = ((int)origin.getY()); // remember y axis weirdness

		double xvalue = xMaxField.getValue();
		double yvalue = yMaxField.getValue();

		if ((xvalue != Double.NaN) && (yvalue != Double.NaN)) {
			
			newscale = new Point2D.Double( (double)xpixels / xvalue,
										   (double)ypixels / yvalue);

			cs.setScale(newscale);
		}
			
	}		


}

class SCTextField 
	extends JTextField
	implements ActionListener
{
	String text; // Save for undo
	ScaleControl parent;

	public SCTextField(ScaleControl parent) {
		
		this.parent = parent;
		addActionListener(this);
		
	}

    public void changeText(String text) {
		setText(text);
		this.text = text;
	}

	public double getValue () {
		double value = Double.NaN;

		try {
			value = Double.parseDouble(getText());
		} catch (Exception exc) {
			value = Double.NaN;
		}

		return value;
	}

	public void actionPerformed (ActionEvent e) {
		boolean goodDouble = true;
		double value = 0.0;

		try {
			value = Double.parseDouble(getText());
		} catch (Exception exc) {
			goodDouble = false;
			setText(text);
		}

		if (goodDouble) {
			text = getText();
			parent.changeCS();
		}

	}
	
}


