/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz 
   $Revision: 1.1.2.1 $
*/

package org.concord.graphfunction.engine;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;
import org.concord.graph.engine.*;
import org.concord.graph.event.*;
import org.concord.graphfunction.event.*;

/* A Quadratic Graphable with some unique qualities.
   - It is based on the root form A (x - R1) (x - R2) + B.
   - It enforced integer values for each coefficent.
*/

public class AreaQuad extends QuadraticFunction {

	private int multiple = 1; // or the A coefficient
	private int constant = 0; // or the B coefficient (constant term)
	private int root1  = 3;
	private int root2  = 2;

	private Color color = Color.blue;

	Vector listeners = new Vector();

	boolean visible = true;

	/** Constructs and initializes an Area Quad with arbitrary
		default values. */
	public AreaQuad() {
		this(1, 3, 2, 0);
	}

	
	/** Constructs and intializaed an Area Quad with the given values.
		The variables "ca" and "cb" are the coeffients. "r1" and "r2"
		are the roots */
	public AreaQuad ( int ca, int r1, int r2, int cb) {
		setValues(ca, r1, r2, cb);
	}

	/** Sets the values of the Area Quad.
		The variables "ca" and "cb" are the coeffients. "r1" and "r2"
		are the roots */
	public void setValues ( int ca, int r1, int r2, int cb) {
		multiple = ca;
		constant = cb;
		root1 = r1;
		root2 = r2;
		notifyChanged();
	}

	public void setMultiple(int ca) {
		multiple = ca;
		notifyChanged();
	}

	public void setConstant(int cb) {
		constant = cb;
		notifyChanged();
	}

	public void setRoot1 ( int r1) {
		root1 = r1;
		notifyChanged();
	}

	public void setRoot2 ( int r2) {
		root2 = r2;
		notifyChanged();
	}

	public int getMultiple() {
		return multiple;
	}

	public int getConstant() {
		return constant;
	}

	public double getCoeffA() {
		return multiple;
	}

	public double getCoeffB() {
		return (double)(-root1 - root2);
	}
	
	public double getCoeffC() {
		return (double)(root1 * root2 * multiple) + (double)constant;
	}

	public void addGraphableListener(GraphableListener gl) {
		listeners.add(gl);
	}

	public void draw(Graphics2D g) {
		super.draw(g);
	}
	
	/*
	 * Removes a GraphableListener.
	 */
	public void removeGraphableListener(GraphableListener gl) {
		listeners.remove(gl);
	}

	public void notifyChanged () {
		EventObject e = new EventObject(this);
		Enumeration list = listeners.elements();
		while (list.hasMoreElements()) {
			((GraphableListener) list.nextElement()).graphableChanged(e);
		}
	}
	
	/** 
	 * Forces classes to redraw any internal image buffer (if implemented). 
	 * The axis is provided to transform coordinates.
	 * This makes sure the background is updated if there is a fundamental 
	 * change that is not checked by this class.
	 */
	public void update(Axis axis) {
		// do nothing
	}

	/** 
	 * Removes the object from any enclosing view.
	 * This also fires an objectRemoved(obj) event.
	 */
	public void remove() {
	}
	
	/** 
	 * Sets whether it is visible in a graph.
	 * This also fires an objectChanged(obj) event.
	 */
	public void setVisible(boolean visible) {
		this.visible = visible;
	}
	
	/** 
	 * Returns true if it is visible in a graph 
	 */
	public boolean isVisible() {
		return visible;
	}

	public String getExpression(int foo) {
		return "FOO";
	}

	public double yval (double xval) {
		return ((multiple) * (xval - (double)root1) * (xval - (double)root2) +
				constant);
	}

	public void loadString (String foo) {}

	public String getLoadString() {
		return "FOO";
	}

	/** 
	 * Returns a copy of itself 
	 */
	public Graphable getCopy() {
		return null;
	}
		
	public boolean mouseDragged(Axis a, Point p) {return false;}
	public boolean mousePressed(Axis a, Point p) {return false;}
	public boolean mouseReleased(Axis a, Point p){return false;}

	public boolean isPointInProximity(Axis a ,java.awt.Point p){
		return false;
	}
}

