/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * @(#)Domain.java
 * 
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/07/20 17:35:37 $
 * $Author: jqian $
 *
 * Copyright 
*/
package org.concord.graphfunction.engine;

/**
 *
 * Date created: 04/07/2004
 *
 * @author Eric Brown-Munoz<p>
 * @author Ingrid C Moncada<p>
 *
 */
public class Domain 
{
	/** Does the segment include the start and end points. */
	protected boolean closedStart = true;
	protected boolean closedEnd = false;
	
	protected double start;
	protected double end;

    protected double interval = Double.NaN; 
    	
   	public double getStart() 
	{
		return start;
	}
	
	public void setStart( double start) 
	{
		this.start = start;
	}
	
	public double getEnd() 
	{
		return end;
	}
	
	public void setEnd( double end) 
	{
		this.end   = end;
	}
	
	public Domain getCopy() 
	{
		Domain retval = new Domain();
		retval.start = start;
		retval.end = end;
		retval.closedStart = closedStart;
		retval.closedEnd = closedEnd;
		return retval;
		}
		
	public boolean isInside(double location) 
	{
		boolean retval = false;
		
		if ((location >= start ) && (location <= end) ) {
			if (((location == start) && !closedStart) || 
				( (location == end ) && !closedEnd)) {
			retval = false;
			} else {
			retval = true;
			}
		}
		return retval;
	}
	
	public boolean isOver(double lastLocation, double location) 
	{
		boolean retval = false;
		
		
		if (isInside(location) || isInside(lastLocation)) {
			retval = true;
		} else if ( 
			   (( start > lastLocation) && (end < location)) || 
			   (( start > location) && (end < lastLocation))) {
			retval = true;
		}
		
		return retval;
	}
	
	/** Returns the intersection between this and inDomain. The intersection
	pays attention to wether each Domain has closed	or open points.
	*/
	
	public Domain getIntersection(Domain inDomain) 
	{
		Domain lower;
		Domain higher;
		Domain retval = null;
	   
        double hstart, lstart, hend, lend; 
         
		if (inDomain.start < start) {
			lower=inDomain;
			higher=this;
		} else if (inDomain.start == start) {
			if (inDomain.closedStart) {
			lower = inDomain;
			higher = this;
			} else {
			lower=this;
			higher = inDomain;
			}
		} else {
			lower=this;
			higher=inDomain;
		}
        
            hstart = higher.start;
            lstart = lower.start;
            hend = higher.end;
            lend = lower.end;
        
        if (!Double.isNaN (interval))
        {
            hstart = (double)Math.round(higher.start / interval) * interval;  
            lstart = (double)Math.round(lower.start / interval) * interval;  
            hend = (double)Math.round(higher.end / interval) * interval;  
            lend = (double)Math.round(lower.end / interval) * interval;  
        }
       if (hstart < lend) {
			if (hend <= lend) {
		    	retval = higher.getCopy();
		    	if (hend == lend) {
		    		if (higher.closedEnd && lower.closedEnd) {
		         		retval.closedEnd = true;
	       			} else {
		        		retval.closedEnd = false;
			    	}
	    		}
			} else { // (higher.end > lend)
			retval = higher.getCopy();
			retval.end = lower.end;
			retval.closedEnd = lower.closedEnd;
			}
		
        } else if (hstart == lend) {
		
			if (higher.closedStart && lower.closedEnd) {
			retval = new Domain();
		
			// Set up a point
			retval.start = higher.start;
			retval.end = higher.start;
			retval.closedStart = true;
			retval.closedEnd = true;
			}
		}
		return retval;
	}	    

}


