/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * @(#)FunctionPainter.java
 * 
 * Last modification information:
 * $Revision: 1.1 $
 * $Date: 2004/05/24 20:55:23 $
 * $Author: imoncada $
 *
 * Copyright 
*/
package org.concord.graphfunction.engine;

import org.concord.graphfunction.event.*;

/**
 * A class that allows us to one paint the part of the line that is passed
 * over by a Value . This value usually represents the Trace (i.e. the 
 * student "repaints" the line by moving the Trace. This class will be 
 * called by the draw method of LinearGraphable 
 *
 * Date created: 04/07/2004
 *
 * @author Eric Brown-Munoz<p>
 * @author Ingrid C Moncada<p>
 *
 */

public class FunctionPainter implements ValueListener 
{
    /** The beginning of the painted region */
    protected double xMin;

    /** The end of the painted region */
    protected double xMax;

    /** The value object that will paint */
    protected Value value;

    /** Enables or disables the FunctionPainter. If it is disabled, the
	Line will also be displayed in its endless infiniteness.
    */
    private boolean on = true;

	public FunctionPainter(Value value)
	{
		setValue(value);
	}
	
	public FunctionPainter(Value value, double start, double end) 
	{
		this(value);
		xMin = start;
		xMax = end;
		notifyChange();
	}
	
	public void setValue(Value value)
	{
		if (this.value != null) {
			this.value.removeValueListener(this);
		}
		
		this.value = value;
		
		reset();
		
		value.addValueListener(this);
	}
	
	public void valueChanged(ValueEvent e) 
	{
		if ( (Double.isNaN(xMin)) || (Double.isNaN(xMax))) {
			xMin = value.getValue(); // we don't use reset since we don't
			xMax = value.getValue(); //    want to throw an event here.
		}
		
		if (value.getValue() < xMin) {
			xMin = value.getValue();
		} 
		else if ( value.getValue() >= xMax) {
			xMax = value.getValue();
		}
	}
	
	/** Resets the FunctionPainter by setting the max and min to the current value
	of the trace. This will effectively "erase" the line so that the user
	can repaint it.
	*/
	public synchronized void reset() 
	{
		xMin = value.getValue();
		xMax = value.getValue();
		notifyChange();
	}
	
	/**
	 * Returns a copy of the painter with the SAME Value object
	*/
	public FunctionPainter getCopy()
	{
		FunctionPainter painter = new FunctionPainter(value);
		painter.xMin = this.xMin;
		painter.xMax = this.xMax;
		//It maintains the "on" state. Maybe it should always be on?
		painter.on = this.on;
		return painter;
	}
	
	/**
	 * Returns if the painter is on or not
	*/
	public boolean isOn()
	{
		return on;
	}
	
	/**
	 * Turns the painter on
	*/
	public void setOn()
	{
		on = true;
		notifyChange();
	}
	
	/**
	 * Turns the painter off
	*/
	public void setOff()
	{
		on = false;
		notifyChange();
	}
	
	protected void notifyChange()
	{
		if (value==null) return;
		value.notifyChange(); // This sets a ValueChanged event to redraw
							   // The graphwindow. This mechanism works for
							   // now. There is probably better.
	}
	
}
