/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.4 $
 * $Date: 2004/06/15 19:16:58 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.engine;

import java.awt.*;

import org.concord.graph.engine.*;

/**
 * A GraphFunction is a function of one variable ( f(x) )
 * It is also a MouseControllable, so it can be drawn and manipulated
 * in a GraphWindow
 * It has a color, en expression and supports more than one expression form.
 * It also has the concept of a "painter" and a "ghost"
 *
 * Date created: 04/07/2004
 *
 * @author Eric Brown-Munoz<p>
 * @author Ingrid C Moncada<p>
 *
 */

public interface GraphFunction extends Graphable, MouseSensitive, Selectable, MouseControllable
{
    /*
	 * Returns the value of the function at 'xval' 
	 */
    public double yval(double xval);

    /*
	 * Creates a Graphical object using the string. The format of the
	 * string is determined by the subclass of Graphical. 
	 */
    public void loadString(String instring);

    /*
	 * Gets a string representation of this object. The format is
	 * determined by the subclass of Graphical 
	 */
    public String getLoadString();

    /** 
	 * Returns the string representation of this function
	 * for a specific expression form
	 */
	public String getExpression(int expressionForm);

	/*
	 * Sets the color of the graph 
	 */
	public void setColor(Color c);

	/*
	 * Returns the color of the graph 
	 */
	public Color getColor();

    /** 
	 * Returns the string representation of this function
	 * depending on the expression form this function has
	 */
	public String getExpression();
	
	/**
	 * Returns the expression form of this function
	*/
	public int getExpressionForm();
	
	/**
	 * Sets the expression form of this function
	*/
	public void setExpressionForm(int form);
	
    /** 
	 * Returns the name of the function (f, g, etc)
	 */
	public String getFunctionName();

    /** 
	 * Changes the name of the function (f, g, etc)
	 */
	public void setFunctionName(String fName);

    /** 
	 * Returns the ordinal of the function (1,2, etc)
	 * if the ordinal number is 1 and the name is f,
	 * the function will appear as f1
	 */
	public int getFunctionOrdinal();

    /** 
	 * Changes the ordinal of the function (1,2, etc)
	 * if the ordinal number is 1 and the name is f,
	 * the function will appear as f1
	 */
	public void setFunctionOrdinal(int ordinal);

	/**
	 * Changes the line stroke
	 */
	public void setLineStroke(Stroke lineStroke);

	/**
	 * Returns the line stroke
	 */
	public Stroke getLineStroke();

	/**
	 * Changes the selected line stroke
	 */
	public void setSelectedLineStroke(Stroke selectedLineStroke);

	/**
	 * Returns the selected line stroke
	 */
	public Stroke getSelectedLineStroke();

	/** 
	 * Resets the LinePainter object if it is defined. 
	 */
	public void resetPainter();

	/**
	 * Turns ON the painter on the function
	 */
	public void setPainterOn();
	
	/**
	 * Turns OFF the painter on the function
	 */
	public void setPainterOff();

	/**
	 * Creates an empty painter that listens to the variable
	*/
	public void setPainter(Value variable, boolean on);

	/**
	 * Sets a painter 
	*/
	public void setPainter(FunctionPainter painter);

	/**
	 * Sets a painter, and turns it on or off
	*/
	public void setPainter(FunctionPainter painter, boolean on);

	/**
	 * Indicates if the function will show a ghost when dragged
	*/
	public void setShowGhost(boolean ghost);

	/**
	 * Returns the ghost of the function
	*/
	public GraphFunctionGhost getGhost();
	
	/**
	 * Sets the ghost for the function
	*/
	public void setGhost(GraphFunctionGhost ghost);

}
