/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/24 21:56:14 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.engine;

import java.awt.*;
import javax.swing.*;
import java.util.*;
import javax.swing.border.*;
import java.lang.reflect.*;

import org.concord.graph.event.*;
import org.concord.graphfunction.event.*;

/**
 * Manager that updates objects according with the color 
 * of a GraphFunction
 *
 * @author Ingrid Moncada<p>
 *
   $Revision: 1.2 $
*/

public class GraphFunctionColorListenerManager
	implements GraphFunctionView
{
	Vector colorObjects;
	Vector methodsColorObjects;
	Vector borderObjects;
	Color currentColor;

	GraphFunction function;
	
	public GraphFunctionColorListenerManager()
	{
		colorObjects = new Vector();
		borderObjects = new Vector();
		methodsColorObjects = new Vector();
	}
	
	public void addColorObject(Object obj)
	{
		addColorObject(obj,"");
	}
	
	public void addColorObject(Object obj, String setColorMethod)
	{
		if (obj==null) return;
		colorObjects.add(obj);
		methodsColorObjects.add(setColorMethod);
		if (currentColor!=null){
			changeColorObject(obj, setColorMethod);
		}
	}

	public void addBorderObject(JComponent c)
	{
		if (c==null) return;
		borderObjects.add(c);
		if (currentColor!=null){
			changeBorderColorObject(c);
		}
	}
	
	/**
	 * Sets the GraphFunction that this view will represent. 
	 * it doesn't listen to changes on this GraphFunction
	*/
	public void setGraphFunction(GraphFunction g)
	{
		if (function!=null){
			function.removeGraphableListener(graphableHandler);
		}
		function = g;
		if (function!=null){
			function.addGraphableListener(graphableHandler);
		}

		changeColorObjects();
	}

	private GraphableListener graphableHandler = new GraphableAdapter() 
	{
		/* Listener to the changes in the graphable.
		Repaints the view when it changes.
		*/
	    public void graphableChanged(EventObject e) 
		{
			//Hack to only take care of color changes
			if (isChangeRelevant((GraphFunction)e.getSource())){
				changeColorObjects();
			}
	    }
	};

	private boolean isChangeRelevant(GraphFunction g)
	{
		if (currentColor==null) return true;
		if (currentColor.equals(g.getColor())) return false;
		return true;
	}
	
	private void changeColorObjects()
	{
		if (function==null) {
			currentColor = Color.black;
		}
		else{
			currentColor = function.getColor();
			if (currentColor==null){
				currentColor = Color.black;
			}
		}
		
		int i;
		
		for (i=0;i<borderObjects.size();i++){
			JComponent c = (JComponent)borderObjects.elementAt(i);
			changeBorderColorObject(c);
		}
		for (i=0;i<colorObjects.size();i++){
			Object c = colorObjects.elementAt(i);
			String method = (String)methodsColorObjects.elementAt(i);
			changeColorObject(c, method);
		}
	}
	
	private void changeBorderColorObject(JComponent c)
	{
		if (c==null) return;
		Border b = c.getBorder();
		if (b instanceof LineBorder){
			c.setBorder(new LineBorder(currentColor, ((LineBorder)b).getThickness(),((LineBorder)b).getRoundedCorners()));
		}
		else if (b instanceof EtchedBorder){
			c.setBorder(new EtchedBorder(((EtchedBorder)b).getEtchType(), Color.white, currentColor));
		}
	}
	
	private void changeColorObject(Object c, String method)
	{
		if (c==null) return;
		if (method==null || method.equals("")){
			method = "setColor";
		}
		try{
			Method m = c.getClass().getMethod(method,new Class[]{currentColor.getClass()});
			m.invoke(c, new Object[]{currentColor});
		}catch(Exception e){
			System.err.println("Error invoking method "+method+" with color:"+currentColor+" in "+c);
			e.printStackTrace();
		}
	}
}

