/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.1 $
 * $Date: 2004/05/24 20:55:23 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.engine;

import java.awt.*;

import org.concord.graph.engine.*;

/**
 * GraphFunctionGhost
 *
 * Date created: 04/07/2004
 *
 * @author Eric Brown-Munoz<p>
 * @author Ingrid Moncada<p>
*/

public interface GraphFunctionGhost extends Drawable
{
	/**
	 * Activates (shows) the ghost
	 */
	public void activate();

	/**
	 * Activates (shows) the ghost for a specific duration
	 * After the time has elapsed, it will automatically deactivate
	 * @param duration	The duration in milliseconds. 
	 * 					If it is < 0, then it will not deactivate
	 */
	public void activate(int duration);

	/**
	 * Deactivates (hides) the ghost
	 */
	public void deactivate();

	/**
	 * Sets the color of the ghost
	 * By default, the color is the same as the original function
	 * @param	color	The color of the ghost. If null, the ghost will have
	 *					the same color as the original function
	 */
	public void setColor(Color color);
	
	/**
	 * Sets the time that the ghost will remain active after deactivated ( deactivate() )
	 * @param durationAfterDeactivate	If the ghost duration is -1, it will 
	 *									NOT disappear after deactivated
	 * 									If the ghost duration is 0, it will disappear 
	 *									immediately when deactivated
	 */
	public void setDuration(int durationAfterDeactivate);

}
