/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * @(#)GraphFunctionList.java
 * 
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/24 21:56:14 $
 * $Author: imoncada $
 *
 * Copyright 
*/
package org.concord.graphfunction.engine;

import java.util.*;

import org.concord.graph.engine.*;

/**
 * A list of GraphFunctions
 *
 * @author Ingrid Moncada<p>
 *
*/
public class GraphFunctionList extends SelectableList
{
	private ColorFactory colorFactory = new ColorFactory();
	
	/**
	 * Adds a new function to the list.
	 * By default, when adding a new function to the list
	 * the function gets selected, and a new color gets assigned
	 */
	public boolean add(Object o)
	{
		return add((GraphFunction)o,true,true);
	}
	
	/**
	 * Adds a new function to the list and it selects it if specified
	 * By default, a new color gets assigned
	 */
	public boolean add(GraphFunction f, boolean select)
	{
		return add(f,select,true);
	}

	/**
	 * Adds a new function to the list. It selects it if specified
	 * and it assigns a new color if specified
	 */
	public boolean add(GraphFunction f, boolean select, boolean changeColor)
	{
		boolean ret;
		ret = super.add(f);

		if (changeColor){
			//Color Factory
			f.setColor(colorFactory.getNextColor());
		}

		if (select){
			f.select();
		}
		return ret;
	}

	public ColorFactory getColorFactory()
	{
		return colorFactory;
	}
	
	/**
	 * Print the contents of the list, including the list of selected functions
	 */
	public void printContent()
	{
		GraphFunction f;
		Object obj;
		System.out.println(size()+" functions:");
		for (int i=0;i<size();i++){
			obj = elementAt(i);
			if (obj instanceof GraphFunction){
				f = (GraphFunction)obj;
				if (f.isSelected()){
					System.out.print(" *");
				}
				else{
					System.out.print("  ");
				}
				System.out.println("  "+ f.getColor() + "  " +
					f.getExpression() + "  " + f);
			}
		}

		int i = 0;
		Enumeration sel = getSelectedObjects().elements();
		while (sel.hasMoreElements ()) {
			obj = sel.nextElement();
			if (obj instanceof GraphFunction){
				f = (GraphFunction)obj;
				System.out.println(" *  "+ f.getColor() + "  " +
					f.getExpression() + "  " + f);
			}
			i++;
		}
		System.out.println(i+" selected functions:");
		
		System.out.println("First selected object:" + this.getFirstSelectedObject());
		System.out.println("First selected index:" + this.getFirstSelectedIndex());
	}

	/** Range determines if you are checking a range or just a point */
	public boolean checkFunction(double xval, boolean range) 
	{
		boolean retval = true;
		Enumeration graphables = this.elements();
		while (graphables.hasMoreElements ()) {
			GraphFunction graph = (GraphFunction)graphables.nextElement();
			// check if each Piecewise is a function
			
			//Only for graphables that are visible!
			if (!graph.isVisible()){
				continue;
			}
			
			if ( Piecewise.class.isAssignableFrom(graph.getClass()) ) {
				if (((Piecewise)graph).checkFunction(xval, range) == false) {
					retval = false;
				}
					
			}
		}// while graphables.hasMoreElements()
		return retval;
	}
	
	/** Works like checkFunction, but this will only return a false
	one time for each bad region. This implements the dialog
	box function. */
	public boolean checkFunctionOnce(double xval) 
	{
		boolean retval = true;
		Enumeration graphables = this.elements();
		while (graphables.hasMoreElements ()) {
			GraphFunction graph = (GraphFunction)graphables.nextElement();
			//Only for graphables that are visible!
			if (!graph.isVisible()){
				continue;
			}
			if (graph instanceof Piecewise){
				// check if each Piecewise is a function
				if (((Piecewise)graph).checkFunctionOnce(xval) == false) {
					retval = false;
					//done for "speed"
					return retval;
				}
			}
		}// while graphables.hasMoreElements()
		return retval;
	}

}
