/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.1 $
 * $Date: 2004/05/24 20:55:23 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.engine;

import java.util.*;

import org.concord.graph.event.*;
import org.concord.graphfunction.event.*;

/**
 * Any class that is a view of a function that belongs to a function list
 * can implement this interface
 * The list manager will update automatically the function when
 * it is selected from the function list (calling the method setGraphFunction)
 *
 * @author Ingrid Moncada<p>
*/

public class GraphFunctionViewManager
	implements SelectableListListener
{
	private Vector views; 
	protected GraphFunctionList glist;

	public GraphFunctionViewManager()
	{
		views = new Vector();
	}

	/**
	 * Adds a view to the manager
	 */
	public void addGraphFunctionView(GraphFunctionView v)
	{
		if (!views.contains(v)){
			views.add(v);
			setSelectedGraphFunction(v);
		}
	}

	/**
	 * Removes a view from the list of views of the manager
	 */
	public void removeGraphFunctionView(GraphFunctionView v)
	{
		if (views.contains(v)){
			views.remove(v);
		}
	}

	/**
	 * Register this class with the graphable list, 
	 * to listen to selections on the list
	*/
    public void setGraphFunctionList(GraphFunctionList list) 
	{
		if (glist!=null){
			glist.removeGraphableListListener(this);
		}
		glist = list;
		if (glist!=null){
			//Current selected
			GraphFunction f;
			if (glist == null) return;
			f = (GraphFunction)glist.getFirstSelectedObject();
			setGraphFunction(f);
			
			glist.addGraphableListListener(this);
		}
//		if (glist.getCurrentSelected() != null) {
//			setGraphable((Graphable)glist.getCurrentSelected());
//		}
    }

	///////////////////////////
	// Grahable List event handlers 
	/**
	 * Fired when a Selectable has been selected. 
	 * The event objects source is the object that has selected.
	 */
	public void listGraphableSelected(EventObject e)
	{
		GraphFunction f = (GraphFunction)e.getSource();	//f could be null, that's ok
		setGraphFunction(f);
	}

	private void setSelectedGraphFunction(GraphFunctionView v)
	{
		GraphFunction f;
		if (glist == null) return;
		f = (GraphFunction)glist.getFirstSelectedObject();
		v.setGraphFunction(f);
	}
	
	private void setGraphFunction(GraphFunction f)
	{
		int i;
		GraphFunctionView view;
		for (i=0;i<views.size();i++){
			view = (GraphFunctionView)views.elementAt(i);
			view.setGraphFunction(f);
		}
	}

	/**
	 * Fired when a Selectable has been deselected. 
	 * The event objects source is the object that has deselected.
	 */
	public void listGraphableDeselected(EventObject e)
	{
		GraphFunction f;
		if (glist == null) return;
		f = (GraphFunction)glist.getFirstSelectedObject();
		if (f==null){
			setGraphFunction(f);
		}
	}

	public void listGraphableAdded(EventObject e){}
	public void listGraphableRemoved(EventObject e){}
	public void listGraphableChanged(EventObject e){}
	///////////////////////////
}
