/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.1 $
 * $Date: 2004/07/27 23:04:53 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.engine;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.event.*;

import org.concord.graph.engine.*;
import org.concord.graph.event.*;
import org.concord.graphfunction.event.*;

public class LineTrace extends Trace {

	GraphArea ga;
	Line2D.Double workLine = new Line2D.Double();
	Point2D.Double top = new Point2D.Double();
	Point2D.Double displayTop = new Point2D.Double();

	double upperY = 0.0;
	double lowerYdisplay = 0.0;

	CoordinateSystem cs;

	public LineTrace() {
		super();
		traceColor = Color.red;
	}

	public LineTrace (Value xvalue) {
		this();
		setXValue(xvalue);
	}

	public void draw(Graphics2D g) {

		if (ga == null) {
			setup();
		}

		Stroke savedStroke = null;
		
		Color savedColor = g.getColor();
		savedStroke = g.getStroke();
		
		g.setColor(traceColor);
		
		if (traceStroke != null) {
			g.setStroke(traceStroke);
		}

		if (isVisible() && (xvalue != null)) {

			double xpos = xvalue.getValue();

			top.setLocation(xpos, upperY);

			cs.transformToDisplay(top,displayTop);

			workLine.setLine(displayTop.x, displayTop.y,
							 displayTop.x, lowerYdisplay);

			g.draw(workLine);

			g.setColor(savedColor);
			g.setStroke(savedStroke);
		}
	}

	void setup() {

		if (ga == null) {
			ga = getGraphArea(); // FIXME if this returns null we are bad.
			ga.addChangeListener ( new ChangeListener() {
					public void stateChanged ( ChangeEvent e) {
						setup();
					}
				});
		}


		cs = ga.getCoordinateSystem();
		upperY = ga.getUpperRightCornerWorld().getY();
		lowerYdisplay = ga.getLowerLeftCornerDisplay().getY();
	}
} // LineTrace
			
			

			
