/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * @(#)LinearEquation.java
 * 
 * Last modification information:
 * $Revision: 1.4 $
 * $Date: 2004/07/27 23:04:53 $
 * $Author: ebrown-munoz $
 *
 * Copyright 
*/
package org.concord.graphfunction.engine;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;

import org.concord.graph.engine.*;
import org.concord.graph.event.*;
import org.concord.graphfunction.event.*;

/**
 * A linear equation like y = m*x + b
 *
 * Date created: 04/07/2004
 *
 * @author Ingrid Moncada<p>
 *
*/

public class LinearEquation 
	extends FunctionEquation 
	implements GraphableListener,
			   ValueListener
{
	BasicStroke bigStroke = new BasicStroke(5.0f);
	BasicStroke yellowStroke = new BasicStroke(3.0f);

	public LinearEquation()
	{
		super();
		setLinear(new LinearFunction());
		setResultValue(new Value());
	}

	public LinearEquation(Value result)
	{
		super();
		setLinear(new LinearFunction());
		setResultValue(result);
	}

	public LinearEquation(LinearFunction function)
	{
		super();
		setLinear(function);
		setResultValue(new Value());
	}

	public LinearEquation(LinearFunction function, Value result)
	{
		super();
		setLinear(function);
		setResultValue(result);
	}
	
	/**
	 * Returns the linear function associated with this equation
	*/
	public LinearFunction getLinear()
	{
		return (LinearFunction)super.getFunction();
	}
	
	/**
	 * Sets the linear function associated with this equation
	*/
	public void setFunction(GraphFunction f)
	{
		if (f instanceof LinearFunction)
		{
			if (getLinear() != null) {
				getLinear().removeGraphableListener(this);
			} 
			setLinear((LinearFunction)f);
			f.addGraphableListener(this);
		}
	}
	
	/**
	 * Sets the linear function associated with this equation
	*/
	public void setLinear(LinearFunction f)
	{
		super.setFunction(f);
	}

	/** Override draw solely to handle the case where slope=0
		FIXME
	*/
	public void draw(Graphics2D g) {
		
		super.draw(g);
		
		LinearFunction lfunction = (LinearFunction)function;

		// FIXME This was quickly ripped from LinearFunction.
		if (lfunction.getSlope() == 0.0) {
			CoordinateSystem cs = 
				lfunction.getGraphArea().getCoordinateSystem();
			double result = getResultValue().getValue();

			double domainStart;
			double domainEnd;
		

			Rectangle clip = g.getClipBounds();
			
			Point2D leftPoint = new Point2D.Double( clip.x , 0.0 );
			Point2D rightPoint = 
				new Point2D.Double( clip.x + clip.width , 0.0 );
		
			// Left and right edges in real coordinates go here.
			Point2D.Double leftScreen = new Point2D.Double();
			Point2D.Double rightScreen = new Point2D.Double();
			cs.transformToWorld ( leftPoint, leftScreen) ;
			cs.transformToWorld ( rightPoint, rightScreen) ;
			domainStart = leftScreen.x;
			domainEnd = rightScreen.x;

			// Translate back to screen coordinates (with yvals) and draw
		
			Point2D start = new Point2D.Double();
			cs.transformToDisplay(new Point2D.Double(domainStart,
													 result),
								  start);
			Point2D end = new Point2D.Double();
			cs.transformToDisplay(new Point2D.Double(domainEnd, 
													 result),
								  end);
		
			Stroke saveStroke = g.getStroke();
			Color saveColor = g.getColor();

			g.setStroke(bigStroke);
			g.setColor(lfunction.getColor());
			g.draw(new Line2D.Double(start, end));

			g.setStroke(yellowStroke);
			g.setColor(Color.yellow);
			g.draw(new Line2D.Double(start, end));
		
			g.setStroke(saveStroke);
			g.setColor(saveColor);
		
		} // if (slope == 0.0)
	} // draw

	///////////////////////////////////
	// methods from interface Equation
	///////////////////////////////////
	/**
	 * Gets the value of X that solves the equation
	*/
	public double getSolution()
	{
		double m,b;
		
		if (function==null || valueResult==null) return Double.NaN;
		
		m = ((LinearFunction)function).getSlope();
		if (m==0) return Double.POSITIVE_INFINITY;

		b = ((LinearFunction)function).getYIntercept();
		return (valueResult.getValue() - b) / m;
	}
	
	protected boolean mouseDraggedEquationPoint(Point location)
	{

		CoordinateSystem cs = getGraphArea().getCoordinateSystem();

		if (function == null) return false;
		LinearFunction f = (LinearFunction)function;

		Point2D.Double transformedLocation = new Point2D.Double();
		cs.transformToWorld(location,transformedLocation);
		
		double changeY = transformedLocation.y - f.clickLocation.y;

		valueResult.setValue(originalValueResult.getValue() + changeY);
		return true;
	}

	///////////////////////////////////
	
	////////////////////////////////////////////////////////////////
	//Methods from LinearFunction class
	
    public double getSlope() 
	{
		return ((LinearFunction)function).getSlope();
    }

    public double getYIntercept() 
	{
		return ((LinearFunction)function).getYIntercept();
    }

    public void setSlope(double slope) 
	{
		((LinearFunction)function).setSlope(slope);
		//This will fire the change event in the function
		//and will be catched by the equation, who will fire it
		//to its listeners
    }


    public void setYIntercept(double yIntercept) 
	{

		((LinearFunction)function).setYIntercept(yIntercept);
		//This will fire the change event in the function
		//and will be catched by the equation, who will fire it
		//to its listeners
    }

	
	////////////////////////////////////////////////////////////////

	////////////////////////////////////////////////////////////////
	// Methods of the interface Graphable
	////////////////////////////////////////////////////////////////
	public Graphable getCopy()
	{
		LinearEquation res = new LinearEquation(
			(LinearFunction)this.function.getCopy(),
			new Value(this.valueResult.getValue()));
		res.setColor(this.getColor());
		res.setAllowMoveEquationPoint(this.allowMoveEquationPoint); 
		res.setExpressionForm(this.getExpressionForm());
		return res;
	}
	////////////////////////////////////////////////////

	public void graphableChanged(EventObject e) {

		super.graphableChanged(e);
		double slope = getSlope();
		double yintercept = ((LinearFunction)function).getYIntercept();
		double value = this.valueResult.getValue();
		if ((getSlope() == 0.0) && (yintercept != value)) {
			valueResult.setValue(yintercept);
		}
 
	}

	public void graphableRemoved(EventObject e) {}

	/**
	 * Listener for the changes in the variable value
	*/
    public void valueChanged(ValueEvent e)
	{
		super.valueChanged(e);
		double slope = getSlope();
		double yintercept = ((LinearFunction)function).getYIntercept();
		double value = this.valueResult.getValue();
		if ((getSlope() == 0.0) && (yintercept != value)) {
			setYIntercept(value);
		}
	}

}
