/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * @(#)LinearPushpinObject.java
 * 
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/06/15 19:16:58 $
 * $Author: ebrown-munoz $
 *
 * Copyright 
*/
package org.concord.graphfunction.engine;

import java.awt.*;
import java.awt.geom.*;

import org.concord.graph.engine.*;


/**
 *
 * Date created: 04/07/2004
 *
 * @author Eric Brown-Munoz<p>
 *
 */
public class LinearPushpinObject 
{
	private LinearFunction line;
	Font pushpinFont = new Font ("Serif", Font.BOLD, 12);
	
	boolean moused = false;
	
	/** if this is false, you can translate the line by moving the pushpin.
	While this is true, the pushpin can only move on the line. */
	private boolean pushpinFixed = true;
	
	double xPosition = 3.0 ; // The position on the line
	
	Point2D.Double clickLocation = new Point2D.Double();
	double origXPosition = 3.0;
	
	public LinearPushpinObject(LinearFunction line) 
	{
		this.line = line;
	}
	
	public void draw(Graphics2D g) 
	{	

		CoordinateSystem cs = getGraphArea().getCoordinateSystem();

		// Get coordinate of pushpin in screen space
		Point2D.Double screenPosition = new Point2D.Double();
		
		double yPosition = line.yval(xPosition);
		
		cs.transformToDisplay(new Point2D.Double(xPosition, yPosition),
							  screenPosition);
		
		g.fillOval ( (int)screenPosition.x - 5 , (int)screenPosition.y -5,
				 10, 10);
		
		String coordinates = "(" + Math.floor(xPosition * 10.0)/10.0 + "," +
			Math.floor(yPosition * 10.0)/ 10.0 + ")";
		
		if (moused) {
			Font saveFont = g.getFont();
			g.setFont(pushpinFont);
			g.drawString( coordinates, (float)screenPosition.x + 15.0f, 
				  (float)screenPosition.y );
			g.setFont(saveFont);
		}
	
	}
	
	public boolean mousePressed(Point location) 
	{
		CoordinateSystem cs = getGraphArea().getCoordinateSystem();
		boolean retval = false;
		
		Point2D.Double screenPosition= new Point2D.Double();
		cs.transformToDisplay(new Point2D.Double(xPosition,
												 line.yval(xPosition)),
							  screenPosition);
		
		if (screenPosition.distance(location) < 6.0) { // pixels
			cs.transformToWorld( location, clickLocation);
		
			retval = true;
			moused = true;
		
			origXPosition = xPosition;
		}
		
		return retval;
	}
	
	public boolean mouseDragged(Point location) 
	{

		CoordinateSystem cs = getGraphArea().getCoordinateSystem();

		boolean retval = true;
		
		// Get click location in screen coordinates.
		Point2D.Double screenLocation = new Point2D.Double();
		cs.transformToWorld(location, screenLocation);
		
		double angle = Math.atan(line.getSlope());
		
		if (pushpinFixed) {
			/* You can not just change the xlocation because this is very
			   wrong -- especially when the line has a steep slope.
			   This basically finds the xcoordinate for a rotated
			   axis where the line is on the x axis */
			xPosition = origXPosition + 
			((screenLocation.x - clickLocation.x) * Math.cos(angle) +
			 (screenLocation.y - clickLocation.y) * Math.sin(angle)) *
			Math.cos(angle);
		} else {
			double slope = line.getSlope();
			xPosition = origXPosition + (screenLocation.x - clickLocation.x);
			line.setYIntercept(screenLocation.y - 
					   line.getSlope() * xPosition);
		}
		
		line.notifyChange();
		
		return retval;
	}
	
	public void setXPosition( double xposition) 
	{
		this.xPosition = xposition;
		line.notifyChange();
	}
	
	/** returns the current xPosition */
	public double getXPosition() 
	{
		return xPosition;
	}
	
	public boolean mouseReleased(Point location) 
	{	
		boolean retval = false;
		
		if (moused = true) {
			retval = true;
			moused = false;
		}
		
		return retval;
	}
	
	public void setFixed(boolean b)
	{
		pushpinFixed = b;
	}

	public boolean getFixed()
	{
		return pushpinFixed;
	}

	public GraphArea getGraphArea() {
		GraphArea retval = null;

		if (line != null) {
			retval = line.getGraphArea();
		}
		return retval;
	}

}


    
