/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.1 $
 * $Date: 2004/05/24 20:55:23 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.engine;

import java.text.*;

/**
 * Utility class that provides basic math functions
 * 
 * @author Ingrid Moncada<p>
*/
public class MathUtil
{
	/**
	 * Returns if two equals are equal (with a threshold of 0.001)
	 */
	public static boolean equalsDouble(double a, double b)
	{
		//Default threshold is 0.001
		return equalsDouble(a,b,0.001);
	}
	
	/**
	 * Returns if two equals are close enough with the the specified threshold
	 * @param threshold  Has to be positive
	 */
	public static boolean equalsDouble(double a, double b, double threshold)
	{
		double c = a-b;
		if (Math.abs(c)>threshold) return false;
		return true;
	}

	public static double roundTenth(double value)
	{
		return roundTenth(value, 10.0);
	}

	public static double roundTenth( double value, double precision) 
	{
		double retval = Math.floor((precision * value) + 0.5) / precision;
//System.out.println("value:"+value+" precision:"+precision+" retval:"+retval);
		return retval;
	}

	public static String getStringRound(double value, int numDecimalsMax) 
	{
		String strDecimals ="";
		for (int i=0;i<numDecimalsMax;i++){
			strDecimals = strDecimals + "#";
		}
		DecimalFormat df = new DecimalFormat("#."+strDecimals);
		//This makes values really close to 0 to be 0
		//Specially done so -0 turns into 0
		if (equalsDouble(value,0.0)) value = 0.0;
		return df.format(value);
//		if (value <= (int)value){
//			return String.valueOf((int)value);
//		}
//		return String.valueOf(value);
	}

	public static int log10(double num)
	{
		double n;
		int i=0;
		
		if (num == 0.0) return 0;
		
		n = Math.abs(num);
		
		if (n >= 1.0){
			while(n >= 10.0){
				i++;
				n = n/10.0;
			}
		}
		else if (n < 1.0){
			//while(n < 1.0){
			while(n > (int)n){
				i--;
				n = n*10.0;
			}
		}
		return i;
	}	
}
