/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * @(#)SegmentDomain.java
 * 
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/07/20 17:35:37 $
 * $Author: jqian $
 *
 * Copyright 
*/
package org.concord.graphfunction.engine;

import java.util.*;
import org.concord.graphfunction.event.*;

/**
 * Manages the Domain for a Piecewise segment.
 *
 * Date created: 04/07/2004
 *
 * @author Eric Brown-Munoz<p>
 * @author Ingrid C Moncada<p>
 *
 */

public class SegmentDomain extends Domain 
{
//	private double interval = 0.1;
	
	private Vector listeners;
	
	public SegmentDomain(double start, double end, double interval) 
	{
		this.start = start;
		this.end = end;
        this.interval = interval;
		listeners = new Vector();
	}
	public SegmentDomain( double start, double end) 
	{
		this.start = start;
		this.end = end;
        this.interval = 0.1;
		listeners = new Vector();
	}
    
	public void addDomainListener(DomainListener dl) 
	{
		if (!listeners.contains(dl)){
			listeners.add(dl);
		}
	}
	
	public void removeDomainListener(DomainListener dl) 
	{
		if (listeners.contains(dl)) {
			listeners.remove(dl);
		}
	}
	
	public void notifyChange() 
	{
		DomainEvent e = 
			new DomainEvent (this);
		Enumeration lists = listeners.elements();
		while (lists.hasMoreElements()) {
			DomainListener gl = (DomainListener)lists.nextElement();
			gl.domainChanged(e);
		}
	}
	
	public void setStart( double start) 
	{
		this.start = start;
		notifyChange();
	}
	
	public void setEnd( double end) 
	{
		this.end   = end;
		notifyChange();
	}
	
	public void setClosedStart(boolean bool) 
	{
		closedStart = bool;
		notifyChange();
	}
	
	public void setClosedEnd(boolean bool) 
	{
		closedEnd = bool;
		notifyChange();
	}
	
	public boolean getClosedStart() 
	{
		return closedStart;
	}
	
	public boolean getClosedEnd() 
	{
		return closedEnd;
	}
	
	public void fix() 
	{
		start = Math.floor(start/interval + 0.5) * interval;   
		end = Math.floor(end/interval + 0.5) * interval;   
	}
}
