/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * @(#)Value.java
 * 
 * Last modification information:
 * $Revision: 1.1 $
 * $Date: 2004/05/24 20:55:23 $
 * $Author: imoncada $
 *
 * Copyright 
*/
package org.concord.graphfunction.engine;

import java.util.*;

import org.concord.graphfunction.event.*;

/**
 * A class that represents a double value.
 * Supports listeners to the value change
 *
 * Date created: 04/07/2004
 *
 * @author Eric Brown-Munoz<p>
 * @author Ingrid C Moncada<p>
 *
 */

public class Value 
{
    private double value = Double.NaN;

    public double lastValid = Double.NaN; // A hack used by PiecewisePanel

    Vector listeners;

    public Value() 
	{
		listeners = new Vector();
    }

	public Value(double val)
	{
		listeners = new Vector();
		setValue(val);
	}
	
	public double getValue() 
	{
		return value;
	}
	
	public synchronized void setValue(double value) 
	{
		this.value = value;
		notifyChange();
	}
	
	public void notifyChange() 
	{
		Enumeration lists = listeners.elements();
		
		ValueEvent e = new ValueEvent(this);
		while (lists.hasMoreElements()) {
			((ValueListener)lists.nextElement()).valueChanged(e);
		}
	}
	
	public void addValueListener(ValueListener listener) 
	{
		if (! listeners.contains(listener)) {
			listeners.add(listener);
		}
	}
	
	public void removeValueListener(ValueListener listener) 
	{
		if (listeners.contains(listener)) {
			listeners.remove(listener);
		}
	}

}
	

     
