/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/07/14 17:18:50 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import org.concord.graphfunction.event.*;

/**
 * @author Eric Brown-Munoz<p>
*/
public class ArrowControl extends JPanel 
	implements MouseListener,
			   FocusListener
{
    int iconHeight = 27;
    int iconWidth = 27;

    JButton upButton;
    JButton downButton;
    JButton rightButton;
    JButton leftButton;


    String buttonPressed;

    TimerTask timerTask;

    Vector listeners;

    java.util.Timer buttonTimer; // Moves button multple times if you hold it down

	public ArrowControl() {

		addFocusListener(this);

		listeners = new Vector(); // This is for ArrowControlListeners
	
		setLayout(null);
		
		ImageIcon upArrow = ResourceLoader.getImageIcon("superlittleuparrow.gif","Up Arrow");
		upButton = new JButton (upArrow);
		upButton.setFocusPainted(false);
		ImageIcon downArrow = ResourceLoader.getImageIcon("superlittledownarrow.gif","Down Arrow");
		downButton = new JButton (downArrow);
		downButton.setFocusPainted(false);
		ImageIcon rightArrow = ResourceLoader.getImageIcon("littlerightarrow.gif","Right Arrow");
		rightButton = new JButton (rightArrow);
		rightButton.setFocusPainted(false);
		ImageIcon leftArrow = ResourceLoader.getImageIcon("littleleftarrow.gif","Left Arrow");
		leftButton = new JButton (leftArrow);
		leftButton.setFocusPainted(false);

		if (upArrow!=null){
			iconHeight = upArrow.getIconHeight() + 6;
			iconWidth  = upArrow.getIconWidth() + 6;
		}
		
		setPreferredSize(new Dimension(iconWidth*2+2, iconHeight*2+2));
		
		upButton.addMouseListener(this);
		downButton.addMouseListener(this);
		rightButton.addMouseListener(this);
		leftButton.addMouseListener(this);
	
		upButton.setActionCommand("up");
		downButton.setActionCommand("down");
		rightButton.setActionCommand("right");
		leftButton.setActionCommand("left");
	
		add(upButton);
		add(downButton);	
		add(leftButton);
		add(rightButton);
		
		//By default, left and right buttons are not visible 
		//Taken out left and right buttons! yeeeeeah!
		leftButton.setVisible(false);
		rightButton.setVisible(false);

		addComponentListener ( new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				int screenWidth = ((Component)e.getSource()).getWidth();
				int screenHeight = ((Component)e.getSource()).getHeight();
		
				int centerX = (screenWidth - iconWidth) / 2;
				int centerY = (screenHeight - iconHeight) / 2;
	
				upButton.setBounds(centerX, 
						   (screenHeight - iconHeight * 2) / 2,
						   iconWidth, iconHeight);
	
				downButton.setBounds(centerX, 
						   screenHeight - iconHeight -
						 (screenHeight - iconHeight * 2) / 2,
						   iconWidth, iconHeight);
	
				leftButton.setBounds((screenWidth - iconWidth * 3) / 2,
						  centerY, 
						  iconWidth, iconHeight);
	
				rightButton.setBounds(screenWidth - iconWidth -
						   (screenWidth - iconWidth * 3) / 2,
						 centerY, 
						 iconWidth, iconHeight);
			}
	    });

		// start timer. the true means the program will end even if the timer
		// isn't finished.
		buttonTimer = new java.util.Timer(true); 

    }
	
	public void setButtonVisible(String buttonDirection, boolean visible)
	{
		if (upButton.getActionCommand().equals(buttonDirection)){
			upButton.setVisible(visible);
		}
		else if (downButton.getActionCommand().equals(buttonDirection)){
			downButton.setVisible(visible);
		}
		else if (leftButton.getActionCommand().equals(buttonDirection)){
			leftButton.setVisible(visible);
		}
		else if (rightButton.getActionCommand().equals(buttonDirection)){
			rightButton.setVisible(visible);
		}
		if (upButton.isVisible() || downButton.isVisible()){
			//setPreferredSize(new Dimension(iconWidth*3+2, iconHeight*2+2));
			validate();
		}		
	}

    public void addArrowControlListener( ArrowControlListener listener) {
		listeners.add(listener);
    }

    public void mousePressed(MouseEvent e) {
		buttonPressed = ((JButton)e.getSource()).getActionCommand();
		notify(buttonPressed);
		if (timerTask != null)  // This should fix the runaway numbers bug
			timerTask.cancel();
		timerTask = new multiTask(this);
		buttonTimer.schedule(timerTask, 750, 50);
    }

    public void mouseReleased(MouseEvent e) {
		timerTask.cancel();
    }

    public void mouseClicked(MouseEvent e) {}

    public void mouseEntered(MouseEvent e) {}

    public void mouseExited(MouseEvent e) {
		if (timerTask != null)
			timerTask.cancel();
	}


	public void focusGained(FocusEvent e) {}

	public void focusLost(FocusEvent e) {
		if (timerTask != null) 
			timerTask.cancel();
	}

    public void notify(String direction) {
		Enumeration items = listeners.elements();
	
		// This should be optimized.
		while (items.hasMoreElements()) {
			ArrowControlListener current = 
			(ArrowControlListener)items.nextElement();
			
			if (direction.equals("up")) {
			current.upPressed( new EventObject(this));
	
			} else if (direction.equals("down")) {
			current.downPressed(new EventObject(this));
	
			} else if (direction.equals("right")) {
			current.rightPressed(new EventObject(this));
	
			} else if (direction.equals("left")) {
			current.leftPressed(new EventObject(this));
			}
		}
		    
    }
	
	/**
	 * Sets if the component is enabled or not
	*/
	public void setEnabled(boolean b)
	{
		super.setEnabled(b);
		upButton.setEnabled(b);
		downButton.setEnabled(b);
		leftButton.setEnabled(b);
		rightButton.setEnabled(b);
	}

    public static void main(String args[]) {
	JFrame frame = new JFrame();
	ArrowControl that = new ArrowControl();
	that.setVisible(true);
	frame.getContentPane().add(that);
	frame.setSize(300,500);
	frame.show();
    }


}

class multiTask extends TimerTask {
    ArrowControl parent;

    public multiTask(ArrowControl parent) {
	this.parent = parent;
    }

    public void run() {
	parent.notify(parent.buttonPressed);
    }

}


