/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
   $Revision: 1.2 $
*/

package org.concord.graphfunction.ui;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import javax.swing.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;

public class Gauge extends JPanel 
	implements ValueListener 
{

	public static int MARGIN = 5; // pixels
	static double LNHALF = 0.693197; // ln (1/2)

	double coefficient = 0.693197 / 50.0;


	Value value = null;
	double bestValue = Double.MAX_VALUE;

	double radius;

	GeneralPath needle;
	Image background;
	int width;
	int height;

	double needleX;
	double needleY;


	public Gauge () {

		addComponentListener(new ComponentAdapter() {
				public void componentResized(ComponentEvent e) {
					createBackground();
					repaint();
				}
			});

		needle = new GeneralPath ();
		needle.moveTo( 0.0f, -0.1f);
		needle.lineTo( -0.1f, 0.0f);
		needle.lineTo( 0.0f, 1.0f);
		needle.lineTo( 0.1f, 0.0f);
		needle.closePath();
	}
		
	public void setValue (Value value) {
		this.value = value;
		value.addValueListener (this);
	}


	/** Sets the value the represents when the needle is pointing straight
		up */

	public void setHalf (double halfPoint) {
		coefficient = LNHALF/halfPoint;
	}

	public void createBackground () {
		width = getWidth();
		height = getHeight();

		background = createImage(width, height);

		Graphics2D g2 = (Graphics2D)background.getGraphics();
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
								 RenderingHints.VALUE_ANTIALIAS_ON);

		int graphicHeight = height - MARGIN * 2;
		int graphicWidth = width - MARGIN * 2;

		if ( (graphicHeight)  <  (graphicWidth/2) ) {
			radius = graphicHeight * 0.9 ;
		} else {
			radius = graphicWidth/2 * 0.9 ;
		}

		needleX = width /2.0;
		needleY = height -MARGIN - radius/9.0;


		Color savedColor = g2.getColor();
		Shape savedClip = g2.getClip();

		g2.clipRect(0,0,width, height - MARGIN);
		g2.setColor(Color.green);
		g2.fill( new Ellipse2D.Double ((double) (width/2.0 - radius),
									   height - MARGIN - (1.1) * radius, 
									   (double) 2.0 * radius, 
									   2 * radius));
		g2.setColor(Color.black);
		g2.draw( new Ellipse2D.Double ((double) (width/2.0 - radius),
									   height - MARGIN - (1.1) * radius, 
									   (double) 2.0 * radius, 
									   2 * radius));

		g2.setColor(Color.gray);
		g2.draw( new Line2D.Double(needleX, needleY, 
								   needleX - radius * 1.0, needleY));
		g2.setClip(savedClip);
		g2.setColor(savedColor);
	}

	public void reset() {
		bestValue = Double.MAX_VALUE;
		repaint();
	}

	public void paint (Graphics g) {

		double myValue = value.getValue();

		if (bestValue > myValue) {
			bestValue = myValue;
		}

		super.paint(g);

		if (background == null) {
			createBackground();
		}

		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
								 RenderingHints.VALUE_ANTIALIAS_ON);
		g2.drawImage(background,0,0,this);

		if (value != null) {
			Color savedColor = g2.getColor();
			AffineTransform savedTransform = g2.getTransform();

			g2.setColor(Color.black);

			g2.translate(needleX, needleY);
			g2.scale(radius, radius);
 			/* g2.rotate( Math.PI/2.0 + Math.PI * 
			   (value.getValue() - min)/((double)(max - min))); */

			g2.rotate( Math.PI/2.0 + 
					   Math.PI * (1 - Math.exp(- coefficient * myValue)));

			g2.fill(needle);

			g2.setTransform(savedTransform);

			g2.setColor(Color.red);
			g2.translate(needleX, needleY);
			g2.rotate(Math.PI/2.0 + 
					   Math.PI * (1 - Math.exp(- coefficient * bestValue)));
			g2.draw(new Line2D.Double(0.0, 0.0, 0.0, radius));


			g2.setTransform(savedTransform);
			g2.setColor(savedColor);
		}
	}

	public void valueChanged (ValueEvent e) {
		repaint ();
	}

	public static void main(String args[]) {

		final JFrame frame = new JFrame();
		final JPanel fa = new Gauge();
		frame.getContentPane().add(fa);
		frame.setSize(600,600);
		frame.show();
		
	}		
}
		
		

		
