/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/24 21:56:14 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui;

import java.awt.*;
import java.util.*;

import javax.swing.*;

import org.concord.graph.event.*;
import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;

/**
 * A label that displays the expression of a function
 * It shows the foreground according to each graphable's color
 * when the graphable is selected it shows a border and in BOLD
 *
 * @author Ingrid Moncada<p>
 *
*/
public class GraphFunctionLabel extends JLabel
	implements GraphFunctionView
{
	GraphFunction function;
	
	public GraphFunctionLabel()
	{
	}
	
	public GraphFunctionLabel(GraphFunction f)
	{
		setGraphFunction(f);
	}

    SelectableAdapter graphableHandler = new SelectableAdapter() 
	{
	    public void graphableChanged(EventObject e) 
		{
			update();
	    }
	    public void graphableSelected(EventObject e) 
		{
			update();
	    }
	    public void graphableDeselected(EventObject e) 
		{
			update();
	    }
	};

	public void setGraphFunction(GraphFunction f)
	{
		setGraphFunction(f,true);
	}

	public void setGraphFunction(GraphFunction f, boolean autoUpdate)
	{
		if (this.function != null) {
		    this.function.removeGraphableListener(graphableHandler);
			this.function.removeSelectableListener(graphableHandler);
		}
		this.function = f;
		if (autoUpdate){
			if (this.function != null) {
				this.function.addGraphableListener(graphableHandler);
				this.function.addSelectableListener(graphableHandler);
			}
		}
		update();
	}
	
	public GraphFunction getGraphFunction()
	{
		return function;
	}
	
	public void updateLook()
	{
		Color functionColor=null;
		Color foreColor=null;
		Color backColor=null;

		if (function==null) return;
		
		functionColor = function.getColor();
		
		if (function.isSelected()) {
			
			//foreColor = functionColor;
			if(foreColor == null){
				foreColor = Color.black;	//XXX
			}

			if (backColor == null){
				backColor = Color.black;	//XXX
			}
			
			super.setBackground(backColor);
			super.setForeground(foreColor);
			
			setFont(this.getFont().deriveFont(Font.BOLD));	//XXX
			super.setBorder(BorderFactory.createLineBorder(functionColor,2));
			
		}
		else {
			
			//foreColor = functionColor;
			if(foreColor == null){
				foreColor = Color.black;	//XXX
			}
			if(backColor == null){
				backColor = Color.black;	//XXX
			}
			
			super.setForeground(foreColor);
			super.setBackground(backColor);
	
			//setFont(table.getFont());
			super.setBorder(null);			//XXX
		}		
	}
	
	public void update()
	{
		if (function==null) return;
		updateLook();
		this.setText(function.getExpression());
	}
}

