/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
   $Revision: 1.1 $
*/
package org.concord.graphfunction.ui;

import java.util.*;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;

import org.concord.graphfunction.engine.*;
import org.concord.graph.engine.*;
import org.concord.graph.event.*;

public class GraphFunctionTable extends JScrollPane 
{
    private JTable table = new JTable();
    private GraphFunctionList functionList = null;

    ListSelectionListener listSelectionListener = new ListSelectionListener() 
	{
	    public void valueChanged(ListSelectionEvent e) 
		{
//System.out.println(" @@ ListSelection.valueChanged event " + e.getValueIsAdjusting());
			if (e.getValueIsAdjusting()) return;
			ListSelectionModel lsm = (ListSelectionModel)e.getSource();
	
			if ( functionList != null) {
				int index;
				int fIndex;
				
				if (table.getSelectionModel().getSelectionMode()==ListSelectionModel.SINGLE_SELECTION){
					index = lsm.getMinSelectionIndex();
					fIndex = functionList.getFirstSelectedIndex();
					if (index >= 0 && fIndex!=index) {
					//The table has something selected but it is different from the selection on the list
						functionList.select(index);
					} 
					else if (fIndex!=index && fIndex >= 0){
					//The table doesn't have anything selected and
					//it is different from the selection on the list
					//	setSelectionIndexInternal(lsm, fIndex);
					}
				}
				else{
					index = lsm.getLeadSelectionIndex();
					if (index >= 0) {
					//The table has something selected
						functionList.select(index);
					} 
				}
			}
			setupColWidth();
	    }
	};

    public GraphFunctionTable() 
	{
		super();
	
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		//table.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		
		//table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
				
		table.setSelectionBackground(Color.white);
		
		setViewportView(table);
		setPreferredSize( new Dimension( 200, 200));
    }

	private void setupColWidth()
	{
		TableColumn column = table.getColumnModel().getColumn(0);
		column.setMaxWidth(50);
	}
	
	public void setGraphFunctionList( GraphFunctionList list ) 
	{
		if (functionList!=null){
			list.removeGraphableListListener(functionListListener);
		}
		
		this.functionList = list;

		GraphFunctionTableRenderer cellRenderer = new GraphFunctionTableRenderer();
		cellRenderer.setGraphFunctionList(list);
		table.setDefaultRenderer(String.class, cellRenderer);
		
		GraphFunctionTableModel tableModel = new GraphFunctionTableModel(list);
		setTableModel(tableModel);

		setupColWidth();

		if (functionList!=null){
			ListSelectionModel lsm = table.getSelectionModel();
			int newIndex = functionList.getFirstSelectedIndex();
			int index = lsm.getMinSelectionIndex();
			if (newIndex >= 0 && newIndex!=index) {
				addSelectionIndexInternal(lsm, newIndex);
			}
			list.addGraphableListListener(functionListListener);
		}
		
    }
    
	GraphableListListener functionListListener = new GraphableListAdapter()
	{
		public void listGraphableSelected(EventObject e) 
		{
//System.out.println(" T Table receives selected event");
			int newIndex = functionList.indexOf((Graphable)e.getSource());
			if (newIndex!=-1){
				ListSelectionModel lsm = table.getSelectionModel();
				int index = lsm.getMinSelectionIndex();
				//if (index!=newIndex){
					addSelectionIndexInternal(lsm, newIndex);
				//}
			}
		}

		public void listGraphableDeselected(EventObject e) 
		{
//System.out.println(" T Table receives deselected event");
			int newIndex = functionList.indexOf((Graphable)e.getSource());
			if (newIndex!=-1){
				ListSelectionModel lsm = table.getSelectionModel();
				int index = lsm.getMinSelectionIndex();
				//if (index!=newIndex){
					removeSelectionIndexInternal(lsm, newIndex);
				//}
			}
		}
	};

	private void addSelectionIndexInternal(ListSelectionModel lsm, int newIndex)
	{
		lsm.removeListSelectionListener(listSelectionListener);
		//lsm.setLeadSelectionIndex(newIndex);
		lsm.addSelectionInterval(newIndex,newIndex);
		lsm.addListSelectionListener(listSelectionListener);
	}
	
	private void removeSelectionIndexInternal(ListSelectionModel lsm, int newIndex)
	{
		lsm.removeListSelectionListener(listSelectionListener);
		lsm.removeSelectionInterval(newIndex,newIndex);
		lsm.addListSelectionListener(listSelectionListener);
	}

	public JTable getTable()
	{
		return table;
	}
	
    public void setTableModel(GraphFunctionTableModel model) 
	{
		table.setModel(model);
		table.getSelectionModel().addListSelectionListener(listSelectionListener);
    }

    public GraphFunctionTableModel getTableModel() 
	{
		return (GraphFunctionTableModel)table.getModel();
    }
}
