/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
   $Revision: 1.2 $
*/
package org.concord.graphfunction.ui;

import java.util.*;

import javax.swing.table.*;

import org.concord.graphfunction.engine.*;
import org.concord.graph.engine.*;
import org.concord.graph.event.*;

public class GraphFunctionTableModel extends AbstractTableModel 
{
	GraphFunctionList graphables;
	TableCellRenderer cellRenderer;
	
	private String colNames[] = {"Visible","Function"};

	public GraphFunctionTableModel() 
	{
	}
	
	public GraphFunctionTableModel(GraphFunctionList list ) 
	{
		setGraphFunctionList(list);
	}
	
	public void setGraphFunctionList( GraphFunctionList list ) 
	{
		graphables = list;
		
		if (list==null) return;
		list.addGraphableListListener(new GraphableListAdapter() 
		{
			int index;
			
			public void listGraphableAdded(EventObject e) 
			{		
				index = graphables.indexOf((Graphable)e.getSource());
				fireTableRowsInserted(index,index);
			}
	
			public void listGraphableRemoved(EventObject e) 
			{
				//The following two lines cause the columns to change size,
				//so I took them out
				//index = graphables.indexOf((Graphable)e.getSource());
				//fireTableRowsDeleted(index,index);
				
				fireTableDataChanged();
			}
	
			public void listGraphableChanged(EventObject e) 
			{
				//This is necessary to update the table with the info of the graphable
				//But if also fires a valueChanged(ListSelectionEvent) event on the table
				//even if the selection is the same.
				//The handler of this event is in GraphableTable, and it doesn't
				//do anything if the selected index is the same
				index = graphables.indexOf((Graphable)e.getSource());
				fireTableRowsUpdated(index,index);
			}
	
			public void listGraphableSelected(EventObject e) 
			{
				//This is not necessary because the Table is listening to selections
				//fireTableDataChanged();
			}
			
			public void listGraphableDeselected(EventObject e) 
			{
				//This is not necessary because the Table is listening to selections
				//fireTableDataChanged();
			}
	
		});
	}
	
	public int getColumnCount() 
	{
		return 2;
	}
	
	public int getRowCount() 
	{
		return graphables.size();
	}
	
	public String getColumnName(int column) 
	{
		return colNames[column];
	}
	
	public void setColumnName(int column, String name) 
	{
		colNames[column] = name;
		fireTableStructureChanged();
	}
	
	public Object getValueAt(int rowIndex, int columnIndex) 
	{
		Object retval = null;
		if (!(graphables.elementAt(rowIndex) instanceof GraphFunction)){
			return null;
		}
		GraphFunction theOne = (GraphFunction)graphables.elementAt(rowIndex);
		if (theOne==null) return null;
		switch (columnIndex) {
			case 0:
				retval = new Boolean(theOne.isVisible());
				break;
			case 1:
				retval = " " + theOne.getExpression();
				break;
		}
	
		return retval;
	}
	
	public Class getColumnClass( int columnIndex) 
	{
		Class retval = String.class;
		if (columnIndex == 0 ) {
			retval = Boolean.class;
		} 
		return retval;
	}
	
	public boolean isCellEditable(int rowIndex, int columnIndex) 
	{
		boolean retval = false;
		if (columnIndex == 0) retval = true;
		return retval;
	}
	
	public void setValueAt( Object aValue, int rowIndex, int columnIndex) 
	{	
		if (columnIndex == 0) 
		{
			Graphable theOne = (Graphable)graphables.elementAt(rowIndex);
			theOne.setVisible(((Boolean)aValue).booleanValue());
		}
	}
}
