/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/24 21:56:14 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui;

import java.awt.Component;
import java.awt.Color;
import java.awt.Font;

import javax.swing.*;
import javax.swing.table.DefaultTableCellRenderer;

import org.concord.graphfunction.engine.*;

/**
 * The renderer for the functionList in the table view
 * It shows the foreground according to each graphable's color
 * when the row is selected it shows a border and in BOLD
 *
 * @author Ingrid Moncada<p>
 *
*/

public class GraphFunctionTableRenderer extends DefaultTableCellRenderer
{
	GraphFunctionList functionList;
	
    public void setGraphFunctionList( GraphFunctionList list ) 
	{
		functionList = list;
	}

    /**
     *
     * Returns the default table cell renderer.
     *
	 * This method is overriden to use the specific colors for each row
	 * @see javax.swing.table.DefaultTableCellRenderer
     */
	public Component getTableCellRendererComponent(JTable table, Object value,
                          boolean isSelected, boolean hasFocus, int row, int column) {

		Color rowColor=null;
		Color foreColor=null;
		Color backColor=null;
		
		if (functionList!=null && row<functionList.size()){
			GraphFunction g = (GraphFunction)functionList.elementAt(row);
			rowColor = g.getColor();
		}
					
		if (isSelected) {
			
			foreColor = rowColor;
			if(foreColor == null){
				foreColor = (table.getSelectionForeground() != null) ? table.getSelectionForeground() : table.getForeground();
			}

			if (backColor == null){
				backColor = (table.getSelectionBackground() != null) ? table.getSelectionBackground() : table.getBackground();
			}
			
			super.setBackground(backColor);
			super.setForeground(foreColor);
			
			setFont(table.getFont().deriveFont(Font.BOLD));
			super.setBorder(BorderFactory.createLineBorder(rowColor));
			
		}
		else {
			
			foreColor = rowColor;
			if(foreColor == null){
				foreColor = table.getForeground();
			}
			if(backColor == null){
				backColor = table.getBackground();
			}
			
			super.setForeground(foreColor);
			super.setBackground(backColor);
	
			setFont(table.getFont());
			super.setBorder(noFocusBorder);
		}		
		
		if (hasFocus) {
		}
	
		setValue(value); 
	
		// ---- begin optimization to avoid painting background ----
		Color back = getBackground();
		boolean colorMatch = (back != null) && ( back.equals(table.getBackground()) ) && table.isOpaque();
			setOpaque(!colorMatch);
		// ---- end optimization to aviod painting background ----
	
		return this;
	}
}


