/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/24 21:56:14 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui;

import java.awt.event.*;
import javax.swing.*;

import org.concord.graph.ui.*;

import org.concord.graphfunction.engine.*;

public class LinearPiecewiseToolBox extends JPanel 
	implements ActionListener 
{
	GraphFunctionList glist= null;
	LinearPiecewise piecewise = null;
	double defaultDomainIncrement = 5.0;
	
	Grid grid;
	
	public LinearPiecewiseToolBox() 
	{
		JButton finite = 
			new JButton(ResourceLoader.getImageIcon("newsegment.gif","Line Segment"));
		
		finite.setActionCommand("finite");
		finite.setToolTipText("Create a line segment");
		finite.addActionListener(this);
		
		add(finite);
			
		JButton infUp = 
			new JButton(ResourceLoader.getImageIcon("newsegmentinfpos.gif","Segment to positive infinity"));
		
		infUp.setActionCommand("infup");
		infUp.setToolTipText("Create a ray approaching positive infinity");
		infUp.addActionListener(this);
		
		add(infUp);
			
		JButton infDown = 
			new JButton(ResourceLoader.getImageIcon("newsegmentinfneg.gif","Segment to negative infinity"));
		
		infDown.setActionCommand("infdown");
		infDown.setToolTipText("Create a ray approaching negative infinity");
		infDown.addActionListener(this);
		
		add(infDown);
			
		/*	JButton delete = 
			new JButton(ResourceLoader.getImageIcon("del.gif","Delete Segment"));
			
		delete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if ( currentPiecewise() != null) {
				currentPiecewise().removeSelected();
				}
			}
			});
		add(delete);
		*/
	}
	
	public void syncGrid(Grid grid)
	{
		this.grid = grid;
	}
	
	public void setPiecewise(LinearPiecewise piecewise) 
	{
		this.piecewise = piecewise;
	}
	
	public void setGraphFunctionList(GraphFunctionList glist) 
	{
		this.glist = glist;
	}
	
	public LinearPiecewise getCurrentPiecewise() 
	{
		if (glist != null) {
			return (LinearPiecewise)glist.getLastSelectedObject();
		} else {
			return piecewise;
		}
	}
	
	public void actionPerformed(ActionEvent e) 
	{
		if ( getCurrentPiecewise() != null) {
			double oldDomainEnd = getCurrentPiecewise().getDomainEnd();
			double oldDomainStart = getCurrentPiecewise().getDomainStart();
			double newDomainStart = 0.0;
			double newDomainEnd = 0.0;
	
			String actionCommand = e.getActionCommand();
	
			double domainIncrement;
			if (grid!=null){
				domainIncrement = grid.getIntervalX();
			} else {
				domainIncrement = defaultDomainIncrement;
			}
				
			if (actionCommand.equals( "infdown")){
				newDomainStart = Double.NEGATIVE_INFINITY;
				if (!Double.isInfinite(oldDomainStart)){
					newDomainEnd = oldDomainStart;
				}
			}
			else if (actionCommand.equals("finite") ){
				if (!Double.isInfinite(oldDomainEnd)){
					newDomainStart = oldDomainEnd;
				}
				else if (!Double.isInfinite(oldDomainStart)){
					newDomainStart = oldDomainStart - domainIncrement;
				}
				newDomainEnd = newDomainStart + domainIncrement;
			}
			else if (actionCommand.equals("infup")) {
				if (!Double.isInfinite(oldDomainEnd)){
					newDomainStart = oldDomainEnd;
				}
				newDomainEnd = Double.POSITIVE_INFINITY;
			}
	
			LinearSegment segment =
			new LinearSegment(newDomainStart, newDomainEnd);
			if (grid!=null){
			double slope = grid.getIntervalY()/grid.getIntervalX();
			segment.setSlope(slope);
			if (Double.isInfinite(newDomainStart)) {
				double rangeEnd = 
				getCurrentPiecewise().yval(newDomainEnd,false);
				if (rangeEnd != Double.NaN) {
				segment.setYIntercept(rangeEnd - slope * newDomainEnd); 
				}
			} else {
				double rangeEnd = 
				getCurrentPiecewise().yval(newDomainStart, false);
	
				
				if (rangeEnd != Double.NaN){
				segment.setYIntercept( rangeEnd - slope*(newDomainStart) );
				}
			}
			//"Snap to grid"
			segment.fix();
			//
			getCurrentPiecewise().add(segment);
			} // grid != null
		} // currentPiecewise() != null
	} // public void actionPerformed
}
