/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/07/14 17:19:56 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui;

import java.awt.event.*;

import javax.swing.*;
import java.util.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;
import org.concord.graph.event.*;

/**
 * @author Eric Brown-Munoz<p>
*/

public class PushpinCheckBox extends JCheckBox 
	implements GraphFunctionView,
			   SelectableListListener
{
	/** The pushpin Graph this box is controling */
	protected LinearFunctionPushpin graph;

	protected GraphFunctionList glist;
	
	public PushpinCheckBox() 
	{
		super();
		
		addItemListener(new ItemListener() 
		{
			public void itemStateChanged(ItemEvent e) 
			{
				if (graph==null) return;
				if (e.getStateChange() == ItemEvent.SELECTED) {
					graph.setPushpin(true);
				} else if (e.getStateChange() == ItemEvent.DESELECTED) {
					graph.setPushpin(false);
				}
			}
		});
	}
	
	public PushpinCheckBox(String name) 
	{
		this();
		setText(name);
	}

	public PushpinCheckBox(LinearFunctionPushpin f) 
	{
		this();
		setGraphFunction(f);
	}
	
	public PushpinCheckBox(String name, LinearFunctionPushpin f) 
	{
		this();
		setText(name);
		setGraphFunction(f);
	}
	
	public void setGraphFunction(GraphFunction f)
	{
		if (f==null){
			graph = null;
			setSelected(false);
			return;
		}
		graph = (LinearFunctionPushpin)f;
		setSelected(graph.isPushpinOn());
	}

	public void setGraphableList ( GraphFunctionList list) {

		GraphFunction gf;

		if (list != glist) {
			if (glist != null) glist.removeGraphableListListener(this);

			this.glist = list;
			list.addGraphableListListener(this);

			gf = (GraphFunction)glist.getFirstSelectedObject();
			setGraphFunction(gf);
		}

		
	}

	public void listGraphableSelected(EventObject e) {
		GraphFunction gf = (GraphFunction)e.getSource();
		setGraphFunction(gf);
	}
	
	public void listGraphableDeselected(EventObject e) {}
	
	public void listGraphableChanged (EventObject e) {}

	public void listGraphableAdded (EventObject e) {
		// FIXME (check if this is really selected
		GraphFunction gf = (GraphFunction)e.getSource();
		setGraphFunction(gf);
	}

	public void listGraphableRemoved (EventObject e) {}

}
