/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/24 21:56:14 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui;

import java.awt.*;

import javax.swing.ImageIcon;

/**
 * Class that loads images
 *
 * @author Ingrid Moncada<p>
*/
public class ResourceLoader
{
	/**
	 * Folder where the images are (relative to this class)
	 */
	protected static String imagesFolder = "images/";
	
    public static ImageIcon getImageIcon(String identifier)
	{
		if (identifier.equals("x")){
			return getImageIcon("x.gif","x");
		}
		else if (identifier.equals("x^2")){
			return getImageIcon("x2.gif","x^2");
		}
		else if (identifier.equals("y")){
			return getImageIcon("y.gif","y");
		}
		else if (identifier.equals("2")){
			return getImageIcon("2.gif","2");
		}
		else{
			return getImageIcon(identifier+".gif",identifier);
		}
	}
	
    public static ImageIcon getImageIcon(String filename, String comment)
	{
		return getImageIcon(ResourceLoader.class, imagesFolder + filename, comment);
	}

	public static boolean setCursorImage(Component component, 
		String filename, String comment, int posX, int posY)
	{
		return setCursorImage(ResourceLoader.class, 
			component, imagesFolder + filename, comment, posX, posY);
	}
	
	public static Cursor getCursorImage(String filename, String comment, int posX, int posY)
	{
		return getCursorImage(ResourceLoader.class, 
			filename, comment, posX, posY);
	}
	
	public static String getStringFolder()
	{
		return imagesFolder;
	}

	public static void setStringFolder(String folder)
	{
		imagesFolder = folder;
	}
	
    public static ImageIcon getImageIcon(Object parent, 
					 String filename, String comment) 
	{ 
		return getImageIcon(parent.getClass(), filename, comment);
    }

    public static ImageIcon getImageIcon(Class parentClass, 
					 String filename, String comment) 
	{ 
		java.net.URL imgURL = parentClass.getResource(filename);
		if (imgURL != null ) {
			return new ImageIcon(imgURL, comment);
		} 
		else {
			System.err.println("Couldn't find file: " + filename);
			return null;
		}
    }

	public static Cursor getCursorImage(Object parent, String filename, String comment, int posX, int posY)
	{
		return getCursorImage(parent.getClass(), filename, comment, posX, posY);
	}
	
	public static Cursor getCursorImage(Class parentClass, String filename, String comment, int posX, int posY)
	{
		Cursor cursor;
		Toolkit toolkit;
		
		toolkit = Toolkit.getDefaultToolkit();

		try{
			Dimension d = toolkit.getBestCursorSize(30,30);
			if (d.height <= 16 || d.width <= 16){
				filename = getFile16Pixels(filename);
			}
		}catch(Exception e){
			e.printStackTrace();
			return null;
		}

		ImageIcon cursorImage = getImageIcon(parentClass, filename, comment);
		
		if (cursorImage==null) return null;
		
		try{
			cursor = toolkit.createCustomCursor(cursorImage.getImage(),new Point(posX,posY),comment);
		}catch(Exception e){
			e.printStackTrace();
			return null;
		}
		return cursor;
	}
	
	public static boolean setCursorImage(Class parentClass, Component parent, 
	                      String filename, String comment, int posX, int posY)
	{
		Cursor cursor = getCursorImage(parentClass, filename, comment, posX, posY);
		
		if (cursor==null) return false;
		
		try{
			parent.setCursor(cursor);
		}catch(Exception e){
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	private static String getFile16Pixels(String filename)
	// If the filename is image.gif
	// This needs to return image16.gif
	// I would use replaceFirst but that is a method of Java 1.4
	// and this should be Java 1.3 compliant
	{
		//Java 1.4
		//filename = filename.replaceFirst("\\.","16.");

		int pos = filename.indexOf(".");
		String newFileName = filename;
		
		if (pos != -1){
			newFileName = filename.substring(0, pos);
			newFileName = newFileName + "16";
			newFileName = newFileName + filename.substring(pos);
		}
		
		return newFileName;
	}

}
