/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * @(#)ValueSlider.java
 * 
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/07/30 20:32:34 $
 * $Author: ebrown-munoz $
 *
 * Copyright 
*/

package org.concord.graphfunction.ui;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.geom.*;
import java.util.*;
import java.awt.event.*;

import org.concord.graph.engine.*;
import org.concord.graph.event.*;
import org.concord.graph.ui.*;
import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;

/**
 * Slider that changes a Value and listens to its changes too
 * It can be synchronized with a graph window and a grid 
 *
 * Date created: 04/07/2004
 *
 * @author Eric Brown-Munoz<p>
 * @author Ingrid C Moncada<p>
 *
 */

public class TenthsValueSlider extends JSlider 
{
	protected ValueSliderModel model;
	private int max; // 
	private GraphArea graphArea = null; // This is optionally set by sync() 
	
	public TenthsValueSlider() {
		super();
		setMajorTickSpacing(20);
		setMinorTickSpacing(10);
		setPaintTicks(true);
	}
	
	public TenthsValueSlider( Value object ) 
	{
		this();
		setValueObject ( object);
	}
	
	public void setValueObject( Value object) 
	{
		model =  new ValueSliderModel (object);
		setModel(model);
	}

	public void setWorldRange ( double worldMinimum, double worldMaximum) {

	    int sliderMin = (int)(worldMinimum *10.0);
		int sliderMax = (int)(worldMaximum *10.0);

		setMinimum(sliderMin);
		setMaximum(sliderMax);

		double scale = 0.10;

		model.setScale(scale);
		model.setOffset(0.0);
	}

	
	public void setValue(double worldValue) 
	{
		model.setWorldValue(worldValue);
	}

	public double getScale () {
		return model.getScale();
	}

	public double getOffset () {
		return model.getOffset();
	}

	public void sync (GraphArea ga) {
		this.graphArea = ga;
		ga.addChangeListener( new ChangeListener () {
				public void stateChanged(ChangeEvent e) {
					setBoundsFromGraphArea(graphArea);
				}
			});
		setBoundsFromGraphArea(ga);
	}

	public  void setBoundsFromGraphArea( GraphArea ga) {
		double min = ga.getLowerLeftCornerWorld().getX();
		double max = ga.getUpperRightCornerWorld().getX();

		setWorldRange(min, max);
	}

	public void setIntervalLabels (boolean labels) {
		model.setIntervalLabels(labels);
	}
		
	
	/** This implements the Value model based on a valueObject 
	 */
	class ValueSliderModel extends DefaultBoundedRangeModel 
	{
		Value valueObject;
		private double scale = 1.0; // The "World" units per Slider unit.
		private double offset = 0.0; // The leftmost value;
		private  boolean intervalLabels = false;
	
		public ValueSliderModel(Value object) 
		{
			super();
			valueObject = object;
			valueObject.addValueListener(new ValueListener () 
			{
				public void valueChanged(ValueEvent e) 
				{
					ValueSliderModel.this.fireStateChanged();
				}
			});
							
		}

		public double getOffset () {
			return offset;
		}

		public void setOffset ( double offset) {
			this.offset = offset;
		}

		// The scale must be an integer to match integer display coords
		//   with integer inervals. Use ceil so the desired range always
		//   is represented (with a bit to spare).
		public void setIntervalLabels (boolean labels) {
			intervalLabels = labels;
			scale = Math.ceil(scale);
		}

		public double getScale () {
			return scale;
		}

		public void setScale ( double scale) {

			if (intervalLabels) {
				this.scale = 1.0/Math.ceil(1.0/scale); 
			} else {
				this.scale = scale;
			}

		}
	
		
		public int getValue() 
		{

			int retval = (int)((valueObject.getValue() - offset) / scale +0.5 );  
			return retval;
		}
	
		public void setValue( int value) 
		{
			double worldValue = (double) value * scale + offset;
			valueObject.setValue(worldValue);
			//fireStateChanged();
		}

		public void setWorldValue (double worldValue) {
			valueObject.setValue(worldValue);
		}
		
		
	} // class ValueSliderModel
}
