/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/24 21:56:14 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui;

import java.util.*;

import org.concord.graph.event.*;
import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;

/**
 * A label that displays (and sets) 
 * the total y value of a function
 *
 * @author Ingrid Moncada<p>
 *
   $Revision: 1.2 $
*/

public class TotalYValueBox extends ValueBox 
	implements GraphFunctionView, ValueListener
{
	protected Value variableValue;	// x variable
	
    public TotalYValueBox(GraphFunction g, Value variableX) 
	{
		super(g);
		variableValue = variableX;
		variableValue.addValueListener(this);
    }

    public TotalYValueBox(Value variableX) 
	{
		super(null);
		variableValue = variableX;
		variableValue.addValueListener(this);
    }

	/**
	 * Sets the graphable  
	*/
    public void setGraphFunction(GraphFunction g) 
	{
		if (source!=null){
			((GraphFunction)source).removeGraphableListener(graphableHandler);
		}
		source = g;
		if (source!=null){
			((GraphFunction)source).addGraphableListener(graphableHandler);
		}
		reset();
    }

    public double getValue() 
	{
		if (source==null || variableValue==null){
			return Double.NaN;
		}
		return ((GraphFunction)source).yval(variableValue.getValue());
    }

    public void setValue(double value) 
	{
		if (source instanceof LinearFunction){
			LinearEquation eq = new LinearEquation(((LinearFunction)source), new Value(value));
			variableValue.setValue(eq.getSolution());
		}
    }

	/**
	 * Listens to the variable X value
	*/
    public void valueChanged(ValueEvent e)
	{
		reset();
	}
		
	/**
	 * Listener to the graphable
	*/
	/***/
	private GraphableListener graphableHandler = new GraphableAdapter () 
	{
		/* Listener to the changes in the graphable.
		Repaints the view when it changes.
		*/
	    public void graphableChanged(EventObject e) 
		{
			reset();
	    }
	};
	
}
