/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/24 21:56:14 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import org.concord.graphfunction.event.*;

/**
 * Generic value input that contains ValueBoxes and JLabels
 * Has an arrow control with arrow buttons to select value to value (left-right)
 * and to modify the value selected (up-down)
 *
 * Use this input to modify values. 
 * Use addLabel to add labels and addValueBox to add ValueBoxes.
 *
 * @author Eric Brown-Munoz<p>
 * @author Ingrid Moncada<p>
 *
*/

public class ValuesInput extends JPanel 
	implements ItemListener
{	
	private Vector propBoxes;  //PropertyBox
	private Vector values;  //ValueBox
	private Vector labels;	//JLabel
	
	private JPanel buttonPanel;
	protected int selectedIndex;
	
	private ArrowControl arrows;
	protected Font labelsFont = new Font("Dialog",Font.BOLD,12);
	
	protected boolean editable = true;
	
	private JLabel messageLabel;
	
    /**
	 * Creates and Initializes a ValuesInput Component 
	*/
	public ValuesInput () 
	{
		this(true);
	}
	/**
	 * Creates and Initializes a ValuesInput Component 
	*/
	public ValuesInput(boolean showArrows) 
	{		
		setOpaque(false);
		
		propBoxes = new Vector();
		values = new Vector();
		labels = new Vector();
		
		setLayout (new BoxLayout(this,BoxLayout.X_AXIS));
	
		buttonPanel = new JPanel();
		buttonPanel.setLayout (new BoxLayout(buttonPanel,BoxLayout.X_AXIS));
		buttonPanel.setOpaque(false);
		
		add(buttonPanel);
		
		messageLabel = new JLabel();
		messageLabel.setVisible(false);
		add(messageLabel);
		
		arrows = new ArrowControl();
	
		selectedIndex = -1;
		
		if (showArrows){
			arrows.setEnabled(false);
			arrows.setOpaque(false);
	
			arrows.addArrowControlListener( new ArrowControlListener() 
			{
				ValueBox valueBox;
				public void upPressed(EventObject e) 
				{
					if (!editable) return;
					if (!(selectedIndex>=0 && selectedIndex<values.size())) return;
					
					valueBox = (ValueBox)values.elementAt(selectedIndex);
					if (valueBox.isEditable()){
						valueBox.increment();
					}
				}
		
				public void downPressed(EventObject e) 
				{
					if (!editable) return;
					if (!(selectedIndex>=0 && selectedIndex<values.size())) return;
					
					valueBox = (ValueBox)values.elementAt(selectedIndex);
					if (valueBox.isEditable()){
						valueBox.decrement();
					}
				}
		
				public void rightPressed(EventObject e) 
				{
					if (!editable) return;
	
					int firstIndex = 0;
					
					if (selectedIndex>=0 && selectedIndex<values.size()) {
						valueBox = (ValueBox)values.elementAt(selectedIndex);
						valueBox.deselect();
						firstIndex = selectedIndex;
					}
					
					selectedIndex ++;
					if (selectedIndex >= values.size()) {
						selectedIndex = 0;
					}
					valueBox = (ValueBox)values.elementAt(selectedIndex);
					while(valueBox!=null && !valueBox.isEditable()){
						selectedIndex ++;
						if (selectedIndex >= values.size()) {
							selectedIndex = 0;
						}
						if (selectedIndex == firstIndex){
							valueBox = null;
						}
						else{
							valueBox = (ValueBox)values.elementAt(selectedIndex);
						}
					}
					
					if (valueBox!=null){
						valueBox = (ValueBox)values.elementAt(selectedIndex);
						valueBox.select();
						arrows.setEnabled(true);
					}
					else{
						selectedIndex = -1;
						arrows.setEnabled(false);
					}
				}
		
				public void leftPressed(EventObject e) 
				{
					int firstIndex = 0;
					
					if (selectedIndex>=0 && selectedIndex<values.size()) {
						valueBox = (ValueBox)values.elementAt(selectedIndex);
						valueBox.deselect();
						firstIndex = selectedIndex;
					}
					
					selectedIndex --;
					if (selectedIndex < 0) {
						selectedIndex = values.size() - 1;
					}
					valueBox = (ValueBox)values.elementAt(selectedIndex);
					while(valueBox!=null && !valueBox.isEditable()){
						selectedIndex --;
						if (selectedIndex < 0) {
							selectedIndex = values.size() - 1;
						}
						if (selectedIndex == firstIndex){
							valueBox = null;
						}
						else{
							valueBox = (ValueBox)values.elementAt(selectedIndex);
						}
					}
					
					if (valueBox!=null){
						valueBox = (ValueBox)values.elementAt(selectedIndex);
						valueBox.select();
						arrows.setEnabled(true);
					}
					else{
						selectedIndex = -1;
						arrows.setEnabled(false);
					}
				}
			});
		
			add(arrows);
		}
	}
	
	/**
	 * Sets if the input is editable or not
	*/
	public void setEditable(boolean b)
	{
		this.editable = b;
		setArrowButtonsVisible(this.editable);
		setSelected(null);
		setValueBoxEditable(b);
	}
	
	/**
	 * Returns if the input is editable or not
	*/
	public boolean isEditable()
	{
		return this.editable;
	}
	
	protected void setValueBoxEditable(boolean b)
	{
		int i;
		for (i=0;i<values.size();i++){
			((ValueBox)values.elementAt(i)).setEditable(b);
		}
	}
	
	/**
	 * Sets the font size of the labels
	*/
	public void setLabelSize(float size)
	{
		int i;
		labelsFont = labelsFont.deriveFont(size);
		ValueBox valueBox;
		for (i=0;i<values.size();i++){
			valueBox = ((ValueBox)values.elementAt(i));
			valueBox.setFont(labelsFont);
	//			valueBox.setPreferredSize(new Dimension(valueBox.getPreferredSize().width + 21, valueBox.getPreferredSize().height));
		}
		JLabel label;
		for (i=0;i<labels.size();i++){
			label = ((JLabel)labels.elementAt(i));
			label.setFont(labelsFont);
	//			label.setPreferredSize(new Dimension(valueBox.getPreferredSize().width + 21, valueBox.getPreferredSize().height));
		}
		PropertyBox propBox;
		for (i=0;i<propBoxes.size();i++){
			propBox = ((PropertyBox)propBoxes.elementAt(i));
			if (propBox instanceof Component){
				((Component)propBox).setFont(labelsFont);
			}
	//			valueBox.setPreferredSize(new Dimension(valueBox.getPreferredSize().width + 21, valueBox.getPreferredSize().height));
		}
	}
	
	/** For debugging purposes only */
	public static void main(String args[]) {
		JFrame frame = new JFrame();
		ValuesInput that = new ValuesInput();
		that.setVisible(true);
		frame.getContentPane().add(that);
		frame.setSize(300,500);
		frame.show();
	}
	
	/**
	 * Removes all the value boxes and labels
	*/
	public void removeAllBoxes()
	{
		int i;
		for (i=0;i<values.size();i++){
			((ValueBox)values.elementAt(i)).removeItemListener(this);
		}
		values.removeAllElements();
		labels.removeAllElements();
		propBoxes.removeAllElements();
		buttonPanel.removeAll();
	}
	
	/**
	 * Hides all boxes
	*/
	public void hideAllBoxes()
	{
		buttonPanel.setVisible(false);
	}
	
	/**
	 * Shows all boxes
	*/
	public void showAllBoxes()
	{
		buttonPanel.setVisible(true);
	}
	
	public void showMessage(String msg)
	{
		messageLabel.setText(msg);
		messageLabel.setVisible(true);
	}

	public void hideMessage()
	{
		messageLabel.setVisible(false);
	}
	
	/**
	 * Adds a PropertyBox
	*/
	public void addPropertyBox(PropertyBox propBox)
	{
		propBoxes.addElement(propBox);
		if (propBox instanceof Component){
			((Component)propBox).setFont(labelsFont);
			buttonPanel.add((Component)propBox);
		}
	}
	
	/**
	 * Adds a PropertyBox to the input
	*/
	public void addPropertyBox(PropertyBox propBox, int index)
	{
		propBoxes.addElement(propBox);
		if (propBox instanceof Component){
			((Component)propBox).setFont(labelsFont);
			buttonPanel.add((Component)propBox,index);
		}
	}
	
	/**
	 * Removes the PropertyBox at the index specified
	*/
	public void removePropertyBox(int index)
	{
		PropertyBox propBox;
		propBox = (PropertyBox)propBoxes.elementAt(index);
		
		if (propBox!=null){
			propBoxes.remove(index);
			buttonPanel.remove((Component)propBox);
		}
	}
	
	/**
	 * Adds a ValueBox
	*/
	public void addValueBox(ValueBox valueBox)
	{
		valueBox.setFont(labelsFont);
		valueBox.setEditable(editable);
		values.addElement(valueBox);
		valueBox.addItemListener(this);
		buttonPanel.add(valueBox);
	}
	
	/**
	 * Adds a ValueBox
	*/
	public void addValueBox(ValueBox valueBox, int index)
	{
		valueBox.setFont(labelsFont);
		valueBox.setEditable(editable);
		values.addElement(valueBox);
		valueBox.addItemListener(this);
		buttonPanel.add(valueBox,index);
	}
	
	/**
	 * Removes the ValueBox at the index specified
	*/
	public void removeValueBox(int index)
	{
		ValueBox valueBox;
		valueBox = (ValueBox)values.elementAt(index);
		
		if (valueBox!=null){
			valueBox.removeItemListener(this);
			values.remove(index);
			buttonPanel.remove(valueBox);
		}
	}
	
	/**
	 * Adds a normal custom Component that is NOT a Value Box or Label
	 * (NOTE: DON'T use this to add LABELS or VALUE BOXES!!
	*/
	public void addCustomComponent(Component c)
	{
		buttonPanel.add(c);
	}
	
	/**
	 * Adds a label to the input
	*/
	public void addLabel(JLabel label)
	{
		label.setFont(labelsFont);
		labels.addElement(label);
		buttonPanel.add(label);
	}
	
	/**
	 * Adds a label to the input
	*/
	public void addLabel(String labelstr)
	{
		JLabel label = new JLabel(labelstr); 
		addLabel(label);
	}
	
	/**
	 * Adds a label to the input
	*/
	public void addLabel(JLabel label, int index)
	{
		label.setFont(labelsFont);
		labels.addElement(label);
		buttonPanel.add(label,index);
	}
	
	/**
	 * Adds a label to the input
	*/
	public void addLabel(String labelstr, int index)
	{
		JLabel label = new JLabel(labelstr); 
		addLabel(label,index);
	}
	
	/**
	 * Removes the label at the index specified
	*/
	public void removeLabel(int index)
	{
		JLabel label;
		label = (JLabel)labels.elementAt(index);
		
		if (label!=null){
			labels.remove(index);
			buttonPanel.remove(label);
		}
	}
	
	/**
	 * Sets the source for all the values 
	*/
	public void setSourceBoxes(Object source)
	{
		PropertyBox propBox;
		for (int i = 0 ; i < values.size(); i++) {
			propBox = (PropertyBox)values.elementAt(i);
			propBox.setSource(source);
		}
		for (int i = 0 ; i < propBoxes.size(); i++) {
			propBox = (PropertyBox)propBoxes.elementAt(i);
			propBox.setSource(source);
		}
	}
	
	/**
	 * Sets the increment for all the values 
	*/
	public void setValueBoxIncrementInterval(double val)
	{
		ValueBox valueBox;
		for (int i = 0 ; i < values.size(); i++) {
			valueBox = (ValueBox)values.elementAt(i);
			valueBox.setIncrementInterval(val);
		}
	}
	
	/**
	 * Sets the number of decimals showing in all the values (rounding) 
	*/
	public void setValueBoxNumDecimalsValue(int n)
	{
		ValueBox valueBox;
		for (int i = 0 ; i < values.size(); i++) {
			valueBox = (ValueBox)values.elementAt(i);
			valueBox.setNumDecimalsValue(n);
		}
	}
	
	/**
	 * Resets all the values 
	*/
	public void resetBoxes() 
	{
		PropertyBox propBox;
		for (int i = 0 ; i < values.size(); i++) {
			propBox = (PropertyBox)values.elementAt(i);
			propBox.reset();
		}
		for (int i = 0 ; i < propBoxes.size(); i++) {
			propBox = (PropertyBox)propBoxes.elementAt(i);
			propBox.reset();
		}
		repaint();
	}
	
	/** Sets the selectedIndex to point to the given box. */
	public void setSelected(ValueBox box) 
	{
		int i;
		ValueBox valueBox;
	
		if (selectedIndex>=0 && selectedIndex<values.size()) {
			valueBox = (ValueBox)values.elementAt(selectedIndex);
			if (valueBox!=null){
				valueBox.deselect();
			}
		}
		selectedIndex = -1;
	
		if (!editable || box==null) {
			
			arrows.setEnabled(false);
			return;
		}
		else{
			arrows.setEnabled(true);
		}
		
		i = values.indexOf(box);
		
		if ( i >= 0 && i < values.size()){	
			selectedIndex = i;
			box.select();
		}
	}
	
	/**
	 * Listener to the selections directly on the box
	*/
	public void itemStateChanged(ItemEvent e)
	{
		if (e.getStateChange() == ItemEvent.SELECTED){
			setSelected((ValueBox)e.getItem());
		}
		else if (e.getStateChange() == ItemEvent.DESELECTED){
			setSelected(null);
		}
	}
	
	/**
	 * Returns the vector of values
	*/
	public Vector getValueBox()
	{
		return values;
	}
	
	/**
	 * Returns the vector of labels
	*/
	public Vector getLabels()
	{
		return labels;
	}
	
	/**
	 * Hides and shows a particular button
	 * @param buttonDirection	Button to hide/show. It can be "up", "down", "left" or "right"
	 * @param visible	Indicates if the button should be visible or not
	*/
	public void setArrowButtonVisible(String buttonDirection, boolean visible)
	{
		arrows.setButtonVisible(buttonDirection, visible);
	}
	
	/**
	 * Hides and shows all the buttons
	 * @param visible	Indicates if the buttons should be visible or not
	*/
	public void setArrowButtonsVisible(boolean visible)
	{
		arrows.setVisible(visible);
	}
	
	public void removeButtons()
	{
		remove(arrows);
		invalidate();
	}
	
	public ValueBox getSelectedBox()
	{
		if (selectedIndex != -1){
			return (ValueBox)values.elementAt(selectedIndex);
		}
		return null;
	}
}
