/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/07/14 17:23:31 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.modelview;

import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import java.util.EventObject;

import javax.swing.*;
import javax.swing.border.*;

import org.concord.graph.event.*;
import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;
import org.concord.graphfunction.ui.*;

/**
 * AreaModelToolBox
 * This is a tool box (TILE BIN) for the area model
 * 
 * @author Ingrid Moncada<p>
*/

public class AreaModelToolBox extends JPanel 
	implements GraphFunctionView, 
			   ActionListener,
			   SelectableListListener
{	
	//Only represents one function
	GraphFunction function;
	
	// ... from a list of functions
	GraphFunctionList glist;
	
	AreaModelView areaModel;

//	JToolBar toolBar;
	JPanel buttonsPanel;
	
	Hashtable buttons;
	
	private JLabel titleLabel;
	
	public AreaModelToolBox()
	{
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		titleLabel = new JLabel("TILE BIN");
		titleLabel.setFont(new Font("Dialog",Font.BOLD,12));
		titleLabel.setAlignmentX(LEFT_ALIGNMENT);
		add(titleLabel);

		buttonsPanel = new JPanel();
		buttonsPanel.setLayout(new BoxLayout(buttonsPanel, BoxLayout.X_AXIS));
		buttonsPanel.setAlignmentX(LEFT_ALIGNMENT);
		add(buttonsPanel);

		buttons = new Hashtable();
	}
	
	public void setGraphFunction(GraphFunction g)
	{
		if (function!=null){
			function.removeGraphableListener(graphableHandler);
		}

		if (g instanceof FunctionEquation){
			function = ((FunctionEquation)g).getFunction();
		}
		else{
			function = g;
		}

		initView();
		setupEnableButtons();
		repaint();

		if (function!=null){
			function.addGraphableListener(graphableHandler);
		}
	}

	public void setGraphFunctionList(GraphFunctionList gl)
	{
		if (glist!=null){
			glist.removeGraphableListListener(this);
		}
		glist = gl;
		if (glist!=null){
			glist.addGraphableListListener(this);
			GraphFunction current = (GraphFunction)gl.getFirstSelectedObject();
			if (current != null) {
				setGraphFunction(current);
			}
		}
	}


	/***/
	private GraphableListener graphableHandler = new GraphableAdapter() 
	{
		/* Listener to the changes in the graphable.
		Repaints the view when it changes.
		*/
	    public void graphableChanged(EventObject e) 
		{
			setupEnableButtons();
			repaint();
	    }
	};
	
	/** Listener to the area model window resizing */
	private ComponentListener areaModelResizeHandler = new ComponentAdapter () 
	{
		/** Listener to the component resizing 
		 * Repaints the view when it changes.
		*/
		public void componentResized(ComponentEvent e) 
		{
			setupEnableButtons();
			repaint();
	    }
	};

	public void setAreaModelView(AreaModelView areaModel)
	{
		if (this.areaModel != null){
			this.areaModel.removeComponentListener(areaModelResizeHandler);
		}
		this.areaModel = areaModel;
		if (this.areaModel != null){
			this.areaModel.addComponentListener(areaModelResizeHandler);
		}
	}

	public void initView()
	{
		if (function==null) return;
		
		boolean eraserSelected=false;
		AbstractButton b = (AbstractButton)buttons.get(new String("-"));
		if (b!=null){
			eraserSelected = b.isSelected();
		}
		
		int i;
		for (i=0; i<buttonsPanel.getComponentCount();i++){
			if (buttonsPanel.getComponent(i) instanceof AbstractButton){
				((AbstractButton)buttonsPanel.getComponent(i)).removeActionListener(this);
			}
		}
		
		buttonsPanel.removeAll();
		buttons.clear();

		Border border;
		Border baseBorder;
		baseBorder = BorderFactory.createEtchedBorder(EtchedBorder.LOWERED); 
		
		if (function instanceof LinearFunction){

			JPanel negPanel = new JPanel();
			negPanel.setLayout(new BoxLayout(negPanel, BoxLayout.X_AXIS));

			AbstractButton button;
			button = addButton(new JButton(), "neghalfcoeff_m.gif","Add - 1/2 to the slope","-m/2");
			buttons.put(new String("-m/2"), button);
			negPanel.add(button);
			button = addButton(new JButton(), "negcoeff_m.gif","Add -1 to the slope","-m");
			buttons.put(new String("-m"), button);
			negPanel.add(button);
			button = addButton(new JButton(), "negcoeff_b.gif","Add -1 to the y intercapt","-b");
			buttons.put(new String("-b"), button);
			negPanel.add(button);

			border = BorderFactory.createTitledBorder(baseBorder,
				" Negative ",TitledBorder.CENTER , 
				TitledBorder.BOTTOM  , new Font("Dialog",Font.PLAIN,11));
			negPanel.setBorder(border);
			
			JPanel posPanel = new JPanel();
			posPanel.setLayout(new BoxLayout(posPanel, BoxLayout.X_AXIS));

			button = addButton(new JButton(), "halfcoeff_m.gif","Add 1/2 to the slope","m/2");
			buttons.put(new String("m/2"), button);
			posPanel.add(button);
			button = addButton(new JButton(), "coeff_m.gif","Add 1 to the slope","m");
			buttons.put(new String("m"), button);
			posPanel.add(button);
			button = addButton(new JButton(), "coeff_b.gif","Add 1 to the y intercept","b");
			buttons.put(new String("b"), button);
			posPanel.add(button);
			
			border = BorderFactory.createTitledBorder(baseBorder,
				" Positive ",TitledBorder.CENTER , 
				TitledBorder.BOTTOM  , new Font("Dialog",Font.PLAIN,11));
			posPanel.setBorder(border);

			buttonsPanel.add(negPanel);
			buttonsPanel.add(Box.createRigidArea(new Dimension(5,0)));
			buttonsPanel.add(posPanel);
			buttonsPanel.add(Box.createRigidArea(new Dimension(5,0)));
		}

		JPanel eraserPanel = new JPanel();
		eraserPanel.setLayout(new BoxLayout(eraserPanel, BoxLayout.X_AXIS));

		JToggleButton eraserB = (JToggleButton)addButton(new JToggleButton(), "eraserpencil.gif","eraserpencilselected.gif","eraser","-");
		buttons.put(new String("-"), eraserB);
		eraserB.setBackground(new Color(220,220,206));
		eraserPanel.add(eraserB);
		
		border = BorderFactory.createTitledBorder(baseBorder,
			" ",TitledBorder.CENTER , 
			TitledBorder.BOTTOM  , new Font("Dialog",Font.PLAIN,11));
		eraserPanel.setBorder(border);

		buttonsPanel.add(eraserPanel);

		//eraserB.setContentAreaFilled(false);
		//eraserB.setBorderPainted(false);
		if (eraserSelected){
			eraserB.setSelected(true);
		}

	}
	
	private AbstractButton addButton(AbstractButton button, String image, String imageDescription, String action)
	{
		return addButton(button, image, "", imageDescription, action);
	}
	
	private AbstractButton addButton(AbstractButton button, String image, String imageSelected, String imageDescription, String action)
	{
		button.setActionCommand(action);
		button.setFocusPainted(false);
		button.setIcon(ResourceLoader.getImageIcon(image, imageDescription));
		if (!imageSelected.equals("")){
			button.setSelectedIcon(ResourceLoader.getImageIcon(imageSelected, imageDescription));
		}
		button.addActionListener(this);
		button.setMargin(new Insets(0,0,0,0));
		return button;
	}
	
    public void actionPerformed (ActionEvent e) 
	{
		String command =((AbstractButton)e.getSource()).getActionCommand();
		
		if (function instanceof LinearFunction){
			LinearFunction linearFunction = (LinearFunction)function;
			if (command.equals("m")){
				linearFunction.setSlope(linearFunction.getSlope()+1.0);
			}
			else if (command.equals("m/2")){
				linearFunction.setSlope(linearFunction.getSlope()+0.5);
			}
			else if (command.equals("b")){
				linearFunction.setYIntercept(linearFunction.getYIntercept()+1.0);
			}
			else if (command.equals("-m")){
				linearFunction.setSlope(linearFunction.getSlope()-1.0);
			}
			else if (command.equals("-m/2")){
				linearFunction.setSlope(linearFunction.getSlope()-0.5);
			}
			else if (command.equals("-b")){
				linearFunction.setYIntercept(linearFunction.getYIntercept()-1.0);
			}
		}
		if (command.equals("-")){
			if (areaModel==null) return;
			if (((AbstractButton)e.getSource()).isSelected()){
				areaModel.setClickBehavior(AreaModelView.DELETE_ONCLICK);
			}
			else{
				areaModel.setClickBehavior(AreaModelView.NONE_ONCLICK);
			}
		}
		setupEnableButtons();
	}
	
	private void setupEnableButtons()
	{
		if (areaModel==null) return;
		
		AbstractButton b;
		
		if (AreaModelView.getNumBoxesHorizontal(function) >= 
			areaModel.getMaxBoxesHorizontal()){
			
			if (function instanceof LinearFunction){
				LinearFunction linearFunction = (LinearFunction)function;
				
				if (linearFunction.getSlope()>=0){
					b = (AbstractButton)buttons.get(new String("m"));
					b.setEnabled(false);
					b = (AbstractButton)buttons.get(new String("m/2"));
					b.setEnabled(false);
				}
				if (linearFunction.getSlope()<=0){
					b = (AbstractButton)buttons.get(new String("-m"));
					b.setEnabled(false);
					b = (AbstractButton)buttons.get(new String("-m/2"));
					b.setEnabled(false);
				}
				
				if (linearFunction.getYIntercept()>=0){
					b = (AbstractButton)buttons.get(new String("b"));
					b.setEnabled(false);
				}
				if (linearFunction.getYIntercept()<=0){
					b = (AbstractButton)buttons.get(new String("-b"));
					b.setEnabled(false);
				}
			}
		}
		else{
			if (function instanceof LinearFunction){
				b = (AbstractButton)buttons.get(new String("m"));
				b.setEnabled(true);
				b = (AbstractButton)buttons.get(new String("m/2"));
				b.setEnabled(true);
				b = (AbstractButton)buttons.get(new String("-m"));
				b.setEnabled(true);
				b = (AbstractButton)buttons.get(new String("-m/2"));
				b.setEnabled(true);
				b = (AbstractButton)buttons.get(new String("b"));
				b.setEnabled(true);
				b = (AbstractButton)buttons.get(new String("-b"));
				b.setEnabled(true);
			}
		}
	}

	public void listGraphableAdded(EventObject e)
	{
		repaint();
	}
	public void listGraphableChanged(EventObject e) 
	{
		repaint();
	}
	public void listGraphableRemoved(EventObject e) 
	{
		repaint();
	}
	public void listGraphableSelected(EventObject e) 
	{
		setGraphFunction((GraphFunction)e.getSource());
		repaint();

	}
	public void listGraphableDeselected(EventObject e) 
	{
	}



} // class AreaModelToolBox











