/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
 * $Revision: 1.3 $
 */

package org.concord.graphfunction.ui.modelview;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;

import org.concord.graphfunction.ui.*;

public class ImageChooser 
	extends JPanel 
	implements ActionListener,
			   ImageObserver
{

	String [] imageNames = { "box", "bill", "orca", "monkey", "basketball", "funrocket" };
	Image [] imageArray;
	Integer [] integerArray;

	JComboBox imageList;

	ModelWindow graphModel;

	public ImageChooser() {
		
		setLayout(new BorderLayout());

		imageArray = new Image[imageNames.length];
		integerArray = new Integer[imageNames.length];

		imageArray[0] = null;
		integerArray[0] = new Integer(0);
		

		for (int i = 1; i < imageNames.length ; i ++ ) {
			integerArray[i] = new Integer(i);
			imageArray[i] = 
				ResourceLoader.getImageIcon(imageNames[i]).getImage();
		}


		imageList = new JComboBox(integerArray);

		imageList.setRenderer(new ImageRenderer());
		imageList.addActionListener(this);
		imageList.setMaximumRowCount(3);

		add(imageList, BorderLayout.PAGE_START);
		
	} // Constructor ImageChooser

	public static void createAndShowGUI() {
		JFrame frame = new JFrame("New Model Image");
		JComponent that = new ImageChooser();
		frame.setContentPane(that);
		frame.pack();
		frame.show();
	}

	public Image getChosenImage () {
	    return imageArray[imageList.getSelectedIndex()];
	}

	public Image goodImage (ImageIcon image) {
		Image newImage;
		int width = image.getIconWidth();
		int height = image.getIconHeight();
		Graphics tg;

		newImage = new BufferedImage(width, height, 
									 BufferedImage.TYPE_BYTE_GRAY);
		tg = newImage.getGraphics();
		tg.drawImage(image.getImage(),0,0,this);
		return newImage;
	}
		

	public void setModel (ModelWindow graphModel) {
		this.graphModel = graphModel;
	}


	public void actionPerformed(ActionEvent e) {

		if (graphModel != null) {
			graphModel.setImage(getChosenImage());
		}
	}

	public boolean imageUpdate(Image img, int infoflags, int x, int y, 
						int width, int height) {
		return false;
	}

	class ImageRenderer
		extends JLabel
		implements ListCellRenderer 
	{

		Image image;

		public ImageRenderer () {
			super();
			setPreferredSize(new Dimension(100,50));
		}

		public Component getListCellRendererComponent(JList list,
													  Object value,
													  int index,
													  boolean isSelected,
													  boolean cellHasFocus) {

			int imageNumber = ((Integer)value).intValue();
			image = imageArray[imageNumber];
			return this;
		}
		
		public void paint (Graphics g) {
			super.paint(g);

			int screenWidth = getWidth();
			int screenHeight = getHeight();
			int imageWidth;
			int imageHeight;

			Graphics2D g2 = (Graphics2D)g;

			if (image != null) {
				imageWidth = image.getWidth(null);
				imageHeight = image.getHeight(null);
				
				g2.drawImage( image, 
							 (screenWidth - imageWidth) / 2,
							 (screenHeight - imageHeight) / 2,
							 null);
			} else {
				imageWidth = screenWidth - 35;
				imageHeight = screenHeight - 20;
				g2.drawRect( (screenWidth - imageWidth) / 2,
							(screenHeight - imageHeight) / 2,
							imageWidth, imageHeight);
			}

		}
	} // class ImageRenderer


} // class ImageChooser


	
