/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/07/14 17:23:32 $
 * $Author: ebrown-munoz $
 *
 * Copyright 
*/
package org.concord.graphfunction.ui.modelview;

import java.util.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import org.concord.graph.ui.*;
import org.concord.graph.event.*;
import org.concord.graph.engine.*;
import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;

/**
 * Window that displays a rectangle that moves up and down
 * according with the yval of a function 
 *
 * Date created: 04/07/2004
 *
 * @author Eric Brown-Munoz<p>
 *
 */

public class ModelWindow extends JPanel 
	implements SelectableListListener, ValueListener,
			   GraphableListener,
			   ImageObserver
{

	public static int MODE_MULTI = 0;
	public static int MODE_SINGLE = 1;
	public static int MODE_CHART = 2;

	public static int DEFAULT_HEIGHT = 20;
	public static int DEFAULT_WIDTH = 30;


	protected int mode = MODE_MULTI;

	protected GraphArea graphArea;

	protected int originx;
	protected int originy;
	
	protected int screenHeight;
	
	protected int objectHeight = DEFAULT_HEIGHT;
	protected int objectWidth = DEFAULT_WIDTH;
	
	protected Vector listeners = new Vector();
	protected Value variable;
	
	protected GraphFunction graphFunction;
	protected GraphFunctionList glist;

	private Image image;

	Point2D.Double screenLocation = new Point2D.Double();
	
	public ModelWindow() 
	{
		this.addComponentListener ( new ComponentAdapter () 
		{
			public void componentResized(ComponentEvent e) 
			{

				//Always center horizontally
				originx = (int)(ModelWindow.this.getWidth() - 
						objectWidth)/2;
				screenHeight = getHeight();
			}
		});
	}
	
	public void setGraphFunctionList(GraphFunctionList gl)
	{
		if (glist!=null){
			glist.removeGraphableListListener(this);
		}
		glist = gl;
		if (glist!=null){
			glist.addGraphableListListener(this);
		}
	}

	public void setGraphFunction(GraphFunction gf) {
		if (graphFunction != null) {
			graphFunction.removeGraphableListener(this);
		}

		graphFunction = gf;

		graphFunction.addGraphableListener(this);
	}

	public void setMode (int mode) {
		this.mode = mode;
		if (mode == MODE_MULTI) {
			graphFunction = (GraphFunction)glist.getFirstSelectedObject();
		}
	}

	public int getMode () {
		return this.mode;
	}
	
	public void setGraphArea(GraphArea ga) 
	{

		this.graphArea = ga;

	}
	
	public void setVariable (Value variable) 
	{
		this.variable = variable;
		
		variable.addValueListener ( new ValueListener () 
		{
			public void valueChanged (ValueEvent e) 
			{
				ModelWindow.this.repaint();
			}
		});
	}

	public void setImage (Image image) {
		this.image = image;
		if (image != null) {
			objectHeight = image.getHeight(this);
			objectWidth = image.getWidth(this);
		} else {
			objectHeight = DEFAULT_HEIGHT;
			objectWidth = DEFAULT_WIDTH;
		}

		originx = (int)(ModelWindow.this.getWidth() - 
						objectWidth)/2;

		repaint();
	}
	
	/**
	 * Paints the objects the same color as the graphables
	 * Paints the objects of the selected graphables on top of the others
	*/
	public void paintComponent(Graphics g) 
	{
		Enumeration graphs;
		super.paintComponent(g);
		Color saveColor = g.getColor();

		if (variable == null) return;

		double xvalue = variable.getValue();

		if (mode == MODE_MULTI) {

			if (glist != null) {	
				graphs = glist.elements();
				while(graphs.hasMoreElements()) {
					GraphFunction oneGraph = 
						(GraphFunction)graphs.nextElement();
					
					if (!oneGraph.isSelected()){
						drawGraphModel(g, oneGraph, xvalue);
					}
				
				} // while
			
				//Now draw the selected objects
				graphs = glist.getSelectedObjects().elements();
				while(graphs.hasMoreElements()) {
					GraphFunction oneGraph = 
						(GraphFunction)graphs.nextElement();
					if (oneGraph!=null){
						drawGraphModel(g, oneGraph, xvalue);
					}
				}
			} else { 
				if (graphFunction != null) {
					drawGraphModel(g, graphFunction, xvalue);
				}
			} // if glist 
		} else if (mode == MODE_SINGLE) {
			if (graphFunction != null) {
				drawGraphModel(g, graphFunction, xvalue);
			}
		}

		g.setColor(saveColor);
	}
	
	protected void drawGraphModel(Graphics g, GraphFunction oneGraph, 
								  double xvalue)
	{
		double yval = oneGraph.yval(xvalue);

		if (oneGraph.isVisible() && (! Double.isNaN(yval))) {

			CoordinateSystem cs = graphArea.getCoordinateSystem();
			cs.transformToDisplay(new Point2D.Double(0.0, yval),
								  screenLocation);

			int ypos = (int)(screenLocation.getY());

			g.setColor(oneGraph.getColor());			
			g.drawLine(0,ypos,getWidth(),ypos);
	
			ypos = ypos - objectHeight/2;
			
			// The color is the same as the graphable

			if (image == null) {
				g.fillRect ((int)originx, ypos, (int)
							objectWidth, objectHeight);
			g.setColor(Color.black);
			g.drawRect ((int)originx, ypos, (int)
				objectWidth, objectHeight);

			} else {
				g.drawImage(image, (int)originx, ypos, this);
			}

		}
	}		
	
	
	public void listGraphableAdded(EventObject e)
	{
		repaint();
	}
	public void listGraphableChanged(EventObject e) 
	{
		repaint();
	}
	public void listGraphableRemoved(EventObject e) 
	{
		repaint();
	}
	public void listGraphableSelected(EventObject e) 
	{
		if (mode == MODE_MULTI) {
			graphFunction = (GraphFunction)glist.getFirstSelectedObject();
		}
		repaint();

	}
	public void listGraphableDeselected(EventObject e) 
	{
	}

	public void graphableChanged (EventObject e) {
		repaint();
	}

	public void graphableRemoved (EventObject e) {
		graphFunction = null;
	}

	public void valueChanged(ValueEvent e) 
	{
		repaint();
	}

	/* The image observer allows one to check to see if the image
	   is currently changing. Since this happens, we can just
	   always return false. Several Image methods require this
	   check.
	*/
	public boolean imageUpdate(Image img,
							   int infoflags,
							   int x,
							   int y,
							   int width,
							   int height) {
		System.out.println("ModelWindow calling imageUpdate");
		return false;
	}
}

