/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.1 $
 * $Date: 2004/05/24 20:55:23 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;

import javax.swing.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;

/**
 * A Panel with radio buttons to choose the expression form
 * It toggles the different symbolic views depending on the 
 * selected expression form.
 *
 * @author Ingrid Moncada<p>
 *
   $Revision: 1.1 $
*/
public class ExpressionFormChooser extends JPanel 
	implements GraphFunctionView, ActionListener
{
	GraphFunction function;
	
	ButtonGroup buttonGroup;
	JPanel buttonPanel;
	Hashtable buttons;
	
	CardLayout inputPanelLayout;
	JPanel inputPanel;
	
	public ExpressionFormChooser()
	{
		this(true);
	}
	
	public ExpressionFormChooser(boolean editable)
	{		
		buttonGroup = new ButtonGroup();
		buttons = new Hashtable();

		buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.Y_AXIS));

		inputPanel = new JPanel();
		inputPanelLayout = new CardLayout();
		inputPanel.setLayout(inputPanelLayout);	
		
		if (editable){
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			
			add(buttonPanel);
			add(inputPanel);
		}
		else{
			setLayout(new BorderLayout());
			add(inputPanel, BorderLayout.WEST);
		}
	}	
	
	public void addInputForm(int form, FunctionInput input, String label)
	{
		String strForm;
		strForm = String.valueOf(form);
		if (input!=null){
			inputPanel.add(input, strForm);
		}
		else{
			inputPanel.add(new JPanel(), strForm);
		}
		JRadioButton button = new JRadioButton(label);
		button.setActionCommand(strForm);
		button.addActionListener(this);
		buttonGroup.add(button);
		buttonPanel.add(button);
		buttons.put(strForm, button);
	}
	
	public void actionPerformed(ActionEvent e)
	{
		String command = ((AbstractButton)e.getSource()).getActionCommand();
		
		showInput(command);
	}
	
	private void showInputButton(String form)
	{
		JRadioButton button = (JRadioButton)buttons.get(form);
		if (button!=null){
			button.setSelected(true);
		}
		showInput(form);
	}
	
	private void showInput(String form)
	{
		inputPanelLayout.show(inputPanel, form);
		
		if (function != null){
			int expressionForm = Integer.parseInt(form);
			if (expressionForm != function.getExpressionForm()){
				function.setExpressionForm(expressionForm);
			}
		}
	}
		
	/**
	 * Sets the function that this view will represent. 
	 * it doesn't listen to changes on this graphable
	*/
	public void setGraphFunction(GraphFunction g)
	{
		function = g;

		if (function!=null){
			showInputButton(String.valueOf(function.getExpressionForm()));
		}
		repaint();
	}
	
	public FunctionInput getCurrentInput()
	{
		return null;
	}
}
