/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/07/14 17:29:54 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import org.concord.graph.event.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;
import org.concord.graphfunction.ui.*;

/**
 * @author Ingrid Moncada<p>
 *
*/

public abstract class FunctionInput extends ValuesInput 
	implements GraphFunctionView, 
			   ItemListener,
			   SelectableListListener
{
    protected GraphFunction subject;
	private GraphFunctionList glist;
	
	private boolean showFunctionName = false; 
   
    /** Creates and Initializes a FunctionInput Component */
    public FunctionInput() 
	{
		super();
		setupSymbolic();
		setGraphFunction(null);
    }

    public FunctionInput(boolean b) 
	{
		super(b);
		setupSymbolic();
		setGraphFunction(null);
    }

    public FunctionInput(GraphFunction subject) 
	{
		super();
		setupSymbolic();
		setGraphFunction(subject);
    }

    GraphableListener graphableHandler = new GraphableAdapter() 
	{
	    public void graphableChanged(EventObject e) 
		{
			updateSymbolic();
			resetBoxes();
	    }
	};

    public void setGraphFunction(GraphFunction subject)
	{
		if (this.subject != null) {
		    this.subject.removeGraphableListener(graphableHandler);
		}
		this.subject = subject;
		if (this.subject != null) {
		    this.subject.addGraphableListener(graphableHandler);
		}
		switchSymbolic(subject);
	}

 	public void setGraphFunctionList(GraphFunctionList gl)
	{
		if (glist!=null){
			glist.removeGraphableListListener(this);
		}
		glist = gl;
		if (glist!=null){
			glist.addGraphableListListener(this);
			Object current = gl.getFirstSelectedObject();
			if (current instanceof GraphFunction) {
				setGraphFunction((GraphFunction)current);
			}
		}
	}


	/**
	 * Called when the function is a different one
	 */
	public void switchSymbolic(GraphFunction subject) 
	{
		//If the function is null, the component will not draw anything
		if (subject==null){
			adjustPreferredSize();
			hideAllBoxes();
			setArrowButtonsVisible(false);
			//addCustomComponent(Box.createRigidArea(this.getPreferredSize()));
			return;
		}
		if (editable){
			setArrowButtonsVisible(true);
		}
		showAllBoxes();
				
		setSourceBoxes(subject);
		
		updateSymbolic();

		resetBoxes();

		adjustPreferredSize();
		revalidate();
	}

	/**
	 * Sets if the input is editable or not
	*/
	public void setEditable(boolean b)
	{
		super.setEditable(b);
		adjustPreferredSize();
	}

	/**
	 * Sets if the input should show the function name or not
	*/
	public void setShowFunctionName(boolean b)
	{
		if (showFunctionName != b){
			showFunctionName = b;
		}
		adjustFunctionName();
	}

	/**
	 * Returns if the input should show the function name or not
	*/
	public boolean getShowFunctionName()
	{
		return showFunctionName;
	}
		
	public void adjustFunctionName()
	{
		if (this.getLabels().size()<=0) return;
		if (getShowFunctionName()){
		//Hack!
			//Remove the y 
			this.removeLabel(0);
			//Insert f(x)
//temp
			this.addLabel(new JLabel(ResourceLoader.getImageIcon("f")),0);
//
			this.addPropertyBox(new FunctionNameBox(),1);
			this.addLabel("(",2);
			this.addLabel(new JLabel(ResourceLoader.getImageIcon("x")),3);
			this.addLabel(")",4);
		}
		else{
		//Hack!
			//Remove the f(x)
			this.removePropertyBox(0);
			this.removeLabel(0);
			this.removeLabel(0);
			this.removeLabel(0);
			//Insert y
			this.addLabel(new JLabel(ResourceLoader.getImageIcon("y")),0);
		}
	}
	
	public void setupFunctionName()
	{
		if (getShowFunctionName()){
		//Add f(x)
//temp
			this.addLabel(new JLabel(ResourceLoader.getImageIcon("f")));
//
			this.addPropertyBox(new FunctionNameBox());
			this.addLabel("(");
			this.addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
			this.addLabel(")");
		}
		else{
		//Add y
			this.addLabel(new JLabel(ResourceLoader.getImageIcon("y")));
		}
	}

	/**
	 * Called when the function has changed
	 */
	protected void updateSymbolic()
	{
	}
	
	/**
	 * Called to initialize just once at the beginning
	 */
	protected void setupSymbolic()
	{
	}

	public abstract void adjustPreferredSize();


	public void listGraphableAdded(EventObject e)
	{
		repaint();
	}
	public void listGraphableChanged(EventObject e) 
	{
		repaint();
	}
	public void listGraphableRemoved(EventObject e) 
	{
		repaint();
	}
	public void listGraphableSelected(EventObject e) 
	{

		setGraphFunction((GraphFunction)e.getSource());

		repaint();

	}
	public void listGraphableDeselected(EventObject e) 
	{
	}
	
	class FunctionNameBox extends JLabel
		implements PropertyBox
	{
		GraphFunction source;
		
		public void setSource(Object source)
		{
			this.source = (GraphFunction)source;
		}
		
		public void reset()
		{
			String strLabel;
			String fName;
			int fOrdinal;
			
			if (source!=null){
				fName = source.getFunctionName();	
				fOrdinal = source.getFunctionOrdinal();

				strLabel = getFunctionNameLabelStr(fName, fOrdinal);
				
				if (getText()==null || !getText().equals(strLabel)){

					setTextAdjustSize(strLabel, fName, fOrdinal);
				}
				
				Color gColor = ((GraphFunction)this.source).getColor();
				//test
				//setOpaque(true);
				//setBackground(gColor);
				//setForeground(((GraphFunction)this.source).getColor());
				
				//setBorder(BorderFactory.createLineBorder(gColor,1));
				
				//setBorder(BorderFactory.createMatteBorder(0,0,1,0,gColor));
			}
			else{
				setText("");
				setBackground(Color.white);
			}
		}
		
//		public void setFont(Font font)
//		{
//			//Font bigger = font.deriveFont(Font.ITALIC, font.getSize()+12.0f);
//			Font bigger = font.deriveFont(6.0f);
//			super.setFont(bigger);
//		}

		private String getFunctionNameLabelStr(String fName, int fOrdinal)
		{
			if (fOrdinal!=-1){
				//Use html to display the ordinal in subscript 
//temp				return "<html>" + fName + "<sub>" + fOrdinal + "</sub></html>";
				
				//return "<html>" + fName + "<font size=2>" + fOrdinal + "</font></html>";
				return "<html>" + "<font size=2>" + fOrdinal + "</font></html>";
				//return fName + fOrdinal;
			}
			else{
				return fName;
			}
		}
		
		private void setTextAdjustSize(String strLabel, String fName, int fOrdinal)
		{
			if (fOrdinal != -1){
//temp				setText(fName + fOrdinal);
				setText(""+fOrdinal);
				Dimension dPref = getMaximumSize();
	
				setText(strLabel);
	
				setMaximumSize(new Dimension(dPref.width, getMaximumSize().height));
				this.revalidate();
				
			}
			else{
				setText(strLabel);
			}
		}
	} // FunctionNameBox
} // FunctionInput
