/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/24 21:56:14 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;
import java.util.Vector;

import javax.swing.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;
import org.concord.graphfunction.ui.*;

/**
 * Base Panel class for displaying the symbolic representation of a function
 * It is a Graph Function View, so it displays the current function selected
 *
 * Date created: 04/07/2004
 *
 * @author Ingrid Moncada<p>
 *
 */

public class FunctionSymbolic extends JPanel
	implements GraphFunctionView
{
	ExpressionFormChooser expressionPanel;
	Vector functionInputs; //of FunctionInput
	float labelSize = 0;
	
	public FunctionSymbolic()
	{
		this(true);
	}
	
	public FunctionSymbolic(boolean editable)
	{
		expressionPanel = new ExpressionFormChooser(editable);
		functionInputs = new Vector();

		if (editable){
			add(expressionPanel);
		}
		else{
			setLayout(new BorderLayout());
			add(expressionPanel, BorderLayout.WEST);
		}
	}

	protected void addFunctionInput(int form, FunctionInput fi, String label)
	{
		if (labelSize!=0){
			fi.setLabelSize(labelSize);
		}
		functionInputs.add(fi);		
		expressionPanel.addInputForm(form,fi,label);
	}
	
	protected void addFunctionInput(FunctionInput fi)
	{
		if (labelSize!=0){
			fi.setLabelSize(labelSize);
		}
		functionInputs.add(fi);		
		add(fi);
	}

	public void setGraphFunction(GraphFunction g)
	{
		expressionPanel.setGraphFunction(g);
		int i;
		for (i=0;i<functionInputs.size();i++){
			((FunctionInput)functionInputs.elementAt(i)).setGraphFunction(g);
		}
	}

	public void setLabelSize(float size)
	{
		labelSize = size;
		
		int i;
		for (i=0;i<functionInputs.size();i++){
			((FunctionInput)functionInputs.elementAt(i)).setLabelSize(size);
		}
	}
	
	public void setValueBoxIncrementInterval(double val)
	{
		int i;
		for (i=0;i<functionInputs.size();i++){
			((FunctionInput)functionInputs.elementAt(i)).setValueBoxIncrementInterval(val);
		}
	}
	
	public void setValueBoxNumDecimalsValue(int n)
	{
		int i;
		for (i=0;i<functionInputs.size();i++){
			((FunctionInput)functionInputs.elementAt(i)).setValueBoxNumDecimalsValue(n);
		}
	}

	public void setEditable(boolean b)
	{
		int i;
		for (i=0;i<functionInputs.size();i++){
			((FunctionInput)functionInputs.elementAt(i)).setEditable(b);
		}
	}

	/**
	 * Sets if the input should show the function name or not
	*/
	public void setShowFunctionName(boolean b)
	{
		int i;
		for (i=0;i<functionInputs.size();i++){
			((FunctionInput)functionInputs.elementAt(i)).setShowFunctionName(b);
		}
	}

	public ValueBox getSelectedBox()
	{
		//FIXME
		FunctionInput f;
		int i;
		for (i=0;i<functionInputs.size();i++){
			f = (FunctionInput)functionInputs.elementAt(i);
			if (f.isVisible()){
				return f.getSelectedBox();
			}
		}
		return null;
	}
}
