/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/24 21:56:14 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;
import java.util.*;

import javax.swing.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;

/**
 * A Panel that knows how to display the symbolic representation of 
 * any function
 * It is a Graph Function View, so it displays the current function selected
 *
 * Date created: 04/07/2004
 *
 * @author Ingrid Moncada<p>
 *
 */
public class FunctionSymbolicPanel extends JPanel
	implements GraphFunctionView
{
	private Hashtable functionInputs;
	
	public FunctionSymbolicPanel()
	{
		functionInputs = new Hashtable();
		
		JPanel fi;

		setLayout(new CardLayout());
		
		//Symbolic representation of a quadratic function
		fi = new QuadraticSymbolic();
		functionInputs.put("QuadraticFunction",fi);
		add(fi,"QuadraticFunction");
		
		//Symbolic representation of a Linear function
		fi = new LinearSymbolic();
		functionInputs.put("LinearFunction",fi);
		add(fi,"LinearFunction");

		//Symbolic representation of a Linear equation
		fi = new LinearEquationInput();
		functionInputs.put("LinearEquation",fi);
		add(fi,"LinearEquation");

		//Symbolic representation of a Linear Piecewise
		//fi = new JPanel();
		//fi = new LinearPiecewiseSymbolic2();
		JScrollPane ps = new LinearPiecewiseSymbolic();
		//fi.add(ps);
		functionInputs.put("LinearPiecewise",ps);
		add(ps,"LinearPiecewise");

		add(new JPanel()," ");
	}
	
	public void setGraphFunction(GraphFunction f)
	{
		String strTypeFunction;
		CardLayout c = (CardLayout)getLayout();
		if (f instanceof QuadraticFunction){
			strTypeFunction = "QuadraticFunction";
		}
		else if (f instanceof LinearFunction){
			strTypeFunction = "LinearFunction";
		}
		else if (f instanceof LinearEquation){
			strTypeFunction = "LinearEquation";
		}
		else if (f instanceof LinearPiecewise){
			strTypeFunction = "LinearPiecewise";
		}
		else{
			strTypeFunction = " ";
		}

		GraphFunctionView fi = (GraphFunctionView)functionInputs.get(strTypeFunction);
		if (fi!=null){
			fi.setGraphFunction(f);
		}
		
		c.show(this,strTypeFunction);
	}
}
