/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.5 $
 * $Date: 2004/08/18 18:20:25 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;
import javax.swing.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;

/**
 * Input for linear equations
 *
 * @author Ingrid Moncada<p>
*/


public class LinearEquationInput extends LinearFunctionEquationInput
{
	ValueBox slopeBox;
	CheckedYInterceptBox interceptBox;
	CheckedValueValueBox resultBox;
	
	public LinearEquationInput()
	{
		super();
	}

	public LinearEquationInput(boolean b)
	{
		super(b);
	}

	public LinearEquationInput(LinearEquation eq)
	{
		super(eq);
	}
	
	protected void setupSymbolic()
	{

		removeAllBoxes();
	
		ValueBox valueBox;

		resultBox = new CheckedValueValueBox();
		resultBox.setSelectedColor(Color.yellow);
		resultBox.setFont(labelsFont);
		addValueBox(resultBox);

		addLabel("  =  ");
	
		slopeBox = new CheckedSlopeBox(); // A kludge to check for zero slope
		slopeBox.setFont(labelsFont);
		addValueBox(slopeBox);
		//By default the first coefficient is selected
		setSelected(slopeBox);

		//addLabel( " x ");
		addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
		
		addLabel("    +  ");
	
		interceptBox = new CheckedYInterceptBox();
		interceptBox.setFont(labelsFont);		
		addValueBox(interceptBox);

		setSelected(resultBox);
	}

	/**
	 * Sets the source for all the values 
	*/
	public void setSourceBoxes(Object source)
	{
		super.setSourceBoxes(source);
		resultBox.setSource(((FunctionEquation)subject).getResultValue());
	}


	
	class CheckedSlopeBox extends ValueBox 
	{
		//The source is a LinearFunction
		protected void setValue(double value)
		{

			if (source instanceof LinearFunction){
				((LinearFunction)source).setSlope(value);
			}
			else if (source instanceof LinearEquation){
				((LinearEquation)source).setSlope(value);
			}
		}
	
		public double getValue()
		{
			if (source instanceof LinearFunction){
				return ((LinearFunction)source).getSlope();
			}
			else if (source instanceof LinearEquation){
				return ((LinearEquation)source).getSlope();
			}
			return Double.NaN;
		}
	} // class CheckedSlopeBox

	class CheckedValueValueBox extends ValueValueBox {
		public void setValue(double value) {
			if (slopeBox.getValue() != 0.0) {
				super.setValue(value);
			}
		}

	} // class CheckedInterceptValueBox

	class CheckedYInterceptBox extends YInterceptBox {
		public void setValue(double value) {
			super.setValue(value);
		}

		public void setValue (double value, boolean check) {
			if (check) {
				setValue(value);
			} else {
				super.setValue(value);
			}
		}

	} // class CheckedInterceptValueBox
}
