/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.4 $
 * $Date: 2004/07/16 20:44:16 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

import org.concord.graph.event.*;
import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;
import org.concord.graphfunction.ui.*;

/** 
 * LinearPiecewiseSymbolic is a component to display a Piecewise on
 * the screen. This is an interactive component that handles
 * mouseclicks. The user can edit either the domain or the
 * function. 
 * This is a "view" of the Piecewise object which is
 * an elemement.
 * This component will pop up a dialog to change a segment when
 * the user clicks.
 *   [We may want this to fit in a scrollpane?] 
 *
 */

public class LinearPiecewiseSymbolic extends JScrollPane 
	implements GraphFunctionView,
			   SelectableListListener,
			   MouseListener
{
    LinearPiecewise piecewise;
    JPanel panel;
    JFrame inputWindow;
    LinearSegmentSlopeInterceptInput inputObject;
    LinearSegmentPointSlopeInput inputObject2;
    TabbedLinearListInput expressionPanel;
	GraphFunctionList glist = null;
	
	Vector segmentLabels;
	
    GraphFunctionLabel selectedSegmentLabel;
	
	JPanel rhPanel;
	JLabel pLabel = new JLabel();

	JTextField header;

	JCheckBox visibleCheckBox = new JCheckBox();

	public LinearPiecewiseSymbolic() 
	{

		// Sometimes Swing identifiers are just too long.
		final int hsbPolicy = JScrollPane.HORIZONTAL_SCROLLBAR_NEVER;
		setHorizontalScrollBarPolicy(hsbPolicy);

		addMouseListener(this);

		segmentLabels = new Vector();
		
		panel = new JPanel();
		panel.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS));
		panel.setBackground(Color.lightGray);
	
		setViewportView(panel);
	
		rhPanel = new JPanel(new GridLayout(1,2));
		rhPanel.add(new JLabel("y="));
		rhPanel.add(new Bracket());
		setRowHeaderView(rhPanel);

		// ---- Header 
		JPanel headerPanel = new JPanel(new BorderLayout());

		header = new HeaderTextField("Piecewise Symbolic");

		JButton headerDelete = 
			new JButton(ResourceLoader.getImageIcon("Remove24"));

		headerDelete.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					piecewise.removeAllElements();
					setupSegments(); // FIXME this is a kludge & may cause bugs

					header.setText("Piecewise Symbolic");
				}
			});
									   
									   
		headerDelete.setMargin(new Insets(0,0,0,0));


		headerPanel.add(header, BorderLayout.CENTER);
		headerPanel.add(headerDelete, BorderLayout.EAST);

		setColumnHeaderView(headerPanel);
	
		visibleCheckBox = new JCheckBox();
		//makeFrame.setBackground(Color.white);
		visibleCheckBox.setMargin(new Insets(2,2,0,0));

		visibleCheckBox.addItemListener(new ItemListener () {

				public void itemStateChanged(ItemEvent e) 
				{
					boolean newlySelected = 
						(e.getStateChange() == ItemEvent.SELECTED);
					if (piecewise != null)
						piecewise.setVisible(newlySelected);
					if (newlySelected)
						piecewise.select();
				}
			});

	
		this.setCorner(JScrollPane.UPPER_LEFT_CORNER,visibleCheckBox);

	} // Constructor

    GraphableListener piecewiseHandler = new GraphableAdapter() 
	{
		public void graphableChanged(EventObject e) 
		{
			if (e instanceof PiecewiseEvent){
				PiecewiseEvent evt = (PiecewiseEvent)e;
				if (evt.getId() == PiecewiseEvent.PIECEWISE_SEGMENT_ADDED){
					setupSegments();
				}
				else if (evt.getId() == PiecewiseEvent.PIECEWISE_SEGMENT_REMOVED){
					setupSegments();
				}
			}
		}

		public void graphableRemoved(EventObject e) 
		{
			setupSegments();
		}
	};

    MouseListener mouseHandler = new MouseAdapter() 
	{
		public void mouseClicked(MouseEvent e) 
		{		
			GraphFunctionLabel newSelected = ((GraphFunctionLabel)e.getSource());
			piecewise.select();
			newSelected.getGraphFunction().select();
			
			if (inputWindow != null) { 
				try{
					inputWindow.setLocationRelativeTo(LinearPiecewiseSymbolic.this);
				}catch(Throwable t){}
				inputWindow.show();

			}

			selectedSegmentLabel = newSelected;
		}
	};

	public void setGraphFunction(GraphFunction f)
	{
		setPiecewise((LinearPiecewise)f);
		visibleCheckBox.setSelected(f.isVisible());
	}

	public void setGraphFunctionList(GraphFunctionList gl)
	{
		if (glist!=null){
			glist.removeGraphableListListener(this);
		}
		glist = gl;
		if (glist!=null){
			glist.addGraphableListListener(this);
		}
	}

	public void setInputWindow (JFrame frame) {
		inputWindow = frame;
	}

	public JFrame getInputWindow () {
		return inputWindow;
	}
	
	public void setPiecewise(LinearPiecewise piecewise) 
	{
		if (this.piecewise!=null){
			this.piecewise.removeGraphableListener(piecewiseHandler);
		}
		
		this.piecewise = piecewise;
		setupSegments();
		
		if (this.piecewise!=null){
			piecewise.addGraphableListener(piecewiseHandler);
		}
		
		if (inputObject != null) {
			inputObject.setLinearPiecewise(piecewise);
			inputObject2.setLinearPiecewise(piecewise);
		}
	}

	public void setupSegments() 
	{
		for (int i=0; i<segmentLabels.size(); i++){
			GraphFunctionLabel l = (GraphFunctionLabel)segmentLabels.elementAt(i);
			l.setGraphFunction(null);
		}
		segmentLabels.removeAllElements();
		
		panel.removeAll();

		if (this.piecewise != null) {
			Enumeration segments = piecewise.getSegments();
			
			while (segments.hasMoreElements()) {
				LinearSegment lps = (LinearSegment)segments.nextElement();
				
				GraphFunctionLabel segmentLabel = new GraphFunctionLabel(lps);
				segmentLabels.add(segmentLabel);
			
				// This can be optimized!
				segmentLabel.addMouseListener( mouseHandler);
			
				panel.add(segmentLabel);
			}

			header.setForeground(this.piecewise.getColor());
			
			rhPanel.setVisible(true);
		}
		else{
			header.setText("");
			rhPanel.setVisible(false);
		}
		//		setColumnHeaderView(header);

		panel.revalidate();
		panel.repaint();

	} // setupSegments

	/** Gets the current Piecewise */
	public Piecewise getPiecewise() 
	{
		return piecewise;
	}
	
	public void newFrame() 
	{
		JFrame frame = new JFrame("Piecewise Symbolic Table");
		// frame.setSize(250,150);
		//This doesnt work in java 1.3
		try{
			frame.setLocationRelativeTo(this);
		}catch(Throwable t){
		}
		LinearPiecewiseSymbolic newPS = new LinearPiecewiseSymbolic();
		newPS.setPiecewise(piecewise);
		frame.getContentPane().add(newPS);
		frame.pack();
		frame.show();
	}

	
	public void listGraphableAdded(EventObject e) {}
	public void listGraphableChanged(EventObject e) {} 
	public void listGraphableRemoved(EventObject e) {}
	public void listGraphableSelected(EventObject e) 
	{
		Object source = e.getSource();
		if (source instanceof GraphFunction) {
			setGraphFunction((GraphFunction)source);
		}
		repaint();

	}
	public void listGraphableDeselected(EventObject e) 
	{
	}

	protected ImageIcon getImageIcon (String filename) {
		java.net.URL imgURL = 
			LinearPiecewiseSymbolic.class.getResource(filename);
		if (imgURL != null ) {
			return new ImageIcon(imgURL);
		} else {
			System.err.println("Couldn't find file: " + imgURL);
			return null;
		}
		
    }

	public void mouseClicked(MouseEvent e) {
		piecewise.select();
	}

	public void mouseEntered(MouseEvent e) {}

	public void mouseExited(MouseEvent e) {}
    
	public void mousePressed(MouseEvent e) {}
    
	public void mouseReleased(MouseEvent e) {} 

	class HeaderTextField extends JTextField {

		boolean editing = false;
		Color editingColor = Color.white;
		Color normalColor;

		public HeaderTextField (String name) {
			super(name);

			normalColor = getBackground();

			setBackground(normalColor);

			// Catch events when the text is changed.
			getDocument().addDocumentListener ( new DocumentListener() {
					public void changedUpdate(DocumentEvent e){
						piecewise.setFunctionName(getText());

					}

					public void insertUpdate(DocumentEvent e) {
						piecewise.setFunctionName(getText());
					}

					public void removeUpdate(DocumentEvent e) {
						piecewise.setFunctionName(getText());
					}

				});
		}

	} // class HeaderTextField

} // public class LinearPiecewiseSymbolic
    
