/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/07/14 17:29:54 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import javax.swing.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;

/**
 * @author Eric Brown-Munoz<p>
 * @author Ingrid Moncada<p>
*/

public class LinearPointSlopeInput extends LinearSlopeInterceptInput 
{
    public LinearPointSlopeInput() 
	{
		super();
    }

    public LinearPointSlopeInput(boolean b) 
	{
		super(b);
    }

    public LinearPointSlopeInput(LinearFunction subject) 
	{
		super(subject);
    }

    protected void setupSymbolic() 
	{	
		removeAllBoxes();
	
		ValueBox valueBox;
		
		addLabel(new JLabel(ResourceLoader.getImageIcon("y")));
		addLabel("  -");
	
		valueBox = new PushpinYBox();
		addValueBox(valueBox);
	
		addLabel("=");
	
		valueBox = new SlopeBox();
		addValueBox(valueBox);
		setSelected(valueBox);
		
		addLabel("  ( ");
		addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
		addLabel("  -");
	
		valueBox = new PushpinXBox();
		addValueBox(valueBox);
	
		addLabel(")");
    }
}

class PushpinXBox extends ValueBox 
{	
	//The source is a LinearFunction
	public double getValue() 
	{
		if (source instanceof LinearFunctionPushpin){
			return ((LinearFunctionPushpin)source).getPushpinX();
		}
		//No pushpin... so? assume x0 = 0 for now...
		return 0.0;
	}
	
	protected void setValue(double value) 
	{
		if (source instanceof LinearFunctionPushpin){
			((LinearFunctionPushpin)source).setPushpinX(value);
		}
	}

}

class PushpinYBox extends ValueBox 
{
	//The source is a LinearFunction
	public double getValue() 
	{
		double xval;
		if (source instanceof LinearFunctionPushpin){
			return ((LinearFunctionPushpin)source).getPushpinY();
		}
		else{
		//No pushpin... so? assume x0 = 0 for now...
			return ((LinearFunction)source).getYIntercept();
		}
    }
	
    // Changing the ps form just moves the y-intercept
    protected void setValue(double value) 
	{
		if (source instanceof LinearFunctionPushpin){
			((LinearFunctionPushpin)source).setPushpinY(value);
		}
		else{
		//No pushpin... so? Nothing, keep assuming x0 = 0 for now...
		}
    }
}


