/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/24 21:56:14 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;

/**
 * Piecewise segment input with a point
 * in the form y = m ( x - x0 ) + y0
 * @author Ingrid Moncada<p>
*/
public class LinearSegmentPointSlopeInput extends LinearSegmentSlopeInterceptInput
{
    protected void setupSymbolic()  
	{
		removeAllBoxes();	
	
		ValueBox valueBox;
		
		valueBox = new PiecewiseSlopeBox();
		addValueBox(valueBox);
		setSelected(valueBox);

		addLabel(" ( x - ");

		valueBox = new XBox();
		addValueBox(valueBox);

		addLabel(") +");

		valueBox = new YBox();
		addValueBox(valueBox);

		addLabel(" for ");

		valueBox = new StartBox();
		addValueBox(valueBox);

		addPropertyBox(new StartComparison());

		addLabel(" x ");

		addPropertyBox(new EndComparison());

		valueBox = new EndBox();
		addValueBox(valueBox);

		selectedIndex = 0;
    }

	public void adjustPreferredSize()
	{
		if (!editable){
			setPreferredSize(new Dimension(360,25));
//			setPreferredSize(new Dimension(this.getPreferredSize().width,25));
		}
		else{
			setPreferredSize(new Dimension(360,40));
//			setPreferredSize(new Dimension(this.getPreferredSize().width,40));
		}
	}	
	
	class YBox extends ValueBox 
	{
		//The source is a LinearSegment
		public double getValue() 
		{
			double value = ((LinearSegment)source).getDomain().getStart();
			if (Double.isInfinite(value)){
				value = ((LinearSegment)source).getDomain().getEnd();
			}
			return ((LinearSegment)source).yval(value, false);
		}
	
		public void setValue(double value) 
		{
			double diff = value - getValue();
			LinearSegment seg = ((LinearSegment)source);
			((LinearSegment)source).setYIntercept(((LinearSegment)source).getYIntercept()+diff);
		}
	}
	
	class XBox extends ValueBox 
	{
		//The source is a LinearSegment
		public double getValue() 
		{
			double value = ((LinearSegment)source).getDomain().getStart();
			if (Double.isInfinite(value)){
				value = ((LinearSegment)source).getDomain().getEnd();
			}
			return value;
		}
	
		public void setValue(double value) 
		{
			double diff = value - getValue();
			LinearSegment seg = ((LinearSegment)source);
			if (!Double.isInfinite(seg.getDomain().getStart())){
				seg.getDomain().setStart(value);
			}
			if (!Double.isInfinite(seg.getDomain().getEnd())){
				seg.getDomain().setEnd(seg.getDomain().getEnd()+diff);
			}
			seg.setYIntercept(seg.getYIntercept()-diff*seg.getSlope());
		}
	}
	
}
