/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.5 $
 * $Date: 2004/07/30 18:10:02 $
 * $Author: jqian $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import org.concord.graph.event.*;
import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;

/** 
 * @author Eric Brown-Munoz<p>
 */

public class LinearSegmentSlopeInterceptInput extends LinearSlopeInterceptInput 
{
	LinearPiecewise piecewise = null;
	
	// The listener that will be added to the current segment
	GraphableListener segmentListener = new GraphableAdapter() 
	{
		public void graphableChanged(EventObject e) 
		{
			LinearSegmentSlopeInterceptInput.this.resetBoxes();
		}
	};
	
	// The listenter that will be added to the piecewise
	SelectableAdapter piecewiseListener = new SelectableAdapter() 
	{
		public void graphableSelected(EventObject e) 
		{
			if (e.getSource() instanceof LinearSegment){
				setGraphFunction((LinearSegment)e.getSource());
			}
		}
	};
	
	public void setLinearPiecewise( LinearPiecewise piecewise) 
	{
		if (this.piecewise!=null) {
			this.piecewise.removeSelectableListener(piecewiseListener);
		}
		
		this.piecewise = piecewise;
		
		if (this.piecewise!=null) {
			// This should call a method in piecewise (to relieve type problem)
			setGraphFunction( (LinearSegment)this.piecewise.getCurrentSegmentSelected());
			
			this.piecewise.addSelectableListener(piecewiseListener);
		}
	}
	
	protected void setupSymbolic() 
	{
		removeAllBoxes();	
	
		ValueBox valueBox;
		
		valueBox = new PiecewiseSlopeBox();
		addValueBox(valueBox);
		setSelected(valueBox);
	
		addLabel("x");
	
		addLabel("+");
	
		valueBox = new YInterceptBox();
		addValueBox(valueBox);
	
		addLabel(" for ");
	
		valueBox = new StartBox();
		addValueBox(valueBox);
	
		addPropertyBox(new StartComparison());
	
		addLabel(" x ");
	
		addPropertyBox(new EndComparison());
	
		valueBox = new EndBox();
		addValueBox(valueBox);
	
		selectedIndex = 0;
	}
	
	public void adjustPreferredSize()
	{
		if (!editable){
			setPreferredSize(new Dimension(320,25));
//			setPreferredSize(new Dimension(this.getPreferredSize().width,25));
		}
		else{
			setPreferredSize(new Dimension(320,40));
//			setPreferredSize(new Dimension(this.getPreferredSize().width,40));
		}
	}	
}

class StartBox extends ValueBox 
{
	//The source is a LinearSegment
    public double getValue() 
	{
        if (Double.isInfinite(((LinearSegment)source).getDomain().getStart()))
        {
            setFont (new Font("Symbol",Font.BOLD,14));
        }
        else
        {
           setFont (new Font("Dialog",Font.BOLD,12));
        } 
		return ((LinearSegment)source).getDomain().getStart();
    }

    public void setValue(double value) 
	{
		LinearSegment seg = ((LinearSegment)source);
		//Only let start be less than end
		if (seg.getDomain().getEnd() - value > this.getIncrementInterval()){
			seg.getDomain().setStart(value);
		}
    }
}

class EndBox extends ValueBox 
{
	//The source is a LinearSegment
	public double getValue() 
	{
        if (Double.isInfinite(((LinearSegment)source).getDomain().getEnd()))
        {
            setFont (new Font("Symbol",Font.BOLD,14));
        }
        else
        {
           setFont (new Font("Dialog",Font.BOLD,12));
        } 
		return ((LinearSegment)source).getDomain().getEnd();
	}
	
	public void setValue(double value) 
	{
		LinearSegment seg = ((LinearSegment)source);
		//Only let end be more than end
		if (value - seg.getDomain().getStart() > this.getIncrementInterval()){
			seg.getDomain().setEnd(value);
		}
	}
}

class PiecewiseSlopeBox extends SlopeBox 
{
	//The source is a LinearSegment
    public void increment() 
	{
		LinearSegment graphable;
		graphable = (LinearSegment)source;
		
		double slope = graphable.getSlope();
		double yIntercept = graphable.getYIntercept();
		graphable.setSlope(slope + getIncrementInterval());
	
		double startPoint = ((LinearSegment) graphable).getDomain().getStart();
		if (! Double.isInfinite(startPoint)) {
			graphable.setYIntercept(yIntercept + startPoint * (- getIncrementInterval()));
		} else {
			double endPoint = ((LinearSegment) graphable).getDomain().getEnd();
			graphable.setYIntercept(yIntercept + endPoint * (- getIncrementInterval()));
		}
		reset();
    }

    public void decrement() 
	{
		LinearSegment graphable;
		graphable = (LinearSegment)source;
		
		double slope = graphable.getSlope();
		double yIntercept = graphable.getYIntercept();
		graphable.setSlope(slope - getIncrementInterval());
		
		double startPoint = ((LinearSegment) graphable).getDomain().getStart();
		if (! Double.isInfinite(startPoint)) {
			graphable.setYIntercept(yIntercept + startPoint * (getIncrementInterval()));
		} else {
			double endPoint = ((LinearSegment) graphable).getDomain().getEnd();
			graphable.setYIntercept(yIntercept + endPoint * (getIncrementInterval()));
		}
		
		reset();
    }

}

abstract class Comparison extends JButton 
    implements PropertyBox, ActionListener//, DomainListener 
{
	//The source is a segment
	SegmentDomain domain;
	
	public Comparison() 
	{
		super();
		addActionListener(this);
		setPreferredSize(new Dimension(25,25));
		setMargin(new Insets(1,1,1,1));
        //System.out.println ("Current Font "+getFont());
        //setFont (new Font("SansSerif",Font.PLAIN,10));
        //System.out.println ("Current Font "+getFont());
	}

	public void setSource(Object source)
	{
		if (source instanceof Segment){
			setDomain(((Segment)source).getDomain());
		}
	}

	public void setDomain(SegmentDomain domain) 
	{
		this.domain = domain;

		reset();
	}

	public abstract void toggle();
	
	// From ActionListener
	public void actionPerformed (ActionEvent e) 
	{
		toggle();
	}
}

class StartComparison extends Comparison 
{
	public void reset() 
	{
		if (domain.getClosedStart()) {
			//setText("\u2264");
            setIcon(ResourceLoader.getImageIcon("lessorequal"));
		} else {
			//setText("<");
            setIcon(ResourceLoader.getImageIcon("lessthan"));
		}
	}
	
	public void toggle() 
	{
		if (domain.getClosedStart()) {
			domain.setClosedStart(false);
		} else {
			domain.setClosedStart(true);
		}
	}
}

class EndComparison extends Comparison 
{
	public void reset() 
	{	
		if (domain.getClosedEnd()) {
			//setText("\u2264");
            setIcon(ResourceLoader.getImageIcon("lessorequal"));
		} else {
			//setText("<");
            setIcon(ResourceLoader.getImageIcon("lessthan"));
		}
	}
	
	public void toggle() 
	{
		if (domain.getClosedEnd()) {
			domain.setClosedEnd(false);
		} else {
			domain.setClosedEnd(true);
		}
	}
}
