/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/07/14 17:29:54 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;

import javax.swing.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;

/**
 * Function input that displays a quadratic in symbolic form
 * 
 * @author Ingrid Moncada<p>
*/

public class QuadraticPolynomialInput extends FunctionInput
{
    /** Creates and Initializes a QuadraticPolynomialInput Component */
    public QuadraticPolynomialInput() 
	{
		super();
    }

    public QuadraticPolynomialInput(boolean editable) 
	{
		super(editable);
    }

    public QuadraticPolynomialInput(QuadraticFunction subject) 
	{
		super(subject);
    }

    public void setGraphFunction(GraphFunction subject) 
	//Only checks if the function is quadratic
	{
		//Checking if the function is quadratic
		if (subject!=null){
			if (!(subject instanceof QuadraticFunction)) return;
		}
		super.setGraphFunction(subject);
	}

	protected void setupSymbolic() 
	{		
		removeAllBoxes();
		
		ValueBox valueBox;

		setupFunctionName();

		addLabel("  =  ");
	
		valueBox = new CoeffABox();
		valueBox.setFont(labelsFont);		
		addValueBox(valueBox);
		//By default the first coefficient is selected
		setSelected(valueBox);

		//addLabel( " x^2 ");
		addLabel(new JLabel(ResourceLoader.getImageIcon("x^2")));
		
		addLabel("     +  ");
	
		valueBox = new CoeffBBox();
		valueBox.setFont(labelsFont);		
		addValueBox(valueBox);
	
		addLabel(new JLabel(ResourceLoader.getImageIcon("x")));

		addLabel("    +  ");
	
		valueBox = new CoeffCBox();
		valueBox.setFont(labelsFont);		
		addValueBox(valueBox);
	}
 		
	public void adjustPreferredSize()
	{
		if (!editable){
			setPreferredSize(new Dimension(330,25));
//			setPreferredSize(new Dimension(this.getPreferredSize().width,25));
		}
		else{
			setPreferredSize(new Dimension(330,40));
//			setPreferredSize(new Dimension(this.getPreferredSize().width,40));
		}
	}	
}

class CoeffABox extends ValueBox 
{
	//The source is a QuadraticFunction
	protected void setValue(double value)
	{
		((QuadraticFunction)source).setCoeffA(value);
	}
	
	public double getValue()
	{
		return ((QuadraticFunction)source).getCoeffA();
	}
}

class CoeffBBox extends ValueBox 
{
	//The source is a QuadraticFunction
	protected void setValue(double value)
	{
		((QuadraticFunction)source).setCoeffB(value);
	}
	
	public double getValue()
	{
		return ((QuadraticFunction)source).getCoeffB();
	}
}

class CoeffCBox extends ValueBox 
{
	//The source is a QuadraticFunction
	protected void setValue(double value)
	{
		((QuadraticFunction)source).setCoeffC(value);
	}
	
	public double getValue()
	{
		return ((QuadraticFunction)source).getCoeffC();
	}
}

