/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/07/14 17:29:54 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;

import javax.swing.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;

/**
 * Function input that displays a quadratic in root form
 * 
 * @author Ingrid Moncada<p>
*/

public class QuadraticRootFormInput extends QuadraticPolynomialInput
{
    /** Creates and Initializes a QuadraticPolynomialInput Component */
    public QuadraticRootFormInput() 
	{
		super();
    }

    public QuadraticRootFormInput(boolean b) 
	{
		super(b);
    }

    public QuadraticRootFormInput(QuadraticFunction subject) 
	{
		super(subject);
    }

    public void setGraphFunction(GraphFunction subject) 
	//Only checks if the function is quadratic
	{
		//Checking if the function is quadratic
		if (subject!=null){
			if (!(subject instanceof QuadraticFunction)) return;
		}
		super.setGraphFunction(subject);
	}

	protected void setupSymbolic() 
	{
		removeAllBoxes();
	
		ValueBox valueBox;

		//addLabel( "y  =  ");
		addLabel(new JLabel(ResourceLoader.getImageIcon("y")));
		addLabel("  =  ");
	
		valueBox = new CoeffARootBox();
		valueBox.setFont(labelsFont);		
		addValueBox(valueBox);

		addLabel(" (  ");

		addLabel(new JLabel(ResourceLoader.getImageIcon("x")));

		addLabel("  -  ");

		valueBox = new Root1Box();
		valueBox.setFont(labelsFont);		
		addValueBox(valueBox);
		//By default the first coefficient is selected
		setSelected(valueBox);

		addLabel(" )");

		addLabel("(  ");

		addLabel(new JLabel(ResourceLoader.getImageIcon("x")));

		addLabel("  -  ");

		valueBox = new Root2Box();
		valueBox.setFont(labelsFont);		
		addValueBox(valueBox);

		addLabel(" )");
    }
	
	public void updateSymbolic()
	{
	//Checks if the quadratic function has roots or not
		if (subject instanceof QuadraticFunction){
			if (!((QuadraticFunction)subject).hasRoots()){
				hideAllBoxes();
				showMessage("This function has no Real Roots");
				setArrowButtonsVisible(false);
				return;
			}
		}
		if (editable){
			setArrowButtonsVisible(true);
		}
		hideMessage();
		showAllBoxes();
		super.updateSymbolic();
	}
	
	public void adjustPreferredSize()
	{
		if (!editable){
			setPreferredSize(new Dimension(420,25));
//			setPreferredSize(new Dimension(this.getPreferredSize().width,25));
		}
		else{
			setPreferredSize(new Dimension(420,40));
//			setPreferredSize(new Dimension(this.getPreferredSize().width,40));
		}
	}	
}

class Root1Box extends ValueBox 
{
	//The source is a QuadraticFunction
	protected void setValue(double value)
	{
		((QuadraticFunction)source).setRoot1(value);
	}
	
	public double getValue()
	{
		return ((QuadraticFunction)source).getRoot1();
	}
}

class Root2Box extends ValueBox 
{
	//The source is a QuadraticFunction
	protected void setValue(double value)
	{
		((QuadraticFunction)source).setRoot2(value);
	}
	
	public double getValue()
	{
		return ((QuadraticFunction)source).getRoot2();
	}
}

class CoeffARootBox extends ValueBox 
{
	//The source is a QuadraticFunction
	protected void setValue(double value)
	{
		((QuadraticFunction)source).setCoeffARoot(value);
	}
	
	public double getValue()
	{
		return ((QuadraticFunction)source).getCoeffA();
	}
}
