/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/07/14 17:29:54 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;

import javax.swing.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;

/**
 * Quadratic input that displays a quadratic in vertex form
 * 
 * @author Ingrid Moncada<p>
*/

public class QuadraticVertexFormInput extends QuadraticPolynomialInput
{
    /** Creates and Initializes a QuadraticPolynomialInput Component */
    public QuadraticVertexFormInput() 
	{
		super();
    }

    public QuadraticVertexFormInput(boolean editable) 
	{
		super(editable);
    }

    public QuadraticVertexFormInput(QuadraticFunction subject) 
	{
		super(subject);
    }

	protected void setupSymbolic() 
	{
		removeAllBoxes();
	
		ValueBox valueBox;

		//addLabel( "y  =  ");
		addLabel(new JLabel(ResourceLoader.getImageIcon("y")));
		addLabel("  =  ");
	
		valueBox = new CoeffAVertexBox();
		valueBox.setFont(labelsFont);		
		addValueBox(valueBox);

		addLabel("  (  ");

		addLabel(new JLabel(ResourceLoader.getImageIcon("x")));

		addLabel("  -  ");

		valueBox = new VertexXBox();
		valueBox.setFont(labelsFont);		
		addValueBox(valueBox);
		//By default the first coefficient is selected
		setSelected(valueBox);

		addLabel("  )  ");

		addLabel(new JLabel(ResourceLoader.getImageIcon("2")));

		addLabel("  +  ");

		valueBox = new VertexYBox();
		valueBox.setFont(labelsFont);		
		addValueBox(valueBox);
    }

	public void updateSymbolic()
	{
	//Checks if the function has vertex or not
		if (subject instanceof QuadraticFunction){
			if (!((QuadraticFunction)subject).hasVertex()){
				hideAllBoxes();
				showMessage("This function has no Vertex Form");
				setArrowButtonsVisible(false);
				return;
			}
		}
		if (editable){
			setArrowButtonsVisible(true);
		}
		hideMessage();
		showAllBoxes();
		super.resetBoxes();
	}
	
	public void adjustPreferredSize()
	{
		if (!editable){
			setPreferredSize(new Dimension(330,25));
//			setPreferredSize(new Dimension(this.getPreferredSize().width,25));
		}
		else{
			setPreferredSize(new Dimension(330,40));
//			setPreferredSize(new Dimension(this.getPreferredSize().width,40));
		}
	}	
}

class VertexXBox extends ValueBox 
{
	//The source is a QuadraticFunction
	protected void setValue(double value)
	{
		((QuadraticFunction)source).setVertexX(value);
	}
	
	public double getValue()
	{
		return ((QuadraticFunction)source).getVertexX();
	}
}

class VertexYBox extends ValueBox 
{
	//The source is a QuadraticFunction	
	protected void setValue(double value)
	{
		((QuadraticFunction)source).setVertexY(value);
	}
	
	public double getValue()
	{
		return ((QuadraticFunction)source).getVertexY();
	}
}

class CoeffAVertexBox extends ValueBox 
{
	//The source is a QuadraticFunction
	protected void setValue(double value)
	{
		((QuadraticFunction)source).setCoeffAVertex(value);
	}
	
	public double getValue()
	{
		return ((QuadraticFunction)source).getCoeffA();
	}
}
