/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/07/19 21:58:21 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import org.concord.graph.event.*;
import org.concord.graph.engine.*;
import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;
import org.concord.graphfunction.ui.*;


public class SegmentInputPane 
	extends JFrame
	implements GraphFunctionView,
			   SelectableListListener,
			   GraphableListener,
			   SelectableListener
 {

	 GraphFunctionList glist;
	 GraphFunction function;

	 TabbedLinearSegmentListInput expressionPanel;
	 JLabel pLabel;

	 Piecewise piecewise;

	 public SegmentInputPane() {
		 super("Edit Segment");

		 setSize(450,150);

		 Container cp = getContentPane();

		 expressionPanel =  new TabbedLinearSegmentListInput();
		 pLabel = new JLabel("Loading...");

		 cp.setLayout(new BorderLayout());

		 pLabel.setBackground(Color.white);
		 pLabel.setOpaque(true);
		 pLabel.setAlignmentX(LEFT_ALIGNMENT);
		 pLabel.setPreferredSize(new Dimension(100,30));
		 cp.add(pLabel,BorderLayout.NORTH);
		 
		 cp.add(expressionPanel,BorderLayout.CENTER);
		 expressionPanel.setAlignmentX(LEFT_ALIGNMENT);
		
	 }

	 public void setPiecewise(Piecewise pw) {
		 if (piecewise != null) {
			 piecewise.removeGraphableListener(this);
		 }

		 piecewise = pw;

		 if (pw != null) {
			 setGraphFunction(piecewise.getCurrentSegmentSelected());
			 pw.addGraphableListener(this);
		 }

	 }
	 
	 public void setGraphFunction(GraphFunction gf) {
		 expressionPanel.setGraphFunction(gf);
		 if (function != null) {
			 function.removeGraphableListener(this);
		 }

		 function = gf;

		 if (function != null) {
			 function.addGraphableListener(this);

			 pLabel.setText(piecewise.getFunctionName() +
							" - Segment "+ 
							(piecewise.getCurrentSegmentSelectedIndex()+1) +
							"  ");

			 pLabel.setForeground(function.getColor());
		 }

		 
	 }

	 public void setGraphFunctionList(GraphFunctionList gl)
	 {
		 if (glist!=null){
			 glist.removeGraphableListListener(this);
		 }
		 glist = gl;
		 if (glist!=null){
			 glist.addGraphableListListener(this);
		 }
	 }
	
	 public void listGraphableAdded(EventObject e) {}
	 public void listGraphableChanged(EventObject e) {} 
	 public void listGraphableRemoved(EventObject e) {}
	 public void listGraphableSelected(EventObject e) 
	 {

		 Object source = e.getSource();

		 if (source instanceof Piecewise) { // We only care about Piecewises
			 setPiecewise((Piecewise)source);
		 }
		 repaint();

	 }
	 public void listGraphableDeselected(EventObject e) {
	 }
	 
	 public void graphableChanged(EventObject e) {
		 if (piecewise != null)
			 setGraphFunction(piecewise.getCurrentSegmentSelected());

	 }

	 public void graphableRemoved(EventObject e) {}

	 public void graphableSelected(EventObject e) {
		 Object source = e.getSource();

		 if (source instanceof LinearPiecewise) {
			 setPiecewise((Piecewise)source);
		 }
	 }
	 public void graphableDeselected(EventObject e) {}

} 
