/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/07/14 17:29:54 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;

import javax.swing.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;

/**
 * Symbolic representation of a transformed function
 *
 * @author Ingrid Moncada<p>
*/

public class TransformedFunctionSymbolic extends FunctionInput 
{
	protected int mode = -1; // The drag mode being displayed.

	public TransformedFunctionSymbolic() 
	{
		super();
	}

	public TransformedFunctionSymbolic(boolean editable) 
	{
		super(editable);
	}

	public TransformedFunctionSymbolic(GraphFunction subject) 
	{
		super(subject);
	}
	
	protected void setupSymbolic()
	{
		TransformedQuadraticFunction tSubject=null;
		
		removeAllBoxes();

		setupFunctionName();

		addLabel("  =  ");

		if (subject instanceof TransformedQuadraticFunction){
			tSubject = (TransformedQuadraticFunction)subject;
			mode = tSubject.getDragMode().getValue();
		}

		if (tSubject!=null){
			if (mode == DragMode.NONE) {
				addLabel(new JLabel(ResourceLoader.getImageIcon("f")));
				addPropertyBox(new OriginalFunctionNameBox());
				addLabel("( ");
				addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
				addLabel(" )");
			} else if (mode == DragMode.TRANSLATE_X) {
				addLabel(new JLabel(ResourceLoader.getImageIcon("f")));
				addPropertyBox(new OriginalFunctionNameBox());
				addLabel("( ");
				addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
				addLabel(" + ");
				addValueBox( new DragFactorBox());
				addLabel(")");
	
			} else if (mode == DragMode.TRANSLATE_Y) {
				addLabel(new JLabel(ResourceLoader.getImageIcon("f")));
				addPropertyBox(new OriginalFunctionNameBox());
				addLabel("( ");
				addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
				addLabel(" ) + ");
				addValueBox( new DragFactorBox());
				
			} else if (mode == DragMode.DILATE_X) {
				addLabel(new JLabel(ResourceLoader.getImageIcon("f")));
				addPropertyBox(new OriginalFunctionNameBox());
				addLabel("( ");
				addValueBox( new DragFactorBox());
				addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
				addLabel(" )");
				
			} else if (mode == DragMode.DILATE_Y) {
				addValueBox( new DragFactorBox());
				addLabel(new JLabel(ResourceLoader.getImageIcon("f")));
				addPropertyBox(new OriginalFunctionNameBox());
				addLabel("( ");
				addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
				addLabel(" )");
				
			} else if (mode == DragMode.REFLECT_X) {
				addLabel(new JLabel(ResourceLoader.getImageIcon("f")));
				addPropertyBox(new OriginalFunctionNameBox());
				addLabel("( - ");
				addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
				addLabel(" )");
				
			} else if (mode == DragMode.REFLECT_Y) {
				addLabel(" -  ");
				addLabel(new JLabel(ResourceLoader.getImageIcon("f")));
				addPropertyBox(new OriginalFunctionNameBox());
				addLabel("( ");
				addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
				addLabel(" )");
				
			}
		}
		
		setLabelSize(labelsFont.getSize());
	}

	public void updateSymbolic() 
	{
		if (mode != ((TransformedQuadraticFunction)subject).getDragMode().getValue()){
			setupSymbolic();
			//adjustFunctionName();
			setSourceBoxes(subject);
		}
		super.updateSymbolic();
	}
	
	public void adjustPreferredSize()
	{
		if (!editable){
			setPreferredSize(new Dimension(330,25));
		}
		else{
			setPreferredSize(new Dimension(330,40));
		}
	}	

	class OriginalFunctionNameBox extends FunctionNameBox
	{
		public void setSource(Object source)
		{
			TransformedQuadraticFunction tSubject;
			if (source instanceof TransformedQuadraticFunction){
				tSubject = (TransformedQuadraticFunction)source;
				super.setSource(tSubject.getOriginal());
			}
			else{
				super.setSource(source);
			}			
		}
	}
	
}

class DragFactorBox extends ValueBox 
{
	//The source is a QuadraticGraphable
	protected void setValue(double value)
	{
	}
	
	public double getValue()
	{
		if (source instanceof TransformedQuadraticFunction){
			return ((TransformedQuadraticFunction)source).getAmountDragged();
		}
		return 0.0;
	}
}
