/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/05/24 21:56:14 $
 * $Author: imoncada $
 *
 * Copyright 
*/
package org.concord.graphfunction.ui.symbolicview;

import java.awt.*;

import javax.swing.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;

/** 
 *
 * Date created: 04/07/2004
 *
 * @author Ingrid Moncada<p>
 *
 */

public class TransformedQuadraticRootFormInput extends QuadraticRootFormInput
{
	protected int mode = -1; // The drag mode being displayed.

    public TransformedQuadraticRootFormInput() 
	{
		super();
    }

    public TransformedQuadraticRootFormInput(TransformedQuadraticFunction subject) 
	{
		super(subject);
    }

	//This class should show a transformed quadratic as 
	// if f(x) = 2 x^2 + 3 x + 4 and g(x) = ( f(x+1) ) + 5
	// it should show:
	//    g(x) = ( 2 (x+1)^2 + 3 (x+1) + 4 ) + 5 
	protected void setupSymbolic() 
	{		
		removeAllBoxes();
		
		//Special
		if (subject==null) return;
		
		ValueBox valueBox;

		mode = ((TransformedQuadraticFunction)subject).getDragMode().getValue();
		
		QuadraticFunction original = ((TransformedQuadraticFunction)subject).getOriginal();

		setupFunctionName();

		addLabel("  =  ");

		if (mode == DragMode.DILATE_Y){
			valueBox = new QuadDragFactorBox();
			valueBox.setFont(labelsFont);		
			addValueBox(valueBox);
			addLabel("(");
		}
		if (mode == DragMode.REFLECT_Y){
			addLabel(" -  (");
		}

		valueBox = new CoeffARootBoxOriginal();
		valueBox.setFont(labelsFont);		
		addValueBox(valueBox);
		//By default the first coefficient is selected
		setSelected(valueBox);

		addLabel(" ( ");

		if (mode == DragMode.TRANSLATE_X){
			addLabel("(");
		}
		
		if (mode == DragMode.DILATE_X){
			addLabel("(");
			valueBox = new QuadDragFactorBox();
			valueBox.setFont(labelsFont);		
			addValueBox(valueBox);
			addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
			addLabel(")");
		}
		else if (mode == DragMode.REFLECT_X){
			addLabel("( - ");
			addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
			addLabel(")");
		}
		else{
			addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
		}

		if (mode == DragMode.TRANSLATE_X){
			addLabel("+");
			valueBox = new QuadDragFactorBox();
			valueBox.setFont(labelsFont);		
			addValueBox(valueBox);
			addLabel(")");
		}
		
		addLabel("  -  ");

		valueBox = new Root1BoxOriginal();
		valueBox.setFont(labelsFont);		
		addValueBox(valueBox);

		addLabel(")");

		addLabel("( ");

		if (mode == DragMode.TRANSLATE_X){
			addLabel("(");
		}
		
		if (mode == DragMode.DILATE_X){
			addLabel("(");
			valueBox = new QuadDragFactorBox();
			valueBox.setFont(labelsFont);		
			addValueBox(valueBox);
			addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
			addLabel(")");
		}
		else if (mode == DragMode.REFLECT_X){
			addLabel("( - ");
			addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
			addLabel(")");
		}
		else{
			addLabel(new JLabel(ResourceLoader.getImageIcon("x")));
		}

		if (mode == DragMode.TRANSLATE_X){
			addLabel("+");
			valueBox = new QuadDragFactorBox();
			valueBox.setFont(labelsFont);		
			addValueBox(valueBox);
			addLabel(")");
		}
		
		addLabel("  -  ");

		valueBox = new Root2BoxOriginal();
		valueBox.setFont(labelsFont);		
		addValueBox(valueBox);

		addLabel(")");
		
		if (mode == DragMode.TRANSLATE_Y){
			addLabel("    +  ");
			valueBox = new QuadDragFactorBox();
			valueBox.setFont(labelsFont);		
			addValueBox(valueBox);
		}

		if (mode == DragMode.DILATE_Y){
			addLabel(")");
		}
		if (mode == DragMode.REFLECT_Y){
			addLabel(")");
		}
    }

	/**
	 * Sets the source for all the values 
	*/
	public void setSourceBoxes(Object source)
	{
		if (this.subject == subject){
			super.setSourceBoxes(subject);
		}
	}
	
	public void updateSymbolic() 
	{
		if (mode != ((TransformedQuadraticFunction)subject).getDragMode().getValue()){
			setupSymbolic();
			//adjustFunctionName();
			setSourceBoxes(subject);
		}
		super.updateSymbolic();
	}

	public void adjustPreferredSize()
	{
		if (!editable){
			setPreferredSize(new Dimension(480,25));
		}
		else{
			setPreferredSize(new Dimension(480,40));
		}
	}	

	class CoeffARootBoxOriginal extends CoeffARootBox
	{
		public void setSource(Object source)
		{
			if (source instanceof TransformedQuadraticFunction){
				super.setSource(((TransformedQuadraticFunction)source).getOriginal());
			}
		}
	}

	class Root1BoxOriginal extends Root1Box
	{
		public void setSource(Object source)
		{
			if (source instanceof TransformedQuadraticFunction){
				super.setSource(((TransformedQuadraticFunction)source).getOriginal());
			}
		}
	}

	class Root2BoxOriginal extends Root2Box
	{
		public void setSource(Object source)
		{
			if (source instanceof TransformedQuadraticFunction){
				super.setSource(((TransformedQuadraticFunction)source).getOriginal());
			}
		}
	}
}
