/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


package org.concord.graphfunction.engine;

import org.concord.graphfunction.engine.LinearFunctionPushpin;

import junit.framework.*;

public class LinearFunctionPushpinTest extends TestCase {

	public static void main(String args[]) {
		junit.textui.TestRunner.run(LinearFunctionPushpinTest.class);
	}

// Tests setSlope and getSlope methods 

	public void testSetSlope() {
		LinearFunctionPushpin p_lfp = new LinearFunctionPushpin ();
		p_lfp.setSlope (200);
		assertEquals ((int)200, (int)p_lfp.getSlope());
	}

// Tests setPushpin and isPushpinOn methods

	public void testSetPushpin() {
		LinearFunctionPushpin p_lfp = new LinearFunctionPushpin ();
		p_lfp.setPushpin (true);
		assertEquals (true, p_lfp.isPushpinOn());
	}

// Tests setPushpinX and setPushpinY methods

	public void testSetPushpinXY() {
		LinearFunctionPushpin p_lfp = new LinearFunctionPushpin ();
		p_lfp.setPushpinX (5);
		p_lfp.setPushpinY (2);
		assertEquals (5, (int)p_lfp.getPushpinX());
		assertEquals (2, (int)p_lfp.getPushpinY());
	}
// Tests set/getYIntercept methods

	public void testSetYIntercept() {
		LinearFunctionPushpin p_lfp = new LinearFunctionPushpin ();
		p_lfp.setYIntercept (23);
		assertEquals (23, (int)p_lfp.getYIntercept());
	}

// Tests setPushpin and setSlope methods

	public void testSetSlopeWithPushpin() {
		LinearFunctionPushpin p_lfp = new LinearFunctionPushpin ();
		p_lfp.setSlope (1);// Sets slope without PushpinOn	
		p_lfp.setYIntercept (1);// Sets Y-Intercept
		//Line is now y=1x+1
		p_lfp.setPushpin (true); // Sets PushpinOn variable
		p_lfp.setPushpinX (-1);
		p_lfp.setPushpinY (0);
		//Pushpin is now set to (-1,0) on line
		//Now change slope, with rotation around pushpin
		p_lfp.setSlope (2);
		//Line should be now have a Y intercept at (0,2)
		assertEquals (2, (int)p_lfp.getYIntercept());
		//Now change slope without rotation around pushpin:
		p_lfp.setPushpin (false);
		p_lfp.setSlope (3);
		//Slope is now 3, and line is y=3x+2
		assertEquals (3, (int)p_lfp.getSlope());
		assertEquals (2, (int)p_lfp.getYIntercept());
	}

// Tests setPushpin and setSlope methods

	public void testSetSlopeWithPushpin2() {
		LinearFunctionPushpin p_lfp = new LinearFunctionPushpin ();
		p_lfp.setSlope (1);// Sets slope without PushpinOn	
		p_lfp.setYIntercept (1);// Sets Y-Intercept
		//Line is now y=1x+1
//		p_lfp.setPushpin (true); // Sets PushpinOn variable
//		p_lfp.setPushpinX (-1);
//		p_lfp.setPushpinY (0);
		//Pushpin is now set to (-1,-1) on line
		//Now change slope to small decimal, with rotation around pushpin
//		p_lfp.setSlope (2);
		//Line should be now have a Y intercept at (0,2)
//		assertEquals (2, (int)p_lfp.getYIntercept());
		//Now change slope without rotation around pushpin:
		p_lfp.setPushpin (false);
		p_lfp.setSlope (3.001);
		//Slope is now 3 or 3.001?
		assertFalse ((double)3 == (double)p_lfp.getSlope());
		assertEquals ((double)3.001, (double)p_lfp.getSlope(), 0.0001);
		assertEquals (1, (int)p_lfp.getYIntercept());
	}


// Purposeful failure for demonstration

//	public void testSetSlopeFail() {
//		LinearFunctionPushpin p_lfp = new LinearFunctionPushpin ();
//		p_lfp.setSlope (200);
//		assertEquals ((int)201, (int)p_lfp.getSlope());
//	}	
}
