/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


package org.concord.graphfunction.ui;

import org.concord.graphfunction.ui.*;
import org.concord.graphfunction.engine.Value;

import junit.framework.*;

public class ValueSliderTest extends TestCase {
                                                                                
        public static void main(String args[]) {
                junit.textui.TestRunner.run(ValueSliderTest.class);
        }
                                                                                
        public void testValueSlider() {
            //Default scale = 1 & offset = 0
            ValueSlider p_vs = new ValueSlider ( new Value (10.0));
            assertEquals(100, p_vs.getMaximum());
            assertEquals ((double)1, (double)p_vs.getScale(), 0.000001);
            //offset = 0, scale = 1, min = 0, max = 100
            p_vs.setValue (35.0);
            assertEquals (35, p_vs.getValue());

            
        }
    
        public void testSetWorldMinimum() {
            //Default scale = 1 & offset = 0
            ValueSlider p_vs = new ValueSlider ( new Value (10.0));
            assertEquals(100, p_vs.getMaximum());
            assertEquals(0, p_vs.getMinimum());
            //offset = 0, scale = 1, min = 0, max = 100
            p_vs.setWorldMinimum (19.34);
            //offset should = 19.34, scale is 19.34->100 = 80.66
            assertEquals (0.8066, p_vs.getScale(), 0.0001);
            assertEquals (19.34, p_vs.getOffset(), 0.0001);
            //set Value to be 63.98
            p_vs.setValue (63.98);

            //getValue should return (int) (63.98 - 19.34) / 0.8066 = 55.3434...
            //rounds to 55?            
            assertEquals (55, p_vs.getValue());   
        
            //set Value to be 64.48
            p_vs.setValue (64.48);
            //getValue should return (int) (63.98 - 19.34) / 0.8066 = 55.9633...
            //rounds to 56?            
            assertEquals (56, p_vs.getValue());   
        }
        public void testSetWorldMaximum() {
            //Default scale = 1 & offset = 0
            ValueSlider p_vs = new ValueSlider ( new Value (10.0));
            assertEquals(100, p_vs.getMaximum());
            assertEquals(0, p_vs.getMinimum());
            //offset = 0, scale = 1, min = 0, max = 100
            p_vs.setWorldMaximum (405.34);
            //offset should = 0, scale is 0->405.34 = 4.057
            assertEquals (4.0534, p_vs.getScale(), 0.0001);
            assertEquals (0.0, p_vs.getOffset(), 0.0001);
            //set Value to be 63.98
            p_vs.setValue (63.98);

            //getValue should return (int) (63.98 - 0) / 4.057 = 15.7703...
            //rounds to 16?            
            assertEquals (16, p_vs.getValue());   
        }
        public void testSetWorldRange() {
            //Default scale = 1 & offset = 0
            ValueSlider p_vs = new ValueSlider ( new Value (10.0));
            assertEquals(100, p_vs.getMaximum());
            assertEquals(0, p_vs.getMinimum());
            //offset = 0, scale = 1, min = 0, max = 100
            //what if max is smaller than min?
            p_vs.setWorldRange (100, 50);
            //offset should = 100, scale is 50->100 = -0.5
            assertEquals (-0.5, p_vs.getScale(), 0.0001);
            assertEquals (100.0, p_vs.getOffset(), 0.0001);
            p_vs.setValue (80);

            //getValue should return (int) (80 - 100) / -0.5 = 40
            assertEquals ("this shouldn't fail", 40, p_vs.getValue());   
        }
}
