/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
 * $Revision: 1.5 $
 */
package org.concord.rttapplets.datafitter;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;

import org.concord.graph.engine.AxisScale;
import org.concord.graphfunction.ui.ResourceLoader;

/**
 * @author ebrownmunoz
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class DataControls extends JPanel 
implements ActionListener {
	
	DataPointManager manager;
	AxisScale axisscale;
	
	
	JToggleButton select; // Default mode. Select object when clicked
	JToggleButton addPoint; // Add a new point
	JToggleButton deletePoint; // Delete a point
	JToggleButton move; // Grab and drag to move the axis
	JToggleButton stretchX; // Stretch the Axis in the X direction
	JToggleButton stretchY; // Stretch the Axis in the Y direction
	private String lastCommand;
	
	public DataControls() {
		
		
		ButtonGroup group = new ButtonGroup();
		
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		select = new DButton("selectionarrow");
		select.setSelected(true);
		select.addActionListener(this);
		select.setActionCommand("disable");
		select.setToolTipText("Drag Objects");
		group.add(select);
		add(select);
		
		addPoint = new DButton("newdatapoint");
		addPoint.setActionCommand("add");
		addPoint.setToolTipText("Add Data Points");
		addPoint.addActionListener(this);
		group.add(addPoint);
		add(addPoint);
		
		deletePoint = new DButton("erasercursor");
		deletePoint.setActionCommand("delete");
		deletePoint.setToolTipText("Delete Data Points");
		deletePoint.addActionListener(this);
		group.add(deletePoint);
		add(deletePoint);
		
		move = new DButton("hand");
		move.setActionCommand("moveaxis");
		move.setToolTipText("Move Axis");
		move.addActionListener(this);
		group.add(move);
		add(move);
		
		stretchX = new DButton("hand_leftright");
		stretchX.setActionCommand("stretchx");
		stretchX.setToolTipText("Stretch X Axis");
		stretchX.addActionListener(this);
		group.add(stretchX);
		add(stretchX);
		
		stretchY = new DButton("hand_topbottom");
		stretchY.setActionCommand("stretchy");
		stretchY.setToolTipText("Stretch Y Axis");
		stretchY.addActionListener(this);
		group.add(stretchY);
		add(stretchY);
	}
	
	public DataControls(DataPointManager manager) {
		this();
		this.manager = manager;
	}
	
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		
		if ( command.equals(lastCommand)) {
			lastCommand = null;
			manager.setMode(DataPointManager.DISABLED);
			axisscale.setDragMode(AxisScale.DRAGMODE_NONE);
			select.setSelected(true);
			
		} else {
			
			if (manager != null)
				manager.setMode(DataPointManager.DISABLED);
			
			if (axisscale != null)
				axisscale.setDragMode(AxisScale.DRAGMODE_NONE);
			
			if (command.equals("disable")) {
				if (manager != null){
					manager.setMode(DataPointManager.DISABLED);
				}
			} else if (command.equals("add")) {
				if (manager != null) {
					manager.setMode(DataPointManager.ADD_POINT);
				}
			} else if (command.equals("delete")) {
				if (manager != null) {
					manager.setMode(DataPointManager.DELETE_POINT);
				}
			} else if (command.equals("moveaxis")) {
				if (axisscale != null) {
					axisscale.setDragMode(AxisScale.DRAGMODE_TRANSLATE);
				}
			} else if (command.equals("stretchx")) {
				if (axisscale != null) {
					axisscale.setDragMode(AxisScale.DRAGMODE_DILATE_X);
				}
			} else if (command.equals("stretchy")) {
				if (axisscale != null) {
					axisscale.setDragMode(AxisScale.DRAGMODE_DILATE_Y);
				}
			} // if command.equals(" ...
			lastCommand = command;
		} // if command.equals(lastCommand...

	}
	
	class DButton extends JToggleButton {
		public DButton(String imageName) {
			ImageIcon icon;
			setPreferredSize(new Dimension(40,40));
			setMargin(new Insets(0,0,0,0));
			
			icon = ResourceLoader.getImageIcon(imageName);
			setIcon(icon);
			
		}
	}
	/**
	 * @return Returns the manager.
	 */
	public DataPointManager getManager() {
		return manager;
	}
	/**
	 * @param manager The manager to set.
	 */
	public void setManager(DataPointManager manager) {
		this.manager = manager;
	}
	
	/**
	 * @param scaler
	 */
	public void setAxisScale(AxisScale scaler) {
		axisscale = scaler;
		
	}
}