/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
 $Revision: 1.14 $
 */

package org.concord.rttapplets.datafitter;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;


import javax.swing.*;

import org.concord.graph.engine.*;
import org.concord.graph.ui.*;
import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.symbolicview.*;
import org.concord.graphfunction.event.*;


public class DataFitPanel extends JPanel 
implements DataSetListener,
ActionListener

{
	LinearFunctionPushpin curve = new LinearFunctionPushpin();
	FitGraphable bestfit;
	DataPointManager manager;
	ErrorLines errors;
	
	GraphWindow graph;
	SimpleGrid grid;
	
	DataSet data;
	DataSetTable table;
	JSplitPane splitPane;
	
	FitMetric fitValue;
	
	public DataFitPanel () 
	{
		JPanel panel = new JPanel();
		GraphArea graphArea;
		CoordinateSystem coords;
		JTabbedPane inputPane = new JTabbedPane ();
		
		graph = new GraphWindow();
		panel.add(graph);
		
		grid = new SimpleGrid();
		grid.setGraphArea(graph.getDefaultGraphArea());
		graph.add(grid);
		
		graphArea = graph.getDefaultGraphArea();
		coords = graphArea.getCoordinateSystem();
		
		
		graphArea.setOriginCentered(false);
		coords.setScale(new Point2D.Double(14.0,14.0));
		
		data = new DataSet();	
		data.addPoint(new DataPoint(1.0,24.4));
		data.addPoint(new DataPoint(2.0,23.7));
		data.addPoint(new DataPoint(3.0,23.4));
		data.addPoint(new DataPoint(4.0,24.0));
		data.addPoint(new DataPoint(5.0,23.0));
		data.addPoint(new DataPoint(6.0,22.5));
		data.addPoint(new DataPoint(7.0,22.4));
		data.addPoint(new DataPoint(8.0,22.37));
		data.addPoint(new DataPoint(9.0,22.03));
		
		data.addDataSetListener(this);
		
		graph.setAntialias(true);
		
		
		
		curve.setPushpin(true);
		curve.setPushpinFixed(false);
		curve.setSelectedLineStroke(curve.getLineStroke());

		fitValue = new FitMetric(data,curve);

		// Error Lines
		errors = new ErrorLines(data, curve, fitValue);
		errors.setVisible(false);
		
		bestfit = new FitGraphable(data, fitValue);
		bestfit.setVisible(false);
		bestfit.setColor(Color.gray);
		
		graph.add(errors);
		graph.add(bestfit);
		graph.add(data);
		

		
		JPanel eastPanel = new JPanel();
		eastPanel.setLayout(new BoxLayout(eastPanel,BoxLayout.Y_AXIS));
		
		JPanel eastControls = new JPanel(new GridLayout(4,1));
				
		// Options
		eastControls.add(new JLabel("Fit Type:"));
		String fitStrings [] = {"Least Squares (Vertical)", "Least Squares (Horizontal)"};
		JComboBox fitTypes = new JComboBox(fitStrings);
		fitTypes.addActionListener(this);
		eastControls.add(fitTypes);
		
		JCheckBox bestButton = new JCheckBox ("Show Best Fit");
		bestButton.addActionListener(this);
		bestButton.setActionCommand("bestfit");
		bestButton.setBorder(BorderFactory.createLineBorder(Color.red));
		
		eastControls.add(bestButton);
		
		JCheckBox errorLinesButton = new JCheckBox("Show Error Lines");
		errorLinesButton.addActionListener(this);
		errorLinesButton.setActionCommand("errorlines");
		
		eastControls.add(errorLinesButton);
				
		eastPanel.add(eastControls);
		
		final Gauge myGauge = new Gauge();
		myGauge.setValue(fitValue);
		myGauge.setPreferredSize(new Dimension(300,300));
		myGauge.setHalf(20.0);
		
		final JPanel gaugeControls = new JPanel(); 
		
		final JLabel gaugeLabel = 
			new JLabel (Double.toString(fitValue.getValue(3)));
		
		fitValue.addValueListener ( new ValueListener () {
			public void valueChanged(ValueEvent e) {
				gaugeLabel.setText(Double.toString(fitValue.getValue(3)));
			}
		});
		
		gaugeControls.add(gaugeLabel);
		
		final JButton gaugeReset = new JButton("Reset");
		gaugeReset.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				myGauge.reset();
			}
		});
		
		
		
		gaugeControls.add(gaugeReset);
		
		eastPanel.add(myGauge);
		eastPanel.add(gaugeControls);
		
		table = new DataSetTable(data);
		eastPanel.add(table);
		
		graph.add(curve);
		
		// Axis Scale
		AxisScale scaler = new AxisScale();
		scaler.setGraphArea(graphArea);
		graph.add(scaler);
		
		// Manager
		manager = new DataPointManager(data, graphArea);
		
		graph.add(manager);
		
		
		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				graph, eastPanel);
		splitPane.setDividerLocation(400);
		
		setLayout(new BorderLayout());
		add(splitPane,BorderLayout.CENTER);
		
		// SouthPanel
		
		JPanel southPanel = new JPanel(new BorderLayout());
		
		DataControls controls = new DataControls(manager);
		controls.setAxisScale(scaler);
		southPanel.add(controls,BorderLayout.WEST);
		
		LinearPointSlopeInput symbolic = new LinearPointSlopeInput();
		symbolic.setGraphFunction(curve);
		
		LinearSlopeInterceptInput siSymbolic = new LinearSlopeInterceptInput();
		siSymbolic.setGraphFunction(curve);
		
		inputPane.addTab("Point-Slope", symbolic);
		inputPane.addTab("Slope-Intercept", siSymbolic);
		
		southPanel.add(inputPane,BorderLayout.CENTER);
		
		add(southPanel,BorderLayout.SOUTH);
		
		graph.addComponentListener(new ComponentAdapter() {
			public void componentResized (ComponentEvent e) {
				double height = (double)graph.getHeight();
				CoordinateSystem cs = 
					graph.getDefaultGraphArea().getCoordinateSystem();
				cs.setOriginOffsetDisplay(new 
						Point2D.Double(20.0, 
								height-20.0));
			}
		});
		
	}
	
	public static void main(String args[]) {
		
		final JFrame frame = new JFrame();
		final JPanel fa = new DataFitPanel();
		frame.getContentPane().add(fa);
		frame.setSize(600,600);
		frame.show();
		
		frame.addWindowListener( new WindowAdapter() {
			public void windowClosing(WindowEvent e){
				
				System.exit(0);
			}			
		});
	}			
	
	public void dataSetChanged (DataSetEvent e) {
		graph.repaint();
	}
	
	
	public void actionPerformed(ActionEvent e) {
		Object eventSource = e.getSource();
		
		if (eventSource instanceof JToggleButton){
			String command = e.getActionCommand();
			if (command.equals("bestfit")) {
				JToggleButton source = (JToggleButton)e.getSource();
				if (source.isSelected()) {
					bestfit.setVisible(true);
				} else {
					bestfit.setVisible(false);
				}	
				graph.repaint();
			} else if (command.equals("errorlines")) {
				JToggleButton source = (JToggleButton)e.getSource();
				if (source.isSelected()) {
					errors.setVisible(true);
				} else {
					errors.setVisible(false);
				}
			}
		} else if (eventSource instanceof JComboBox) {
			JComboBox source = (JComboBox)eventSource;
			int index = source.getSelectedIndex();
			
			if (index == 0)
				fitValue.setFitType(FitMetric.LEAST_SQUARES);
			else if (index == 1)
				fitValue.setFitType(FitMetric.LEAST_SQUARES_HORIZONTAL);
			
			// data.noti
		}
	}
	
	
	
}
