/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.10 $
 * $Date: 2004/09/07 15:49:34 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.rttapplets.datafitter;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.*;

import org.concord.graph.engine.CoordinateSystem;
import org.concord.graph.engine.DefaultSelectable;
import org.concord.graph.engine.Graphable;
import org.concord.graph.engine.MouseControllable;
import org.concord.graph.engine.Selectable;

/**
 * Represents a Graphable 2 Dimensional Data Point. This point contains
 * X and Y values.
 *
 * Date created: 
 *
 * @author Eric Brown-Munoz<p>
 *
 */
 
public class DataPoint 
	extends DefaultSelectable
	implements Selectable, 
			   Graphable,
			   MouseControllable
{
	Point2D.Double point;
	boolean isOn = true;
	DataSet parent;
	private Color onColor = Color.red;
	private Color offColor = Color.gray;
	private Point transformedPoint = 
		new Point();
	private Point2D.Double transformedDPoint =
		new Point2D.Double();
	private static final int XPOINTSIZE = 11;
	private static final int YPOINTSIZE = 11;
	private double roundCoeff = .1;

	public DataPoint(double x, double y) 
	{
		super();
		point = new Point2D.Double();
		setX(x);
		setY(y);
	}

	/** Returns a copy of itself as defined by the
	 * Graphable interface.
	 */
	public Graphable getCopy() {
		DataPoint retval = new DataPoint(point.x, point.y);
		return retval;
	}
	
	/**
	 * @return Returns the point.
	 */
	public Point2D.Double getPoint() {
		return point;
	}
	
	/**
	 * @param point The point to set.
	 */
	public void setPoint(Point2D.Double point) {
		this.point = point;
	}
	
	/** Draw a DataPoint on the g. This uses the GraphArea
	 * Set in DefaultGraphaphable.
	 */
	public void draw(Graphics2D g) {
		CoordinateSystem cs = getGraphArea().getCoordinateSystem();
		Color savedColor = g.getColor();
		
		g.setColor(Color.white);
		
		if (isOn) {
			g.setColor(onColor);
		} else { 
			g.setColor(offColor);
		}

		cs.transformToDisplay(point,transformedPoint);
		g.fillOval(transformedPoint.x - XPOINTSIZE/2 -1, 
				   transformedPoint.y - YPOINTSIZE/2 -1,
				   XPOINTSIZE, YPOINTSIZE);
		g.setColor(Color.black);
		g.drawOval(transformedPoint.x - XPOINTSIZE/2 -1, 
				   transformedPoint.y - YPOINTSIZE/2 -1,
				   XPOINTSIZE, YPOINTSIZE);
		g.fillOval(transformedPoint.x,
				   transformedPoint.y,
				   1, 1);

		g.setColor(savedColor);
		
	}
	
	/** Return true if Point p is near the DataPoint */
	public boolean isPointInProximity(Point p) {
		boolean retval = false;
		CoordinateSystem cs = getGraphArea().getCoordinateSystem();
		
		cs.transformToDisplay(point, transformedPoint);

		if (transformedPoint.distance(p) < 7.0)
			retval = true;
		
		return retval;
	}

	/**
	 * @param newX
	 */
	public void setX(double newX) {
		double x = Math.floor(newX / roundCoeff + 0.5) * roundCoeff;
		point.x = x;
		notifyChange();
	}
	
	public void setY(double newY) {
		double y = Math.floor(newY / roundCoeff + 0.5) * roundCoeff;
		point.y = y;
		notifyChange();
	}
	
	public double getX() {
		return point.x;
	}

	public double getY() {
		return point.y;
	}

	/**
	 * @return Returns the parent.
	 */
	public DataSet getParent() {
		return parent;
	}
	/**
	 * @param parent The parent to set.
	 */
	public void setParent(DataSet parent) {
		this.parent = parent;
	}
	/* (non-Javadoc)
	 * @see org.concord.graph.engine.MouseControllable#mousePressed(java.awt.Point)
	 */
	public boolean mousePressed(Point p) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.concord.graph.engine.MouseControllable#mouseDragged(java.awt.Point)
	 */
	public boolean mouseDragged(Point p) {
		CoordinateSystem cs = getGraphArea().getCoordinateSystem();

		cs.transformToWorld(p,transformedDPoint);
		setX(transformedDPoint.x);
		setY(transformedDPoint.y);
		notifyChange();
		
		return true;
	}

	/* (non-Javadoc)
	 * @see org.concord.graph.engine.MouseControllable#mouseReleased(java.awt.Point)
	 */
	public boolean mouseReleased(Point p) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.concord.graph.engine.MouseControllable#isMouseControlled()
	 */
	public boolean isMouseControlled() {
		// TODO Auto-generated method stub
		return false;
	}
	
	
}
