/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
 * $Revision: 1.2 $
 */
package org.concord.rttapplets.datafitter;

import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.*;

import org.concord.graph.engine.AxisScale;
import org.concord.graph.engine.CoordinateSystem;
import org.concord.graph.engine.DefaultGraphable;
import org.concord.graph.engine.DefaultSelectable;
import org.concord.graph.engine.GraphArea;
import org.concord.graph.engine.Graphable;
import org.concord.graph.engine.MouseControllable;
import org.concord.graph.ui.GraphWindow;

/** Add DataPointManager to GraphWindow to give users the
 * ability to add and delete datapoints. DataPointManager
 * must be at the top of the Vector so that it gets
 * mouseclicks first (which selected).
 * 
 * @author ebrownmunoz
 *
 */

public class DataPointManager extends DefaultSelectable implements
		MouseControllable {

	public static int DISABLED = 0;
	public static int ADD_POINT = 1;
	public static int DELETE_POINT = 2;
	
	
	int mode = DISABLED;
	GraphArea ga;
	DataSet dataset;
	
	/**Constructs and Initializes a DataPointManager
	 * 
	 */
	public DataPointManager() {
		super();
	}

	public DataPointManager(DataSet dataset, GraphArea ga) {
		this();
		this.ga = ga;
		this.dataset = dataset;
	}
	
	/**
	 * @return Returns the dataset.
	 */
	public DataSet getDataset() {
		return dataset;
	}

	/**
	 * @param dataset The dataset to set.
	 */
	public void setDataset(DataSet dataset) {
		this.dataset = dataset;
	}
	
	public boolean mousePressed(Point p) {
			
		boolean retval = false;
		
		if (mode == DELETE_POINT ) {
			if (dataset != null) {
				dataset.removePoint(p);
			}
			retval = true;
		} else if (mode == ADD_POINT ) {
			if (dataset != null) {
				CoordinateSystem cs = getGraphArea().getCoordinateSystem();
				Point2D.Double tpoint = new Point2D.Double(); // This is not expensive here
				cs.transformToWorld(p, tpoint);
				
				dataset.addPoint(tpoint.x, tpoint.y);
			}
			retval = true;
		}
		return retval;
	}

	/* (non-Javadoc)
	 * @see org.concord.graph.engine.MouseControllable#mouseDragged(java.awt.Point)
	 */
	public boolean mouseDragged(Point p) {
		// TODO Auto-generated method stub
		return true;
	}

	/* (non-Javadoc)
	 * @see org.concord.graph.engine.MouseControllable#mouseReleased(java.awt.Point)
	 */
	public boolean mouseReleased(Point p) {
		// TODO Auto-generated method stub
		return true;
	}

	
	/**
	 * @return Returns the mode.
	 */
	public int getMode() {
		return mode;
	}
	/**
	 * @param mode The mode to set.
	 */
	public void setMode(int mode) {
		this.mode = mode;
	}
	
	public boolean isMouseControlled() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.concord.graph.engine.MouseSensitive#isPointInProximity(java.awt.Point)
	 */
	public boolean isPointInProximity(Point p) {
		if (mode == DISABLED)
			return false;
		else 
			return true; // All Points are in proximity
	}

	/* (non-Javadoc)
	 * @see org.concord.graph.engine.Graphable#getCopy()
	 */
	public Graphable getCopy() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.concord.graph.engine.Drawable#draw(java.awt.Graphics2D)
	 */
	public void draw(Graphics2D g) {
		// TODO Auto-generated method stub

	}

}
