/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.8 $
 * $Date: 2004/09/07 15:50:19 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.rttapplets.datafitter;

import java.awt.*;
import java.util.*;
import org.concord.graph.engine.*;
import org.concord.graph.event.*;
import org.concord.graphfunction.engine.*;


/** 
 * A set of data that can be displayed with GraphWindow. 
 *
 * Date created: 
 *
 * @author Eric Brown-Munoz<p>
 *	
 */

public class DataSet 
	extends SelectableList
{
	private static int XPOINTSIZE = 11;
	private static int YPOINTSIZE = 11;

	private static int HORIZONTAL = 0;
	private static int VERTICAL = 1;
	
	private Vector listeners = new Vector();

	private Color onColor  = Color.red; // Color for "on points"
	private Color offColor = Color.lightGray; // Color for "off points"


	public DataPoint addPoint(DataPoint newPoint) {
		int newRow = size(); // inherited from Vector
		newPoint.parent = this;
		add(newPoint);
		notifyListeners(new DataSetEvent(this,newRow,DataSetEvent.INSERT));
		return newPoint;
	}

	public DataPoint addPoint (double x, double y) {
		return addPoint(new DataPoint(x,y));
	}


	public void removePoint (int row) {
		remove(row);
		notifyListeners(new DataSetEvent(this,row,DataSetEvent.DELETE));
	}

	/** Delete the datapoint nearest to this screen location. 
	*/
	public void removePoint (Point location) {
		int row;
		DataPoint found = findPoint(location);
		if (found != null) {
			row = indexOf(found);
			removePoint(row);
		}
	}

	public int getSize() {
		return size();
	}

	public void addDataSetListener (DataSetListener list) {

		if (! listeners.contains(list)) {
			listeners.add(list);
		}
	}

	public DataPoint findPoint(Point location) {
		DataPoint retval = null;
		
		Enumeration selectables = elements();
		while (selectables.hasMoreElements()) {
			Object each = selectables.nextElement();
			if ( (each instanceof DataPoint) &&
					((DataPoint)each).isPointInProximity(location)) {
				System.out.println("DateSet: found point " + location);
				retval = (DataPoint)each;
				break;
			}
		}
		return retval;
	}
	
	/** Return the index of the given point in the DataSet.
	 * 
	 * @param point
	 * @return
	 */
	public int findIndex (DataPoint point) {
		return indexOf(point);
	}

	public void removeDataSetListener (DataSetListener list) {

		if (listeners.contains(list)) {
			listeners.remove(list);
		}
	}

	public void notifyListeners( DataSetEvent e) {
		Enumeration dsenum = listeners.elements();

		while (dsenum.hasMoreElements()) {
			DataSetListener eachlist = (DataSetListener)dsenum.nextElement();
			eachlist.dataSetChanged(e);
		}
	}

	public void setOn (int row, boolean isOn) {
		DataPoint point = (DataPoint)elementAt(row);
		point.isOn = isOn;
		notifyListeners(new DataSetEvent(this,row,row,DataSetEvent.UPDATE));
	}

	public boolean getOn (int row) {
		DataPoint point = (DataPoint)elementAt(row);
		return point.isOn;
	}

	public void setX(int row, double newX) {
		DataPoint point = (DataPoint)elementAt(row);
		point.setX(newX);
		notifyListeners(new DataSetEvent(this,row,row,DataSetEvent.UPDATE));
	}

	public void setY(int row, double newY) {
		DataPoint point = (DataPoint)elementAt(row);
		point.setY(newY);
		notifyListeners(new DataSetEvent(this,row,row,DataSetEvent.UPDATE));

	}

	/** Calculates the best fit using least-squares regression. Puts
		the answer in fitline and returns fitline. */

	public Enumeration listPoints () {
		return elements();
	}

	public DataPoint getPoint (int index) {
		if (index < size()) {
			return (DataPoint)elementAt(index);
		} else {
			return null;
		}
	}

	public void readFile ( String filename ) {
		//See Erics file in Ephemera

	}

	/**
	 * NOt implemented yet
	 */
	public Graphable getCopy()
	{
		return null;
	}

	/*
	 * Adds a GraphableListener to listen for events 
	 */
	public void addGraphableListener(GraphableListener gl)
	{}
	
	/*
	 * Removes a GraphableListener.
	 */
	public void removeGraphableListener(GraphableListener gl)
	{}
	
	/** 
	 * Forces classes to redraw any internal image buffer (if implemented). 
	 * The axis is provided to transform coordinates.
	 * This makes sure the background is updated if there is a fundamental 
	 * change that is not checked by this class.
	 */
	public void update() {}


	/** 
	 * Removes the object from any enclosing view.
	 * This also fires an objectRemoved(obj) event.
	 */
	public void remove()
	{}
	
	/** 
	 * Sets whether it is visible in a graph.
	 * This also fires an objectChanged(obj) event.
	 */
	public void setVisible(boolean visible)
	{}
	
	/** 
	 * Returns true if it is visible in a graph 
	 */
	public boolean isVisible()
	{
		return true;
	}


}
    
