/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.4 $
 * $Date: 2004/09/07 15:51:22 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.rttapplets.datafitter;

import java.awt.*;
import java.awt.event.*;
import java.util.EventObject;

import javax.swing.*;

import org.concord.graph.engine.SelectableList;
import org.concord.graph.event.SelectableListListener;


/**
 * Class name and description
 *
 * Date created: 
 *
 * @author Eric Brown-Munoz<p>
 *
 */

public class DataSetTable extends JPanel 
	implements ActionListener, SelectableListListener
{
	JTable table;
	JScrollPane scrollpane;
	DataSet dataSet;

	public DataSetTable (DataSet dataSet) {
		this.dataSet = dataSet;
		dataSet.addGraphableListListener(this);

		setLayout(new BorderLayout());

		// Set up the Swing table in a Scrollpane
		DataSetTableModel model = new DataSetTableModel(dataSet);
		table = new JTable (model);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		DefaultCellEditor editor = (DefaultCellEditor)table.getDefaultEditor(Double.class);
		editor.setClickCountToStart(0);

		scrollpane = new JScrollPane(table);

		add(scrollpane, BorderLayout.CENTER);

		// Buttons under the table
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new GridLayout(2,1));

		JButton newButton = new JButton("New Point");
		newButton.addActionListener(this);
		newButton.setActionCommand("new");
		buttonPanel.add(newButton);
	  
		JButton deleteButton = new JButton("Delete Point");
		deleteButton.addActionListener(this);
		deleteButton.setActionCommand("delete");
		buttonPanel.add(deleteButton);
		
		add(buttonPanel,BorderLayout.SOUTH);
	}

	public void actionPerformed (ActionEvent e) {
		String command = e.getActionCommand();

		if (command.equals("new")) {
			int newrow = dataSet.getSize();
			dataSet.addPoint(new DataPoint(0,0));
			table.setRowSelectionInterval(newrow, newrow);
			table.editCellAt(newrow, 2);


		}

		if (command.equals("delete")) {
			int row = table.getSelectedRow();
			if (row >= 0) {
				dataSet.removePoint(table.getSelectedRow());
			}
		}
	}

	public static void main(String args[]) {

		DataSet foo = new DataSet();

		// Test Data - Nothing more
		foo.addPoint (new DataPoint(3,6));
		foo.addPoint (new DataPoint(5,6));
		foo.addPoint (new DataPoint(7,6));
		foo.addPoint (new DataPoint(7,8));
		foo.addPoint (new DataPoint(9,6));
		foo.addPoint (new DataPoint(11,6));
		foo.addPoint (new DataPoint(1,6));

		final JFrame frame = new JFrame();
		final JPanel fa = new DataSetTable(foo);
		frame.getContentPane().add(fa);
		frame.setSize(600,600);
		frame.show();
		
	}

	/* (non-Javadoc)
	 * @see org.concord.graph.event.SelectableListListener#listGraphableSelected(java.util.EventObject)
	 */
	public void listGraphableSelected(EventObject e) {
		DataPoint dp = (DataPoint)e.getSource();
		int newrow = dp.getParent().getFirstSelectedIndex();
		table.setRowSelectionInterval(newrow, newrow);
		
	}

	/* (non-Javadoc)
	 * @see org.concord.graph.event.SelectableListListener#listGraphableDeselected(java.util.EventObject)
	 */
	public void listGraphableDeselected(EventObject e) {
		DataPoint dp = (DataPoint)e.getSource();
		int newrow = dp.getParent().getFirstSelectedIndex();
		if (newrow > 0)
			table.setRowSelectionInterval(newrow, newrow);
		else
			table.clearSelection();
	}

	/* (non-Javadoc)
	 * @see org.concord.graph.event.GraphableListListener#listGraphableAdded(java.util.EventObject)
	 */
	public void listGraphableAdded(EventObject e) {
		DataPoint dp = (DataPoint)e.getSource();
		int newrow = dp.getParent().getFirstSelectedIndex();
		if (newrow > 0)
			table.setRowSelectionInterval(newrow, newrow);
		else
			table.clearSelection();
	}

	/* (non-Javadoc)
	 * @see org.concord.graph.event.GraphableListListener#listGraphableRemoved(java.util.EventObject)
	 */
	public void listGraphableRemoved(EventObject e) {
		table.clearSelection();
	}

	/* (non-Javadoc)
	 * @see org.concord.graph.event.GraphableListListener#listGraphableChanged(java.util.EventObject)
	 */
	public void listGraphableChanged(EventObject e) {
		// TODO Auto-generated method stub
		
	}			

}
