/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.5 $
 * $Date: 2004/08/26 22:03:36 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.rttapplets.datafitter;


import java.util.*;
import javax.swing.table.*;
import javax.swing.event.*;
import org.concord.graph.event.GraphableListListener;


/**
 * Class name and description
 *
 * Date created: 
 *
 * @author Eric Brown-Munoz<p>
 *
 */

public class DataSetTableModel 
	implements TableModel 
{
	static int COLUMN_COUNT = 4;

	Vector listeners = new Vector();;
	String columnNames [] = {"On/Off","#","X","Y"};
	Class columnClasses [] = { Boolean.class, Integer.class,
							   Double.class, Double.class};
	DataSet data;

	public DataSetTableModel (DataSet dataSet) {

		this.data = dataSet;
		dataSet.addGraphableListListener ( new GraphableListListener () {
				public void listGraphableAdded(EventObject de) {
					TableModelEvent me = null;
					int row = data.indexOf(de.getSource());

					me = new TableModelEvent(DataSetTableModel.this,
											 row,
											 row,
											 TableModelEvent.ALL_COLUMNS,
											 TableModelEvent.INSERT);
					notifyListeners(me);
				}
				
				public void listGraphableChanged(EventObject de) {
				
					TableModelEvent me = null;
					int row = data.indexOf(de.getSource());
					int column = TableModelEvent.ALL_COLUMNS;
						 
					me = new TableModelEvent(DataSetTableModel.this,
											 row,
											 row,
											 column,
											 TableModelEvent.UPDATE );
					notifyListeners(me);
				}
				
			    public void listGraphableRemoved(EventObject de) {
			    	TableModelEvent me=null;
			    	int row = data.indexOf(de.getSource());
			    	
			    	me = new TableModelEvent(DataSetTableModel.this,
										     row,
											 row,
											 TableModelEvent.ALL_COLUMNS,
											 TableModelEvent.DELETE);
			    	notifyListeners(me);
			    }
			 

			});
	}

	public void addTableModelListener (TableModelListener l) {
		if (! listeners.contains(l)) {
			listeners.add(l);
		}

	}

	void notifyListeners (TableModelEvent e ) {
		Enumeration listenum = listeners.elements();

		while (listenum.hasMoreElements()) {
			TableModelListener tmlist = 
				(TableModelListener)listenum.nextElement();
			tmlist.tableChanged(e);
		}
	}

	public Class getColumnClass(int columnIndex) {
		return columnClasses[columnIndex];
	}

	public String getColumnName (int columnIndex) {
		return columnNames[columnIndex];
	}

	public void removeTableModelListener (TableModelListener l) {
		listeners.remove(l);
	}

	public int getColumnCount () {
		return COLUMN_COUNT;
	}

	public int getRowCount () {
		if (data != null) {
			return data.getSize();
		} else {
			return 0;
		}
	
	}

	public Object getValueAt(int rowIndex, int columnIndex) {

		Object retval = null;

		if (data != null) {
			DataPoint row = data.getPoint(rowIndex);
			switch (columnIndex) {
			case 0: 
				retval = new Boolean(data.getOn(rowIndex));
				break;
			case 1:
				retval =  new Integer(rowIndex + 1);
				break;
			case 2:
				retval =  new Double(row.getX());
				break;
			case 3:
				retval = new Double(row.getY());
				break;
			}

		} else {
			return null;
		}
		return retval;
	}

	public void setValueAt(Object value, int rowIndex, int columnIndex) {

		if (data != null) {
			switch (columnIndex) {
			case 0: 
				data.setOn(rowIndex,((Boolean)value).booleanValue());
				break;
			case 1:
				break;
			case 2:
				data.setX(rowIndex,((Double)value).doubleValue());
				break;
			case 3:
				data.setY(rowIndex,((Double)value).doubleValue());
				break;
			}

		} 
	}

	public boolean isCellEditable (int rowIndex, int columnIndex) {
		if (columnIndex == 1) {
			return false;
		} else {
			return true;
		}
	}
   

}
