/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/*
 * Author: Eric Brown-Munoz
 * $Revision: 1.6 $
 * 
 */
package org.concord.rttapplets.datafitter;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.*;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;
import java.util.Enumeration;
import java.util.EventObject;

import org.concord.graph.engine.CoordinateSystem;
import org.concord.graph.engine.DefaultGraphable;
import org.concord.graph.engine.Graphable;
import org.concord.graph.event.GraphableListListener;
import org.concord.graph.event.GraphableListener;
import org.concord.graphfunction.engine.LinearFunction;

/**
 * @author ebrownmunoz
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class ErrorLines extends DefaultGraphable 
	implements GraphableListListener,
		GraphableListener, ImageObserver
{
	
	
	private DataSet data;
	private LinearFunction fit;
	private FitMetric fitmetric;
	private CoordinateSystem cs;
	private Image buffer;
	private Point2D.Double location;
	private Point2D.Double linePoint = new Point2D.Double();
	private Point2D.Double transformedLocation = new Point2D.Double();
	private Point2D.Double transformedLinePoint = new Point2D.Double();
	private Color errorColor = Color.green;
	
	public ErrorLines() {
		
	}
	
	public ErrorLines(DataSet data, LinearFunction fit, FitMetric fitmetric) {
		this();
		setData(data);
		setFit(fit);
		setFitmetric(fitmetric);
	}
	
	public Graphable getCopy() {
		return null;
	}
	
	
	/**
	 * @return Returns the data.
	 */
	public DataSet getData() {
		return data;
	}
	/**
	 * @param data The data to set.
	 */
	public void setData(DataSet data) {
		this.data = data;
		data.addGraphableListListener(this);
	}
	
	
	/**
	 * @return Returns the mode.
	 */
	
	
	public LinearFunction getFit() {
		return fit;
	}
	
	
	/**
	 * @return Returns the errorColor.
	 */
	public Color getErrorColor() {
		return errorColor;
	}
	/**
	 * @param errorColor The errorColor to set.
	 */
	public void setErrorColor(Color errorColor) {
		this.errorColor = errorColor;
	}
	/**
	 * @param fit The fit to set.
	 */
	public void setFit(LinearFunction fit) {
		this.fit = fit;
		data.addGraphableListener(this);
	}
	
	/**
	 * @return Returns the fitmetric.
	 */
	public FitMetric getFitmetric() {
		return fitmetric;
	}
	/**
	 * @param fitmetric The fitmetric to set.
	 */
	public void setFitmetric(FitMetric fitmetric) {
		this.fitmetric = fitmetric;
	}
	
	public void draw(Graphics2D g) {
		CoordinateSystem cs = getGraphArea().getCoordinateSystem();
		Dimension size = getGraphArea().getSize();
		double yIntersection;
		double fitSlope;
		
		if ((data != null) && (fit != null)) {
			Enumeration points = data.elements();
			Color savedColor = g.getColor();
			
			g.setColor(errorColor);
			if ((fitmetric.getFitType() == FitMetric.ABSOLUTE_VALUE) || // is this Vertical?
					(fitmetric.getFitType() == FitMetric.LEAST_SQUARES)) {
				while (points.hasMoreElements()) {
					DataPoint point = (DataPoint)points.nextElement();
					location = point.getPoint();
					cs.transformToDisplay(location, transformedLocation);
					
					linePoint.setLocation(location.getX(), fit.yval(location.getX()));
					
					cs.transformToDisplay (linePoint, transformedLinePoint);
					
					g.drawLine( (int)transformedLocation.x, (int)transformedLocation.y, 
							(int)transformedLinePoint.x, (int)transformedLinePoint.y );
				}
			} else if ((fitmetric.getFitType() == FitMetric.ABSOLUTE_VALUE_HORIZONTAL) || 
					(fitmetric.getFitType() == FitMetric.LEAST_SQUARES_HORIZONTAL)) {
				
				while (points.hasMoreElements()) {
					DataPoint point = (DataPoint)points.nextElement();
					location = point.getPoint();
					cs.transformToDisplay(location, transformedLocation);
					
					fitSlope = fit.getSlope();
					if (fitSlope != 0.0) {
						yIntersection = (location.getY() - fit.getYIntercept() )/ fitSlope;
					} else {
						yIntersection = graphArea.getUpperRightCornerWorld().getX();
					}

					linePoint.setLocation(yIntersection, location.getY());					
					cs.transformToDisplay (linePoint, transformedLinePoint);
					
					g.drawLine( (int)transformedLocation.x, (int)transformedLocation.y, 
							(int)transformedLinePoint.x, (int)transformedLinePoint.y );
				}
			}
			
			g.setColor(savedColor);
			//notifyChange();
		}
	}
		
	/* (non-Javadoc)
	 * @see org.concord.graph.event.GraphableListListener#listGraphableAdded(java.util.EventObject)
	 */
	public void listGraphableAdded(EventObject e) {
		// TODO Auto-generated method stub
	}
	
	/* (non-Javadoc)
	 * @see org.concord.graph.event.GraphableListListener#listGraphableRemoved(java.util.EventObject)
	 */
	public void listGraphableRemoved(EventObject e) {
		// TODO Auto-generated method stub
	}
	
	/* (non-Javadoc)
	 * @see org.concord.graph.event.GraphableListListener#listGraphableChanged(java.util.EventObject)
	 */
	public void listGraphableChanged(EventObject e) {
	}
	
	/* (non-Javadoc)
	 * @see org.concord.graph.event.GraphableListener#graphableChanged(java.util.EventObject)
	 */
	public void graphableChanged(EventObject e) {
		System.out.println("Errorlines: GraphableChanged");
			
	}
	
	/* (non-Javadoc)
	 * @see org.concord.graph.event.GraphableListener#graphableRemoved(java.util.EventObject)
	 */
	public void graphableRemoved(EventObject e) {
		// TODO Auto-generated method stub
	}

	/** A stub to fulfill the ImageObserver requirement for drawImage */
	public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height) {
		return false;
	}
	
}
