/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/*
 * Author: Eric Brown-Munoz
 * $Revision: 1.3 $
 */
package org.concord.rttapplets.datafitter;

import java.util.EventObject;

import org.concord.graph.event.GraphableListListener;
import org.concord.graph.event.GraphableListener;
import org.concord.graphfunction.engine.LinearFunction;
import org.concord.graphfunction.event.ValueEvent;
import org.concord.graphfunction.event.ValueListener;

/**
 * Manages and draws a graphical representation of the
 * bestfit based on a given DataSet.
 *  
 * @author ebrownmunoz
 *
 */
public class FitGraphable extends LinearFunction
	implements GraphableListListener,
		ValueListener
{
	DataSet data;
	FitMetric fit;
	
	/**
	 * Construct and Initialize a FitGraphable
	 */
	public FitGraphable() {
		super();
		setSelectionEnabled(false);
		setVisible(false); // Not visible until Dataset is set
	}
	
	/** Construct and Intialize a FitGraphable
	 * with the given dataset.
	 * 
	 * @param dataset
	 */
	public FitGraphable(DataSet dataset, FitMetric fit) {
		this();
		this.data = dataset;
		setFit(fit);
		dataset.addGraphableListListener(this);
		fit.getBestFit(this);
		setVisible(true);
	}
	
	
	/**
	 * @return Returns the fit.
	 */
	public FitMetric getFit() {
		return fit;
	}
	/**
	 * @param fit The fit to set.
	 */
	public void setFit(FitMetric fit) {
		if (fit != null)
			fit.removeValueListener(this);
		
		this.fit = fit;
		fit.addValueListener(this);
	}
	/**
	 * @return Returns the data.
	 */
	public DataSet getData() {
		return data;
	}
	/**
	 * @param data The data to set.
	 */
	public void setData(DataSet data) {
		this.data = data;
	}

	public void listGraphableAdded(EventObject e) {
		fit.getBestFit(this);
		
	}

	public void listGraphableRemoved(EventObject e) {
		// TODO Auto-generated method stub
		fit.getBestFit(this);
	}

	public void listGraphableChanged(EventObject e) {
		// TODO Auto-generated method stub
		fit.getBestFit(this);
	}

	/* (non-Javadoc)
	 * @see org.concord.graphfunction.event.ValueListener#valueChanged(org.concord.graphfunction.event.ValueEvent)
	 */
	public void valueChanged(ValueEvent e) {
		fit.getBestFit(this);
		notifyChange();
	}
}
