/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.7 $
 * $Date: 2004/09/03 14:51:52 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
 */
package org.concord.rttapplets.datafitter;

import java.util.*;
import java.awt.geom.*;

import org.concord.graph.engine.*;
import org.concord.graph.event.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;


/**
 * Class name and description
 *
 * Date created: 
 *
 * @author Eric Brown-Munoz<p>
 *
 */

public class FitMetric extends Value 
{
	
	public static int LEAST_SQUARES = 0;
	public static int LEAST_SQUARES_HORIZONTAL = 1;
	public static int ABSOLUTE_VALUE = 2;
	public static int ABSOLUTE_VALUE_HORIZONTAL = 3;
	
	private DataSet data;
	private LinearFunction curve;
	private int fitType = LEAST_SQUARES;
	
	public FitMetric( DataSet data, LinearFunction curve ) 
	{
		this.data = data;
		data.addGraphableListListener(new GraphableListListener() 
				{
			
			public void listGraphableAdded(EventObject e) {
				calculate();
				
			}
			
			public void listGraphableRemoved(EventObject e) {
				calculate();
			}
			
			public void listGraphableChanged(EventObject e) {
				calculate();
			}
				} );
		
		this.curve = curve;
		
		curve.addGraphableListener(new GraphableAdapter() 
				{
			public void graphableChanged(EventObject e) 
			{
				calculate();
			}
				} );
		
		calculate();
	}
	
	
	
	/**
	 * @return Returns the mode.
	 */
	public int getFitType() {
		return fitType;
	}
	/**
	 * @param mode The mode to set.
	 */
	public void setFitType(int fitType) {
		this.fitType = fitType;
		setValue(getValue());
		calculate();
		notifyChange();
	}
	
	/** Returns the value (error) of the current fit. */
	public void calculate() 
	{
		double error = 0.0;
		int number = 0;
		
		Enumeration pointsEnum = data.listPoints();
		
		if (curve.getSlope() == 0.0) {
			setValue(Double.POSITIVE_INFINITY);
		} else {
			
			while (pointsEnum.hasMoreElements() ) {
				DataPoint eachPoint = 
					(DataPoint)pointsEnum.nextElement();
				
				if (eachPoint.isOn) {
					number++;
					if (fitType == LEAST_SQUARES) {
						error += Math.pow(eachPoint.getY() - 
								curve.yval(eachPoint.getX()),2.0);
					} else if ( fitType == LEAST_SQUARES_HORIZONTAL) {
						if (Math.abs((curve.getSlope()) - 0.0) < 0.0000001 ) {
							error = Double.POSITIVE_INFINITY;
							break;
						} else {
							
							double xval = (eachPoint.getY() - curve.getYIntercept())/ curve.getSlope();
							error += Math.pow((xval - eachPoint.getX()),2.0);
						}
					} else if ( fitType == ABSOLUTE_VALUE) {
						error += Math.abs(eachPoint.getY() - curve.yval(eachPoint.getX()));
					}
				}
			}
			
			setValue(error/number);
		}
	}
	
	public LinearFunction getBestFit(LinearFunction fitline) 
	{
		double slope = 1.0;
		double intercept = 0.0;
		
		double x;
		double y;
		double sx = 0.0; // sum of x
		double sy = 0.0 ; // sum of y
		double sx2 = 0.0; // sum of x^2
		double sxy = 0.0 ; // sum of x * y
		
		Enumeration pointslist = data.elements();
		
		int number = 0;
		
		while (pointslist.hasMoreElements()) {
			DataPoint eachpoint = (DataPoint)pointslist.nextElement();
			
			if (eachpoint.isOn) {
				if (fitType == LEAST_SQUARES ) {
					x = eachpoint.getX();
					y = eachpoint.getY();
					sx += x;
					sy += y;
					sx2 += x * x;
					sxy += x * y;
					number ++;
				} else if (fitType == LEAST_SQUARES_HORIZONTAL) {
					x = eachpoint.getY(); // swap x and y to get best fit of inverse
					y = eachpoint.getX();
					sx += x;
					sy += y;
					sx2 += x * x;
					sxy += x * y;
					number ++;
				}
			}
		} // while
		
		if ((fitType == LEAST_SQUARES) || (fitType == LEAST_SQUARES_HORIZONTAL)) {
		
			slope = ((sxy - sx * sy / (double)number) /
					(sx2 - sx * sx / (double)number));
		
			intercept = (sy / (double)number) - slope * (sx / (double)number);
		
			if (fitType == LEAST_SQUARES_HORIZONTAL) {
				
				if (slope != 0.0) {
					intercept = - intercept/slope;
					slope = 1.0/slope;
				}
				
			}
		}
		
		fitline.setSlope(slope);
		fitline.setYIntercept(intercept);
		
		return fitline;
		
	}
	
	public double getValue(int places) 
	{
		double value = getValue();
		double coeff = Math.pow(10,places);
		return Math.round(value * coeff) / coeff;
	}
}
