/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
 $Revision: 1.3 $
 */

package org.concord.rttapplets.datafitter;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import javax.swing.*;

import org.concord.graph.engine.*;
import org.concord.graph.ui.*;

/** A simple grid. This is explicitly for the DataFitter Applet
 since the public grids are way too complex and still don't
 meet the needs of this applet.
 
 I plan to generalize this later, but for now it is exactly
 what I want.
 */

public class SimpleGrid extends DefaultGraphable
implements ImageObserver
{
	
	Font labelFont = new Font("SansSerif",Font.PLAIN,10);
	FontMetrics labelMetrics;
	
	Color gridColor = Color.gray;
	Stroke gridStroke = new BasicStroke(1.0f);
	Color axisColor = Color.black;
	Stroke axisStroke = new BasicStroke(3.0f);
	private Color labelColor = Color.black;
	
	private boolean needsUpdate = true;
	
	private int sWidth;
	private int sHeight;
	
	// Nice values for the scale used by setInterval
	final double [] niceValues = {0.001, 0.002, 0.005, 0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 0.1,
			2.0, 5.0, 10.0, 20.0, 50.0, 100.0, 500.0, 1000.0, 2000.0};
	
	
	Point2D.Double axisPoint = new Point2D.Double(0.0,0.0);
	
	Image buffer = null;
	private double targetInterval = 20.0;
	
	
	public SimpleGrid() {
		super();
	}
	
	public void update() {
		needsUpdate = true;
	}
	
	public void createBuffer () {
		
		GraphArea ga = getGraphArea();
		CoordinateSystem cs = ga.getCoordinateSystem();
		
		double scalex = cs.getScale().getX();
		double scaley = cs.getScale().getY();
		
		double xInterval = niceInterval(scalex);
		double yInterval = niceInterval(scaley);
		
		double xLineSpacing = scalex * xInterval;
		double yLineSpacing = scaley * yInterval;
		
		double xNumberStart;
		double yNumberStart;
		
		
		
		
		int width = (int)ga.getUpperRightCornerDisplay().getX();
		int height = (int)ga.getLowerLeftCornerDisplay().getY();
		
		
		
		if ((width != 0) || (height != 0)) {
			
			/* Create image for background */
			buffer = new BufferedImage( width, height, 
					BufferedImage.TYPE_3BYTE_BGR );
			
			Graphics2D bg = (Graphics2D)buffer.getGraphics();
			labelMetrics = bg.getFontMetrics( labelFont);
			
			bg.setColor(Color.white);
			bg.fillRect(0,0,width,height);
			
			
			/* Font for label */
			bg.setFont(labelFont);
			labelMetrics = bg.getFontMetrics(labelFont);
			
			/* Get axisPoint in screen coordinates */
			Point2D.Double screenAxisPoint = new Point2D.Double();
			cs.transformToDisplay(axisPoint,screenAxisPoint);
			
			double axisx = screenAxisPoint.x;
			double axisy = screenAxisPoint.y;
			
			/* Draw grid lines */
			bg.setColor(gridColor);
			
			double startx = axisx % (xInterval * scalex);
			double starty = axisy % (yInterval * scaley);
			
			int xi = 0;
			double xpos;
			
			while (( xpos = startx + xi * xInterval * scalex ) < width) {
				bg.drawLine((int)xpos, 0, (int)xpos, height  );
				xi++;
			}
			
			int yi = 0;
			double ypos;
			
			while (( ypos = starty + yi * yInterval * scaley) < height) {
				bg.drawLine(0, (int)ypos, width, (int)ypos);
				yi++;
			}
			
			bg.setStroke(axisStroke);
			bg.setColor(axisColor);
			
			/* Draw AxisLines */
			if ((axisx > 0.0) && (axisx < width)) {
				bg.drawLine((int)axisx, 0, (int)axisx, height);
			}
			
			if ((axisy > 0.0) && (axisy < height)) {
				bg.drawLine(0,(int)axisy, width, (int)axisy);
			}
			
			labelMetrics = bg.getFontMetrics();
			
			/* Draw Labels */
			if ((axisy > 0.0) && (axisy < (height - 15))) {
				xNumberStart = - Math.floor(axisx / (xInterval*scalex)) * xInterval;
				
				
				xi = 0;
				
				
				while (( xpos = startx + xi * xInterval * scalex) < width) {
					String labelString = Double.toString(xNumberStart + xi * xInterval);
					if ((int)xpos != axisx) {
						sWidth = labelMetrics.stringWidth(labelString);
						sHeight = labelMetrics.getHeight();
						
						bg.setColor(Color.white);
						bg.fillRect((int)(xpos - sWidth/2), (int)axisy + 5, sWidth, sHeight);
						bg.setColor(labelColor);
						bg.drawString( labelString, (int)(xpos - sWidth/2), (int) axisy + sHeight + 5);
					}
					xi++;
				}
			}
			
			if ((axisx > 0.0) && (axisx < (width - 15))) {
				yNumberStart = - Math.floor(axisy / (yInterval*scaley)) * yInterval;
				
				yi = 0;
				
				
				while (( ypos = starty + yi * yInterval * scaley) < height) {
					
					if ((int) ypos != axisy) {
						String labelString = Double.toString(-(yNumberStart + yi * yInterval));
						
						sWidth = labelMetrics.stringWidth(labelString);
						sHeight = labelMetrics.getHeight();
						
						bg.setColor(Color.white);
						bg.fillRect((int)axisx - sWidth - 5, (int) ypos - sHeight - 2, sWidth, sHeight);
						bg.setColor(labelColor);
						bg.drawString( labelString, (int) axisx - sWidth -5 , (int) ypos - 2);
					}
					yi++;
				}
			}
		} else {
			buffer = null;
		}
		
		needsUpdate = false;
	} // oldupdate()
	
	
	
	public void draw(Graphics2D g) {
		
		if ((buffer == null ) || needsUpdate) {
			createBuffer();
		}
		
		if (buffer != null ) // createBuffer() fixes buffer.
			g.drawImage(buffer,0,0,this);
		
		
	}
	
	
	public double niceInterval (double scale) {
		int i;
		double interval = targetInterval / scale;
		
		for ( i = 0; (niceValues[i] < interval) && (i < niceValues.length); i++);
		
		return niceValues[i];
	}
	
	public Graphable getCopy() {
		return null;
	}
	
	/* (non-Javadoc)
	 * @see java.awt.image.ImageObserver#imageUpdate(java.awt.Image, int, int, int, int, int)
	 */
	public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height) {
		// TODO Auto-generated method stub
		return false;
	}
	
}





