/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz 
   $Revision: 1.2 $
*/

package org.concord.rttapplets.fanalyzer;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/** This class ensures that the one component inside is
	the largest square that will fit in the Panel. The
	component will center itself in the space available.
*/

public class MagicCenteringSquarePanel 
	extends JPanel
	implements ComponentListener
{

	JComponent component;

	public MagicCenteringSquarePanel () {
		
		setLayout(null);
		addComponentListener(this);
	}

	public MagicCenteringSquarePanel (JComponent component) {

		this();
		add(component);
			
	}
	
	public void add (JComponent component) {
		this.component = component;
		super.add(component);
		layoutComponent();
	}

	private void layoutComponent () {
		if (component != null) {
			int width=getWidth();
			int height=getHeight();

			int side = (width < height) ? width : height;

			int posx = (width - side)/ 2;
			int posy = (height - side)/ 2;

			component.setBounds(posx, posy, side, side);
			validate();
		}
	}


	public void componentHidden(ComponentEvent e) {}

	public void componentMoved(ComponentEvent e) {}

	public void componentResized(ComponentEvent e){
		layoutComponent();
	}

	public void componentShown(ComponentEvent e){}
}

		
