/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
 * $Revision: 1.4 $
 */

package org.concord.rttapplets.fanalyzer;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.text.*;
import java.awt.geom.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;

import org.concord.rttapplets.ui.*;
import org.concord.graph.engine.*;
import org.concord.graph.ui.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;
import org.concord.graphfunction.ui.symbolicview.*;

public class SimpleFA extends JPanel 
	implements ActionListener
{

	JSplitPane splitPane;
	GraphFunctionList gflist;
	Grid grid;
	GraphWindow graph;
	GraphArea graphArea;
	SimpleFAModel areaModel;
	Value variable;


	public SimpleFA () {

		setLayout(new BorderLayout());

		// leftPane and rightPane (of SplitPane)
		JPanel leftPane = new JPanel(new BorderLayout());
		JPanel rightPane = new JPanel(new BorderLayout());

		// ----- variable
		variable = new Value();
		//X starts in 1.0
		variable.setValue(1.0);

		// ----- gflist
		gflist = new GraphFunctionList();
		gflist.getColorFactory().setStartColorIndex(3);

		// ----- A Dialog for Zero Slope;
		ZeroSlopeDialog zsd = new ZeroSlopeDialog (this, gflist);

		// ---- graph & grid

		JPanel graphPane = new JPanel(new BorderLayout());
		graph = new GraphWindow ();
		graph.setSelectionMode(SelectableList.SELECTIONMODE_NONE);
		grid = new Grid();
		graph.addDecoration(grid);
		graph.add(gflist);
		graph.setEditable(false); // No mouse interaction
		graphArea = graph.getDefaultGraphArea();
		graph.setBorder(BorderFactory.createLineBorder(Color.black));

		graphPane.addComponentListener(new GraphComponentListener());
		graphPane.add(graph, BorderLayout.CENTER);

		EquationTrace trace = new EquationTrace(variable);
		trace.setGraphFunctionList(gflist);
		graph.add(trace);


		addEquation(); // This adds the first new equation to gflist


		// --- SymbolicPanel
		JPanel symbolicPanel = new JPanel( new GridBagLayout());
		GridBagConstraints spc = new GridBagConstraints();
		spc.anchor = GridBagConstraints.WEST;
		spc.gridx = 0;
		symbolicPanel.add(new JLabel("Equation: "),spc);
		symbolicPanel.add(new JLabel("Function: "),spc);
		symbolicPanel.add(new JLabel("Evaluation: "),spc);

		spc.gridx = 1;
		spc.gridy = GridBagConstraints.RELATIVE;
		spc.gridwidth = 2;

		LinearEquationInput equationInput = new BorderedLinearEquationInput();
		equationInput.setGraphFunctionList(gflist);
		equationInput.setLabelSize(14.0f);

		//Set increments for input boxes FIXME (big hack that caused problem)
		Vector vals = equationInput.getValueBox();
		//Increment for coeff m
		((ValueBox)vals.elementAt(1)).setIncrementInterval(0.5);
		//Increment for coeff b
		((ValueBox)vals.elementAt(2)).setIncrementInterval(1.0);


		symbolicPanel.add(equationInput, spc);

		LinearFunctionEquationInput functionInput = 
			new LinearFunctionEquationInput(false);
		functionInput.setLabelSize(14.0f);
		functionInput.setGraphFunctionList(gflist);
		functionInput.setEditable(false);
		symbolicPanel.add(functionInput, spc);

		LinearEvalLabel evalLabel = new LinearEvalLabel();
		//evalLabel.setLabelSize(14.0f);
		evalLabel.setGraphFunctionList(gflist);
		evalLabel.setVariableValue(variable);

		symbolicPanel.add(evalLabel, spc);

		leftPane.add(symbolicPanel, BorderLayout.NORTH);

		// ---- Area Model
		areaModel = new SimpleFAModel();
		areaModel.setBorder(BorderFactory.createLineBorder(Color.black,1));
		areaModel.setBackground(Color.white);
		areaModel.setVariable(variable);
		areaModel.setGraphFunctionList(gflist);
		
		leftPane.add(areaModel, BorderLayout.CENTER);

		// --- bottomLeft Panel
		JPanel bottomLeft = new JPanel();
		bottomLeft.setLayout(new BorderLayout());
		
		// ---- ValueSlider
		TenthsValueSlider slider = new TenthsValueSlider(variable);
		slider.setPaintTicks(false);
		slider.sync(graphArea);
		
		graphPane.add(slider, BorderLayout.SOUTH);

		// ---- AreaModelToolbar
		SimpleModelToolbox toolbar = new SimpleModelToolbox();
		toolbar.setGraphFunctionList(gflist);
		toolbar.setAreaModelView(areaModel);
		
		bottomLeft.add(toolbar, BorderLayout.CENTER);

		leftPane.add(bottomLeft, BorderLayout.SOUTH);

		// ---- Start of rightPane
		rightPane.add(new MagicCenteringSquarePanel(graphPane), 
					  BorderLayout.CENTER);

		// ---- underGraph
		JPanel underGraph = new JPanel(new BorderLayout());
		
		// ---- FunctionTable
		GraphFunctionTable table = new GraphFunctionTable();
		table.setGraphFunctionList(gflist);
		table.setPreferredSize(new Dimension(200,100));
		underGraph.add(table, BorderLayout.CENTER);

		// ---- Buttons!
		JPanel buttonPanel = new JPanel(new GridLayout(2,2));
		JButton newButton = new JButton("New Equation");
		newButton.setActionCommand("newFunction");
		newButton.addActionListener(this);

		JButton duplicateButton = new JButton("Duplicate Equation");
		duplicateButton.setActionCommand("duplicateFunction");
		duplicateButton.addActionListener(this);
		
		JButton deleteButton = new JButton("Delete Equation");
		deleteButton.setActionCommand("deleteFunction");
		deleteButton.addActionListener(this);

		//JButton repaintButton = new JButton("Clear line");
		//repaintButton.setActionCommand("repaint");
		//repaintButton.addActionListener(this);

		buttonPanel.add(newButton);
		buttonPanel.add(duplicateButton);
		buttonPanel.add(deleteButton);
		//buttonPanel.add(repaintButton);



		underGraph.add(buttonPanel,BorderLayout.SOUTH);

		rightPane.add(underGraph, BorderLayout.SOUTH);

		rightPane.setMinimumSize(new Dimension(0,0));

		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, 
								   leftPane, rightPane);

		this.add(splitPane, BorderLayout.CENTER);
	}


	
	private void addEquation()
	{
		LinearEquation eq;
		eq = new LinearEquation(new Value(0.0));
		eq.setGraphArea(graphArea);
		eq.setPainter(variable, false);
		eq.setExpressionForm(LinearEquation.FORM_FUNCTION);
		gflist.add(eq);
	}
	

	//Testing purposes
    public static void main(String args[]) 
    {
		final SimpleFA fa = new SimpleFA();

		RTTFrame frame = new RTTFrame(fa, "Function Analyzer");
		frame.show();
	}

	public void actionPerformed (ActionEvent e) {

		String command =((AbstractButton)e.getSource()).getActionCommand();
		
		if (command.equals("newFunction")){
			addEquation();
		}
		else if (command.equals("duplicateFunction")){
			LinearEquation current = 
				(LinearEquation)gflist.getLastSelectedObject();
			if (current!=null){
				gflist.add( current.getCopy());
			}
		}
		else if (command.equals("deleteFunction")){
			gflist.removeAllSelected();
		}
		else if (command.equals("repaint")){
			LinearEquation current = (LinearEquation)gflist.getLastSelectedObject();
			if (current!=null){
				current.setPainterOn();
				current.resetPainter();
			}
		}
	}	


	class GraphComponentListener implements ComponentListener {
		public void componentHidden(ComponentEvent e) {}

		public void componentMoved(ComponentEvent e) {}

		public void componentResized(ComponentEvent e) {
			CoordinateSystem cs = graphArea.getCoordinateSystem();
			double height = (double)graph.getHeight();
			double width = (double)graph.getWidth();
			double scale = width / 15.0;
			Point2D origin = new Point2D.Double(scale * 4.0, 
												height - scale * 4.0);

			cs.setScale( new Point2D.Double(scale, scale));
			cs.setOriginOffsetDisplay(origin);
		}

		public void componentShown(ComponentEvent e) {}
	}

} // class SimpleFA 
	

		
