/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
 * $Revision: 1.1 $
*/

package org.concord.rttapplets.fanalyzer;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;

import org.concord.graph.event.*;
import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;
import org.concord.graphfunction.ui.*;

/**
 * SimpleModelToolbox
 * This is a tool box (TILE BIN) for the area model
 * 
*/

public class SimpleModelToolbox extends JPanel 
	implements GraphFunctionView, 
			   ActionListener,
			   SelectableListListener
{	
	//Only represents one function
	GraphFunction function;
	
	// ... from a list of functions
	GraphFunctionList glist;
	
	SimpleFAModel areaModel;

//	JToolBar toolBar;
	JPanel buttonsPanel;
	
	Vector buttons = new Vector(); // To set up enable/diable
	
	private JLabel titleLabel;
	
	public SimpleModelToolbox()
	{

		// ---- Title Box
		setLayout(new BorderLayout());

		titleLabel = new JLabel("TILE BIN");
		titleLabel.setFont(new Font("Dialog",Font.BOLD,12));
		titleLabel.setAlignmentX(LEFT_ALIGNMENT);
		add(titleLabel, BorderLayout.NORTH);

		// ---- buttons Panel
		buttonsPanel = new JPanel();
		buttonsPanel.setLayout(new BoxLayout(buttonsPanel, BoxLayout.X_AXIS));
		buttonsPanel.setAlignmentX(LEFT_ALIGNMENT);

		// ---- Negative Buttons
		JPanel negPanel = new JPanel();
		negPanel.setLayout(new BoxLayout(negPanel,BoxLayout.X_AXIS));		


		Border baseBorder = 
			BorderFactory.createEtchedBorder(EtchedBorder.LOWERED); 

		Border border = 
			BorderFactory.
			createTitledBorder(baseBorder, " Negative ",
							   TitledBorder.CENTER , 
							   TitledBorder.BOTTOM  , 
							   new Font("Dialog",Font.PLAIN,11));

		negPanel.setBorder(border);

		addJButton(negPanel,"neghalfcoeff_m.gif","-m/2");
		addJButton(negPanel,"negcoeff_m.gif","-m");
		addJButton(negPanel,"negcoeff_b.gif","-b");

		buttonsPanel.add(negPanel);
		
		// ---- Positive Buttons
		JPanel posPanel = new JPanel();
		posPanel.setLayout(new BoxLayout(posPanel,BoxLayout.X_AXIS));

		border = 
			BorderFactory.
			createTitledBorder(baseBorder, " Positive ",
							   TitledBorder.CENTER , 
							   TitledBorder.BOTTOM  , 
							   new Font("Dialog",Font.PLAIN,11));

		posPanel.setBorder(border);

		addJButton(posPanel,"halfcoeff_m.gif","m/2");
		addJButton(posPanel,"coeff_m.gif","m");
		addJButton(posPanel,"coeff_b.gif","b");

		buttonsPanel.add(posPanel);

		// ---- EraserPanel
		JPanel eraserPanel = new JPanel();
		eraserPanel.setLayout(new BoxLayout(eraserPanel,BoxLayout.X_AXIS));
		
		border = 
			BorderFactory.
			createTitledBorder(baseBorder, " ",
							   TitledBorder.CENTER , 
							   TitledBorder.BOTTOM  , 
							   new Font("Dialog",Font.PLAIN,11));

		eraserPanel.setBorder(border);

		JToggleButton eraserButton = new JToggleButton();
		eraserButton.setMargin(new Insets(0,0,0,0));
		eraserButton.setIcon(ResourceLoader.getImageIcon("eraserpencil.gif", 
												 "button image"));
		eraserButton.setSelectedIcon(ResourceLoader.
									 getImageIcon("eraserpencilselected.gif", 
												  "button image"));
		eraserButton.setActionCommand("erase");
		eraserButton.addActionListener(this);
		eraserPanel.add(eraserButton);

		buttonsPanel.add(eraserPanel);
		
		add(buttonsPanel, BorderLayout.CENTER);



	}

	public void addJButton( JPanel cont, String image, String action) {
		JButton butt = new JButton();
		butt.setIcon(ResourceLoader.getImageIcon(image, "button image"));
		butt.setMargin(new Insets(0,0,0,0));
		butt.setActionCommand(action);
		butt.addActionListener(this);
		cont.add(butt);

		buttons.add(butt);
	}

	
	public void setGraphFunction(GraphFunction g)
	{
		if (g instanceof FunctionEquation){
			function = ((FunctionEquation)g).getFunction();
		}
		else{
			function = g;
		}

		repaint();

	}

	public void setGraphFunctionList(GraphFunctionList gl)
	{
		if (glist!=null){
			glist.removeGraphableListListener(this);
		}
		glist = gl;
		if (glist!=null){
			glist.addGraphableListListener(this);
			GraphFunction current = (GraphFunction)gl.getFirstSelectedObject();
			if (current != null) {
				setGraphFunction(current);
			}
		}
	}


	public void setAreaModelView(SimpleFAModel areaModel)
	{
		this.areaModel = areaModel;
	}

    public void actionPerformed (ActionEvent e) 
	{
		String command =((AbstractButton)e.getSource()).getActionCommand();
		
		if (function instanceof LinearFunction){
			LinearFunction linearFunction = (LinearFunction)function;
			if (command.equals("m")){
				linearFunction.setSlope(linearFunction.getSlope()+1.0);
			}
			else if (command.equals("m/2")){
				linearFunction.setSlope(linearFunction.getSlope()+0.5);
			}
			else if (command.equals("b")){
				linearFunction.setYIntercept(linearFunction.getYIntercept()+1.0);
			}
			else if (command.equals("-m")){
				linearFunction.setSlope(linearFunction.getSlope()-1.0);
			}
			else if (command.equals("-m/2")){

				linearFunction.setSlope(linearFunction.getSlope()-0.5);
			}
			else if (command.equals("-b")){
				linearFunction.setYIntercept(linearFunction.getYIntercept()-1.0);
			}
		}
		if (command.equals("erase")){
			if (areaModel==null) return;
			if (((AbstractButton)e.getSource()).isSelected()){
				areaModel.setClickBehavior(SimpleFAModel.DELETE_ONCLICK);
			}
			else{
				areaModel.setClickBehavior(SimpleFAModel.NOTHING_ONCLICK);
			}
		}
	}
	
	public void listGraphableAdded(EventObject e)
	{
		repaint();
	}
	public void listGraphableChanged(EventObject e) 
	{
		repaint();
	}
	public void listGraphableRemoved(EventObject e) 
	{
		repaint();
	}
	public void listGraphableSelected(EventObject e) 
	{
		setGraphFunction((GraphFunction)e.getSource());
		repaint();

	}
	public void listGraphableDeselected(EventObject e) 
	{
	}



} // class SimpleModelToolbox

