/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.3 $
 * $Date: 2004/08/18 18:24:58 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.rttapplets.lineartransformer;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import javax.swing.*;
import org.concord.graph.engine.*;
import org.concord.graph.ui.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;
import org.concord.graphfunction.ui.modelview.*;
import org.concord.graphfunction.ui.symbolicview.*;

/**
 * Main panel for the Linear Transformer Grapher
 *
 * @author Ingrid Moncada<p>
 * @author Eric Brown-Munoz<p>
*/

public class LinearTransformer extends JPanel 
	implements ActionListener
{
	String interactiveName;

    GraphFunctionList gflist;
	GraphFunctionViewManager functionManager;

    // We need to these available for the button Handlers.
    JPanel inputPanel;
    CardLayout inputPanelLayout;
    JCheckBox keepGhostButton;

	Grid grid;
	GraphWindow graph;
	
	//Initial ghost duration (1 sec)
	int ghostDuration = 1000;
	
//	SaveAsImageButton saveButton;
	
    // The x variable;
    private Value variable;

	private boolean autoPaint = false;
	private boolean showGhosts = true;

    public LinearTransformer() 
	{
		this("RTT Linear Transformer");
	}
	
	public LinearTransformer(String interactiveName)
	{
		this.interactiveName = interactiveName;
	
		// Set Contraints for GridBagLayout for all panels
		GridBagConstraints graphConstraints = new GridBagConstraints();
		graphConstraints.gridx = 1;
		graphConstraints.gridy = 0;
		graphConstraints.gridwidth = 2;
		graphConstraints.gridheight = 3;
		graphConstraints.fill = GridBagConstraints.BOTH;
		graphConstraints.weightx = 100;
		graphConstraints.weighty = 100;
	
		GridBagConstraints controlsConstraints = 
			(GridBagConstraints)graphConstraints.clone();
		controlsConstraints.gridx = 3;
		controlsConstraints.gridwidth = 1;
		controlsConstraints.gridheight = 4;
		controlsConstraints.fill = GridBagConstraints.VERTICAL;
		controlsConstraints.weightx = 0;
	
		GridBagConstraints modelConstraints = 
			(GridBagConstraints)graphConstraints.clone();
		modelConstraints.gridx = 0;
		modelConstraints.gridwidth = 1;
		modelConstraints.gridheight = 3;
		modelConstraints.fill = GridBagConstraints.BOTH;
		modelConstraints.weightx = 20;
	
		GridBagConstraints inputConstraints = 
			(GridBagConstraints)graphConstraints.clone();
		
		inputConstraints.gridx = 0;
		inputConstraints.gridy = 3;
		inputConstraints.gridwidth = 3;
		inputConstraints.gridheight = 1;
		inputConstraints.fill = GridBagConstraints.HORIZONTAL;
		inputConstraints.weighty = 0;
		inputConstraints.anchor = GridBagConstraints.SOUTH;
		
		////////////////////////////////////////////////////////////////////////////////

		variable = new Value();
		variable.setValue(5.0);
		
		gflist = new GraphFunctionList();
		functionManager = new GraphFunctionViewManager();
		functionManager.setGraphFunctionList(gflist);

		addNewFunction();

		////////////////////////////////////////////////////////////////////////////////
		//Graph
		graph = new GraphWindow();

		grid = new Grid();
		graph.addDecoration(grid);		
	
		final JPanel graphContainer = new JPanel();
	
		graphContainer.setBorder(BorderFactory.createLineBorder(Color.black));
	
		// This ensures the graph is centered on resize
		graph.getDefaultGraphArea().setOriginCentered(true);	

		CoordinateSystem coord = graph.getDefaultGraphArea().getCoordinateSystem();
		coord.setScale(new Point2D.Double(18.0,18.0));
		//((Axis)coord).setFixOriginScreen(true);

		graph.setMouseBehavior(DefaultGraphMouseManager.ONCLICK_SELECT);

		graph.add(gflist);
		
		graph.setVisible(true);
	
		graphContainer.setLayout(new BoxLayout(graphContainer,
							   BoxLayout.Y_AXIS));
		graphContainer.add(graph);

		Trace trace = new Trace(gflist, variable);
		trace.setNumDecimalsXCoordinate(1);
		trace.setNumDecimalsYCoordinate(1);

		graph.add(trace);
	
		////////////////////////////////////////////////////////////////////////////////
	
		ValueSlider slider = new ValueSlider(variable);
		//slider.sync(graph);
		graphContainer.add(slider);
	
		setLayout(new GridBagLayout());
	
		add(graphContainer,graphConstraints);
		
		////////////////////////////////////////////////////////////////////////////////
	

		LinearSymbolic inputPanel = new LinearSymbolic();
		functionManager.addGraphFunctionView(inputPanel);
		add(inputPanel,inputConstraints);
		
		////////////////////////////////////////////////////////////////////////////////

		JPanel controls = new JPanel();
		controls.setLayout(new BoxLayout(controls, BoxLayout.Y_AXIS));
		
		PushpinCheckBox pushpinButton = new PushpinCheckBox("Pushpin");
		functionManager.addGraphFunctionView(pushpinButton);
	
		JCheckBox autoPaintCheckBox = new JCheckBox("Auto-paint");
		autoPaintCheckBox.setActionCommand("autopaint");
		autoPaintCheckBox.addActionListener(this);
	
		JCheckBox ghostButton = new JCheckBox("Show Ghosts", true);
		ghostButton.setActionCommand("ghost");
		ghostButton.addActionListener(this);
	
		keepGhostButton = new JCheckBox("Keep Ghosts");
		keepGhostButton.setActionCommand("keepghost");
		keepGhostButton.addActionListener(this);

		JButton resetPainter = new JButton("Clear Line");
		resetPainter.setActionCommand("repaint");
		resetPainter.addActionListener(this);
	
		JButton reflectX = new JButton("Reflect X");
		reflectX.setActionCommand("reflectx");
		reflectX.addActionListener(this);
	
		JButton reflectY = new JButton("Reflect Y");
		reflectY.setActionCommand("reflecty");
		reflectY.addActionListener(this);
	
		JPanel topControls = new JPanel();
		GridLayout gridLay = new GridLayout(4,2);
		topControls.setLayout(gridLay);
		//topControls.setPreferredSize(new Dimension(240,100));
		topControls.setSize(new Dimension(240,100));
	
		gridLay.setHgap(20);
		gridLay.setVgap(4);
		topControls.add(pushpinButton);
		topControls.add(autoPaintCheckBox);
		topControls.add(ghostButton);
		topControls.add(resetPainter);
		topControls.add(keepGhostButton);
		topControls.add(reflectX);
		topControls.add(Box.createRigidArea(new Dimension(5,5)));
		topControls.add(reflectY);
		
		controls.add(topControls);
	
		////////////////////////////////////////////////////////////////////////////////

		GraphFunctionTable table = new GraphFunctionTable();
		table.setGraphFunctionList(gflist);
		table.setPreferredSize(new Dimension(200,200));
		controls.add(table);
		
		JButton createGraph = new JButton ("New Function");
		createGraph.setActionCommand("new");
		createGraph.addActionListener(this);
	
		JButton saveGraph = new JButton("Duplicate Function");
		saveGraph.setActionCommand("copy");
		saveGraph.addActionListener(this);
	
		JButton deleteGraph = new JButton ("Delete Function");
		deleteGraph.setActionCommand("delete");
		deleteGraph.addActionListener(this);
	
		JButton deleteAll = new JButton ("Delete All");
		deleteAll.setActionCommand("deleteall");
		deleteAll.addActionListener(this);
		
		JPanel bottomControls = new JPanel();
		gridLay = new GridLayout(4,1);
		bottomControls.setLayout(gridLay);
		bottomControls.setPreferredSize(new Dimension(150,120));
		bottomControls.setMaximumSize(new Dimension(150,120));
		bottomControls.setSize(new Dimension(150,120));
	
		gridLay.setVgap(4);
		bottomControls.add(createGraph);
		bottomControls.add(saveGraph);
		bottomControls.add(deleteGraph);
		bottomControls.add(deleteAll);

		////////////////////////////////////////////////////////////////////////////////
		
//		JPanel saveButtonPanel = new JPanel();
//		saveButtonPanel.setLayout(new BorderLayout());
//		saveButton = new SaveAsImageButton(this,interactiveName);
		//saveButton.setAlignmentX(RIGHT_ALIGNMENT);
//		saveButtonPanel.add(saveButton,BorderLayout.SOUTH);
	
//		JPanel bottomButtons = new JPanel();
//		bottomButtons.setLayout(new BorderLayout());
//		bottomButtons.add(bottomControls, BorderLayout.CENTER);
//		bottomButtons.add(saveButtonPanel, BorderLayout.EAST);
	
//		controls.add(bottomButtons);
		
//		add(controls,controlsConstraints);
	
		////////////////////////////////////////////////////////////////////////////////

		ModelWindow model = new ModelWindow();
		model.setGraphFunctionList(gflist);
		model.setVariable(variable);
		//model.sync(graph);
	
		add(model,modelConstraints);

		////////////////////////////////////////////////////////////////////////////////

		//Auto-paint true by default
		autoPaintCheckBox.setSelected(true);
		//Since this doesn't trigger the action event (unless you do doClick()), then:
		autoPaint = true;
		setupPainter();
		
		validate();
    }

    public void actionPerformed (ActionEvent e) 
	{
		String command =((AbstractButton)e.getSource()).getActionCommand();
	
		if (command.equals("copy")) {
			GraphFunction current = (GraphFunction)gflist.getLastSelectedObject();
			if (current!=null){
				addCopy(current);
			}
		} else if (command.equals("ghost")) {
			showGhosts = ((AbstractButton)e.getSource()).isSelected();
			enableGhosts(showGhosts);
			keepGhostButton.setEnabled(showGhosts);
		} else if (command.equals("keepghost")) {
			boolean keepGhosts = ((AbstractButton)e.getSource()).isSelected();
			keepGhosts(keepGhosts);
		} else if (command.equals("new")) {
			addNewFunction();
		} else if (command.equals("delete")) {
			gflist.removeAllSelected();
		} else if (command.equals("deleteall")) {
			gflist.removeAllElements();
		} else if (command.equals("reflectx")) {
			LinearFunctionPushpin current = (LinearFunctionPushpin)gflist.getLastSelectedObject();
			if (current!=null){
				
//				//Mode 1: Activate the ghost for 1 second
//				current.getGhost().activate(ghostDuration);

				//Mode 2: Activate and deactivate the ghost
				//(This is used when if the ghost has a duration already)
				current.getGhost().activate();
				current.reflectX();
				current.getGhost().deactivate();
				
			}	    
		} else if (command.equals("reflecty")) {
			LinearFunctionPushpin current = (LinearFunctionPushpin)gflist.getLastSelectedObject();
			if (current!=null){
				
//				//Mode 1: Activate the ghost for 1 second
//				current.getGhost().activate(ghostDuration);

				//Mode 2: Activate and deactivate the ghost 
				//(This is used when if the ghost has a duration already)
				current.getGhost().activate();
				current.reflectY();
				current.getGhost().deactivate();
				
			}	    
	
		} else if (command.equals("repaint")) {
			LinearFunctionPushpin current = (LinearFunctionPushpin)gflist.getLastSelectedObject();
			if (current!=null){
				//current.setPainterOn();
				current.resetPainter();
			}
		} else if (command.equals("autopaint")) {
			autoPaint = ((AbstractButton)e.getSource()).isSelected();
			setupPainter();
		}
    }
	
	private GraphFunction addNewFunction()
	{
	    LinearFunctionPushpin npl = new LinearFunctionPushpin();
		npl.setPainter(new FunctionPainter(variable,variable.getValue()-2, variable.getValue()+2));
		if (autoPaint){
			npl.setPainterOff();
		}
		npl.setShowGhost(showGhosts);
		
		//Mode 2: the ghost has a duration of 1 sec
		npl.getGhost().setDuration(ghostDuration);
		
	    gflist.add(npl);
	    
	    return npl;
	}

	private GraphFunction addCopy(GraphFunction f)
	{
		GraphFunction g;
		g = (GraphFunction)f.getCopy();
		g.setShowGhost(showGhosts);

		//Mode 2: the ghost has a duration of 1 sec
		g.getGhost().setDuration(ghostDuration);

		gflist.add(g);

		return g;
	}

	private void enableGhosts(boolean b)
	{
		int i;
		GraphFunction g;
		for (i=0; i<gflist.size();i++){
			g = (GraphFunction)gflist.elementAt(i);
			g.setShowGhost(b);
		}
	}
	
	private void keepGhosts(boolean b)
	{
		int i;
		GraphFunction g;
		for (i=0; i<gflist.size();i++){
			g = (GraphFunction)gflist.elementAt(i);
			if (b){
				ghostDuration = -1;
			}
			else{
				ghostDuration = 1000;
			}
			g.getGhost().setDuration(ghostDuration);
		}
	}

	private void setupPainter()
	{
		int i;
		for (i=0; i<gflist.size();i++){
			if (gflist.elementAt(i) instanceof LinearFunction){
				LinearFunction l = (LinearFunction)gflist.elementAt(i);
				if (autoPaint){
					l.setPainterOff();
				}
				else{
					l.setPainterOn();
				}
			}
		}
	}
	
	//Testing purposes
    public static void main(String args[]) 
    {
		JFrame frame = new JFrame();
		LinearTransformer fa = new LinearTransformer();
		frame.getContentPane().add(fa);
		frame.setSize(700,450);
		frame.show();
		
		frame.addWindowListener( new WindowAdapter() {
			public void windowClosing(WindowEvent e){
				System.exit(0);
			}			
		});
    }

}
