/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.9 $
 * $Date: 2004/08/18 18:24:58 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.rttapplets.lineartransformer;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import javax.swing.*;
import javax.swing.border.*;
import org.concord.graph.engine.*;
import org.concord.graph.ui.*;

import org.concord.rttapplets.ui.*;
import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;
import org.concord.graphfunction.ui.modelview.*;
import org.concord.graphfunction.ui.symbolicview.*;

/**
 * Main panel for the Linear Transformer Grapher
 *
 * @author Eric Brown-Munoz<p>
*/

public class SimpleLT  extends JPanel 
	implements ActionListener
{
	String interactiveName;

    GraphFunctionList gflist;
	GraphFunctionViewManager functionManager;

    // We need to these available for the button Handlers.
    TabbedLinearListInput inputPanel;
    CardLayout inputPanelLayout;
    JCheckBox keepGhostButton;

	Grid grid;
	GraphWindow graph;
	
	//Initial ghost duration (1 sec)
	int ghostDuration = 1000;
	
	//	SaveAsImageButton saveButton;
	
    // The x variable;
    private Value variable;

	private boolean autoPaint = false;
	private boolean showGhosts = true;

    public SimpleLT() 
	{
		this("RTT Linear Transformer");
	}
	
	public SimpleLT(String interactiveName)
	{

		this.interactiveName = interactiveName;
		JPanel mainPanel = new JPanel();
		JPanel controlPanel = new JPanel();
		JPanel topControls = new JPanel();


		JSplitPane splitPane;

		mainPanel.setLayout(new BorderLayout());
		controlPanel.setLayout(new BoxLayout(controlPanel,BoxLayout.Y_AXIS));
	
		setLayout(new BoxLayout(this,BoxLayout.X_AXIS));
   
		

		variable = new Value();
		variable.setValue(5.0);
		
		gflist = new GraphFunctionList();

		addNewFunction();

		// ------ 
		//Graph
		graph = new GraphWindow();

		grid = new Grid();
		graph.addDecoration(grid);		
	
		final JPanel graphContainer = new JPanel();
	
		graphContainer.setBorder(BorderFactory.createLineBorder(Color.black));
	
		// This ensures the graph is centered on resize
		graph.getDefaultGraphArea().setOriginCentered(true);	

		CoordinateSystem coord = graph.getDefaultGraphArea().getCoordinateSystem();
		coord.setScale(new Point2D.Double(18.0,18.0));
		//((Axis)coord).setFixOriginScreen(true);

		graph.setMouseBehavior(DefaultGraphMouseManager.ONCLICK_SELECT);

		graph.add(gflist);

		
		graph.setVisible(true);
	
		graphContainer.setLayout(new BoxLayout(graphContainer,
							   BoxLayout.Y_AXIS));
		graphContainer.add(graph);

		Trace trace = new Trace(gflist, variable);
		trace.setNumDecimalsXCoordinate(1);
		trace.setNumDecimalsYCoordinate(1);

		graph.add(trace);


	
		// --------------
	
		TenthsValueSlider slider = new TenthsValueSlider(variable);
		slider.sync(graph.getDefaultGraphArea());
		slider.setPaintTicks(false);
		graphContainer.add(slider);
	
		mainPanel.add(graphContainer,BorderLayout.CENTER);
		
		//  -------

		JPanel modelPanel = new JPanel(new BorderLayout());
	    modelPanel.setPreferredSize(new Dimension(50,500));

		ModelWindow model = new ModelWindow();
		model.setGraphArea(graph.getDefaultGraphArea());
		model.setGraphFunctionList(gflist);
		model.setVariable(variable);
		model.setPreferredSize(new Dimension(50,500));

		modelPanel.add(model, BorderLayout.CENTER);

		ImageChooser changeModel = new ImageChooser();
		changeModel.setModel(model);

		modelPanel.add(changeModel, BorderLayout.SOUTH);
		
		// ------
	

		inputPanel = new TabbedLinearListInput();
		inputPanel.setGraphableList(gflist);
		mainPanel.add(inputPanel,BorderLayout.SOUTH);

		// ------------
		topControls.setLayout(new GridLayout(3,2,4,4));
		topControls.setBorder(BorderFactory.createLineBorder(Color.lightGray,4));

		PushpinCheckBox pushpin = new PushpinCheckBox("Pushpin");
		pushpin.setBackground(Color.white);
		pushpin.setGraphableList(gflist);
		topControls.add(pushpin);

		/*JButton pReset = new JButton("Zero Pushpin");
		  pReset.setBackground(Color.white);
		  pReset.setActionCommand("pushpinreset");
		  pReset.addActionListener(this);
		  topControls.add(pReset);
		*/

		JButton repaint = new JButton("Repaint Line");
		repaint.setEnabled(true);
		repaint.setBackground(Color.white);
		repaint.setActionCommand("repaint");
		repaint.addActionListener(this);

		topControls.add(repaint);

		JButton reflectX = new JButton("Reflect X");
		reflectX.setBackground(Color.white);
		reflectX.setActionCommand("reflectx");
		reflectX.addActionListener(this);
		topControls.add(reflectX);
	
		JButton reflectY = new JButton("Reflect Y");
		reflectY.setBackground(Color.white);
		reflectY.setActionCommand("reflecty");
		reflectY.addActionListener(this);
		topControls.add(reflectY);

		controlPanel.add(topControls);

		JPanel ghostPanel = new JPanel();

		ghostPanel.add(new JLabel("Ghosts:"));

		String [] ghostStrings = { "None", "2 Seconds", "Persistent"};
		
		JComboBox ghostCombo = new JComboBox(ghostStrings);
		ghostCombo.setBackground(Color.white);
		ghostCombo.addActionListener(this);
		
		ghostPanel.add(ghostCombo);
		enableGhosts(false);

		controlPanel.add(ghostPanel);

		// ------------
		GraphFunctionTable table = new GraphFunctionTable();
		table.setGraphFunctionList(gflist);
		table.setPreferredSize(new Dimension(200,200));
		controlPanel.add(table);

		// --------
		JPanel bottomControls = new JPanel(new GridLayout(2,2,4,4));
		bottomControls.setBorder(BorderFactory.createLineBorder(Color.lightGray,4));
		JButton createGraph = new JButton ("New");
		createGraph.setActionCommand("new");
		createGraph.setBackground(Color.white);
		createGraph.addActionListener(this);
	
		JButton saveGraph = new JButton("Duplicate");
		saveGraph.setActionCommand("copy");
		saveGraph.setBackground(Color.white);
		saveGraph.addActionListener(this);
	
		JButton deleteGraph = new JButton ("Delete");
		deleteGraph.setActionCommand("delete");
		deleteGraph.setBackground(Color.white);
		deleteGraph.addActionListener(this);

		JButton deleteAllGraph = new JButton ("Delete All");
		deleteAllGraph.setActionCommand("deleteall");
		deleteAllGraph.setBackground(Color.white);
		deleteAllGraph.addActionListener(this);
	
		bottomControls.add(createGraph);
		bottomControls.add(saveGraph);
		bottomControls.add(deleteGraph);
		bottomControls.add(deleteAllGraph);
		// ------------

		controlPanel.add(bottomControls);
		controlPanel.setMinimumSize(new Dimension(0,0));


		JPanel leftPane = new JPanel();
		leftPane.setLayout(new BoxLayout(leftPane, BoxLayout.X_AXIS));
		leftPane.add(modelPanel);
		leftPane.add(mainPanel);

		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, leftPane,
								   controlPanel);
		splitPane.setResizeWeight(1.0);

		add(splitPane);
		// ------ 

	} // Constructor

    public void actionPerformed (ActionEvent e) 
	{

		Object source = e.getSource();

		if (source instanceof JComboBox) {
			String comboString = (String)(((JComboBox)source).getSelectedItem());
			if (comboString.equals("None")) {
				enableGhosts(false);
			} else if (comboString.equals("2 Seconds")) {
				enableGhosts(true);
				keepGhosts(false);
			} else if (comboString.equals("Persistent")) {
				enableGhosts(true);
				keepGhosts(true);
			}
		} else {
				
			String command =((AbstractButton)e.getSource()).getActionCommand();
	
			if (command.equals("copy")) {
				GraphFunction current = (GraphFunction)gflist.getLastSelectedObject();
				if (current!=null){
					addCopy((SnappingLinearFunctionPushpin)current);
				}
			} else if (command.equals("ghost")) {
				enableGhosts(showGhosts);
				keepGhostButton.setEnabled(showGhosts);
			} else if (command.equals("keepghost")) {
				boolean keepGhosts = ((AbstractButton)e.getSource()).isSelected();
				keepGhosts(keepGhosts);
			} else if (command.equals("new")) {
				addNewFunction();
			} else if (command.equals("delete")) {
				gflist.removeAllSelected();
			} else if (command.equals("deleteall")) {
				gflist.removeAllElements();
			} else if (command.equals("reflectx")) {
				LinearFunctionPushpin current = (LinearFunctionPushpin)gflist.getLastSelectedObject();
				if (current!=null){
				
					//				//Mode 1: Activate the ghost for 1 second
					//				current.getGhost().activate(ghostDuration);
					
					//Mode 2: Activate and deactivate the ghost
					//(This is used when if the ghost has a duration already)

					GraphFunctionGhost tghost = current.getGhost();
					if (tghost != null) {
						tghost.activate();
					}

					current.reflectX();

					if (tghost != null) {
						tghost.deactivate();
					}

				}	    
			} else if (command.equals("reflecty")) {
				LinearFunctionPushpin current = (LinearFunctionPushpin)gflist.getLastSelectedObject();
				if (current!=null){
					
					//				//Mode 1: Activate the ghost for 1 second
					//				current.getGhost().activate(ghostDuration);

					//Mode 2: Activate and deactivate the ghost 
					//(This is used when if the ghost has a duration already)

					GraphFunctionGhost tghost = current.getGhost();
					if (tghost != null) {
						tghost.activate();
					}

					current.reflectY();

					if (tghost != null) {
						tghost.deactivate();
					}

				}	    
	
			} else if (command.equals("repaint")) {
				LinearFunctionPushpin current = (LinearFunctionPushpin)gflist.getLastSelectedObject();
				if (current!=null){

					current.setPainterOn();
					current.resetPainter();
				}
			} else if (command.equals("pushpinreset")) {
				LinearFunctionPushpin current = (LinearFunctionPushpin)gflist.getLastSelectedObject();
				if (current!=null){
					current.getPushpin().setXPosition(0.0);
				}
			} else if (command.equals("autopaint")) {
				autoPaint = ((AbstractButton)e.getSource()).isSelected();
			}
		} // instanceof JComboBox
	}
	
	private GraphFunction addNewFunction()
	{
	    LinearFunctionPushpin npl = new SnappingLinearFunctionPushpin();
		npl.setSlopeInterval(.01);

		npl.setShowGhost(showGhosts);
		npl.setPainter(variable,false);
		
		if (showGhosts)
			npl.getGhost().setDuration(ghostDuration);
		
	    gflist.add(npl);
	    
	    return npl;
	}

	private GraphFunction addCopy(SnappingLinearFunctionPushpin f)
	{
		//FIXME This should be cleaned and generalized
		SnappingLinearFunctionPushpin g;
		g = (SnappingLinearFunctionPushpin)f.getCopy();

		g.setSlopeInterval(.01);
		g.setShowGhost(showGhosts);
		g.setPainter(variable,false);

		if (showGhosts)
			g.getGhost().setDuration(ghostDuration);

		gflist.add(g);

		return g;
	}

	private void enableGhosts(boolean b)
	{
		int i;
		GraphFunction g;

		showGhosts = b;

		for (i=0; i<gflist.size();i++){
			g = (GraphFunction)gflist.elementAt(i);
			g.setShowGhost(b);
		}
	}
	
	private void keepGhosts(boolean b)
	{
		int i;
		GraphFunction g;
		for (i=0; i<gflist.size();i++){
			g = (GraphFunction)gflist.elementAt(i);
			if (b){
				ghostDuration = -1;
			}
			else{
				ghostDuration = 1000;
			}
			g.getGhost().setDuration(ghostDuration);
		}
	}

	
	//Testing purposes
    public static void main(String args[]) 
    {

		SimpleLT lt = new SimpleLT();
		RTTFrame frame = new RTTFrame(lt, "Linear Transformer");
		frame.show();
    }

}
