/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/08/02 17:42:15 $
 * $Author: jqian $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.rttapplets.piecewise;

import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.awt.geom.*;

import javax.swing.*;
import org.concord.graph.engine.*;
import org.concord.graph.ui.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;
import org.concord.graphfunction.ui.*;
import org.concord.graphfunction.ui.modelview.*;
import org.concord.graphfunction.ui.symbolicview.*;

/**
 * Main panel for the Linear Transformer Grapher
 *
 * @author Ingrid Moncada<p>
 * @author Eric Brown-Munoz<p>
*/

public class PiecewisePanel extends JPanel 
	implements ActionListener 
{
	String interactiveName;
	
	GraphFunctionList gflist;
	GraphFunctionViewManager functionManager;
	
	Grid grid;
	GraphWindow graph;
	private AxisScale axisScale;
	
	JToggleButton scaleButtonX;
	JToggleButton scaleButtonY;
//	SaveAsImageButton saveButton;
	
	public PiecewisePanel() 
	{
		this("Piecewise Linear Grapher");
	}
	
	public PiecewisePanel(String interactiveName)
	{
		this.interactiveName = interactiveName;
	
		// Set Contraints for GridBagLayout for all panels
		GridBagConstraints graphConstraints = new GridBagConstraints();
		graphConstraints.gridx = 1;
		graphConstraints.gridy = 0;
		graphConstraints.gridwidth = 2;
		graphConstraints.gridheight = 3;
		graphConstraints.fill = GridBagConstraints.BOTH;
		graphConstraints.weightx = 100;
		graphConstraints.weighty = 100;
		
		GridBagConstraints controlsConstraints = 
			(GridBagConstraints)graphConstraints.clone();
		controlsConstraints.gridx = 3;
		controlsConstraints.gridwidth = 1;
		controlsConstraints.gridheight = 4;
		controlsConstraints.weightx = 1;
		controlsConstraints.weightx = 1;
		
		GridBagConstraints modelConstraints = 
			(GridBagConstraints)graphConstraints.clone();
		modelConstraints.gridx = 0;
		modelConstraints.gridwidth = 1;
		modelConstraints.gridheight = 3;
		modelConstraints.fill = GridBagConstraints.BOTH;
		modelConstraints.weightx = 20;
		
		GridBagConstraints buttonConstraints = 
			(GridBagConstraints)graphConstraints.clone();
		
		buttonConstraints.gridx = 0;
		buttonConstraints.gridy = 3;
		buttonConstraints.gridwidth = 3;
		buttonConstraints.gridheight = 1;
		buttonConstraints.fill = GridBagConstraints.HORIZONTAL;
		buttonConstraints.weighty = 0;
		buttonConstraints.anchor = GridBagConstraints.SOUTH;
		
		gflist = new GraphFunctionList();
		functionManager = new GraphFunctionViewManager();
		functionManager.setGraphFunctionList(gflist);
		
		addNewPiecewise();
		
		JPanel controls = new JPanel();
		controls.setLayout(new BoxLayout(controls, BoxLayout.Y_AXIS));
		
		LinearPiecewiseSymbolic psymb = new LinearPiecewiseSymbolic();
		functionManager.addGraphFunctionView(psymb);
		//psymb.setGraphFunctionList(gflist);
		controls.add(psymb);
		
		GraphFunctionTable gtable = new GraphFunctionTable();
		gtable.setGraphFunctionList(gflist);
		controls.add(gtable);
		
		JPanel graphPanel = new JPanel();
		graphPanel.setLayout(new BoxLayout(graphPanel, BoxLayout.Y_AXIS));
		
		////////////////////////////////////////////////////////////////////////////////
		//Graph
		graph = new GraphWindow();
		graph.setAntialias(true);

		graph.getDefaultGraphArea().setOriginCentered(false);

		CoordinateSystem coord = graph.getDefaultGraphArea().getCoordinateSystem();
		coord.setScale(new Point2D.Double(7.0,7.0));

		grid = new Grid();
		graph.addDecoration(grid);		

		grid.setLabelFormat(new DecimalFormat("#"));
		grid.setLabelSize(10.0f);
		grid.setInterval(5.0,5.0);
		
		graph.add(gflist);
		
		graphPanel.add(graph);
		
		//////////////////////////////////////////////////////////////
		//Buttons
		
		JButton deleteSegment= new JButton("Delete Piece");
		deleteSegment.setActionCommand("delsegment");
		deleteSegment.addActionListener(this);
		deleteSegment.setToolTipText("Delete the selected piece");
		
		JButton newPiecewise= new JButton("New Function");
		newPiecewise.setActionCommand("create");
		newPiecewise.addActionListener(this);
		newPiecewise.setToolTipText("Create a new Piecewise Function");
		
		JButton deletePiecewise= new JButton("Delete Function");
		deletePiecewise.setActionCommand("delete");
		deletePiecewise.addActionListener(this);
		deletePiecewise.setToolTipText("Delete the selected Piecewise Function and all its pieces");
		
		JButton deleteAll= new JButton("Delete All");
		deleteAll.setActionCommand("deleteall");
		deleteAll.addActionListener(this);
		deleteAll.setToolTipText("Delete all Functions");
		
		scaleButtonX = new JToggleButton("Change X Scale");
		scaleButtonX.setActionCommand("scalex");
		scaleButtonX.addActionListener(this);
		scaleButtonY = new JToggleButton("Change Y Scale");
		scaleButtonY.setActionCommand("scaley");
		scaleButtonY.addActionListener(this);
		
		JButton changeLabelX = new JButton("Label X Axis");
		changeLabelX.setActionCommand("labelx");
		changeLabelX.addActionListener(this);
		JButton changeLabelY = new JButton("Label Y Axis");
		changeLabelY.setActionCommand("labely");
		changeLabelY.addActionListener(this);
		
		JPanel buttons = new JPanel();
		GridLayout gridLay = new GridLayout(5,2);
		buttons.setLayout(gridLay);
		
//		JPanel saveButtonPanel = new JPanel();
//		saveButtonPanel.setLayout(new BorderLayout());
//		saveButton = new SaveAsImageButton(this,interactiveName);
		//saveButton.setAlignmentX(RIGHT_ALIGNMENT);
//		saveButtonPanel.add(saveButton,BorderLayout.EAST);
		
		//gridLay.setVgap(4);
		//gridLay.setHgap(5);
		buttons.add(deleteSegment);
		buttons.add(scaleButtonX);
		buttons.add(newPiecewise);
		buttons.add(scaleButtonY);
		buttons.add(deletePiecewise);
		buttons.add(changeLabelX);
		buttons.add(deleteAll);
		buttons.add(changeLabelY);
		buttons.add(Box.createRigidArea(new Dimension(10,10)));
//		buttons.add(saveButtonPanel);
		
		buttons.setSize(265,145);
		buttons.setPreferredSize(new Dimension(265,145));
		buttons.setMaximumSize(new Dimension(265,145));
		
		controls.add(buttons);
		
		final Value variable = new Value();
		
		variable.addValueListener(new ValueListener() 
		{
			public void valueChanged(ValueEvent e) {
		
				if (!gflist.checkFunctionOnce(variable.getValue())){
					try {
						messageDialog(graph,"Because there is more than one y-value for at least \none x-value, "+
							"this is not the graph of a function.  You \nmay have to change the domain for one or more \nsegments.",
							  "Not a function",
							  JOptionPane.ERROR_MESSAGE);
					} catch (Exception re) {
						repaint();
					}
				}
			}
		});
				
		Trace trace = new Trace (gflist, variable);
		graph.add(trace);
		
		final ValueSlider slider = new ValueSlider(variable);
		graphPanel.add(slider);
		
		//Scaling object
		axisScale = new AxisScale();
		axisScale.setDragMode(AxisScale.DRAGMODE_NONE);

		graph.add(axisScale);
		//

		graph.setVisible(true);
		
		ModelWindow model = new ModelWindow();
		model.setGraphFunctionList(gflist);
		//model.setAxis(graph.graphAxis);
		model.setVariable( variable);
		model.setGraphArea(graph.getDefaultGraphArea());
		
		LinearPiecewiseToolBox bpanel = new LinearPiecewiseToolBox();
		bpanel.setGraphFunctionList(gflist);
		bpanel.syncGrid(grid);
			
		setLayout(new GridBagLayout());
		add(graphPanel,graphConstraints);
		add(controls,controlsConstraints);
		add(bpanel,buttonConstraints);
		add(model,modelConstraints);
		
		validate();
	
	}
	
	public void actionPerformed(ActionEvent e) 
	{
		String action = e.getActionCommand();
		if (action.equals("create")) {
			addNewPiecewise();
            System.out.println ("entered");
		}
		else if (action.equals("delsegment")) {
			LinearPiecewise current = (LinearPiecewise)gflist.getLastSelectedObject();
			if (current!=null){
				current.removeSelected();
			}
		}
		else if (action.equals("delete")) {
			gflist.removeAllSelected();
		}
		else if (action.equals("deleteall")) {
			gflist.removeAllElements();
		}
		else if (action.equals("scalex")) {
			if (((JToggleButton)e.getSource()).isSelected()){
				axisScale.setDragMode(AxisScale.DRAGMODE_DILATE_X);
				scaleButtonY.setSelected(false);
			}
			else{
				axisScale.setDragMode(AxisScale.DRAGMODE_NONE);
			}
		}			
		else if (action.equals("scaley")) {
			if (((JToggleButton)e.getSource()).isSelected()){
				axisScale.setDragMode(AxisScale.DRAGMODE_DILATE_Y);
				scaleButtonX.setSelected(false);
			}
			else{
				axisScale.setDragMode(AxisScale.DRAGMODE_NONE);
			}
		}			
		else if (action.equals("labelx")) {
			String axisLabel;
			axisLabel = grid.getLabelXAxis();
			axisLabel = String.valueOf(JOptionPane.showInputDialog(null,
				"X Axis Label:", "Change X Axis Label",JOptionPane.QUESTION_MESSAGE,null,null,axisLabel));
			grid.setLabelXAxis(axisLabel);
			graph.repaint();
		}
		else if (action.equals("labely")) {
			String axisLabel;
			axisLabel = grid.getLabelYAxis();
			axisLabel = String.valueOf(JOptionPane.showInputDialog(null,
				"Y Axis Label:", "Change Y Axis Label",JOptionPane.QUESTION_MESSAGE,null,null,axisLabel));
			grid.setLabelYAxis(axisLabel);
			graph.repaint();
		}
	}
	
	private void addNewPiecewise()
	{
		// Set up a Piecewise
		LinearPiecewise pw = new LinearPiecewise();
		// Set up a Piecewise Segment
		LinearSegment mygraph = new LinearSegment(0,5);
		pw.add(mygraph);
	
		gflist.add(pw);
		pw.select();
	}
	
	//Testing purposes
	public static void main(String args[]) 
	{
		JFrame frame = new JFrame();
		PiecewisePanel fa = new PiecewisePanel();
		frame.getContentPane().add(fa);
		frame.setSize(700,500);
		frame.show();
	}
	
	void messageDialog(final Component c, final String message, final String title, final int type)
		throws Exception
	{

		Runnable modaldialog = new Runnable () 
		{
			public void run() {
				JOptionPane.showMessageDialog(c, message, title, type);
			}
		};
	
		SwingUtilities.invokeLater(modaldialog);
	}
    
}
