/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
 * $Revision: 1.2 $
 */

package org.concord.rttapplets.piecewise;

import java.util.*;

import org.concord.graph.engine.*;
import org.concord.graphfunction.engine.*;
import org.concord.graph.event.*;
import org.concord.graphfunction.event.*;


public class SegmentAutoSelect 
	implements ValueListener
{

	private GraphFunctionList gflist;
	private Value variable;

	public SegmentAutoSelect () {
	}

	public SegmentAutoSelect (GraphFunctionList list, Value variable) {
		setGraphFunctionList(list);
		setValue(variable);
	}

	public void setGraphFunctionList ( GraphFunctionList list) {
		gflist = list;
	}

	public void setValue ( Value value) {
		if (variable != null) 
			variable.removeValueListener(this);
		variable = value;
		value.addValueListener(this);
	}

	public void valueChanged(ValueEvent e) {
		if ((gflist != null) && (variable != null) )
			{
				double value = variable.getValue();
				GraphFunction wasSelected = 
					(GraphFunction)gflist.getFirstSelectedObject();

				Enumeration graphFunctions = gflist.elements();

				while (graphFunctions.hasMoreElements()) {
					GraphFunction next = 
						(GraphFunction)graphFunctions.nextElement();
					if (next instanceof Piecewise) 
						{
							Segment segment = 
								((Piecewise)next).getSegment(value);
							if (segment != null)
								segment.select();
						}
				}

				if (wasSelected != null) 
					wasSelected.select();

			}
				

	}
		
}

