/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.7 $
 * $Date: 2004/08/18 18:24:58 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.rttapplets.piecewise;

import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.awt.geom.*;

import javax.swing.*;
import org.concord.graph.engine.*;
import org.concord.graph.ui.*;

import org.concord.rttapplets.ui.*;
import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.event.*;
import org.concord.graphfunction.ui.*;
import org.concord.graphfunction.ui.modelview.*;
import org.concord.graphfunction.ui.symbolicview.*;

/**
 * Main panel for the Linear Transformer Grapher
 *
*/

public class SimpleLPG extends JPanel 
	implements ActionListener,
			   ComponentListener
{
	String interactiveName;
	
	GraphFunctionList gflist;
	
	Grid grid;
	GraphWindow graph;

	GraphArea ga;

	double initialScaleX = 22.0;
	double initialScaleY = 22.0;
	private AxisScale axisScale;

	private Value variable;

	private CoordinateSystem cs;

	LinearPiecewise pw1;
	LinearPiecewise pw2;
	LinearPiecewise pw3;
	LinearPiecewise pw4;

	ScaleControl scaleDialog;
	AxisLabelControl labelDialog;

	SegmentAutoSelect autoSelect;

	JToggleButton scaleButtonX;
	JToggleButton scaleButtonY;

	private boolean dialogUp = false; // When true, block further dialogs


	//	SaveAsImageButton saveButton;
	
	public SimpleLPG() 
	{
		this("RTT Piecewise Linear Grapher");
	}
	
	public SimpleLPG(String interactiveName)
	{
		this.interactiveName = interactiveName;

		JSplitPane splitPane;
		JPanel mainPanel = new JPanel();
		JPanel controlPanel = new JPanel();
		JPanel modelPanel = new JPanel();

		// This makes sure the axis is OK on resize (a kludge to
		// get around graph weirdness).
		addComponentListener(this);

		gflist = new GraphFunctionList();
		
		pw1 = addNewPiecewise();
		pw2 = addNewPiecewise();
		pw2.setVisible(false);
		pw3 = addNewPiecewise();
		pw3.setVisible(false);
		pw4 = addNewPiecewise();
		pw4.setVisible(false);
		
		controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.Y_AXIS));


		SegmentInputPane segmentInput = new SegmentInputPane();
		segmentInput.setGraphFunctionList(gflist);

		segmentInput.setGraphFunctionList(gflist);

		
		LinearPiecewiseSymbolic psymb1 = new LinearPiecewiseSymbolic();
		psymb1.setGraphFunction(pw1);
		controlPanel.add(psymb1);
		psymb1.setInputWindow(segmentInput);

		LinearPiecewiseSymbolic psymb2 = new LinearPiecewiseSymbolic();
		psymb2.setGraphFunction(pw2);
		controlPanel.add(psymb2);
		psymb2.setInputWindow(segmentInput);

		LinearPiecewiseSymbolic psymb3 = new LinearPiecewiseSymbolic();
		psymb3.setGraphFunction(pw3);
		controlPanel.add(psymb3);
		psymb3.setInputWindow(segmentInput);

		LinearPiecewiseSymbolic psymb4 = new LinearPiecewiseSymbolic();
		psymb4.setGraphFunction(pw4);
		controlPanel.add(psymb4);
		psymb4.setInputWindow(segmentInput);

		JPanel buttonPane = new JPanel(new GridLayout(2,2));
  
		scaleButtonX = new JToggleButton("X Scale");
		scaleButtonX.setBackground(Color.white);
		scaleButtonX.setActionCommand("scalex");
		scaleButtonX.addActionListener(this);
		scaleButtonY = new JToggleButton("Y Scale");
		scaleButtonY.setBackground(Color.white);
		scaleButtonY.setActionCommand("scaley");
		scaleButtonY.addActionListener(this);


		JButton labelButton = new JButton("Axis Labels");
		labelButton.addActionListener(this);
		labelButton.setActionCommand("label");
		labelButton.setBackground(Color.white);

		scaleButtonX.setPreferredSize(new Dimension(100,10));
		scaleButtonY.setPreferredSize(new Dimension(100,10));
		labelButton.setPreferredSize(new Dimension(100,10));

		buttonPane.add(scaleButtonX);
		buttonPane.add(scaleButtonY);
		buttonPane.add(labelButton);
		controlPanel.add(buttonPane);

		
		JPanel graphPanel = new JPanel();
		graphPanel.setLayout(new BoxLayout(graphPanel, BoxLayout.Y_AXIS));
		
		// --------
		//Graph
		graph = new GraphWindow();
		graph.setAntialias(true); // FIXME

		ga = graph.getDefaultGraphArea();

		ga.setOriginCentered(false);

		cs = ga.getCoordinateSystem();
		cs.setScale(new Point2D.Double(7.0,7.0));

		grid = new Grid();
		graph.addDecoration(grid);		

		//grid.setLabelFormat(new DecimalFormat("#"));
		//grid.setLabelSize(10.0f);
		//grid.setSmartInterval(true);
		grid.setInterval(5.0,5.0);

		graph.add(gflist);
		
		graphPanel.add(graph);
		
		//////////////////////////////////////////////////////////////
		//Buttons
		
		JButton deleteSegment= new JButton("Delete");
		deleteSegment.setActionCommand("delsegment");
		deleteSegment.addActionListener(this);
		deleteSegment.setToolTipText("Delete the selected piece");
		
		
		//-----
		variable = new Value(1.0);
		variable.addValueListener(new ValueListener() 
			{
				public synchronized void valueChanged(ValueEvent e) {
		
					if (!gflist.checkFunctionOnce(variable.getValue())){
						try {
							if (!dialogUp ) {
								dialogUp = true;
								messageDialog(graph,"There is more than one y-value for at least \none x-value. "+
											  "This is not the graph of a function.  You \nmay have to change the domain for one or more \nsegments.",
											  "Not a function",
											  JOptionPane.ERROR_MESSAGE);
							}
						} catch (Exception re) {
							repaint();
						}
					} else {
						dialogUp= false;
					}
				} // valueChanged
			});

		// ---- A Class to automatically select segments based on trace
		autoSelect = 
			new SegmentAutoSelect(gflist,variable);
	

		// ---- The Trace and Slider
		Trace trace = new Trace (gflist, variable);
		graph.add(trace);
		
		TenthsValueSlider slider = new TenthsValueSlider(variable);
        slider.setPaintTicks (false);
		slider.sync(ga);
		graphPanel.add(slider);
		
		graph.setVisible(true);

		// ---- Axis Control
		axisScale = new AxisScale();
		axisScale.setDragMode(AxisScale.DRAGMODE_NONE);
		graph.add(axisScale);
		
		ModelWindow model = new ModelWindow();
		model.setGraphFunctionList(gflist);

		model.setVariable( variable);
		model.setGraphArea(ga);

		modelPanel.setLayout(new BorderLayout());

		modelPanel.setMaximumSize(new Dimension(110,800));

		modelPanel.add(model, BorderLayout.CENTER);

		ImageChooser modelChange = new ImageChooser();
		modelChange.setModel(model);
		modelChange.setPreferredSize(new Dimension(110,50));
		modelPanel.add(modelChange,BorderLayout.SOUTH);
		
		LinearPiecewiseToolBox bpanel = new LinearPiecewiseToolBox();
		bpanel.setGraphFunctionList(gflist);
		bpanel.syncGrid(grid);
		bpanel.add(deleteSegment);
			
		setLayout(new BorderLayout());

		mainPanel.setLayout(new BorderLayout());

		mainPanel.add(graphPanel, BorderLayout.CENTER);
		mainPanel.add(bpanel, BorderLayout.SOUTH);


		JPanel leftPane = new JPanel();
		leftPane.setLayout(new BoxLayout(leftPane, BoxLayout.X_AXIS));
		leftPane.add(modelPanel);
		leftPane.add(mainPanel);

		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, 
								   leftPane, controlPanel);
		splitPane.setResizeWeight(1.0);

		setLayout(new BorderLayout());

		add(splitPane);

		pw1.select();


		scaleDialog = new ScaleControl(ga);
		scaleDialog.hide();
		scaleDialog.sync(graph);

		labelDialog = new AxisLabelControl(grid);
		labelDialog.hide();
		labelDialog.sync(graph);

		validate();
	
	} // Constructor
	
	public void actionPerformed(ActionEvent e) 
	{
		String action = e.getActionCommand();
		if (action.equals("create")) {
			addNewPiecewise();
		}
		else if (action.equals("delsegment")) {
			LinearPiecewise current = (LinearPiecewise)gflist.getLastSelectedObject();
			if (current!=null){
				current.removeSelected();
			}
		}
		else if (action.equals("delete")) {
			gflist.removeAllSelected();
		}
		else if (action.equals("scale")) {
			scaleDialog.show();
		}

		else if (action.equals("scalex")) {
			if (((JToggleButton)e.getSource()).isSelected()){
				axisScale.setDragMode(AxisScale.DRAGMODE_DILATE_X);
				scaleButtonY.setSelected(false);
			}
			else{
				axisScale.setDragMode(AxisScale.DRAGMODE_NONE);
			}
		}			
		else if (action.equals("scaley")) {
			if (((JToggleButton)e.getSource()).isSelected()){
				axisScale.setDragMode(AxisScale.DRAGMODE_DILATE_Y);
				scaleButtonX.setSelected(false);
			}
			else{
				axisScale.setDragMode(AxisScale.DRAGMODE_NONE);
			}
		}			

		else if (action.equals("label")) {
			labelDialog.show();
		}
	}
	
	private LinearPiecewise addNewPiecewise()
	{
		// Set up a Piecewise
		LinearPiecewise pw = new LinearPiecewise();
		// Set up a Piecewise Segment
		LinearSegment mygraph = new LinearSegment(0,5);
		pw.add(mygraph);
	
		gflist.add(pw);
		pw.select();

		return pw;
	}
	
	//Testing purposes
	public static void main(String args[]) 
	{
		javax.swing.SwingUtilities.invokeLater(new Runnable () {
				public void run() {
					SimpleLPG lpg = new SimpleLPG();

					RTTFrame frame = new RTTFrame(lpg, "Piecewise Linear Grapher");
				
					frame.show();
				}
			});
			
	}
	
	void messageDialog(final Component c, final String message, final String title, final int type)
		throws Exception
	{

		Runnable modaldialog = new Runnable () 
		{
			public void run() {
				JOptionPane.showMessageDialog(c, message, title, type);
			}
		};
	
		SwingUtilities.invokeLater(modaldialog);
	}

	public void componentHidden(ComponentEvent e) {}

	public void componentMoved(ComponentEvent e) {}

	public void componentResized(ComponentEvent e) {
		double height = (double)graph.getHeight();
		cs.setOriginOffsetDisplay(new Point2D.Double(30.0, height - 30.0));
	}

	public void componentShown(ComponentEvent e) {

	}

    
}
