/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
   $Revision: 1.2 $
*/

package org.concord.rttapplets.qgrapher;

import java.awt.*;
import java.util.EventObject;
import javax.swing.*;
import javax.swing.event.*;
import org.concord.graphfunction.engine.Value;
import org.concord.graph.engine.GraphArea;
import org.concord.graph.event.GraphableAdapter;

public class AnimateTrace 
	implements ChangeListener,
			   Runnable
{
	public static int STOPPED = 0;
	public static int PAUSED = 1;
	public static int RUNNING = 2;

	public Value value = null;
	public int status = STOPPED;
	GraphArea sync = null;

	QualGraphable syncGraph = null;

	Thread animate = null;

	double referenceTime = 0.0; // Seconds on clock when current thread started
	long   systemTime;     // last read of systemClock (in milliseconds)
	double endTime = Double.POSITIVE_INFINITY; // How many seconds for the animation (as set)
	double stopTime; // The real time to stop (as determined by logic)

	
	public AnimateTrace() {
	}

	public AnimateTrace(Value value) {
		this.value = value;
	}

	public void setEndTime(double time) {
		endTime = time;
	}

	public void setSyncGraph(QualGraphable qgraph) {
		syncGraph = qgraph;
		qgraph.addGraphableListener ( new GraphableAdapter () { // FIXME
				public void graphableChanged(EventObject e) {
					double graphend = syncGraph.getEnd();
					if (graphend < endTime) {
						stopTime = graphend;
					} else {
						stopTime = endTime;
					}
				}
			});
	}

	/** Get the end time from the size of the GraphArea. Pass in
	    a null to stop syncing. */
	public void syncToGraphArea (GraphArea ga) {
		if (ga != null) {
			ga.removeChangeListener(this);
		}

		sync = ga;
		ga.addChangeListener(this);
		setEndTime(sync.getUpperRightCornerWorld().getX());

	}

	public double getEndTime () {
		return endTime;
	}

	public void start () {
		if (syncGraph != null) {
			double graphend = syncGraph.getEnd();
			if (graphend < endTime) {
				stopTime = graphend;
			} else {
				stopTime = endTime;
			}
		} else {
			stopTime = endTime;
		}

		if (status == STOPPED ) {
			referenceTime = 0.0;
			animate = new Thread(this);
			animate.start();
		} else if (status == RUNNING) {
			// do nothing
		} else if (status == PAUSED) {
			animate = new Thread(this);
			animate.start();
		}
	}

	public void pause () {
		if (status == RUNNING) {
			status = PAUSED;
		} else if (status == PAUSED) {
			start();
		}
	}

	public void stop () {
		status = STOPPED;
	}

	public synchronized void run() {

		double time = 0.0;

		if (value != null ) {

			systemTime = System.currentTimeMillis();
			status = RUNNING;

			while(status == RUNNING) {
				time = referenceTime + (double) (System.currentTimeMillis() -
												 systemTime) / 1000.0;
				value.setValue(time);

				if (true) {
					try {
						Thread.sleep(50);
					} catch (InterruptedException e) {
						System.err.println("Sleep interrupted");
					}

					if (time > stopTime) {
						status = STOPPED;
					}
				}
			}

			referenceTime = time;
		}

	}

	public void stateChanged( ChangeEvent e) {
		// Sets the end time to the rightmost value of the current GraphArea
		if (sync != null) {
			setEndTime(sync.getUpperRightCornerWorld().getX());
		}
	}
		

		
}
