/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


package org.concord.rttapplets.qgrapher;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.geom.*;

import java.net.URL;

public class ChartModel extends JPanel implements Model, ImageObserver {

    int number; // The official value to chart
    Image objImage;
    int numberOfObjects = 10;

    int objWidth;
    int objHeight;

    int margin = 10; // How far from the top or bottom
    int screenWidth;
    int screenHeight;
    int yinterval; // How far from top of one object to the top of other
                   // is this negative?? 
                   //= - (screenHeight - 2*margin - objHeight)/ (N - 1)

    int ystart; //Where the first object goes
    int xpos; // a position to put object in the center


    Graphics2D g2; // The graphics of this component
    Image doubleBuffer; // set up with redraw event handler
    Graphics2D gg; // or just a Graphics?

    Color backgroundColor = Color.blue;

    public ChartModel () {
	this("orca.gif");
    }

    public ChartModel(String graphic) {

	if (graphic == null) {
	    graphic = "orca.gif";
	}
	try {
	    objImage = loadImage(graphic);
	} catch (Exception e) {
	    objImage = loadImage("orca.gif");
	}

	objHeight = objImage.getHeight(this);
	objWidth = objImage.getWidth(this);

	this.addComponentListener(new ComponentAdapter() {
		public void componentResized(ComponentEvent e) {
		    resize();
		}
	    });
    }
	

    private synchronized void resize() {
	screenHeight = getHeight();
	if (screenHeight > 0) {
	    screenWidth = getWidth();
	    xpos = (screenWidth - objWidth) / 2;
	    ystart = screenHeight - margin - objHeight;
	    yinterval = -(ystart - margin) / (numberOfObjects - 1);
	    g2 = (Graphics2D)getGraphics();
	    doubleBuffer = createImage(screenWidth, screenHeight);
	    gg = (Graphics2D) doubleBuffer.getGraphics();
	}
    }

    public void paint (Graphics g) {
	Graphics2D g2 = (Graphics2D)g;

	gg.setColor(backgroundColor);
	gg.fillRect(0,0,screenWidth, screenHeight);// Background solid color for now.

	for (int i = 0 ; i < number; i++) {
	    gg.drawImage(objImage, xpos, ystart+i*yinterval, this);
	}

	g2.drawImage(doubleBuffer,0,0,this); //put on screen with no clipping.

    }

    public synchronized void move(double value) {
	int newnumber = (int) (value + 0.5); // round to nearest
	Rectangle clipRect;

	if ( number != newnumber) {
	    // Think about threads here?
	    if (newnumber > number) { // we are adding objects

		clipRect = 
		    new Rectangle(xpos,
				  ystart + (newnumber-1)*yinterval,
				  objWidth,
				  (newnumber - number)* (-yinterval) + 
				  objHeight);

		for (int i = number; i < newnumber; i++) {
		    gg.drawImage(objImage, xpos, ystart+i*yinterval,this);
		}

	    } else {

		// Covers all the objects slated for deletion
		clipRect = 
		    new Rectangle(xpos,
				  ystart + (number-1) * yinterval,
				  objWidth,
				  (number - newnumber) * (-yinterval) + objHeight);
		
		gg.setColor(backgroundColor);
		gg.fill( clipRect);
		
		for (int i = newnumber -1 + objHeight/yinterval; i < newnumber; i++) {
		    if (i >= 0) {
			gg.drawImage(objImage, xpos, ystart+i*yinterval,this);
		    }
		}

	    }

	    number = newnumber;
	    
	    Shape saveClip = g2.getClip();
	    
	    g2.setClip(clipRect);
	    
	    g2.drawImage(doubleBuffer,0,0,this);

	    g2.setClip(saveClip);
	} // if (number != newnumber
    
    }

    public void setOrientation (int dummy) {
	// For now do nothing
    }

    public void setImage(String name) {
	objImage = loadImage(name);
	objHeight = objImage.getHeight(this);
	objWidth = objImage.getWidth(this);

	resize();
    }


    private Image loadImage(String name) {
	Image retval;
	URL url = Model.class.getResource(name); 
	retval=Toolkit.getDefaultToolkit().getImage(url);
	MediaTracker tracker = new MediaTracker(this);
	tracker.addImage(retval,0);
	try {
	    tracker.waitForAll();
	} catch (Exception e) {
	    System.err.println("Problem loading Image");
	    retval = null;
	} 
	return retval;
    }


}
	
