/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz

   A Graphable object for use with the CC Graph
   package. Based on the QGrapher Applet.
*/

/** The QualGraphable is a continuous piecewise Graph.
	It consists of "handles" that are moveable connected
	by "segments" that are either straight lines or
	quadratic curves. 
*/

package org.concord.rttapplets.qgrapher;

import java.awt.*;
import java.awt.geom.*;
import java.util.Vector;

import org.concord.graph.engine.*;

public class Handle
{

    Point2D.Double position = new Point2D.Double(); 
    boolean visible;
    QualGraphable parent;
	int size = 9;
	double sensitivity= 4.0; // How far can a mouse click be (display coords)


	private Point2D.Double transPosition = new Point2D.Double();

	public Handle() {
	}

	public Handle(QualGraphable parent, Point2D.Double position) {
		setParent(parent);
		setPosition(position);
	}

	public void setParent(QualGraphable parent) {
		this.parent = parent;
	}

	public QualGraphable getParent() {
		return parent;
	}

	public void setPosition (Point2D.Double position) {
		this.position.x = position.x;
		this.position.y = position.y;
	}

	public Point2D.Double getPosition () {
		return position;
	}

	public double getX() {
		return position.x;
	}

	public double getY() {
		return position.y;
	}

	public synchronized void draw (Graphics2D g) {
		CoordinateSystem cs = parent.getCoordinateSystem();
		int offset = size/2;

		cs.transformToDisplay(position,transPosition);

		g.fillRect((int)transPosition.x - offset, 
				   (int)transPosition.y - offset,
				   size, size);

	}

	public boolean isPointInProximity (Point location) {
		double dist = location.distance(transPosition);
		return (dist <= sensitivity);
	} 
		

}
