/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/* Author: Eric Brown-Munoz
   Date  : August 15,2003

   This implements the Model class which is a part of the
   QGrapher applet.

   Copyright (C) 2003 Eric Brown-Munoz

   This program is free software; you can redistribute it and/or modify 
   it under the terms of the GNU General Public License as published by 
   the Free Software Foundation; either version 2 of the License, or 
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

$Revision: 1.1 $
*/
package org.concord.rttapplets.qgrapher;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.URL;

import org.concord.graph.engine.*;

import org.concord.graph.ui.*;


public class ObjectModel extends JPanel implements Model, ImageObserver {
    private double originx;
    private double originy = 30.0;
    private double posy;

    private double position = 0.0;
    private double increment;

    private int orientation; // 0 - Vertical, 1 - Horizontal
    public static final int Vertical = 0;
    public static final int Horizontal = 1;

    private double objectWidth=30.0;
    private double objectHeight=30.0;

    private MediaTracker tracker;
    private Image Background;
    private Image object;

    private boolean centerx = true;
    private boolean centery = false;
    private boolean setIncrement=false;

    private GraphArea syncga;

    private Rectangle2D.Double drawnRect;

    public ObjectModel() {

		addComponentListener (new ComponentAdapter () {
				public void componentResized (ComponentEvent e) {

					if ( Background == null) {
						Background = createImage (getWidth(),getHeight());
						Graphics g = Background.getGraphics();
						g.setColor(ObjectModel.this.getBackground());
						g.fillRect(0,0,getWidth(),getHeight());
					}
		    
					if (object != null) {
						objectWidth = object.getWidth(ObjectModel.this);
						objectHeight = object.getHeight(ObjectModel.this);
					}

					if (centerx) {
						originx = (double)(getWidth()-objectWidth)/2.0;
					}

					if (centery) {
						originy = (double)(getHeight()-objectHeight)/2.0;
					}

					posy = getHeight() - originy;

					if (orientation == Vertical) {
						increment = (getHeight() - 2 * originy)/10.0;
					} else {
						increment = (getWidth() - 2*originx)/10.0;
					}

					doSync();

				}
			});

    }

    public void setOrientation (int orient) {

	if ((orient == Vertical) || (orient == Horizontal)) {
	    if (orient == Vertical) {
		centery = false;
		if (centerx) {
		    originx = (double)(getWidth()-objectWidth)/2.0;
		}

		if (!setIncrement) {
		    increment = (getHeight() - 2 * originy)/10.0;
		}
	    }

	    if (orient == Horizontal) {
		centerx = false;
		if (centery) {
		    originy = (double)(getHeight()-objectHeight)/2.0;
		}

		if (!setIncrement) {
		    increment = (getWidth() - 2 * originx)/10.0;
		}

	    }
	    
	    orientation = orient;

	    posy = getHeight() - originy;

	}
    }

    public void setBackground (String background) {
	this.Background = loadImage(background); // XXXX
    }

    public void setImage (String name) {
		object = loadImage(name);
		if (centerx) {
			originx = Math.abs((getWidth() - object.getWidth(this))/2.0);
		}
		
		if (centery) {
			originy = Math.abs((getHeight() - object.getHeight(this))/2.0);
		}
    }

    public void paintComponent (Graphics g){

		super.paintComponent( g);

		Graphics2D g2 = (Graphics2D)g;

		g2.drawImage(Background,0,0,this);
	
		Rectangle2D.Double newRect = getRect();

		Shape savedclip = g2.getClip();
		if (drawnRect != null) {
			g2.setClip(newRect.createUnion(drawnRect).getBounds2D());
		}

		if (object == null) {
			g2.fill(newRect);
		} else {

			if (orientation == Vertical) {

				System.out.println("draw: posy " + posy);
				System.out.println("draw: increment " + increment);
				System.out.println("draw: position " + position);
									   

				g2.drawImage(object,
							 (int)originx,
							 (int)(posy - increment * position),
							 (ImageObserver)this);
			} else {
				g2.drawImage(object,
							 (int)(originx + increment * position),
							 (int)posy,
							 (ImageObserver)this);
			}
		}

		drawnRect = newRect;
		g2.setClip(savedclip);
		
    }

    public void move (double newpos) {
		this.position = newpos;
		this.repaint();
    }

    public void setPosition (double position) {
		this.position=position;
		drawnRect = null;
    }

    public double getPosition (double position) {
		return position;
    }

    public void sync (GraphArea ga) {

		syncga = ga;
    }
	
	public void sync (GraphWindow gw) {
		syncga = gw.getDefaultGraphArea();
	}
 
    private void doSync() {
		if (syncga != null) {
			Point2D.Double gwOrigin = new Point2D.Double();
			CoordinateSystem cs = syncga.getCoordinateSystem();

			cs.transformToDisplay(new Point2D.Double(0.0, 0.0), gwOrigin);

			posy = gwOrigin.y;
		
			originy = getHeight() - posy;
		}
    }


    public Rectangle2D.Double getRect( double position) {
		if (orientation == 0 ) { // Vertical
			return new Rectangle2D.Double(originx, posy-position*increment,
										  objectWidth, objectHeight);
		} else { // Horizontal
			return new Rectangle2D.Double(originx + position*increment, posy,
										  objectWidth, objectHeight);
	}
    }

    public Rectangle2D.Double getRect() {
		return getRect(this.position);
    }

    public void setup() {

    }

    public Dimension getMinimumSize() {
	return new Dimension(100,100);
    }

    private Image loadImage(String name) {
	Image retval;
	URL url = ObjectModel.class.getResource(name); 
	retval=Toolkit.getDefaultToolkit().getImage(url);
	tracker = new MediaTracker(this);
	tracker.addImage(retval,0);
	try {
	    tracker.waitForAll();
	} catch (Exception e) {
	    System.err.println("Problem loading Image");
	    retval = null;
	} 
	return retval;
    }
} // public class ObjectModel
