/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/** Author: Eric Brown-Munoz
            Sept 5, 2003

   Part of QGrapher, an qualitative graph for teaching algebra.

   Copyright (C) 2003, Eric Brown-Munoz

   This program is free software; you can redistribute it and/or modify 
   it under the terms of the GNU General Public License as published by 
   the Free Software Foundation; either version 2 of the License, or 
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

$Revision: 1.3 $
*/
package org.concord.rttapplets.qgrapher;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

/** A class to implement the buttons for QGrapher. For now this has specific
   buttons. This should be generalized in the future.
*/

public class QButtonBar 
	extends JPanel
	implements ActionListener
{

    private Vector qButtonListeners = new Vector();
	QualGraphable graph;
	String[] type = { "line_straight", "line_up", "line_down", "rightquad_up",
					   "leftquad_up", "leftquad_down", "rightquad_down"};
	
    JButton delete = null;

	
    public QButtonBar()
	{
		int n = type.length;
		setLayout(new GridLayout(1,n));
		setMinimumSize(new Dimension(38*n,38));
		ImageIcon buttonIcon;
		ImageIcon buttons = getImageIcon("images/buttons.gif");
	
		for (int i = 0 ; i < n ; i++)
		{
			Image tempImage = new BufferedImage(36,36,BufferedImage.TYPE_INT_RGB);
			Graphics2D tempg = (Graphics2D)tempImage.getGraphics();
	
			tempg.drawImage(buttons.getImage(),(int)(-35*i),0,this);
			buttonIcon = new ImageIcon(tempImage);
	
			JButton newButton = new JButton(buttonIcon);
			newButton.addActionListener(this);
			newButton.setActionCommand(type[i]);
			this.add(newButton);
		}
	
		delete = new JButton(getImageIcon("images/back.gif"));
		delete.addActionListener(this);
		delete.setActionCommand("delete");
		this.add(delete);

    } // public QButtonBar
	
    /** Constructs and initializes a new QButtonBar */
    public QButtonBar( QualGraphable qg ) {
		this();
		setQualGraphable(qg);
    } // public QButtonBar

    public void setQualGraphable(QualGraphable qg)
	{
		graph = qg;
    }
	
    protected ImageIcon getImageIcon (String filename) {
	java.net.URL imgURL = QButtonBar.class.getResource(filename);
	if (imgURL != null ) {
	    return new ImageIcon(imgURL);
	} else {
	    System.err.println("Couldn't find file: " + imgURL);
	    return null;
	}

    }

    protected void grayLabel (JLabel object) {
		ImageIcon iconObj = (ImageIcon)object.getIcon();
		Image image = iconObj.getImage();
		ImageFilter grayfilter = new GrayFilter(true,60);
		
		Image newimage = createImage(new FilteredImageSource(image.getSource(),
															 grayfilter));
		iconObj.setImage(newimage);
    }

	public void actionPerformed (ActionEvent e) {
		
		String command = e.getActionCommand();
		Handle last = null;
		double last_x = 0.0;
		double last_y = 0.0;

		if (graph != null) 
			last = graph.lastHandle();

		if (last != null) {
			last_x = last.getX();
			last_y = last.getY();
		}

		if        (command.equals("line_straight")) {
			graph.newSegment(last_x + 2.0, last_y, QualSegment.LINE);
		} else if (command.equals("line_up")) {
			graph.newSegment(last_x + 2.0, last_y + 2.0, QualSegment.LINE);
		} else if (command.equals("line_down")) {
			graph.newSegment(last_x + 2.0, last_y - 2.0 , QualSegment.LINE);
		} else if (command.equals("rightquad_up")) {
			graph.newSegment(last_x + 2.0, last_y + 2.0 , QualSegment.RIGHT);
		} else if (command.equals("rightquad_down")) {
			graph.newSegment(last_x + 2.0, last_y - 2.0 , QualSegment.RIGHT);
		} else if (command.equals("leftquad_up")) {
			graph.newSegment(last_x + 2.0, last_y + 2.0 , QualSegment.LEFT);
		} else if (command.equals("leftquad_down")) {
			graph.newSegment(last_x + 2.0, last_y - 2.0 , QualSegment.LEFT);
		} else if (command.equals("delete")) {
			graph.removeLastSegment();
		}

	}


} // class QButtonBar
