/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/




package org.concord.rttapplets.qgrapher;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;

import org.concord.rttapplets.ui.*;
import org.concord.graph.ui.*;
import org.concord.graphfunction.ui.modelview.*;
import org.concord.graphfunction.engine.*;
import org.concord.graph.engine.*;



public class QGrapher extends JPanel implements ComponentListener {

	GraphWindow graph;
	QualGraphable qgraph;
	QButtonBar buttons;
	CoordinateSystem cs;
	GraphArea ga;
	Grid grid;
	AnimateTrace animation;

	Trace trace;

	Value variable = new Value(3.0);

	ModelWindow model;

	public QGrapher () {

		setLayout(new BorderLayout());
		JPanel modelPanel = new JPanel();
		JPanel graphPanel = new JPanel();
		JPanel mControlPanel = new JPanel();
		TraceAnimationControl playControl;

		mControlPanel.setLayout(new BoxLayout(mControlPanel,BoxLayout.Y_AXIS));

		graphPanel.setLayout(new BorderLayout());

		addComponentListener(this);

		graph = new GraphWindow();
		grid = new Grid();
		grid.setShowAxisLabels(false);
		grid.setShowGridLabels(false);
		graph.add(grid);

		ga = graph.getDefaultGraphArea();
		cs = ga.getCoordinateSystem();
		cs.setOriginOffsetDisplay(new Point2D.Double(20.0, -300.0));

		qgraph = new QualGraphable();
		qgraph.setGraphArea(ga);

		graph.add(qgraph);

		qgraph.newSegment(5.0, 5.0, 1);  // FIXME XXXX
		qgraph.newSegment(10.0, 10.0, 0);

		graphPanel.add(graph,BorderLayout.CENTER);

		buttons = new QButtonBar(qgraph);
		buttons.setPreferredSize(new Dimension(300,75));
		graphPanel.add(buttons, BorderLayout.SOUTH);

		modelPanel.setLayout(new BorderLayout());
		model = new ModelWindow();
		model.setPreferredSize(new Dimension(100,300));
		model.setGraphArea(ga);
		model.setGraphFunction(qgraph);
		model.setVariable(variable);

		variable.setValue(0.0);

		//trace = new Trace(qgraph, variable);
		trace = new LineTrace(variable);
		trace.setShowCoordinates(false);
		graph.add(trace);


		modelPanel.setPreferredSize(new Dimension(100,100));
		modelPanel.add(model, BorderLayout.CENTER);

		animation = new AnimateTrace(variable);
		animation.setSyncGraph(qgraph);

		playControl = new TraceAnimationControl();
		playControl.setPreferredSize(new Dimension(100,25));

		playControl.setAnimateTrace(animation);
		mControlPanel.add(playControl);

		ImageChooser chooser = new ImageChooser();
		chooser.setModel(model);
		mControlPanel.add(chooser);

		modelPanel.add(mControlPanel, BorderLayout.SOUTH);


		add(modelPanel,BorderLayout.WEST);
		add(graphPanel,BorderLayout.CENTER);

	}

	public void componentHidden(ComponentEvent e) {}

	public void componentMoved(ComponentEvent e) {}

	public void componentResized(ComponentEvent e) {
		double height = (double)graph.getHeight();
		cs.setOriginOffsetDisplay(new Point2D.Double(20.0, height - 20.0));
	}

	public void componentShown(ComponentEvent e) {}

	public static void main (String[] args) {

		QGrapher qg = new QGrapher();

		RTTFrame frame = new RTTFrame(qg, "RTT Qualitative Grapher");
		frame.show();
	}
}
