/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/




package org.concord.rttapplets.qgrapher;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class TraceAnimationControl 
	extends JPanel 
	implements ActionListener
{

	AnimateTrace trace;
	
	public TraceAnimationControl () {

		Insets zeroInsets = new Insets(0,0,0,0);
		

		ImageIcon playIcon = 
			createImageIcon("images/Play24.gif","Play button");
		JButton playButton = new JButton(playIcon);

		ImageIcon pauseIcon = 
			createImageIcon("images/Pause24.gif","Pause button");
		JButton pauseButton = new JButton(pauseIcon);
		ImageIcon stopIcon = 
			createImageIcon("images/Stop24.gif","Stop button");
		JButton stopButton = new JButton(stopIcon);

		playButton.setActionCommand("play");
		pauseButton.setActionCommand("pause");
		stopButton.setActionCommand("stop");

		playButton.setBackground(Color.white);
		pauseButton.setBackground(Color.white);
		stopButton.setBackground(Color.white);

		playButton.setMargin(zeroInsets);
		pauseButton.setMargin(zeroInsets);
		stopButton.setMargin(zeroInsets);



		setLayout(new BoxLayout(this,BoxLayout.X_AXIS));

		playButton.addActionListener(this);
		add(playButton);

		pauseButton.addActionListener(this);
		add(pauseButton);

		stopButton.addActionListener(this);
		add(stopButton);
	}

	public void setAnimateTrace (AnimateTrace trace) {
		this.trace = trace;
	}

	/** Returns an ImageIcon, or null if the path was invalid. */
	protected static ImageIcon createImageIcon(String path,
                                           String description) {
		java.net.URL imgURL = TraceAnimationControl.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL, description);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}

	}

	public void actionPerformed (ActionEvent e) {
		String action = e.getActionCommand();

		if (action.equals("play")) {
			trace.start();
		} else if (action.equals("pause")) {
			trace.pause();
		} else if (action.equals("stop")) {
			trace.stop();
		}
	}
}
