/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.4 $
 * $Date: 2004/08/24 20:01:59 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.rttapplets.quadgrapher;

import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.awt.geom.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;

import org.concord.graph.engine.*;
import org.concord.graph.ui.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;
import org.concord.graphfunction.ui.symbolicview.*;

/**
 * Main panel for the Quadratic Grapher
 *
 * @author Ingrid Moncada<p>
*/

public class QuadGrapher extends JPanel 
	implements ActionListener
{
	String interactiveName;

    GraphFunctionList gflist;
	GraphFunctionViewManager functionManager;

    // We need to these available for the button Handlers.
    JPanel inputPanel;
    CardLayout inputPanelLayout;
	JToggleButton scaleButtonX;
	JToggleButton scaleButtonY;
	JButton scaleButtonReset;
	JButton incrementChangeButton;

	Grid grid;
	GraphWindow graph;
	double initialScaleX = 22.0;
	double initialScaleY = 22.0;
	private AxisScale axisScale;
	
	QuadraticSymbolic functionInput;
	double valueBoxIncrement = 0.1;

	// The x variable;
	private Value variable;

	JPanel transformContainer;
	JButton cmdTransformFunction;
	QuadraticFunctionTransformerPanel transformDialog;
	
	//Number of functions created
	int numberFunction = 0;
	
	CoordinateSystem coord;
	GraphArea graphArea;

/* Multi ghosts are taken out
//	MultiGhost ghosts = new MultiGhost();
*/
	public QuadGrapher()
	{
		this("RTT Quadratic Panel");
	}
	
	public QuadGrapher(String interactiveName)
	{
		this.interactiveName = interactiveName;

		gflist = new GraphFunctionList();

		functionManager = new GraphFunctionViewManager();
		functionManager.setGraphFunctionList(gflist);

		addQuadraticFunction();

		variable = new Value();
		//X starts in 1.0
		variable.setValue(1.0);
		
		GraphFunctionColorListenerManager colorListener = new GraphFunctionColorListenerManager();
		functionManager.addGraphFunctionView(colorListener);
		
		Trace trace = new Trace(gflist, variable);
		trace.setNumDecimalsXCoordinate(1);
		trace.setNumDecimalsYCoordinate(2);
		
		setLayout(new BorderLayout());

		////////////////////////////////////////////////////////////////////////////////
		//Graph		
		graph = new GraphWindow();
		graph.setDefaultGraphArea(new GraphArea(new Axis()));

		grid = new Grid();
		graph.addDecoration(grid);		
		////////////////////////////////////////////////////////////////////////////////

		//Function expression

		JPanel functionContainer = new JPanel();
		functionContainer.setLayout(new BoxLayout(functionContainer, BoxLayout.X_AXIS));
		
		functionInput = new QuadraticSymbolic();
		functionInput.setLabelSize(16.0f);
		//functionInput.setAlignmentX(LEFT_ALIGNMENT);
		//functionInput.setGraphableList(gflist);
		functionManager.addGraphFunctionView(functionInput);
		functionInput.setValueBoxNumDecimalsValue(2);

		incrementChangeButton = new JButton(String.valueOf(valueBoxIncrement));
		incrementChangeButton.setActionCommand("setincrement");
		incrementChangeButton.addActionListener(this);
		incrementChangeButton.setPreferredSize(new Dimension(70,40));
		incrementChangeButton.setMargin(new Insets(0,0,0,0));
		
		JButton roundCoeffButton = new JButton(getString("Round"));
		roundCoeffButton.setActionCommand("round");
		roundCoeffButton.addActionListener(this);
		//roundCoeffButton.setPreferredSize(new Dimension(60,40));
		roundCoeffButton.setMargin(new Insets(0,5,0,5));
		
		functionContainer.add(functionInput);
		functionContainer.add(roundCoeffButton);
		functionContainer.add(incrementChangeButton);
			
		functionContainer.setBackground(new Color(222,222,222));
		functionContainer.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		
		////////////////////////////////////////////////////////////////////////////////

		////////////////////////////////////////////////////////////////////////////////
		//Graph
		final JPanel graphContainer = new JPanel();	

		grid.setInterval(1.0,1.0);
		grid.setLabelFormat(new DecimalFormat("#"));
		grid.setLabelSize(10.0f);

		//This was triggering an exception. There is a java bug with Antialiasing
		//when the quadratic has big values
		//http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4696289
		//Right now this is handled in the Quadratic Function
		graph.setAntialias(true);
		
		//This maintains the y axis in the middle all the time
		graph.getDefaultGraphArea().setXCentered(true);
		//This maintains the x axis in the middle all the time
		graph.getDefaultGraphArea().setYCentered(true);

		graphArea = graph.getDefaultGraphArea();
		coord = graphArea.getCoordinateSystem();
		coord.setScale(new Point2D.Double(initialScaleX, initialScaleY));
		((Axis)coord).setFixOriginScreen(true);

		graph.setMouseBehavior(DefaultGraphMouseManager.ONCLICK_SELECT);

		//Scaling object
		axisScale = new AxisScale();
		axisScale.setDragMode(AxisScale.DRAGMODE_NONE);
		//

		graph.add(gflist);
		graph.add(trace);
		graph.add(axisScale);

		graph.setVisible(true);

		graph.setBorder(BorderFactory.createLineBorder(Color.black, 1));

		graphContainer.setLayout(new BoxLayout(graphContainer,BoxLayout.Y_AXIS));
		
		graphContainer.add(graph);	
		
		add(graphContainer,BorderLayout.CENTER);

		//Slider to set value of variable X
		ValueSlider slider = new ValueSlider(variable);
		//slider.setPaintTicks(true);
		//slider.setPaintLabels(true);
		slider.sync(graphArea);
		//slider.setInterval(0.1);

		graphContainer.add(slider);
		
		final JPanel controlsContainer = new JPanel();
		//controlsContainer.setPreferredSize(new Dimension(250,900));

		controlsContainer.setLayout(new GridBagLayout());
		final GridBagConstraints gc = new GridBagConstraints();
		gc.fill = GridBagConstraints.BOTH;
		gc.anchor = GridBagConstraints.NORTHEAST;

		////////////////////////////////////////////////////////////////////////////////
		//General buttons for the list
		
		transformContainer = new JPanel();
		transformContainer.setLayout(new GridLayout(1,1));
		transformContainer.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
		
		cmdTransformFunction = new JButton(getString("TransformFunction"));
		cmdTransformFunction.setActionCommand("transformFunction");
		cmdTransformFunction.addActionListener(this);
		transformContainer.add(cmdTransformFunction);

		JPanel functionButtons = new JPanel();
		functionButtons.setLayout(new GridLayout(4,1));
		functionButtons.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));

		JButton cmdNewFunction = new JButton(getString("NewFunction"));
		cmdNewFunction.setActionCommand("newFunction");
		cmdNewFunction.addActionListener(this);
		functionButtons.add(cmdNewFunction);

		JButton cmdCopyFunction = new JButton(getString("DuplicateFunction"));
		cmdCopyFunction.setActionCommand("copyFunction");
		cmdCopyFunction.addActionListener(this);
		functionButtons.add(cmdCopyFunction);

		JButton cmdDeleteFunction = new JButton(getString("DeleteFunction"));
		cmdDeleteFunction.setActionCommand("deleteFunction");
		cmdDeleteFunction.addActionListener(this);
		functionButtons.add(cmdDeleteFunction);

		JButton cmdDeleteAllFunction = new JButton(getString("DeleteAll"));
		cmdDeleteAllFunction.setActionCommand("deleteAllFunction");
		cmdDeleteAllFunction.addActionListener(this);
		functionButtons.add(cmdDeleteAllFunction);


		JPanel scaleButtons = new JPanel();
		scaleButtons.setLayout(new GridLayout(3,1));
		scaleButtons.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));

		add(functionContainer,BorderLayout.SOUTH);

		
		scaleButtonX = new JToggleButton(getString("ChangeXScale"));
		scaleButtonX.setActionCommand("scalex");
		scaleButtonX.addActionListener(this);
		scaleButtonY = new JToggleButton(getString("ChangeYScale"));
		scaleButtonY.setActionCommand("scaley");
		scaleButtonY.addActionListener(this);
		scaleButtonReset = new JButton(getString("ResetScale"));
		scaleButtonReset.setActionCommand("scalereset");
		scaleButtonReset.addActionListener(this);


		scaleButtons.add(scaleButtonX);
		scaleButtons.add(scaleButtonY);
		scaleButtons.add(scaleButtonReset);

		//Temp
		//JButton debugButton = new JButton("Debug");
		//debugButton.setActionCommand("debug");
		//debugButton.addActionListener(this);
		//scaleButtons.add(debugButton);


/* Multi ghosts are taken out for now because they were slow
   and people didn't like them anyway 

//		final JPanel ghostContainer = new JPanel();

//		final JCheckBox ghostCheckBox = new JCheckBox("Show Ghosts");
//		ghostCheckBox.setActionCommand("showghosts");
//		ghostCheckBox.addActionListener(this);
//		ghostContainer.add(ghostCheckBox);

//		final JButton ghostClearButton = new JButton("Clear Ghosts");
//		ghostClearButton.setActionCommand("clearghosts");
//		ghostClearButton.addActionListener(this);
//		ghostContainer.add(ghostClearButton);
		
//		// Ghost handler 
//		graph.add(ghosts);
//		glist.addGraphableListener(new GraphableAdapter() {
//			public void graphableChanged(GraphableEvent e) {
//				if (e.isFunctionChange()) {
//					ghosts.addGhost((Graphable)e.getSource());
//				}
//			}
//		});

		gc.gridx = 0;
		gc.gridy = 0;
		gc.gridheight = 1;
		controlsContainer.add(ghostContainer,gc);

*/

		gc.gridy = 0;
		gc.gridheight=1;
		controlsContainer.add(transformContainer,gc);

		gc.gridy = 1;
		gc.gridheight=4;
		controlsContainer.add(functionButtons,gc);

		gc.gridy = 5;
		gc.gridheight=3;
		controlsContainer.add(scaleButtons,gc);

		//Function List
		GraphFunctionTable functionList = new GraphFunctionTable();
		functionList.setGraphFunctionList(gflist);

		//Temp for now 
		TableColumn column = functionList.getTable().getColumnModel().getColumn(0);
		column.setMaxWidth(50);
		
		//functionList.setPreferredSize(new Dimension(80, 100));
		functionList.setPreferredSize(new Dimension(250, 200));

		gc.gridy = 8;
		gc.gridheight = 8;
		controlsContainer.add(functionList, gc);
		////////////////////////////////////////////////////////////////////////////////

		add(controlsContainer, BorderLayout.EAST);
		////////////////////////////////////////////////////////////////////////////////
		
		//Show transformer is NOT enabled by default
		setShowTransformer(false);

		validate();
	}

    public void actionPerformed(ActionEvent e) 
	{
		String command =((AbstractButton)e.getSource()).getActionCommand();
		
		if (command.equals("newFunction")){
			addQuadraticFunction();
		}
		else if (command.equals("copyFunction")){
			GraphFunction current = (GraphFunction)gflist.getLastSelectedObject();
			if (current!=null){
				addCopy(current);
			}
		}
		else if (command.equals("deleteFunction")){
			gflist.removeAllSelected();
		}
		else if (command.equals("deleteAllFunction")){
			gflist.removeAllElements();
			numberFunction = 0;
		}
/* Multi Ghosts are taken out
//		else if ( command.equals("clearghosts") ) {
//			ghosts.clear();
//			graph.repaint();
//		} else if ( command.equals("showghosts") ) {
//			if (((AbstractButton)e.getSource()).isSelected()) {
//				ghosts.setEnabled(true);
//			} else {
//				ghosts.setEnabled(false);
//				ghosts.clear();
//				graph.repaint();
//			}
//		}
*/
		else if (command.equals("scalex")) {
			if (((JToggleButton)e.getSource()).isSelected()){
				axisScale.setDragMode(AxisScale.DRAGMODE_DILATE_X);
				scaleButtonY.setSelected(false);
			}
			else{
				axisScale.setDragMode(AxisScale.DRAGMODE_NONE);
			}
		}			
		else if (command.equals("scaley")) {
			if (((JToggleButton)e.getSource()).isSelected()){
				axisScale.setDragMode(AxisScale.DRAGMODE_DILATE_Y);
				scaleButtonX.setSelected(false);
			}
			else{
				axisScale.setDragMode(AxisScale.DRAGMODE_NONE);
			}
		}			
		else if (command.equals("scalereset")) {
			grid.setInterval(1.0,1.0);
			CoordinateSystem coord = graph.getDefaultGraphArea().getCoordinateSystem();
			coord.setScale(new Point2D.Double(initialScaleX, initialScaleY));
		}
		else if (command.equals("setincrement")) {
			//Set increments for the input boxes
			String strIncrement;
			try{
				Object[] possibilities = {"0.01", "0.1", "1"};
				//strIncrement = String.valueOf(
					//JOptionPane.showInputDialog(null,
					//"Coefficients increment:", "Change of coefficient increment",
					////JOptionPane.QUESTION_MESSAGE,null,null,String.valueOf(valueBoxIncrement)));
					//JOptionPane.QUESTION_MESSAGE,null,possibilities,String.valueOf(valueBoxIncrement)));

				strIncrement = (String)possibilities[
					JOptionPane.showOptionDialog(null,
						"Select how much the coefficients should increment:", 
						"Change of coefficient increment",
						JOptionPane.YES_NO_OPTION,
						JOptionPane.QUESTION_MESSAGE,null,possibilities,String.valueOf(valueBoxIncrement))];
				valueBoxIncrement = Double.valueOf(strIncrement).doubleValue();
				
			}catch(Exception ex){
				return;
			}
			functionInput.setValueBoxIncrementInterval(valueBoxIncrement);
			incrementChangeButton.setText(strIncrement);
		}
		else if (command.equals("transformFunction")) {
		//Show Quadratic Function Transformer
			if (transformDialog==null){
				transformDialog = new QuadraticFunctionTransformerPanel();
				//We need the action listener on the dialog because it is NOT modal
				//transformDialog.addActionListener(this);
			}
			QuadraticFunction current = (QuadraticFunction)gflist.getLastSelectedObject();
			transformDialog.setOriginalFunction(current,gflist.getColorFactory().getNextColor());
			//transformDialog.resetGraphArea(graphArea);
			//transformDialog.setIntervalGrid(grid.getIntervalX(), grid.getIntervalY());
			//The dialog is NOT modal because modal dialogs in Java are too confusing for the user
			//specially in a browser (applet). When a modal dialog loses the focus, it's hard to gain 
			//it again. Since it is not modal, that's why we need the action listener on the dialog
			//transformDialog.showAsDialog(false);
		}
		else if (command.equals("ok")) {
		//Ok action from the dialog!!
			QuadraticFunction newQ;
			if (transformDialog != null && transformDialog.getAccepted()){
				newQ = transformDialog.getTransformedFunction();
				if (newQ != null){
					//Add the new function to the list, selecting it (true) but NOT changing its color (false) 
					gflist.add(newQ, true, false);
					newQ.setShowGhost(true);
					newQ.setDragMode(DragMode.NONE);
				}
			}
		}
		else if (command.equals("round")) {
		//Round according to the interval value
			ValueBox box = functionInput.getSelectedBox();
			box.roundWithDecimals();
			box.roundWithIncrementInterval();
		}
		//else if (command.equals("debug")) {
		//	QuadraticFunction current = (QuadraticFunction)gflist.getLastSelectedObject();
		//	System.out.println("coeffs:"+current.getCoeffA()+" "+current.getCoeffB()+" "+current.getCoeffC());
		//	System.out.println("roots:"+current.getRoot1()+ " " + current.getRoot2());
		//}
	}
	
	private GraphFunction addQuadraticFunction()
	{
		///////////////////
		//Quadratic
		QuadraticFunction q;
		q = new QuadraticFunction();
		q.setExpressionForm(QuadraticFunction.FORM_POLYNOM);
		q.setShowGhost(true);
		q.setDragMode(DragMode.NONE);
		q.setFunctionName("f");
		numberFunction++;
		q.setFunctionOrdinal(numberFunction);
		gflist.add(q);
		///////////////////
		return q;
	}
	
	private GraphFunction addCopy(GraphFunction f)
	{
		GraphFunction g;
		g = (GraphFunction)f.getCopy();
		g.setShowGhost(true);
		((QuadraticFunction)g).setDragMode(DragMode.NONE);
		numberFunction++;
		g.setFunctionOrdinal(numberFunction);
		gflist.add(g);
		return g;
	}

	public void setShowTransformer(boolean b)
	{
		transformContainer.setVisible(b);
		cmdTransformFunction.setVisible(b);
	}
	
	//Testing purposes
    public static void main(String args[]) 
	{
		final JFrame frame = new JFrame();
		final QuadGrapher fa = new QuadGrapher();
		frame.getContentPane().add(fa);
		frame.setSize(700,600);
		frame.show();
		
		frame.addWindowListener( new WindowAdapter() 
		{
			public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}			
		});
    }
	
	/**
	 * STRINGS --- This should be moved to a resource file!!!
	 * (for Localization)
	 */
	private String getString(String strStringName)
	{
		if (strStringName.equals("Round")){
			return "Round";
		}
		if (strStringName.equals("TransformFunction")){
			return "Transform Function";
		}
		if (strStringName.equals("NewFunction")){
			return "New Function";
		}
		if (strStringName.equals("DuplicateFunction")){
			return "Duplicate Function";
		}
		if (strStringName.equals("DeleteFunction")){
			return "Delete Function";
		}
		if (strStringName.equals("DeleteAll")){
			return "Delete All";
		}
		if (strStringName.equals("ChangeXScale")){
			return "Change X Scale";
		}
		if (strStringName.equals("ChangeYScale")){
			return "Change Y Scale";
		}
		if (strStringName.equals("ResetScale")){
			return "Reset Scale";
		}
		return "";
	}

}
