/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.2 $
 * $Date: 2004/08/24 20:02:59 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/

package org.concord.rttapplets.quadgrapher;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.text.*;

import javax.swing.*;

import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;
import org.concord.graphfunction.ui.symbolicview.*;
import org.concord.graph.ui.*;
import org.concord.graph.engine.*;






/**
 * Main panel for the Quadratic Transformer
 *
 * @author Ingrid Moncada<p>
*/

public class QuadraticFunctionTransformerPanel extends JPanel 
	implements ActionListener
{
	String interactiveName;

    protected GraphFunctionList gflist;
    private GraphArea ga;
    private CoordinateSystem cs;
    
    // We need to these available for the button Handlers.
    JPanel inputPanel;
    CardLayout inputPanelLayout;
//	SaveAsImageButton saveButton;
	JToggleButton scaleButtonX;
	JToggleButton scaleButtonY;
	JButton scaleButtonReset;
	JButton incrementChangeButton;

	Grid grid;
	GraphWindow graph;
	double initialScaleX = 22.0;
	double initialScaleY = 22.0;
	
	QuadraticPolynomialInput functionInputF;
	TransformedFunctionSymbolic functionInputGF;
	TransformedQuadraticPolynomialInput functionInputGF2;
	QuadraticPolynomialInput functionInputG;
	
	GraphFunctionColorListenerManager colorListenerF;
	GraphFunctionColorListenerManager colorListenerG;
	
	JPanel dragModeContainer;
	JRadioButton emptyRadioB;
	
	double valueBoxIncrement = 0.1;

	QuadraticFunction functionF;
	TransformedQuadraticFunction functionG;
	
	Value variable;

	DragMode dragMode = new DragMode(DragMode.NONE);
	
	JDialog dialog;
	private boolean accepted = false;

	public QuadraticFunctionTransformerPanel()
	{
		this("Quadratic Function Transformer");
	}
	
	public QuadraticFunctionTransformerPanel(String interactiveName)
	{
		this.interactiveName = interactiveName;
	
		variable = new Value();
		//X starts in 1.0
		variable.setValue(1.0);

		gflist = new GraphFunctionList();

//		functionF = addQuadFunction();
//		functionF.setSelectable(false);
//		functionF.setLineStroke(new BasicStroke(7.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));
		
//		functionG = addTransformedQuadFunction();
		//functionG.setSelectedLineStroke(new BasicStroke(3.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));

		colorListenerF = new GraphFunctionColorListenerManager();
//		colorListenerF.setGraphFunction(functionF);
		
		colorListenerG = new GraphFunctionColorListenerManager();
//		colorListenerG.setGraphFunction(functionG);

		Trace trace = new Trace(gflist, variable);
		
		//Grid bag constraints for all the components		
		//createConstraints(int gridx, int gridy, int gridwidth, int gridheight, int fill, double weightx, double weighty, int anchor)

		//Transformations panel (radio buttons)
		GridBagConstraints dragModeConstraints = createConstraints(0,0,1,1,GridBagConstraints.HORIZONTAL,0,10, GridBagConstraints.NORTH);
		dragModeConstraints.insets = new Insets(40,30,0,0);
		//Graph
		GridBagConstraints graphConstraints = createConstraints(1,0,1,1,GridBagConstraints.BOTH,10,10, GridBagConstraints.CENTER);

		//Label value of x
		GridBagConstraints valueXLabelConstraints =createConstraints(0,1,1,1,GridBagConstraints.BOTH,1,1, GridBagConstraints.CENTER);

		//Slider value of x
		GridBagConstraints valueXSliderConstraints =createConstraints(1,1,1,1,GridBagConstraints.HORIZONTAL,1,1, GridBagConstraints.CENTER);
		//valueXSliderConstraints.insets = new Insets(0,0,0,46);

		//FunctionSymbolics
		GridBagConstraints functionConstraints = createConstraints(0,2,2,3,GridBagConstraints.NONE,0,1, GridBagConstraints.CENTER);
		
		//List Buttons
		GridBagConstraints buttonsConstraints =	createConstraints(0,5,2,1,GridBagConstraints.NONE,1,1, GridBagConstraints.CENTER);
		
		setLayout(new GridBagLayout());

		////////////////////////////////////////////////////////////////////////////////
		
		////////////////////////////////////////////////////////////////////////////////
		//Graph
		MainPanel mainPanelGraph = new MainPanel();
		graph = mainPanelGraph.getGraph();
		grid = mainPanelGraph.getGrid();
		////////////////////////////////////////////////////////////////////////////////

		////////////////////////////////////////////////////////////////////////////////
		//Function expression
		JPanel functionContainer = new JPanel();
		JPanel functionContainer1 = new JPanel();
		JPanel functionContainer2 = new JPanel();

		functionContainer.setLayout(new BoxLayout(functionContainer, BoxLayout.Y_AXIS));
		functionContainer1.setLayout(new BoxLayout(functionContainer1, BoxLayout.Y_AXIS));
		functionContainer2.setLayout(new BoxLayout(functionContainer2, BoxLayout.Y_AXIS));
		
		//functionInputF = new QuadraticSymbolic();
		functionInputF = new QuadraticPolynomialInput();
		functionInputF.setLabelSize(16.0f);
//		functionInputF.setGraphFunction(functionF);
		//functionInputF.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		functionInputF.setBackground(new Color(222,222,222));
		functionInputF.setEditable(false);
		functionInputF.setShowFunctionName(true);

		functionInputGF = new TransformedFunctionSymbolic();
		functionInputGF.setLabelSize(16.0f);
//		functionInputGF.setGraphFunction(functionG);
		//functionInputGF.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		functionInputGF.setBackground(new Color(222,222,222));
		functionInputGF.setShowFunctionName(true);
		functionInputGF.setEditable(false);

		functionInputGF2 = new TransformedQuadraticPolynomialInput();
		functionInputGF2.setLabelSize(16.0f);
//		functionInputGF2.setGraphFunction(functionG);
		//functionInputGF2.setBackground(new Color(222,222,222));
		functionInputGF2.setEditable(false);
		functionInputGF2.setShowFunctionName(true);

		//functionInputG = new QuadraticSymbolic();
		functionInputG = new QuadraticPolynomialInput();
		functionInputG.setLabelSize(16.0f);
//		functionInputG.setGraphFunction(functionG);
		//functionInputG.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		functionInputG.setBackground(new Color(222,222,222));
		functionInputG.setEditable(false);
		functionInputG.setShowFunctionName(true);
		//colorListenerG.addColorObject(functionInputG,"setBackground");

		//////////////////////		
		functionInputF.setAlignmentX(LEFT_ALIGNMENT);

		functionContainer1.add(new JLabel("Original function:"));
		functionContainer1.add(functionInputF);

		functionContainer1.setBorder(BorderFactory.createLineBorder(Color.black,3));
		colorListenerF.addBorderObject(functionContainer1);
		//////////////////////		
		
		//////////////////////		
		functionInputGF.setAlignmentX(LEFT_ALIGNMENT);
		functionInputGF2.setAlignmentX(LEFT_ALIGNMENT);
		functionInputG.setAlignmentX(LEFT_ALIGNMENT);
		
		functionContainer2.add(new JLabel("Transformed function:"));
		functionContainer2.add(functionInputGF);
		functionContainer2.add(functionInputGF2);
		functionContainer2.add(functionInputG);

		functionContainer2.setBorder(BorderFactory.createLineBorder(Color.black,3));
		colorListenerG.addBorderObject(functionContainer2);
		//////////////////////		

		//////////////////////		
		functionContainer1.setAlignmentX(LEFT_ALIGNMENT);
		functionContainer1.setMaximumSize(new Dimension(Short.MAX_VALUE, Short.MAX_VALUE));
		functionContainer2.setAlignmentX(LEFT_ALIGNMENT);
		functionContainer2.setMaximumSize(new Dimension(Short.MAX_VALUE, Short.MAX_VALUE));

		functionContainer.add(functionContainer1);
		functionContainer.add(Box.createRigidArea(new Dimension(0,5)));
		functionContainer.add(functionContainer2);
		//////////////////////		

		add(functionContainer,functionConstraints);
		////////////////////////////////////////////////////////////////////////////////

		////////////////////////////////////////////////////////////////////////////////
		//Graph
		final JPanel graphContainer = new JPanel();	
		graphContainer.setBorder(BorderFactory.createLineBorder(Color.black));

		grid.setInterval(1.0,1.0);
		grid.setLabelFormat(new DecimalFormat("#"));
		grid.setLabelSize(10.0f);
		
		ga = graph.getDefaultGraphArea();
		cs = ga.getCoordinateSystem();

		ga.setXCentered(true);
		ga.setYCentered(false);
		cs.setScale(new Point2D.Double(initialScaleX, initialScaleY));
		cs.setOriginOffsetDisplay(new Point2D.Double(9.0, 6.0));
		graph.setMouseBehavior(DefaultGraphMouseManager.ONCLICK_SELECT);

		graph.setSelectablesList(gflist);

		graph.setVisible(true);
		graph.add(trace);

		graphContainer.setLayout(new BoxLayout(graphContainer,
							   BoxLayout.Y_AXIS));
		graphContainer.add(graph);	

		add(graphContainer,graphConstraints);
		////////////////////////////////////////////////////////////////////////////////

		////////////////////////////////////////////////////////////////////////////////
		//Label of variable X
		JPanel valueXLabelContainer = new JPanel();
		
		JLabel lblValueX = new JLabel("Value of ");
		lblValueX.setFont(lblValueX.getFont().deriveFont(14.0f));
		ValueValueBox valueX = new ValueValueBox(variable);
		valueX.setFont(valueX.getFont().deriveFont(16.0f));
		valueX.setListenToValue();
		valueX.setEditable(false);
		JLabel lblX = new JLabel(ResourceLoader.getImageIcon("x"));
		JLabel lblValueXColon = new JLabel(":  ");

		JPanel valueXSliderContainer = new JPanel();
		valueXSliderContainer.setLayout(new BoxLayout(valueXSliderContainer, BoxLayout.X_AXIS));

		valueXLabelContainer.add(lblValueX);
		valueXLabelContainer.add(lblX);
		valueXLabelContainer.add(lblValueXColon);
		valueXLabelContainer.add(valueX);

		add(valueXLabelContainer,valueXLabelConstraints);
		////////////////////////////////////////////////////////////////////////////////

		////////////////////////////////////////////////////////////////////////////////
		//Slider to set value of variable X
		ValueSlider slider = new ValueSlider(variable);
		slider.setPaintTicks(true);
		slider.setPaintLabels(true);
		slider.sync(ga);
		//for now
		//slider.setInterval(0.1);

		valueXSliderContainer.add(slider);
		
		add(valueXSliderContainer,valueXSliderConstraints);
		////////////////////////////////////////////////////////////////////////////////

		////////////////////////////////////////////////////////////////////////////////
		//General buttons
		JPanel buttonsContainer = new JPanel();
		buttonsContainer.setLayout(new BoxLayout(buttonsContainer, BoxLayout.Y_AXIS));
		
		JButton cmdSaveFunction = new JButton("Save g as f");
		cmdSaveFunction.setActionCommand("saveFunctionGAsF");
		cmdSaveFunction.addActionListener(this);

		//buttonsContainer.add(cmdSaveFunction);
		
		add(buttonsContainer,buttonsConstraints);
		////////////////////////////////////////////////////////////////////////////////
		
		////////////////////////////////////////////////////////////////////////////////
		// Radio buttons for drag mode
		JPanel leftSideContainer = new JPanel();
		leftSideContainer.setLayout(new BoxLayout(leftSideContainer, BoxLayout.Y_AXIS));
		
		JButton startOverButton = new JButton("Start Over");
		startOverButton.setActionCommand("startover");
		startOverButton.addActionListener(this);
		
		leftSideContainer.add(startOverButton);
		leftSideContainer.add(Box.createRigidArea(new Dimension(0,30)));
		
		dragModeContainer = new JPanel();
		dragModeContainer.setLayout(new BoxLayout(dragModeContainer, BoxLayout.Y_AXIS));

		JLabel dragModeLabel = new JLabel("Transformations of f(x):");
		dragModeContainer.add(dragModeLabel);
		
		ButtonGroup buttonGroup = new ButtonGroup();
	
		JRadioButton radioB;
		
		emptyRadioB = new JRadioButton("");
		emptyRadioB.setSelected(true);
		emptyRadioB.setVisible(false);
		buttonGroup.add(emptyRadioB);
		dragModeContainer.add(emptyRadioB);

		dragModeContainer.add(new JLabel("Translations:"));

		radioB = new JRadioButton("f(x+a)");
		radioB.setActionCommand("translatex");
		radioB.addActionListener(this);
		buttonGroup.add(radioB);
		dragModeContainer.add(radioB);

		radioB = new JRadioButton("f(x)+a");
		radioB.setActionCommand("translatey");
		radioB.addActionListener(this);
		buttonGroup.add(radioB);
		dragModeContainer.add(radioB);

		dragModeContainer.add(new JLabel("Dilations:"));
		
		radioB = new JRadioButton("f(ax)");
		radioB.setActionCommand("dilatex");
		radioB.addActionListener(this);
		buttonGroup.add(radioB);
		dragModeContainer.add(radioB);

		radioB = new JRadioButton("af(x)");
		radioB.setActionCommand("dilatey");
		radioB.addActionListener(this);
		buttonGroup.add(radioB);
		dragModeContainer.add(radioB);

		dragModeContainer.add(new JLabel("Reflections:"));

		radioB = new JRadioButton("f(-x)");
		radioB.setActionCommand("reflecty");
		radioB.addActionListener(this);
		buttonGroup.add(radioB);
		dragModeContainer.add(radioB);

		radioB = new JRadioButton("-f(x)");
		radioB.setActionCommand("reflectx");
		radioB.addActionListener(this);
		buttonGroup.add(radioB);
		dragModeContainer.add(radioB);
		
		leftSideContainer.add(dragModeContainer);
		
		add(leftSideContainer, dragModeConstraints);
		////////////////////////////////////////////////////////////////////////////////

		//It is not ready to release to internal:
		//functionInputGF.setVisible(false);
		
		validate();
	}

	private GridBagConstraints createConstraints(int gridx, int gridy, int gridwidth, int gridheight, int fill, double weightx, double weighty, int anchor){
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = gridx;
		c.gridy = gridy;
		c.gridwidth = gridwidth;
		c.gridheight = gridheight;
		c.fill = fill;
		c.weightx = weightx;
		c.weighty = weighty;
		c.anchor = anchor;
		return c;
	}
	
    public void actionPerformed (ActionEvent e) {
		String command =((AbstractButton)e.getSource()).getActionCommand();
		
		if (command.equals("newFunction")){
			gflist.add(newQuadFunction());
		}
		else if (command.equals("copyFunction")){
			copyFunction();
		}
		else if (command.equals("deleteFunction")){
			gflist.removeAllSelected();
		}
		else if (command.equals("deleteAllFunction")){
			gflist.removeAllElements();
		}
		else if (command.equals("repaint")){
			GraphFunction current = (GraphFunction)gflist.getLastSelectedObject();
			if (current!=null){
				current.setPainterOn();
				current.resetPainter();
			}
		}
		else if (command.equals("scalex")) {
			
		}			
		else if (command.equals("scaley")) {
			
		}			
		else if (command.equals("scalereset")) {
			grid.setInterval(1.0,1.0);
			cs.setScale(new Point2D.Double(initialScaleX, initialScaleY));
		}
		else if (command.equals("setincrement")) {
			//Set increments for the input boxes
			String strIncrement;
			strIncrement = String.valueOf(JOptionPane.showInputDialog(null,
				"Coefficients increment:", "Change of coefficient increment",
				JOptionPane.QUESTION_MESSAGE,null,null,String.valueOf(valueBoxIncrement)));
			try{
				valueBoxIncrement = Double.valueOf(strIncrement).doubleValue();
				
			}catch(Exception ex){
				return;
			}
			functionInputGF.setValueBoxIncrementInterval(valueBoxIncrement);
			incrementChangeButton.setText(strIncrement);
		}
		else if (command.equals("translatex")) {
			dragMode.setValue(DragMode.TRANSLATE_X);
			functionG.setDragMode(dragMode);
			
			enableDragModeButtons(false);
		}
		else if (command.equals("translatey")) {
			dragMode.setValue(DragMode.TRANSLATE_Y);
			functionG.setDragMode(dragMode);
			
			enableDragModeButtons(false);
		}
		else if (command.equals("dilatex")) {
			dragMode.setValue(DragMode.DILATE_X);
			functionG.setDragMode(dragMode);
			
			enableDragModeButtons(false);
		}
		else if (command.equals("dilatey")) {
			dragMode.setValue(DragMode.DILATE_Y);
			functionG.setDragMode(dragMode);
			
			enableDragModeButtons(false);
		}
		else if (command.equals("reflectx")) {
			if (functionG instanceof QuadraticFunction){
				dragMode.setValue(DragMode.REFLECT_Y);	//REFLECT_Y  is  -f(x)  is  reflectX()
				functionG.setDragMode(dragMode);
				((QuadraticFunction)functionG).reflectX();
			
				enableDragModeButtons(false);
			}
		}
		else if (command.equals("reflecty")) {
			//hack. this method should be in graph function
			if (functionG instanceof QuadraticFunction){
				dragMode.setValue(DragMode.REFLECT_X);	//REFLECT_X  is  f(-x)  is  reflectY()
				functionG.setDragMode(dragMode);
				((QuadraticFunction)functionG).reflectY();
		
				enableDragModeButtons(false);
			}
		}
		else if (command.equals("startover")) {
			startOver(true);
		}
		else if (command.equals("cancel")) {
			accepted = false;
			dialog.hide();
		}
		else if (command.equals("ok")) {
			accepted = true;
			dialog.hide();
		}
	}	
	
	public void setOriginalFunction(QuadraticFunction original, Color newColor)
	{
		gflist.removeAllElements(false);
		if (original==null) {
			functionF = null;
			functionG = null;
			functionInputF.setGraphFunction(functionF);
			colorListenerF.setGraphFunction(functionF);
			functionInputGF.setGraphFunction(functionG);
			functionInputGF2.setGraphFunction(functionG);
			functionInputG.setGraphFunction(functionG);
			colorListenerG.setGraphFunction(functionG);
			return;
		}
				
		functionF = (QuadraticFunction)original.getCopy();
		functionF.setLineStroke(new BasicStroke(7.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));

		gflist.add(functionF,false,false);

		functionInputF.setGraphFunction(functionF);
		colorListenerF.setGraphFunction(functionF);
				
		functionG = new TransformedQuadraticFunction((QuadraticFunction)original);

		startOver(false);

		functionG.setColor(newColor);
//
		functionG.setLineStroke(functionG.getSelectedLineStroke());
		functionG.setHighlightedLineStroke(new BasicStroke(7.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));

		functionG.setFunctionName(functionF.getFunctionName());
		functionG.setFunctionOrdinal(functionF.getFunctionOrdinal()+1);
		
		functionInputGF.setGraphFunction(functionG);
		functionInputGF2.setGraphFunction(functionG);
		functionInputG.setGraphFunction(functionG);
		colorListenerG.setGraphFunction(functionG);
	
		gflist.add(functionG,false,false);		
	}
	
	public QuadraticFunction getTransformedFunction()
	{
		if (!accepted) return null;
		return (QuadraticFunction)functionG.getCopy();
	}
	
	public void showAsDialog()
	{
		if (dialog == null){
			dialog = new JDialog();
		}
		dialog.getContentPane().removeAll();
		dialog.getContentPane().add(this);
		dialog.setModal(true);
		dialog.setTitle("Function Transformation");
		dialog.setSize(600,600);
		try{
			dialog.setLocationRelativeTo(this);
		}catch(Throwable t){}

		//OK and Cancel buttons
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		JButton okButton = new JButton("OK");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		okButton.setPreferredSize(cancelButton.getPreferredSize());

		buttonPanel.add(cancelButton);
		buttonPanel.add(okButton);
		
		dialog.getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		accepted = false;
		dialog.show();
	}

	private void startOver(boolean reset)
	{
		dragMode.setValue(DragMode.NONE);
		if (functionG==null) return;
		
		functionG.setDragMode(dragMode);
		if (reset){
			functionG.resetFromOriginal();			
		}
		
		emptyRadioB.setSelected(true);
		enableDragModeButtons(true);
	}
	
	protected QuadraticFunction newQuadFunction()
	{
		///////////////////
		//Quadratic
		QuadraticFunction q;
		q = new QuadraticFunction();
		q.setExpressionForm(QuadraticFunction.FORM_POLYNOM);		
		q.setFunctionOrdinal(1);
		q.setFunctionName("f");
		q.setCoeffs(1,-4,3);
		///////////////////
		
		return q;
	}

	private GraphFunction copyFunction()
	{
		GraphFunction g=null;
		GraphFunction current = (GraphFunction)gflist.getLastSelectedObject();
		if (current!=null){
			g = (GraphFunction)current.getCopy();
			gflist.add(g);
		}
		//TEMP
		//((QuadraticFunction)g).setCoeffs(1,2,1);
		
		return g;
	}

	public void enableDragModeButtons(boolean b)
	{
		int i;
		Component c;
		if (dragModeContainer!=null){
			for (i=0;i<dragModeContainer.getComponentCount();i++){
				c = dragModeContainer.getComponent(i);
				c.setEnabled(b);
			}
		}
	}
	
	public boolean getAccepted()
	{
		return accepted;
	}
	
	public void setAllowSaveImage(boolean b)
	{
//		if (saveButton!=null){
//			saveButton.setVisible(b);
//		}
	}
	
	//Testing purposes
    public static void main(String args[]) {
		final JFrame frame = new JFrame();
		final QuadraticFunctionTransformerPanel fa = new QuadraticFunctionTransformerPanel();
		frame.getContentPane().add(fa);
		frame.setSize(600,600);
		frame.show();
		
		QuadraticFunction q = fa.newQuadFunction();
		q.setFunctionName("f");
		q.setColor(fa.gflist.getColorFactory().getNextColor());
		fa.setOriginalFunction(q,fa.gflist.getColorFactory().getNextColor());
		
		frame.addWindowListener( new WindowAdapter() {
			public void windowClosing(WindowEvent e){
				
				//ComponentScreenshot.saveScreenshotAsImageFile(fa, "c:\\test.png","png");
				
				System.exit(0);
			}			
		});
    }
	

}
