/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/*
 * Created on Aug 23, 2004
 *
 * Author: Eric Brown-Munoz
 *         Ingrid Moncada
 * 
 * $Revision: 1.2 $
 * 
 */
package org.concord.rttapplets.quadgrapher;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.*;
import javax.swing.*;
import org.concord.graphfunction.engine.*;
import org.concord.graphfunction.ui.*;
import org.concord.graphfunction.ui.symbolicview.*;
import org.concord.graph.ui.*;

import org.concord.graph.engine.*;



import javax.swing.JPanel;

/**
 * A Simple Panel to implement Ingrid's Quadratic Tansformer.
 * 
 */
public class SimpleQuadTransformer extends JPanel 
	implements ActionListener {

	QuadraticFunction curve;
	GraphArea ga;
	GraphWindow graph;
	Grid grid;
	
	Value variable;
	TransformedQuadraticFunction transformedCurve;
	
	
	
	
	/**
	 * Construct and Initialize a SimpleQuadTransformer Panel
	 */
	
	public SimpleQuadTransformer() {
		super();
		
		setLayout(new BorderLayout());
		
		JPanel graphPanel = new JPanel(new BorderLayout());
		
		// Graph and Grid
		graph = new GraphWindow();
		graph.setAntialias(true);
		ga = graph.getDefaultGraphArea();
		ga.setOriginCentered(true);
		grid = new Grid();
		graph.addDecoration(grid);
		
		// Variable
		variable = new Value();
		
		
		
		// Curve
		curve = new QuadraticFunction();
		curve.setColor(Color.red);
		graph.add(curve);
		
		transformedCurve = 
			new TransformedQuadraticFunction((QuadraticFunction)curve);
		curve.setColor(Color.green);
		graph.add(transformedCurve);
		
				
//		 Trace
		Trace trace = new Trace(curve, variable);
		graph.add(trace);
		
		// Slider
		ValueSlider slider = new ValueSlider(variable);
		slider.sync(ga);
		
		graphPanel.add(graph, BorderLayout.CENTER);
		graphPanel.add(slider, BorderLayout.SOUTH);
		
		add(graphPanel,BorderLayout.CENTER);
		
		// Controls
		JPanel controls = new JPanel();
		JButton startOver = new JButton("Start Over");
		startOver.setActionCommand("startover");
		
		controls.add(startOver);
		
		add(controls, BorderLayout.WEST);
		
		// Functions
		JPanel functionPanel = new JPanel();
		functionPanel.setLayout(new BoxLayout(functionPanel,BoxLayout.Y_AXIS));
		
		QuadraticPolynomialInput original = new QuadraticPolynomialInput();
		original.setEditable(false);
		original.setBorder(BorderFactory.createLineBorder(Color.red));
		original.setGraphFunction(curve);
		original.setLabelSize(14.0f);
		original.setShowFunctionName(true);
		
		TransformedFunctionSymbolic transformed = new TransformedFunctionSymbolic();
		transformed.setLabelSize(14.0f);
		transformed.setGraphFunction(transformedCurve);
		transformed.setShowFunctionName(true);
		transformed.setEditable(false);
		
		
		
		
	
		
		functionPanel.add(original);
		functionPanel.add(transformed);
		
		add(functionPanel,BorderLayout.SOUTH);
		
	}
	
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub

	}

	public static void main(String[] args) {
		JFrame frame = new JFrame("RTT Quadratic Transformer");
		SimpleQuadTransformer that = new SimpleQuadTransformer();
		frame.getContentPane().add(that);
		frame.pack();
		frame.validate();
		frame.show();
	}
		
}		

