/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.1 $
 * $Date: 2004/05/24 21:38:38 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.rttapplets.savestate;

import java.awt.*;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.awt.image.BufferedImage;

import javax.swing.*;
import javax.imageio.ImageIO;

import org.shetline.io.*;

/**
 * A class that takes a screen shot of a Component
 * and saves it as a jpg, png or gif image
 * @author Ingrid Moncada<p>
 *
   $Revision: 1.1 $
*/

public class ComponentScreenshot
{	
	public static void saveImageAsFile(Image image, String filename)
		throws Throwable
	{
		saveImageAsFile(image, filename, "");
	}
	
	public static void saveImageAsFile(Image image, String filename, String type)
		throws Throwable
	{
		FileOutputStream file;
		
		try{
			file = new FileOutputStream(filename); 
			
			if (type==null || type.equals("")){
				saveImageAsOutputStream(image, file);
			}
			else{
				saveImageAsOutputStream(image, file, type);
			}
				
			file.close();
			
		}catch(Throwable e){
			e.printStackTrace();
			throw e;
		}
	}

	/**
	 * Saves the image in the output stream in the "best" format
	*/
	public static void saveImageAsOutputStream(Image image, OutputStream out)
		throws Throwable
	{
		//Try to save it as .png first
		try{
			saveImageAsOutputStream(image, out, "png");
		}catch(Throwable e){
		//If it fails, try to save it as .gif
			saveImageAsOutputStream(image, out, "gif");
		}
	}

	/**
	 * Saves the image in the output stream in the given format
	*/
	public static void saveImageAsOutputStream(Image image, OutputStream out, String type)
		throws Throwable
	{		
		try{

			if (image instanceof BufferedImage){
				if (type.toLowerCase().equals("gif")) {
				//Saving as GIF using the third party library
				
					int errCode = GIFOutputStream.writeGIF(out,(BufferedImage)image, GIFOutputStream.STANDARD_256_COLORS);
					
					if (errCode!=GIFOutputStream.NO_ERROR){
					//Throw exception with the correct error message
						String errorMessage = "";
						if (errCode == GIFOutputStream.IMAGE_LOAD_FAILED){
							errorMessage = "Image load failed.";
						}
						else if (errCode == GIFOutputStream.TOO_MANY_COLORS){
							errorMessage = "Too many colors (more than 256).";
						}
						else if (errCode == GIFOutputStream.INVALID_COLOR_MODE){
							errorMessage = "Invalid color mode.";
						}
						else{
							errorMessage = "Error code "+errCode +".";
						}
						errorMessage = "Error while saving gif image: " + errorMessage;
						throw new Exception(errorMessage);						
					}
				} 
				else {
				//Try to save using ImageIO
				
					ImageIO.write(((BufferedImage)image),type,out);
				}
			}
				
		}catch(Throwable e){
			e.printStackTrace();
			throw e;
		}
	}

	public static void saveScreenshotAsImageFile(Component c, String filename, String type)
		throws Throwable
	{
		Image image = getScreenshot(c);
		saveImageAsFile(image, filename, type);
	}

	public static Image getScreenshot(Component c)
		throws Exception
	{
		return getScreenshot(c,false);
	}
	
	public static Image getScreenshot(Component comp, boolean includeParents)
		throws Exception
	{
		Component c;
		
		c = comp;
		if (includeParents){
			//Hack?
			while (c.getParent()!=null && c.getParent() instanceof JPanel){
				c = c.getParent();
			}
		}
		
		BufferedImage image = null;
		try{
			image = new BufferedImage(c.getWidth(), c.getHeight(), BufferedImage.TYPE_INT_RGB);
		
			// Create a graphics contents on the buffered image
			Graphics g = image.createGraphics();
	
			c.paint(g);
			
		}catch(Exception e){
			e.printStackTrace();
			throw e;
		}
		return image;
	}
	
}
