/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.1 $
 * $Date: 2004/05/24 21:38:38 $
 * $Author: imoncada $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.rttapplets.savestate;

import java.awt.image.*;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * 
 * @author Eric Brown-Munoz<p>
 * @author Ingrid Moncada<p>
*/
public class DataServer
{
    private String baseURL;
	private Throwable lastException;
	private ErrorStream errorStream;
	
	public DataServer(String url)
	{
		baseURL = url;
	}
	
	public DataServer(String url, ErrorStream errorStream)
	{
		this(url);
		this.errorStream = errorStream;
	}
	
	public ErrorStream getErrorStream()
	{
		return errorStream;
	}
	
	public boolean testUrlConnection()
	{
		return testUrlConnection(baseURL);
	}

	public static boolean testUrlConnection(String url)
	{
		if (url==null) return false;

		String strUrl="";
		URL urlTest;
		HttpURLConnection conn;
		InputStream s_response;
		int code = 0;

		try {
			//Testing a folder... I need to put the / at the end
			strUrl = url;
			if (strUrl.charAt(strUrl.length()-1) != '/'){
				strUrl = strUrl + "/";
			}
			
			//System.out.println("Testing url connection: "+strUrl+" ...");
			urlTest = new URL(strUrl);
			conn = (HttpURLConnection)urlTest.openConnection();
			code = conn.getResponseCode();
			System.out.println("Url connection OK: "+strUrl+" response code: "+code);
			if (!(code == 200 || code==403)){
				s_response = conn.getInputStream();
			}
			
			//System.out.println("Url folder OK!: "+strUrl);
		} catch(Exception e) {
			System.err.println("Trouble opening URL connection. Response code:"+code);
			e.printStackTrace();
			//lastException = e;
			return false;
		}

		//lastException = null;
		return true;
	}
	
	/**
	 * Returns each line of the output of the script as an element on the vector
	*/
	public Vector transferData(String script)
	{
		String strUrl="";
		URL url = null;
		URLConnection conn = null;
		OutputStream s_output = null;
		InputStream s_response = null;
	
		Vector retval = new Vector();
	
		try {
			strUrl = baseURL+"/"+script;
			url = new URL(strUrl);
			//System.out.println("Opening connection to: " + strUrl);
			conn = url.openConnection();
			conn.setDoOutput(false);
			s_response = conn.getInputStream();
		} catch(Throwable e) {
			System.err.println("Trouble opening URL connection: "+ strUrl);
			e.printStackTrace();
			if (errorStream != null){
				errorStream.println("Trouble opening URL connection: "+ strUrl);
				e.printStackTrace(new PrintStream(errorStream));
			}
			lastException = e;
			return null;
		}
	
		try {
			String contentType = conn.getContentType(); // ignore for now
	
			BufferedReader in = new BufferedReader( new InputStreamReader(s_response));
	
			String line;
			while ((line = in.readLine()) != null) {
				retval.addElement(line);
			}
		} catch (Throwable e) {
			System.err.println("Trouble with IO: " + strUrl);
			e.printStackTrace();
			if (errorStream != null){
				errorStream.println("Trouble with IO: "+ strUrl);
				e.printStackTrace(new PrintStream(errorStream));
			}
			lastException = e;
			return null;
		}
		
		lastException = null;
		return retval;//(String[])retval.toArray();
	}

	public String[] transferData(String script, Vector outputs)
	{
		String strUrl="";
		URL url = null;
		URLConnection conn = null;
		OutputStream s_output = null;
		InputStream s_response = null;
		String [] dummy = {""};
	
		Vector retval = new Vector();
	
		try {
			strUrl = baseURL+"/"+script;
			url = new URL(strUrl);
			conn = url.openConnection();
			conn.setDoOutput(true);
			s_output = conn.getOutputStream();
		} catch(Throwable e) {
			System.err.println("Trouble opening URL connection: "+ strUrl);
			e.printStackTrace();
			if (errorStream != null){
				errorStream.println("Trouble opening URL connection: "+ strUrl);
				e.printStackTrace(new PrintStream(errorStream));
			}
			lastException = e;
			return null;
		}
	
		try {
		    BufferedWriter out = 
			new BufferedWriter(new OutputStreamWriter(s_output));

		    // This is very kludgey! The bufferend image is
		    //   binary and MUST be the last thing sent.
		    //   This is true now, and will clean this up 
		    //   when we have time.
		    Enumeration eachout = outputs.elements();

		    boolean ifFirst = true;

		    while(eachout.hasMoreElements()) {
			Object each = eachout.nextElement();
			if (each == null){
				throw new NullPointerException("Trying to save a null object.");
			}
			else if (each instanceof BufferedImage) {
			    if (!ifFirst) {
				out.write("&");
			    }

			    out.flush();
				
				//Save the image with the "best" format
				ComponentScreenshot.saveImageAsOutputStream((BufferedImage)each, s_output);

			} else {
			    if (!ifFirst) {
				out.write("&");
			    }

			    ifFirst = false;

			    out.write(URLEncoder.encode(each.toString()));
			    // 'each' is usually a string -  toString() just returns self.
			}
		    }
			
		    out.close();
		    s_output.close();
	
		    s_response = conn.getInputStream();
		    String contentType = conn.getContentType(); // ignore for now

		    BufferedReader in = new BufferedReader( new InputStreamReader(s_response));
			
		    String line;
		    while ((line = in.readLine()) != null) {
			    retval.addElement(line);
		    }
		   

		    in.close();
		    s_response.close();

		} catch (Throwable e) {
		    System.err.println("Trouble with IO: " + strUrl);
			e.printStackTrace();
			if (errorStream != null){
				errorStream.println("Trouble with IO: "+ strUrl);
				e.printStackTrace(new PrintStream(errorStream));
			}
			lastException = e;
		    return null;
		}

		
		lastException = null;
		return (String[])retval.toArray(dummy);
	}

	public String getErrorMessage()
	{
		if (errorStream != null){
			return errorStream.toString();
		}
		
		OutputStream outp = new ByteArrayOutputStream();
		PrintStream printp = new PrintStream(outp);
		lastException.printStackTrace(printp);
		return outp.toString();
	}
	
	public void clearErrorMessage()
	{
		if (errorStream != null){
			errorStream.clear();
		}		
	}
}
