/* Copyright (c) 2004 The Concord Consortium, Inc., 
   10 Concord Crossing, Concord, MA 01741.

   This library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software 
   Foundation; version 2.1 of the License.

   This library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A 
   PARTICULAR PURPOSE. See the GNU Lesser General Public 
   License for more details.

   The GNU Lesser General Public License can be found at 
   http://opensource.org/licenses/lgpl-license.php; for a 
   paper copy, write to the Free Software Foundation, Inc., 
   59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/


/**
 * Last modification information:
 * $Revision: 1.4 $
 * $Date: 2004/08/11 15:14:37 $
 * $Author: ebrown-munoz $
 *
 * Copyright 2004 The Concord Consortium
*/
package org.concord.rttapplets.ui;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.io.PrintStream;

import javax.swing.*;
import javax.swing.event.*;

import org.concord.rttapplets.savestate.*;

import org.concord.graphfunction.ui.ResourceLoader;

/**
 *
 * 
 * @author Ingrid Moncada<p>
*/
public class SaveAsImagePanel extends JPanel 
	implements ActionListener, CaretListener
{
	private JComboBox cohort;
	private JTextField username;
	private JTextField title;
	private JTextArea comment;
	private ErrorStream errorStream;
	
	JButton okButton;
	
	private JDialog dialog;
	
	boolean accepted = false;
	
	SaveStateServer ds;
	Component screenComponent;
	String interactiveName;
	
	private String imageId;
	
	Image image;
	
	private boolean showImage = false;
	
	private boolean warnEmptyCohort = false;
	
	private Vector componentsToExclude;
	
	public SaveAsImagePanel(Component c, String interactiveName, SaveStateServer ds)
	{
		this(c,interactiveName);
		this.ds = ds;
		errorStream = ds.getErrorStream();
	}
	
	public SaveAsImagePanel(Component c, String interactiveName)
	{
		componentsToExclude = new Vector();
		this.screenComponent = c;
		this.interactiveName = interactiveName;
		
		JLabel titleLabel = new JLabel("<html><center>Please fill out the information you want <br>to save with this image.</html>");
		//titleLabel.setPreferredSize(new Dimension(80,80));
		JPanel titlePanel = new JPanel();
		JLabel imageLabel = new JLabel();
		imageLabel.setIcon(ResourceLoader.getImageIcon("camera1.gif",""));
		titlePanel.add(imageLabel);
		titlePanel.add(titleLabel);

		cohort = new JComboBox();
		cohort.addActionListener(this);
		username = new JTextField();
		username.addCaretListener(this);
		title = new JTextField();
		title.addCaretListener(this);
		comment = new JTextArea();
		comment.setLineWrap(true);
		comment.setRows(4);
		comment.setWrapStyleWord(true);
		JScrollPane commentPane = new JScrollPane(comment,
                    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                    JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		commentPane.setPreferredSize(null);
		
		JPanel fieldsPanel = new JPanel();
		
		//createConstraints(int gridx, int gridy, int gridwidth, int gridheight, int fill, double weightx, double weighty, int anchor)
		fieldsPanel.setLayout(new GridBagLayout());
		
		GridBagConstraints gbc;

		gbc = createConstraints(0,0,1,1,GridBagConstraints.HORIZONTAL,1,1, GridBagConstraints.WEST);
		gbc.insets = new Insets(0,15,0,0);
		fieldsPanel.add(new JLabel("Group"), gbc);
		gbc = createConstraints(0,1,1,1,GridBagConstraints.HORIZONTAL,1,1, GridBagConstraints.WEST);
		gbc.insets = new Insets(0,15,0,0);
		fieldsPanel.add(new JLabel("User Name"), gbc);
		gbc = createConstraints(0,2,1,1,GridBagConstraints.HORIZONTAL,1,1, GridBagConstraints.WEST);
		gbc.insets = new Insets(0,15,0,0);
		fieldsPanel.add(new JLabel("Title of the image"), gbc);
		gbc = createConstraints(0,3,1,1,GridBagConstraints.HORIZONTAL,1,1, GridBagConstraints.WEST);
		gbc.insets = new Insets(0,15,0,0);
		fieldsPanel.add(new JLabel("Comment"), gbc);
		
		gbc = createConstraints(1,0,1,1,GridBagConstraints.HORIZONTAL,5,1, GridBagConstraints.WEST);
		gbc.insets = new Insets(0,0,0,15);
		fieldsPanel.add(cohort, gbc);		
		gbc = createConstraints(1,1,1,1,GridBagConstraints.HORIZONTAL,5,1, GridBagConstraints.WEST);
		gbc.insets = new Insets(0,0,0,15);
		fieldsPanel.add(username, gbc);
		gbc = createConstraints(1,2,1,1,GridBagConstraints.HORIZONTAL,5,1, GridBagConstraints.WEST);
		gbc.insets = new Insets(0,0,0,15);
		fieldsPanel.add(title, gbc);
		gbc = createConstraints(1,3,1,1,GridBagConstraints.HORIZONTAL,5,1, GridBagConstraints.WEST);
		gbc.insets = new Insets(0,0,0,15);
		fieldsPanel.add(commentPane, gbc);
		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		okButton = new JButton("OK");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		okButton.setPreferredSize(cancelButton.getPreferredSize());
		okButton.setEnabled(false);

		buttonPanel.add(cancelButton);
		buttonPanel.add(okButton);
		
		setLayout(new BorderLayout());
		add(titlePanel, BorderLayout.NORTH);
		add(fieldsPanel, BorderLayout.CENTER);
		add(buttonPanel, BorderLayout.SOUTH);

		FillListThread f = new FillListThread();
		f.start();
	}
	
	public void addComponentToExclude(Component c)
	{
		if (c==null) return;
		if (!componentsToExclude.contains(c)){
			componentsToExclude.add(c);
		}
	}

	private void showComponentsToExclude(boolean b)
	{
		int i;
		for (i=0;i<componentsToExclude.size();i++){
			((Component)componentsToExclude.elementAt(i)).setVisible(b);
		}
	}
	
	public void showAsDialog()
	{
		//Warn in case the cohorts are empty
		if (!warnEmptyCohort && cohort.getItemCount()==0){
			showErrorMessageDialog("list");
			warnEmptyCohort = true;
			return;
		}

		if (dialog == null){
			dialog = new JDialog();
		}
		dialog.getContentPane().removeAll();
		dialog.getContentPane().add(this);
//		dialog.setModal(true);
		dialog.setTitle("Save screenshot as image");
		dialog.setSize(400,300);
		try{
			dialog.setLocationRelativeTo(this);
		}catch(Throwable t){}

		dialog.show();
	}

	private void showMessageDialog()
	{
		if (imageId==null || imageId.equals("")){
			showErrorMessageDialog("save");
		}
		else{
			showMessageDialogImageURL();
		}
	}
	
	private void showErrorMessageDialog(String type)
	{
		String imageURL;
		String[] strIntroText;
		String strMessage;
		String strTitle;
		
		if (type.equals("save")){
			strTitle = "Error saving image";
			strIntroText = new String[3];
			strIntroText[0] = "Sorry, your image could NOT be saved.";
			strIntroText[1] = "An error ocurred during saving the image.";
			strIntroText[2] = "You can save the following detailed information about the error and contact support.";
		}
		else{
			strTitle = "Error saving image";
			strIntroText = new String[2];
			strIntroText[0] = "An error ocurred. You probably won't be able to save images.";
			strIntroText[1] = "You can save the following detailed information about the error and contact support.";
		}
		
		if (ds==null){
			ds = new SaveStateServer(new ErrorStream());
			errorStream = ds.getErrorStream();
		}
		
		strMessage = ds.getErrorMessage();
		ds.clearErrorMessage();

		if (dialog != null){
			dialog.hide();
		}
		JFrame d = new MessageDialog(strTitle, strIntroText, strMessage);
	}

	private void showMessageDialogImageURL()
	{
		String imageURL;
		String[] strIntroText;
		String strMessage;
		String strTitle;

		System.out.println("SaveAsImagePanel in showMessageDialogImageURL");
		
		imageURL = AppProperties.getScriptsUrl()+"/"+ AppProperties.SCRIPT_URL_getImage +"?id="+imageId;
		strTitle = "Image saved succesfully";
		strIntroText = new String[1];
		strIntroText[0] = 
			"<html>" +
			"The image has been saved. To view the image, copy the URL<br>" +
			"below and paste it into a browswer window.<br><br>" +
			"To point other people to the saved image, copy the URL below<br>"+
			"and paste it into an e-mail or a posting to a discussion<br>"+
			"thread.<br><br>" +
			"You can also access this and other saved images (and<br>" +
			"determine their URL) at any time via<br>" +
			"http://internal.rtt.concord.org/Interactives/cgi-bin/showimages.";

		strMessage = imageURL;

		if (dialog != null){
			dialog.hide();
		}

		System.out.println("SaveAsImagePanel in showMessageDialogImageURL starting Frame");
		JFrame d = new MessageDialog(strTitle, strIntroText, strMessage);
		
		/*
		JDialog d = dialog;
		if (d == null){
			d = new JDialog();
		}

		String strText = "The image has been saved!<br>You can use the following link to view it:<br>";
		strText = "<html>"+strText+"</html>";
		JLabel titleLabel = new JLabel(strText);
		//JLabel imageIdLabel = new JLabel();
		JTextField imageIdLabel = new JTextField();
		imageIdLabel.setEditable(false);
		imageIdLabel.setBackground(Color.white);
		
		String strMessage;
		strMessage = AppProperties.getScriptsUrl()+"/getimage.php?id="+imageId;
		//strMessage = "<html><a href=\""+strMessage+"\">"+strMessage+"</a></html>";
		imageIdLabel.setText(strMessage);
		JButton okButton = new JButton("OK");
		okButton.setActionCommand("cancel");
		okButton.addActionListener(this);
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(okButton);
		
		d.getContentPane().removeAll();
		d.setSize(450,180);
		d.getContentPane().add(titleLabel, BorderLayout.NORTH);
		d.getContentPane().add(imageIdLabel, BorderLayout.CENTER);
		d.getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		d.setModal(true);
		try{
			d.setLocationRelativeTo(this);
		}catch(Throwable t){}
		d.show();
		*/
	}

    public void actionPerformed (ActionEvent e) 
	{
		if (e.getSource() == cohort){
			enableOkButton();
			return;
		}
		
		String command =((AbstractButton)e.getSource()).getActionCommand();
		accepted = command.equals("ok");
		
		if (!accepted){
			if (dialog!=null){
				dialog.hide();
				//dispose();
			}
			if (!showImage){
				return;
			}
		}

		saveAsImage();
				
		showMessageDialog();

		if (showImage){
			JLabel imageLabel = new JLabel();
			imageLabel.setIcon(new ImageIcon(image));
			JDialog d = new JDialog();
			d.setSize(800,600);
			d.getContentPane().add(imageLabel);
			d.setModal(true);
			d.show();
		}

		if (dialog!=null){
			//dialog.hide();
		}
	}
	
	public void caretUpdate(CaretEvent e)
	{
		enableOkButton();
	}
	
	private void enableOkButton()
	{
		if (username.getText().equals("")){
			okButton.setEnabled(false);
			return;
		}
		else if (title.getText().equals("")){
			okButton.setEnabled(false);
			return;
		}
		else{
			Object obj = cohort.getSelectedItem();
			if (obj instanceof Cohort){
			}
			else{
				okButton.setEnabled(false);
				return;
			}
		}
		okButton.setEnabled(true);
	}

	public boolean getAccepted()
	{
		return accepted;
	}
	
	private void saveAsImage()
	{
		if (ds==null){
			ds = new SaveStateServer(new ErrorStream());
			errorStream = ds.getErrorStream();
		}

		//Make some components invisible
		showComponentsToExclude(false);
		//
		
		try{
			image = ComponentScreenshot.getScreenshot(screenComponent, true);
			
		}catch(Exception e){
		//Error taking the screenshot
			showComponentsToExclude(true);
			e.printStackTrace();
			if (errorStream!=null){
				errorStream.println("Error taking the screenshot of the component: "+ screenComponent);
				e.printStackTrace(new PrintStream(errorStream));
			}
			imageId = "";
			return;
		}
		showComponentsToExclude(true);

		/*
		//TEST!!!!
		try{
			//test
			ComponentScreenshot.saveImageAsFile(image,"ingridtestnew.gif","gif");
			ComponentScreenshot.saveImageAsFile(image,"ingridtestnew.png","png");
			//
		}catch(Exception e){}
		*/
		
		String cohortId = "";
		
		if (cohort.getSelectedItem()!=null){
			Object obj = cohort.getSelectedItem();
			if (obj instanceof Cohort){
				cohortId = ((Cohort)obj).id;
			}
		}

		imageId = ds.saveStateAsImage(cohortId , 
			username.getText(), title.getText(), comment.getText(), interactiveName,
			image);
		
		System.out.println("Image id:"+imageId);
	}
	
	public void setShowImage(boolean b)
	{
		showImage = b;
	}
	
	class Cohort
	{
		String id;
		String name;
		
		public String toString(){ return name;}
	}

	private GridBagConstraints createConstraints(int gridx, int gridy, int gridwidth, int gridheight, 
		int fill, double weightx, double weighty, int anchor){
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = gridx;
		c.gridy = gridy;
		c.gridwidth = gridwidth;
		c.gridheight = gridheight;
		c.fill = fill;
		c.weightx = weightx;
		c.weighty = weighty;
		c.anchor = anchor;
		return c;
	}

	private void fillCohortList()
	{
		if (ds==null){
			ds = new SaveStateServer(new ErrorStream());
			errorStream = ds.getErrorStream();
		}
		
		Vector result = ds.getCohortList();

		if (result==null) {
			return;
		}
		
		cohort.removeAllItems();
		cohort.addItem("(Select Group)");
		
		int i;
		for (i=0; i<result.size(); i+=2){
			Cohort objCohort = new Cohort();
			objCohort.id = (String)result.elementAt(i); //result[i];
			objCohort.name = (String)result.elementAt(i+1); //result[i+1];
			cohort.addItem(objCohort);
		}
	}
	
	class FillListThread extends Thread{
		public void run()
		{
			fillCohortList();
		}
	}
}



